
package  giden.CORE;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;

/**
 * GdnTextWindow is a general purpose Frame for displaying
 * the contents of text files for various GIDEN subsytems.
 * 
 * @author Dilworth
 * @version 1.02
 */

public class GdnTextWindow extends Frame implements ActionListener, WindowListener
{
    /**
     * Text area holding the displayed text
     */
    TextArea Text;
    /**
     * Button user pushes to dismiss this frame.
     */
    Button OK_BUTTON;
    /**
     * Creates the popup frame with the given title, location,
     * and size.  The a TextArea is added with a vertical
     * scrollbar.  The text is currently hardcoded and set
     * to fit in frame area without needing the horizontal scroll
     * 
     * @param title text title for the frame
     * @param x frame x location
     * @param y frame y location
     * @param width frame width
     * @param height frame height
     */
    public GdnTextWindow(String path, String file, String title, int x, int y, int width, int height)
    {
        Create(path,file,title,x,y,width,height);
    }
    
    public GdnTextWindow(String path, String file, String title, Point point, int width, int height)
    {
        Create(path,file,title,point.x,point.y,width,height);
    }
    
    
    protected void Create(String path, String file, String title, int x, int y, int width, int height)
    {
        setTitle(title);
        setBounds(x,y,width,height);
        setLayout(new BorderLayout());
        OK_BUTTON = new Button("Dismiss");
        OK_BUTTON.addActionListener(this);
        addWindowListener(this);
        Text = new TextArea("",0,0,TextArea.SCROLLBARS_VERTICAL_ONLY);
        Text.setEditable(false);
        readHelpText(path,file);                   // create help text
        FlowLayout button_layout = new FlowLayout();
        button_layout.setVgap(1);
        Panel buttons = new Panel(button_layout);
        buttons.add(OK_BUTTON);
        add(BorderLayout.CENTER,Text);
        add(BorderLayout.SOUTH,buttons);  // add button
        
    }
    
    /**
     * Responds the user pushing the OK button.
     * Hides the frame
     */
    public void actionPerformed(ActionEvent event)
    {
        makeVisible(false);
    }
    /**
     * Creates help text.  Currently this text is hard coded.
     */
    void readHelpText(String path, String filename)
    {
        String line;
        GdnCsvInput file = new GdnCsvInput(path,filename);
        while(true)
        {
            line = file.readLine();
            if(line==null)
                break;
            Text.append(line);
            Text.append("\n");
        }
    }
    /** Makes the frame visible */
    public void makeVisible(boolean visible)
    {
        if(visible)
        {
            setVisible(false);
            setVisible(true);
            toFront();
        }
        else
        {
            setVisible(false);
        }
    }
    public void windowActivated(WindowEvent event)
    {
        println("windowActivated "+event.toString());
        
    }
    public void windowClosed(WindowEvent event)
    {
        println("windowClosed "+event.toString());
    }
    public void windowClosing(WindowEvent event)
    {
        setVisible(false);
        //println("windowClosing "+event.toString());
    }
    public void windowDeactivated(WindowEvent event)
    {
        //setVisible(false);
        println("windowDeactivated "+event.toString());
    }
    public void windowDeiconified(WindowEvent event)
    {
    }
    public void windowIconified(WindowEvent event)
    {
        println("windowIconified "+event.toString());
        
    }
    public void windowOpened(WindowEvent event)
    {
        println("windowOpened "+event.toString());
        
    }   
    /** print diagnostic messages */
    private void println(String message)
    {
        //System.out.println(message);
    }
}
