package  giden.CORE;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
import giden.userSOLVERS.*;

/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * SolverMenuBase is a base class for creating and menu
 * entries for types of problems and specific solvers
 * of those problems.  Implementors use the CreateMenu
 * method in the SolverMenu class to add problems and solvers
 * to the GIDEN Solver menu.  SolverMenu inherits
 * this class and makes direct calls in to it.  This
 * class provides two methods AddProblem and AddSolver
 * that must be called in the correct order.  The AddProblem
 * method is called first and creates a menu
 * selection inside the GIDEN Solver menu.  After a
 * method has been created, calls to AddSolver will
 * create menu selections for that type of problem.
 * After a user has selected a solver, GIDEN will call
 * the CreateSolver method in the SolverMenu class and
 * pass it the name of the solver that has been selected.
 * The implementor is responsible for creating that solver
 * and returning its base class pointer to GIDEN.
 * <p>
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @see Network
 * @see SolverMenu
 *
 * @version 	4.0a 2004 08 27
 * @author 	    D. S. Dilworth
 */


public class SolverMenuBase
{
    final int MaxProblems=16;
    final int MaxSolvers=16;
    String Problems[];
    String Solvers[][];
    boolean Available[][];
    int ProblemIndex;
    int SolverIndex;
    int LastSolverIndex;

    public SolverMenuBase()
    {
        Problems = new String[MaxProblems];
        Solvers = new String[MaxProblems][MaxSolvers];
        Available = new boolean[MaxProblems][MaxSolvers];     
        ProblemIndex=0;
        SolverIndex=0;
        LastSolverIndex=-1;
               
        /*        
        AddProblem("Problem#1");
        AddSolver("Solver1-1",true);
        AddSolver("Solver1-2",true);
        AddProblem("Problem#2");
        AddSolver("Solver2-1",false);
        AddSolver("Solver2-2",true);
        
        String problem,solver;
        problem=FirstProblem();
        while(problem!=null)
        {
            solver=FirstSolver();
            while(solver!=null)
            {
                solver=NextSolver();
            }
            problem=NextProblem();
        }
        */
        
    }
    //----------------------------------------------------------
    /** Adds the name for a Problem-type to the GIDEN Solver menu.
     * The given name will be added as a menu pick in the Solver menu.
     * This method is used to add Problem-types to the GIDEN Solver menu.
     * The AddSolver method adds particular Solvers to the submenu
     * of the most recently added Problem-type menu.
     * @param name  The name of the Problem-type to appear on the Solver menu
     * @see SolverMenuBase#AddSolver */
    //----------------------------------------------------------
    public void AddProblem(String name)
    {
        Problems[ProblemIndex]=name;
        ProblemIndex++;
        SolverIndex=0;
    }
    //----------------------------------------------------------
    /** Adds the name for a particular Solver to the GIDEN Solver->Problem submenu.
     * The given name will be added as a menu pick below the
     * most recent Problem-type specified through the AddProblem method.
     * This method is used to add Solver names to GIDEN Solver->Problem submenu.
     * The AddProblem method creates the Problem-Type menu picks,
     * and must be called prior to calling AddSolver
     * @param name       The name of the Solver to appear on the Solver->Problem submenu
     * @param available  True if this Solver menu should be activated
     * @see SolverMenuBase#AddProblem */
    //----------------------------------------------------------
    public void AddSolver(String name, boolean available)
    {
        Solvers[ProblemIndex-1][SolverIndex]=name;
        Available[ProblemIndex-1][SolverIndex]=available;
        SolverIndex++;
    }
    //----------------------------------------------------------
    /** Adds the name for a particular Solver to the GIDEN Solver->Problem submenu.
     * The given name will be added as a menu pick below the
     * most recent Problem-type specified through the AddProblem method.
     * This method is used to add Solver names to GIDEN Solver->Problem submenu.
     * The AddProblem method creates the Problem-Type menu picks,
     * and must be called prior to calling AddSolver
     * @param name       The name of the Solver to appear on the Solver->Problem submenu
     * @see SolverMenuBase#AddProblem */
    //----------------------------------------------------------
    public void AddSolver(String name)
    {
        AddSolver(name,true);
    }
    /** Internal method */
    public String FirstProblem()
    {
        ProblemIndex=0;
        return NextProblem();
    }
    /** Internal method */
    public String NextProblem()
    {
        SolverIndex=0;
        LastSolverIndex=-1;
        return Problems[ProblemIndex++];
    }
    /** Internal method */
    public String FirstSolver()
    {
        SolverIndex=0;
        return NextSolver();
    }
    /** Internal method */
    public String NextSolver()
    {
        LastSolverIndex=SolverIndex;
        return Solvers[ProblemIndex-1][SolverIndex++];
    }
    boolean GetAvailable()
    {
        return Available[ProblemIndex-1][LastSolverIndex];
    }
}

