package  giden.CORE;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;
public class SolverServices extends SolverThread
{
    UserBase UserBaseArray[];
    int UserBaseCount;
    int UserBaseMax;
    String HelpFile;
    
    SolverServices(GUISolver gui)
    {
        super(gui);
        UserBaseMax=10;
        UserBaseArray=null;
        RemoveUserBases();
        Network net = GUI.GetUiNet().GetNet();
        if(net.GetSolverServices()==null)
            net.PutSolverServices(this);
    }
    Network GetNet()
    {
        return GUI.GetUiNet().GetNet();
    }
    
    UiNodeInfo GetDefaultUiNodeInfo()
    {
        return GUI.GetDefaultUiNodeInfo();
    }
    
    UiEdgeInfo GetDefaultUiEdgeInfo()
    {
        return GUI.GetDefaultUiEdgeInfo();
    }
    
    public void AddSolverMenuInfo(String filename)
    {
        GUI.AddSolverMenuInfo(filename);
    }
    
    public void AddSolverMenuInfo()
    {    
        String dotname;
        dotname = GetSolver().getClass().getName();
        String slashname = dotname.replace('.','/');
        HelpFile = "/"+slashname+".html";
        AddSolverMenuInfo(HelpFile);
    }
    
    
    
    /*-----------------------------------------------------------
    | USERBASE SUPPORT
    -----------------------------------------------------------*/
    
    public void RemoveUserBases()
    {
        if(UserBaseArray==null)
            UserBaseArray = new UserBase[UserBaseMax];
        for(int i=0;i<UserBaseMax;i++)
            UserBaseArray[i]=null;
        UserBaseCount=0;
    }
    /** internal method */
    public void DeleteEdgeX(Edge edge)
    {
        boolean success=false;

        for(int i=0;i<UserBaseMax;i++)
        {
            if(UserBaseArray[i]!=null)
            {
                UserIO user_info = (UserIO) UserBaseArray[i];
                user_info.GetUiNet().DeleteEdgeX(edge);
                success=true;
            }
        }
        if(success==false)      // in gui mode .. not UserBaseArrays ready;
        {
            GUI.GetUiNet().DeleteEdgeX(edge);
        }
    }
    
    /** internal method */
    public void DeleteNodeX(Node node)
    {
        boolean success=false;
        for(int i=0;i<UserBaseMax;i++)
        {
            if(UserBaseArray[i]!=null)
            {
                UserIO user_info = (UserIO) UserBaseArray[i];
                user_info.GetUiNet().DeleteNodeX(node);
                success=true;
            }
        }
        if(success==false)      // in gui mode .. not UserBaseArrays ready;
        {
            GUI.GetUiNet().DeleteNodeX(node);
        }
    }
    
    void RemoveUserBase(UserBase user)  // doesn't inform UserBase
    {
        for(int i=0;i<UserBaseMax;i++)
        {
            if(UserBaseArray[i]==user)
            {
                UserBaseArray[i]=null;
                UserBaseCount--;
                break;
            }
        }
    }
    public void AddUserBase(UserBase user)
    {
        boolean success=false;
        for(int i=0;i<UserBaseMax;i++)
        {
            if(UserBaseArray[i]==null)
            {     
                UserBaseArray[i]=user;
                UserBaseArray[i].GetNetworkDrawInfo();
                UserBaseArray[i].ClearSetArrays();
                UserBaseCount++;
                success=true;
                break;
            }
        }
        if(success==false)
        {
            // it wasn't found
        }
    }
    
    /** internal method */
    public void RestoreNetworks()
    {
        for(int i=0;i<UserBaseMax;i++)
        {
            if(UserBaseArray[i]!=null)
            {
                UserBaseArray[i].RestoreNetwork();
            }
        }
    }
    void DrawNetworks()
    {
        for(int i=0;i<UserBaseMax;i++)
        {
            if(UserBaseArray[i]!=null)
            {
                UserBaseArray[i].DrawNetwork();
            }
        }
    }
    
   // void StartOfSolver()
   // {
   // }
    
    /** Creates an Auxilary window with the given window name and Network */
    public UserBase CreateAuxWindow(String name, Network net)
    {
        return CreateAuxWindow(name,new GUINetwork(net));
    }
    
    private UserBase CreateAuxWindow(String name, GUINetwork aux)
    {
    // create the solver window
        GUISolver aux_window = (GUISolver) GUI.parent.createSolverWindowDsd(name, aux);
        aux_window.setupAuxMode();
    // strip the menu
        int n = aux_window.mb.getMenuCount();
        for(int i=0;i<n;i++)
            aux_window.mb.remove(n-i-1);
    // redraw, how is the size set?
        aux_window.pack();
        aux_window.repaint();
    // get the UserBase
    aux_window.SolverControl=this;
        UserBase base = new UserIO((SolverServices)aux_window.SolverControl,aux_window);
    // assign UserBase to this GUINetwork
        aux.UserInfo=base;  
    // setup animation
        base.AnimationArray=UserBaseArray[0].AnimationArray;
        base.AnimationTotal=UserBaseArray[0].AnimationTotal;
    // add UserBase to Services
        AddUserBase(base);
        return base;
    }
    
    /** Deletes and Auxilary window created by a user solver */
    public void DeleteAuxWindow(UserBase user)
    {
        if(user!=null)
        {
            GUISolver gui = user.GetGUI();
            if(gui!=null)
            {
                gui.windowClosing(null);
            }
        }
        
    }
    //-----------------------------------------------------------
    /* Display the given line in the Pseudcode window */
    //-----------------------------------------------------------
    /** internal method */
    public void ShowPseudoCodeText(int index)
    {
        GUI.ShowPseudoCodeText(index);
    }
    //-----------------------------------------------------------
    /* Display the given line in the Pseudcode window */
    //-----------------------------------------------------------
    /** internal method */
    public void ShowPseudoCodeText(String token)
    {
        GUI.ShowPseudoCodeText(token);
    }
    void Repaint(String who)
    {
        GUI.Repaint(who);
    }
    String GetActionMode()
    {
        if(LastWasPcNext)
            return GetNextString();
        return GUI.getAnimationMode();
    }
}
