package  giden.CORE;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.util.Hashtable;

/*==============================================================
| UiNodeEdgeIdManager class
|---------------------------------------------------------------
| Provides management of unique node and edge identifiers
|---------------------------------------------------------------
==============================================================*/
public class UiNodeEdgeIdManager
{
    // hashtables to check uniqueness of identifiers quickly
    private Hashtable node_id;
    private Hashtable edge_id;

    // private information used to generate unique identifiers; this
    // may change as we modify our requirements for this identifier
    // class to accomodate other design concerns (e.g., merging
    // networks)
    private int n_count;  // counter used to generate unique node id
    private int e_count;  // counter used to generate unique edge id
    private String n_prefix;  // user-specified node prefix text
    private String e_prefix;  // user-specified edge prefix text


    //----------------------------------------------------------
    //----------------------------------------------------------
    UiNodeEdgeIdManager()
    {
        node_id = new Hashtable();
        edge_id = new Hashtable();

        n_count = 1;
        e_count = 1;
        n_prefix = new String("");
        e_prefix = new String("");
    }
    
    void DsdClearNodes()
    {
        node_id = new Hashtable();
        n_count=1;
    }
    
    
    void DsdClearEdges()
    {
        edge_id = new Hashtable();
        e_count=1;
    }
    
    void DsdPresetNodeId(int id)
    {
        n_count=id;
    }
    void DsdPresetEdgeId(int id)
    {
        e_count=id;
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    UiNodeEdgeIdManager(String np, String ep)
    {
        this();

        n_prefix = new String(np);
        e_prefix = new String(ep);
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    private void resetEdgeCount() {e_count = 1;}
    private void resetNodeCount() {n_count = 1;}

    //----------------------------------------------------------
    //----------------------------------------------------------
    public void SetNodeIdPrefix(String np) {
        resetNodeCount();
        n_prefix = new String(np);
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    public void SetEdgeIdPrefix(String ep) {
        resetEdgeCount();
        e_prefix = new String(ep);
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiNode PutNodeId(UiNode node, String id) 
    {
        resetNodeCount();
        return (UiNode) node_id.put(id,node);
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiEdge PutEdgeId(UiEdge edge, String id)
    {
        resetEdgeCount();
        return (UiEdge) edge_id.put(id,edge);
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    public String GenerateNodeId(UiNode node) 
    {
        String id;

        do {
            id = new String(n_prefix+Integer.toString(n_count++));
        } while (node_id.containsKey(id));

        node_id.put(id,node);

        return id;
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    public String GenerateEdgeId(UiEdge edge) 
    {
        String id;

        do {
            id = new String(e_prefix+Integer.toString(e_count++));
        } while (edge_id.containsKey(id));

        edge_id.put(id,edge);

        return id;
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    public void ClearNodeId(String key) 
    {
        resetNodeCount();
        node_id.remove(key);
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    public void ClearEdgeId(String key) 
    {
        resetEdgeCount();
        edge_id.remove(key);
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiNode GetNode(String key) {
        return (UiNode) node_id.get(key);
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiEdge GetEdge(String key) {
        return (UiEdge) edge_id.get(key);
    }
}
