package  giden.CORE;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
import java.awt.Rectangle;
import java.awt.Color;
import java.awt.Point;

/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * UserIO is class that provides methods for interacting with the user.
 * <p>
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @version 	4.0a 2004 08 27
 * @author 	    D. S. Dilworth
 */



public class UserIO extends UserBase
{
    protected NodeArray NodeSetIndex;
    protected EdgeArray EdgeSetIndex;
    protected String StatusString;
    protected UiNodeArray SolverNodeInfo;
    protected UiEdgeArray SolverEdgeInfo;
    protected UiNodeArray NetworkNodeInfo;
    protected UiEdgeArray NetworkEdgeInfo;
    protected GUISolver GUI;
    protected UiNetwork UiNet;
    protected String PcToken;
    protected int PcIndex;
    protected UiSolverResult Result;
    protected RowDialogMain RowDialog;
    
    /*===========================================================
    | NEW METHODS
    ===========================================================*/
    
    /** Returns the default draw information for nodes */
    public UiNodeInfo GetDefaultUiNodeInfo()
    {
        return Services.GetDefaultUiNodeInfo();
    }
    
    /** Returns the default draw information for edges */
    public UiEdgeInfo GetDefaultUiEdgeInfo()
    {
        return Services.GetDefaultUiEdgeInfo();
    }
    
    /** Add the node to the window at the (x,y), and using the
     default draw information. */
    public UiNode AddNode(Node node, int x, int y)
    {
        Point pt = new Point(x,y);
        return AddNode(node,pt);
    }
    
    /** Add the node to the window at the given point, and using the
        default draw information */
    public UiNode AddNode(Node node, Point pt)
    {
        UiNode result=null;
        
        UiNodeInfo nodeinfo = GetDefaultUiNodeInfo();
        nodeinfo.x=pt.x;
        nodeinfo.y=pt.y;
        nodeinfo.label=new Integer(node.GetIndex()).toString();
        if(UiNet.CreateNode(node,nodeinfo))
            result=UiNet.Node2UiNode(node);
        return result;
            
    }
    /** Add the node to the window, draw as defined by info  */
    public UiNode AddNode(Node node, UiNodeInfo info)
    {
        UiNode result=null;
        if(UiNet.CreateNode(node,info))
            result=UiNet.Node2UiNode(node);
        return result;
    }
    
    /** Add the edge to the window  */
    public UiEdge AddEdge(Edge edge)
    {
        UiEdge result=null;
        UiEdgeInfo edgeinfo = GetDefaultUiEdgeInfo();
        edgeinfo.label = new Integer(edge.GetIndex()).toString();
        return AddEdge(edge,edgeinfo);
    }
 
    
    /** Add the edge to the window, draw as defined by info  */
    public UiEdge AddEdge(Edge edge, UiEdgeInfo info)
    {
        UiEdge result=null;
        if(UiNet.CreateEdge(edge,info))
            result=UiNet.Edge2UiEdge(edge);
        return result;
    }
    
    /** Returns true if the node is a member of this window */
    public boolean IsNodeMember(Node node)
    {
        UiNodeInfo info = GetNodeInfo(node);
        return info!=null;
    }
        
    /*=========================================================*/
    
    
    /** Creates a user window, called from SolverMenu */
    public UserIO(SolverServices services,GUISolver gui)
    {
        super(services);
        GUI=gui;
        UiNet=GUI.UiNet;
        Result=null;
        NullBase=false;     // true for UserIO object
    }
    protected void Clear()
    {
        super.Clear();
        StatusString = "";
        PcToken=null;
        PcIndex=-1;
        ClearSetArrays();
    }
    protected void ClearSetArrays()
    {
        NodeSetIndex = new NodeArray(100,"NodeSetIndex");
        EdgeSetIndex = new EdgeArray(100,"EdgeSetIndex");
    }
    
    /*===========================================================
    | Methods supported by UserBase
    | - placed here for JavaDoc
    ===========================================================*/
    
    public RowDialogMain CreateRowDialog(String name)
    {
        ExecBase solver = Services.GetSolver();
        RowDialog = new RowDialogMain(name,this,GetTopRight(),solver);
        return RowDialog;
    }
    
    /** Create a named AnimationSet and define color */
    public ColorSet CreateColorSet(Color color)
    {
        return super.CreateColorSet(color);
    }
    /** Create a named final AnimationSet and define color */
    public FinalSet CreateFinalSet(String label, Color color)
    {
        return super.CreateFinalSet(label,color);
    }
    /** Create a named AnimationSet and define color */
    public AnimationSet CreateAnimationSet(String label, Color color)
    {
        return super.CreateAnimationSet(label,color);
    }
    /** Internal Method */
    public void NotifyOfSolverWarning(String message)
    {
        super.NotifyOfSolverWarning(message);
    }
    /** Internal Method */
     public void NotifyOfSolverError(String message)
    {
        super.NotifyOfSolverError(message);
    }
    /** Internal Method */
    public void NotifyOfSolverNotice(String message)
    {
        super.NotifyOfSolverNotice(message);
    }
    /** Create a named final AnimationSet and define color */
    /** Create a named AnimationSet and define color */
    
    
    /*===========================================================
    ===========================================================*/
    
    public Network GetNet()
    {
        return GetUiNet().GetNet();
    }
    
    /** Gets the UiNet for this window */
    public UiNetwork GetUiNet()
    {
        return UiNet;
    }
    /** Returns to GUI for this window */
    public /*protected*/ GUISolver GetGUI()
    {
        return GUI;
    }
    
    /** Internal Method */
    public Point GetTopRight()
    {
        Rectangle bounds = GetGUI().getBounds();
        Point point = new Point();
        point.x = bounds.x+bounds.width;
        point.y = bounds.y;
        return point;
    }
    
    
    
    /*===========================================================
    | UPDATE METHODS
    ===========================================================*/
    
    /** Update the UI to show changes to animation controls */
    public void Update()
    {
        int i;
        boolean drawnetwork=false;
        String label=Services.GetActionMode();
        if(label.equals(Services.GetTraceString()))
        {
            for(i=0;i<AnimationTotal;i++)
            {
                if(AnimationArray[i].IsBreakOn())
                {
                    AnimationArray[i].PutTraced(false);
                    AnimationArray[i].PutChanged(false);
                    drawnetwork=true;
                }
            }
        }
        if(label.equals(Services.GetContinuousAnimationGoString()))
        {
            Services.Message("UserIO.Update() Illegal GO state");
            drawnetwork=true;
        }
        if(label.equals(Services.GetContinuousAnimationPauseString()) &&
           !Services.GUI.isAnimationDetailEnabled())
        {
            for(i=0;i<AnimationTotal;i++)
            {
                if(AnimationArray[i].IsBreakOn())
                {
                    AnimationArray[i].PutTraced(false);
                    AnimationArray[i].PutChanged(false);
                    drawnetwork=true;
                }
            }
        }
        if(label.equals(Services.GetFinalString())) // move earlier and return
        {
            drawnetwork=false;
        }
        if(drawnetwork)
        {
            for(i=0;i<AnimationTotal;i++)   // crc->dsd 010509
            {
                {
                    AnimationArray[i].PutTraced(false);
                    AnimationArray[i].PutChanged(false);
                }
            }
            
            Services.DrawNetworks();
            Services.WaitForActionButton();
        }
    }

    protected void StepUpdate()
    {
        String label=Services.GetActionMode();
        if(label.equals(Services.GetStepString()))
        {
            DrawNetwork();
            Services.WaitForActionButton();
        }
        // owenj (24-May-1999): Dave agrees to clean up so that this
        // isn't circumventing the isolation layer
        if(label.equals(Services.GetContinuousAnimationPauseString())
           && Services.GUI.isAnimationDetailEnabled()) {
            DrawNetwork();
            Services.WaitForActionButton();
        }
    }
    
    protected void PcNextUpdate()
    {
        String label=Services.GetActionMode();
        if(label.equals(Services.GetNextString()))
        {
            DrawNetwork();
            Services.WaitForActionButton();
        }
    }

    
    /*===========================================================
    | ANIMATION AND FINAL SETS
    ===========================================================*/

    //-----------------------------------------------------------
    /** Create animation sets */
    //-----------------------------------------------------------
    public void CreateAnimationButtons()
    {
        Color color[] = new Color[SetMax];
        String label[] = new String[SetMax];
        int i;
        
        for(i=0;i<AnimationTotal;i++)
        {
            //color[i]=AnimationColors[i];
            //label[i]=AnimationLabels[i];
            color[i]=AnimationArray[i].GetColor();
            label[i]=AnimationArray[i].GetLabel();
            
        }
        GUISetSwatch[] sets = CreateAnimationSetButtons(
            color,label,AnimationTotal);
        
        for(i=0;i<AnimationTotal;i++)
        {
            AnimationArray[i].PutSwatch(sets[i]);
            AnimationArray[i].PutIndex(i);
        }
    }
    //-----------------------------------------------------------
    /** Create animation sets */
    //-----------------------------------------------------------
    public  void CreateFinalButtons()
    {
        Color color[] = new Color[SetMax];
        String label[] = new String[SetMax];
        int i,j;
        
        j=0;
        for(i=0;i<FinalTotal;i++)
        {
            if(FinalArray[i].GetEnabled())
            {
                color[j]=FinalArray[i].GetColor();
                label[j]=FinalArray[i].GetLabel();
                j++;
            }
        }
        CreateFinalButtons(color,label,j);
    }

    GUISetSwatch[] CreateAnimationSetButtons(
        Color colors[],String labels[],int count)
    {
        GUISetSwatch result[] = GUI.createAlgorithmAnimationSetButtons(
            colors,labels,count);
        return result;
    }
    protected void CreateFinalButtons(
        Color colors[],String labels[],int count)
    {
        GUI.createFinalAnimationSetButtons(colors,labels,count);
    }
    /** Update an AnimationSet */
    public void Trace(AnimationSet set)
    {
        set.PutTraced(true);
    }
    /** Single step trace all sets */
    public void SingleStep()
    {
        int i;
        for(i=0;i<AnimationTotal;i++)
        {
            AnimationArray[i].PutChanged(true);
        }
        StepUpdate();
    }
    /**
     * Single step trace this animation set
     * 
     * @param set the animation set to mark as stepped
     */
    public void SingleStep(AnimationSet set)
    {
        set.PutChanged(true);
        StepUpdate();
    }
    /**
     * Single step trace this node and set
     * 
     * @param node the node to move to the given set
     * @param set the animation set to mark as stepped
     */
    public void SingleStep(Node node, AnimationSet set)
    {
        if(MoveToSet(node,set))
        {
            set.PutChanged(true);
            StepUpdate();
        }
    }
    /**
     * Single step trace this edge and set
     * 
     * @param edge the nedge to move to the given set
     * @param set the animation set to mark as stepped
     */
    public void SingleStep(Edge edge, AnimationSet set)
    {
        if(MoveToSet(edge,set))
        {
            set.PutChanged(true);
            StepUpdate();
        }
    }
    /**
     * Move this node to the animation set
     * 
     * @param node the node to move to a new set
     * @param set the set to move the node into
     */
    public boolean MoveToSet(Node node, AnimationSet set)
    {
        boolean result=false;
        
        Object obj = NodeSetIndex.GetObj(node);
        if(obj!=null)
        {
            int oldset = NodeSetIndex.GetInt(node);
            if(oldset==set.GetIndex())
                return result;
        }
        if(UiNet.Node2UiNode(node)!=null)
        {
            NodeSetIndex.Put(node,set.GetIndex());
            SetNodeColor(node,set.GetColor());
            result=true;
        }
        return result;
    }
    /**
     * Move this edge to this set
     * 
     * @param edge the edge to move to a new set
     * @param set the set to move the edge into
     */
    public boolean MoveToSet(Edge edge, AnimationSet set)
    {
        boolean result=false;
        
        Object obj = EdgeSetIndex.GetObj(edge);
        if(obj!=null)
        {
            int oldset = EdgeSetIndex.GetInt(edge);
            if(oldset==set.GetIndex())
                return result;
        }   
        if(UiNet.Edge2UiEdge(edge)!=null)
        {
            EdgeSetIndex.Put(edge,set.GetIndex());
            SetEdgeColor(edge,set.GetColor());
            result=true;
        }
        return result;
    }
    /*
    Removes the node from any set that it is a member of
    public boolean RemoveFromSet(Node node)
    {
        boolean result=false;
        UiNode ui_node = UiNet.Node2UiNode(node);
        if(ui_node!=null);
        {
            ///CODED, NOT TESTED
            Object obj = NodeSetIndex.GetObj(node);
            if(obj!=null)
            {
                int oldset = EdgeSetIndex.GetInt(node);
                AnimationSet set = AnimationArray[oldset];
                set.PutChanged(true);
                UiNodeInfo info = (UiNodeInfo)NetworkNodeInfo.GetObj(ui_node);
                NodeSetIndex.Put(node,null);  // null?
                SetNodeColor(node,info.color);      // get color from old set
                result=true;
            }
        }
        return result;
    }

    // Removes the node from any set that it is a member of
    public boolean RemoveFromSet(Edge edge)
    {
        return false;
    }
     */
    
    /**
     * Move all nodes and edges into the given set
     * 
     * @param set the set to move all nodes and edge into
     */
    public void MoveAll(AnimationSet set)
    {
        Network Net = UiNet.GetNet();
        
        Node node;
        NodeListIter nit = Net.GetNodeListIter();    // node list iterator
        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
            MoveToSet(node,set);
            
        Edge edge;
        EdgeListIter eit = Net.GetEdgeListIter();    // edge list iterator
        for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
            MoveToSet(edge,set);
    }
        
    /*===========================================================
    | STATUS LINE AND NODE/EDGE CLICKS
    ===========================================================*/
    
    /** Copies the message to the status line buffer */
    public void StatusLine(String message)
    {
        StatusString=message;
    }
    /**
     * Clear the status line
     */
    public void ClearStatusLine()
    {
        StatusLine("");
    }
    /** Prompt user with default answer, wait for text, returns text */
    public String UserTextInput(String question,String answer)
    {
        DrawNetwork();
        return Services.UserTextInput(question,answer);
    }
    /** Prompt user, wait for mouse click, returns coordinates */
    public Point UserMouseInput(String question)
    {
        return null;
    }
    /** Prompts the user and returns the clicked node or null */
    public Node UserMouseNodeInput(String prompt)
    {
        ClearStatusLine();
        DrawNetwork();
        Node result=null;
        
        String who="UserMouseNodeInput("+prompt+")";
        Services.Message(who);
        GUI.UserMouseNodeInput(prompt);
        String text=Services.EventWait("UserMouseNodeInput");
        Services.Message(who+" -> "+text);
        if(GUI.ClickedNode!=null)
        {
            result=UiNet.UiNode2Node(GUI.ClickedNode);
        }
        return result;
    }
    /** Prompts the user and returns the clicked edge or null */
    public Edge UserMouseEdgeInput(String prompt)
    {
        ClearStatusLine();
        DrawNetwork();
        Edge result=null;
        
        String who="UserMouseEdgeInput("+prompt+")";
        Services.Message(who);
        GUI.UserMouseEdgeInput(prompt);
        String text=Services.EventWait("UserMouseEdgeInput");
        Services.Message(who+" -> "+text);
        if(GUI.ClickedEdge!=null)
        {
            result=UiNet.UiEdge2Edge(GUI.ClickedEdge);
        }
        return result;
    }

    /*===========================================================
    | PSEUDO CODE AND DIALOG BOX
    ===========================================================*/
            
    /**
     * Display the line with given token in the PseudoCode window
     * 
     * @param token the string token associated with the text line
     */
    public void ShowPseudoCodeText(String token)
    {
        PcToken=token;
        PcIndex=-1;
        PcNextUpdate();
    }
    
    /**
     * Create a dialog box and wait for user response
     * 
     * @param textname the text name for the solver
     * @param solver the base class for the solver
     * @param net the network the solver is currently processing
     * @param input class with solver input information
     */
    public void CreateSolverInputDialog(String textname, ExecBase solver, Network net, GidenSolverInput input)
    {
        new GUISolverInputDialog(net.UiNet.GUI, textname,solver, net, input);
    }

   
    /*===========================================================
    | NODE/EDGE INFO AND PRIMITIVES
    ===========================================================*/

    /** Gets draw info for the given node */
    public UiNodeInfo GetNodeInfo(Node node)
    {
        UiNodeInfo info=null;
        UiNode uinode = UiNet.Node2UiNode(node);
        if(uinode!=null)
            info=(UiNodeInfo)SolverNodeInfo.GetObj(uinode);
        return info;
    }
    /** Gets draw info for the given edge */
    public UiEdgeInfo GetEdgeInfo(Edge edge)
    {
        return (UiEdgeInfo)SolverEdgeInfo.GetObj(UiNet.Edge2UiEdge(edge));
    }
    /** Puts draw info for the given node */
    public void PutNodeInfo(Node node, UiNodeInfo info)
    {
        SolverNodeInfo.Put(UiNet.Node2UiNode(node),info);
    }
    /** Puts draw info for the given edge */
    public void PutEdgeInfo(Edge edge, UiEdgeInfo info)
    {
        SolverEdgeInfo.Put(UiNet.Edge2UiEdge(edge),info);
    }
    /** Puts the color of a node
     * @deprecated */
    public void SetNodeColor(Node node,Color color)
    {
        PutNodeColor(node,color);
    }
    /** Puts the color of a node */
    public void PutNodeColor(Node node,Color color)
    {
        UiNodeInfo info = GetNodeInfo(node);
        if(info!=null)
        {
            info.color=color;
            PutNodeInfo(node,info);
        }
    }   
    /** Puts the color of an edge
     * @deprecated */
    public void SetEdgeColor(Edge edge,Color color)
    {
        PutEdgeColor(edge,color);
    }
    /** Puts the color of an edge */
    public void PutEdgeColor(Edge edge,Color color)
    {
        UiEdgeInfo info = GetEdgeInfo(edge);
        if(info!=null)
        {
            info.color=color;
            PutEdgeInfo(edge,info);
        }
    }
    /** Puts edge visibility, misspelled
    @deprecated */
    public void PutEdgeVisiblty(Edge edge,boolean visible)
    {
        PutEdgeVisibility(edge,visible);
    }
    
    /** Puts edge visibility */
    public void PutEdgeVisibility(Edge edge,boolean visible)
    {
        UiEdgeInfo info = GetEdgeInfo(edge);
        if(info!=null)
        {
            info.visible=visible;
            PutEdgeInfo(edge,info);
        }
    }
    /**
     * Changes the string label for this node
     * 
     * @param node the node to be assigned the label
     * @param label the new text label for this node
     */
    public void PutLabel(Node node, String label)
    {
        UiNodeInfo info = GetNodeInfo(node);
        if(info!=null)
        {
            info.label=label;
            PutNodeInfo(node,info);
        }
    }
    /**
     * Changes the integer label for this node
     * 
     * @param node the node to be assigned the label
     * @param label the new integer label for this node
     */
    public void PutLabel(Node node, int label)
    {
        PutLabel(node,TX.S(label));
    }
    /**
     * Changes the string label for this edge
     * 
     * @param edge the edge to be assigned the label
     * @param label the new text label for this edge
     */
    public void PutLabel(Edge edge, String label)
    {
        UiEdgeInfo info = GetEdgeInfo(edge);
        if(info!=null)
        {
            info.label=label;
            PutEdgeInfo(edge,info);
        }
    }
    /**
     * Changes the integer label for this edge
     * 
     * @param edge the edge to be assigned the label
     * @param label the new integer label for this edge
     */
    public void PutLabel(Edge edge, int label)
    {
        PutLabel(edge,TX.S(label));
    }
    /**
     * Changes the network to directed or undirected
     * 
     * @param flag true if the edge should be directed, otherwise false
     */
    public void PutDirected(boolean flag)
    {
        ExecBase solver = Services.GetSolver();
        Network Net = solver.GetNet();
        Net.PutDirected(flag);
    }

    /*===========================================================
    ===========================================================*/

    protected void GetNetworkDrawInfo()
    {
        UiNode node;
        UiEdge edge;
        UiNodeListIter nit;
        UiEdgeListIter eit;
        
        NetworkNodeInfo=UiNet.CreateNodeArray("NetworkNodeInfo");
        NetworkEdgeInfo=UiNet.CreateEdgeArray("NetworkEdgeInfo");
        SolverNodeInfo =UiNet.CreateNodeArray("SolverNodeInfo");
        SolverEdgeInfo =UiNet.CreateEdgeArray("SolverEdgeInfo");

        nit = UiNet.GetUiNodeListIter();    // node list iterator
        for(node=nit.GetFirstUiNode();node!=null;node=nit.GetNextUiNode())
        {
            AddNetworkSolverNode(node);
        }
        eit = UiNet.GetUiEdgeListIter();    // edge list iterator
        for(edge=eit.GetFirstUiEdge();edge!=null;edge=eit.GetNextUiEdge())
        {
            AddNetworkSolverEdge(edge);
        }
    }
    protected void UpdateNetworkDrawInfo(UiNodeArray nodeinfo, UiEdgeArray edgeinfo)
    {
        UiNode node;
        UiEdge edge;
        UiNodeListIter nit;
        UiEdgeListIter eit;
       
        if(nodeinfo==null || edgeinfo==null)
            return;
        nit = UiNet.GetUiNodeListIter();    // node list iterator
        for(node=nit.GetFirstUiNode();node!=null;node=nit.GetNextUiNode())
        {
            if(nodeinfo.GetObj(node)!=null)
                node.PutUiInfo((UiNodeInfo)nodeinfo.GetObj(node));
        }
        eit = UiNet.GetUiEdgeListIter();    // edge list iterator
        for(edge=eit.GetFirstUiEdge();edge!=null;edge=eit.GetNextUiEdge())
        {
            if(edgeinfo.GetObj(edge)!=null)
                edge.PutUiInfo((UiEdgeInfo)edgeinfo.GetObj(edge));
        }
    }
    protected void RestoreNetwork()
    {
        //if(UiBase.NullBase==false)
        UpdateNetworkDrawInfo(NetworkNodeInfo,NetworkEdgeInfo);
        UiNet.DeleteNodeArray("NetworkNodeInfo");
        UiNet.DeleteEdgeArray("NetworkEdgeInfo");
        UiNet.DeleteNodeArray("SolverNodeInfo");
        UiNet.DeleteEdgeArray("SolverEdgeInfo");
        NetworkNodeInfo=null;
        NetworkEdgeInfo=null;
        SolverNodeInfo=null;
        SolverEdgeInfo=null;
        ClearStatusLine();
       
    }
    /** Internal Method */
    public void DrawNetwork()
    {
        GUI.putStatusLineInfo(StatusString);
        UpdateNetworkDrawInfo(SolverNodeInfo,SolverEdgeInfo);
        if(PcIndex>-1)
        {
            Services.ShowPseudoCodeText(PcIndex);
            PcIndex=-1;
        }
        if(PcToken!=null)
        {
            Services.ShowPseudoCodeText(PcToken);
            PcToken=null;
        }
        GUI.Repaint("UserBase.DrawNetwork");
    }
    protected void AddNetworkSolverNode(UiNode node)
    {
        NetworkNodeInfo.Put(node,node.GetUiInfo());
        SolverNodeInfo.Put(node,node.GetUiInfo());
    }
    protected void AddNetworkSolverEdge(UiEdge edge)
    {
        NetworkEdgeInfo.Put(edge,edge.GetUiInfo());
        SolverEdgeInfo.Put(edge,edge.GetUiInfo());
    }
    
    
}