package giden.GRDS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
import java.awt.Point;

/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * Graph is class for managing (creating, accessing, and deleting)
 * Nodes and Ndges.   A Graph object contains a LinkList of its
 * Nodes, a LinkList of its Edges, and a LinkList of incident edges, for
 * each Node, accessible via the Node object.
 * Nodes and Edges are created using methods provided
 * by this class, and those methods automatically update the
 * related Graph data structures.
 * The Node and Edge lists are accessed through NodeListIters and
 * EdgeListIters (iterators)
 * provided by the Graph object.  The number of created
 * Nodes or Edges or the current number of Nodes or Edges are
 * also available.
 * <p>
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @see Edge
 * @see EdgeListIter
 * @see GraphState
 * @see InListIter
 * @see LinkList
 * @see Network
 * @see Node
 * @see NodeListIter
 * @see OutListIter
 *
 * @version 	4.0a 2004 08 27
 * @author 	    D. S. Dilworth
 */


/*===============================================================
===============================================================*/
public class Graph implements java.io.Serializable
{
    protected int NodeIndex;
    protected int EdgeIndex;
    private LinkList NodeList;
    private LinkList EdgeList;
    // internal data member */
    public /*protected*/ UiNetwork UiNet;  // remove link
    private boolean Directed;
    SolverServices Services;

    //-----------------------------------------------------------
    /** Creates an empty graph (a graph with no Nodes and no Edges)  */
    //-----------------------------------------------------------
    public Graph()
    {
        NodeIndex=1;
        EdgeIndex=1;
        NodeList = new DoubleLinkList();
        EdgeList = new DoubleLinkList();
        UiNet=null;
        Directed=true;  // PutDirected UiNet ready yet
        Services=null;
    }
    /** internal method */
    public void PutSolverServices(SolverServices services)
    {
        Services=services;
    }
    /** Returns the solver services object for creating new windows */
    public SolverServices GetSolverServices()
    {
        return Services;
    }
    
    //----------------------------------------------------------
    //----------------------------------------------------------
    void UpdateNodeId(Node node)
    {
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    void UpdateEdgeId(Edge edge)
    {
    }
    /** Internal method */
    public void DeleteGraph()
    {
        DeleteAllNodes();
        NodeIndex=1;
        EdgeIndex=1;
    }
    //----------------------------------------------------------
    /** Returns true if the graph is directed, false otherwise */
    //----------------------------------------------------------
    public boolean GetDirected()
    {
        return Directed;
    }
    
    //----------------------------------------------------------
    /** Changes the graph's directed flag, true means it's directed */
    //----------------------------------------------------------
    public void PutDirected(boolean directed)
    {
        Directed=directed;
        UiNet.GUI.setDirected(Directed);
    }
    

    /*= = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    | Nodes
    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =*/

    //-----------------------------------------------------------
    /** Gets the current number of Nodes in this Graph.  This number
     * is incremented with each successful CreateNode call and is
     * decremented with each successful DeleteNode call.
     * @return  The current number of Nodes in this Graph.
     * @see Graph#GetCreatedNodesCount */
    //-----------------------------------------------------------
    public int GetNodeCount()
    {
        return NodeList.GetSize();
    }

    //-----------------------------------------------------------
    /** Gets the total number of Nodes ever created by this Graph.
     * This number is incremented with each successful CreateNode
     * call and is never decremented.
     * Useful for setting the size of node arrays.
     * @return The total number of Nodes ever created by this Graph
     * @see Graph#GetEdgeCount */
     //-----------------------------------------------------------
    public int GetCreatedNodesCount()
    {   return NodeIndex;  }

    /** Internal method */
    public int GetNodeIndex()
    {
        return NodeIndex;
    }
    //-----------------------------------------------------------
    /** Creates a Node with the given coordinates
     * @return  A new Node object with coordinates (x,y). */
    //-----------------------------------------------------------
    //public Node CreateNode(int x, int y)
    //{
    //    Node node = CreateNode();
    //    Point pt = new Point(x,y);
    //    UiNet.CreateNode(node,pt);
    //    return node;
    //}

    //-----------------------------------------------------------
    /** Creates a Node in this graph
     * @return  The new node */
    //-----------------------------------------------------------
    public Node CreateNode()
    {
        ListEntry entry;

        Node node = new Node(this,NodeIndex++);

        entry = NodeList.AddItem(node);

        node.PutEntry(entry);
        
        //UpdateNodeId(node);
        return node;
    }
    /** Internal method */
    public void IncrementNodeIndex()
    {
        NodeIndex++;
    }

    //-----------------------------------------------------------
    /** Gets the iterator for this Graph's Node list
     * @return  A NodeListIter for this Graph object  */
    //-----------------------------------------------------------
    public NodeListIter GetNodeListIter()
    {
        return new NodeListIter(NodeList);
    }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    private LinkList GetNodeList()
    {
        return NodeList;
    }
    
    //-----------------------------------------------------------
    //-----------------------------------------------------------
    void DeleteNode(Node node, boolean call_ui)
    {
        Edge current;
        Edge next;
        
        //if(UiNet!=null && call_ui)
        //    UiNet.DeleteNodeX(node);
        if(Services!=null)
            Services.DeleteNodeX(node);

        EdgeListIter adj = new EdgeListIter(node.GetAdjList());
        current = adj.GetFirstEdge();
        while(current!=null)
        {
            next = adj.GetNextEdge();
            DeleteEdge(current,call_ui);
            current=next;
        }
        NodeList.DeleteEntry(node.GetEntry());
    }


    //-----------------------------------------------------------
    /** Deletes the given Node and all its incident Edges from
     *  this graph */
    //-----------------------------------------------------------
    public void DeleteNode(Node node)
    {
        DeleteNode(node,true);
    }
    /** Internal method */
    public void DeleteAllNodes()
    {
        NodeListIter nit = GetNodeListIter();
        Node node,next;
        node = nit.GetFirstNode();
        while(node!=null)
        {
            next = nit.GetNextNode();
            DeleteNode(node);
            node=next;
        }
        NodeIndex=1;
    }

    /*= = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    | Edges
    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =*/

    //-----------------------------------------------------------
    /** Gets the current number of Edges in this Graph.  This number
     * is incremented with each successful CreateEdge call and is
     * decremented with each successful DeleteEdge call.
     * @return  The current of the number of Edges in this Graph.
     * @see Graph#GetCreatedEdgesCount */
    //-----------------------------------------------------------
    public int GetEdgeCount()
    {   return EdgeList.GetSize();  }

    //-----------------------------------------------------------
    /** Gets the total number of edges ever created.  This number
     * is incremented with each successful CreateEdge call and is
     * never decremented.  Useful for
     *  setting the size of Edge arrays.
     * @return The number of Edges created by this Graph
     * @see Graph#GetEdgeCount */
    //-----------------------------------------------------------
    public int GetCreatedEdgesCount()
    {   return EdgeIndex;  }

    /** Internal method */
    public int GetEdgeIndex()
    {   return EdgeIndex;  }


    //-----------------------------------------------------------
    /* Creates an Edge with given label position joining two Nodes
     * @return A new Edge joining the source and target Nodes, with
     * label coordinates (x,y).  */
    //-----------------------------------------------------------
    /*
    public Edge CreateEdge(Node source, Node target, int x, int y)
    {
        Edge n_edge;

        n_edge=CreateEdge(source,target);
        
        return n_edge;
    }
    */
    
    //-----------------------------------------------------------
    /** Creates an Edge joining two Nodes
     * @return A new Edge.  */
    // ----------------------------------------------------------
    public Edge CreateEdge(Node source, Node target)
    {
        Edge edge = new Edge(this,EdgeIndex++);

        edge.PutSource(source);
        edge.PutTarget(target);

        edge.PutEntry(EdgeList.AddItem(edge));
        edge.PutSourceEntry(source.PutEdge(edge));
        edge.PutTargetEntry(target.PutEdge(edge));
        //UpdateEdgeId(edge);
        
        return edge;
    }
    /** Internal method */
    public void IncrementEdgeIndex()
    {
        EdgeIndex++;
    }
    /** Internal method */
   public Edge ReverseEdge(Edge edge)
    {
        Node source;
        Node target;
        ListEntry sourceentry,targetentry;

        source=edge.GetSource();
        target=edge.GetTarget();
        sourceentry=edge.GetSourceEntry();
        targetentry=edge.GetTargetEntry();
        edge.PutSource(target);
        edge.PutTarget(source);
        edge.PutSourceEntry(targetentry);
        edge.PutTargetEntry(sourceentry);
        
        return edge;
    }
    

    //-----------------------------------------------------------
    /** Gets the iterator for this graph's edge list
     * @return  an EdgeListIter for this graph  */
    //-----------------------------------------------------------
    public EdgeListIter GetEdgeListIter()
    {
        return new EdgeListIter(EdgeList);
    }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    private LinkList GetEdgeList()
    {
        return EdgeList;
    }

    //-----------------------------------------------------------
    //-----------------------------------------------------------
    void DeleteEdge(Edge edge, boolean call_ui)
    {
        ListEntry entry;
        Node source,target;

        //if(UiNet!=null && call_ui)
        if(Services!=null)
            Services.DeleteEdgeX(edge);
        
        edge.DetachNodes();
        EdgeList.DeleteEntry(edge.GetEntry());
    }
    //-----------------------------------------------------------
    /** Deletes the edge from this graph */
    //-----------------------------------------------------------
    public void DeleteEdge(Edge edge)
    {
        DeleteEdge(edge,true);
    }
    /** Internal method */
    public void DeleteAllEdges()
    {
        EdgeListIter eit = GetEdgeListIter();
        Edge edge,next;
        edge = eit.GetFirstEdge();
        while(edge!=null)
        {
            next = eit.GetNextEdge();
            DeleteEdge(edge);
            edge=next;
        }
        EdgeIndex=1;
    }
}

