package giden.GRDS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * IndexArray is a base class that is inherited by the NodeArray
 * and EdgeArray classes.  An IndexArray stores and retrieves
 * information using an object of type Index (e.g. Node and Edge).
 * The length of the array is automatically resized to
 * accomodate accesses that are beyond the size of the current
 * array.  The resizing involves creating a new array of a
 * larger size, copying the contents of the old array to the
 * new array, and then discarding the old array.  Aside from
 * time to resize the array, the resizing is transparent to the
 * application.
 * <p>
 * Any type of Java Object can be stored in an IndexArray
 * (including Node and Edge objects).  A method is also
 * provided to convert variables of type "int" into an
 * object of type "Integer", and then store it in the IndexArray
 * (methods for other types of non-objects may be added in the future).
 *<p>
 * Any type of supported variable can be stored using the overloaded
 * Put method.  Objects can be retrieved using the Get method and
 * then casted to the appropriate type.  Several methods are
 * provided to perform this retrieve and cast operation (GetInt,
 * GetNode, GetEdge).
 * <p>
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 *
 * @see EdgeArray
 * @see Index
 * @see Network
 * @see NodeArray
 *
 * @version 	4.0a 2004 08 27
 * @author 	    D. S. Dilworth
 */


// you can Put ints or objects, but must Get specific objects

/*===============================================================
===============================================================*/
public class IndexArray implements java.io.Serializable
{
    /** Integer data type for IndexArray */
    public static final int INTEGER_TYPE=0x1;
    /** Float data type for IndexArray */
    public static final int FLOAT_TYPE=0x2;
    /** Double data type for IndexArray */
    public static final int DOUBLE_TYPE=0x4;
    /** String data type for IndexArray */
    public static final int STRING_TYPE=0x8;
    /** Node data type for IndexArray */
    public static final int NODE_TYPE=0x100;
    /** Edge data type for IndexArray */
    public static final int EDGE_TYPE=0x200;
    /** Superset of all data types*/
    public static final int ANY_TYPE=0xFFFF;
    
    Object Values[];
    private int Size;
    private String Name;
    private String Alias;
    // owenj (28-May-1999; to make public) private Object FillObject;
    /** internal data member */
    public Object FillObject;  // 29sep99 dsd made it public->package
    /** internal data member */
    public int FillType;
    private UiIndexArray UiArray;
    //-----------------------------------------------------------
    /** Creates an IndexArray with the given initial size and name
     * and initializes the array components to null.  If the given
     * size is less than 100, then a size of 100 is used.  Each array
     * access is checked to determine whether it is beyond the bounds
     * of the array.  To accommodate array accesses beyond its bounds,
     * the array is automatically resized, and the new components are
     * initialized to the most recent Object used to Fill this array
     * (default is null).
     * @param size   Initial size of the array
     * @param name   Name of the array */
    //-----------------------------------------------------------
    public IndexArray(int size, String name)
    {
        if (size < 100)
            size=100;
        Size=size;
        //Size=10;        // !! only for alpha version
        Values = new Object[Size];
        Fill(null);
        Name=name;
        Alias=null;
        UiArray=null;
        FillType=0;
    }
    /** internal method */
    public UiIndexArray GetUiIndexArray()
    {
        return UiArray;
    }
    /** internal method */
    public void PutUiIndexArray(UiIndexArray uiarray)
    {
        UiArray = uiarray;
    }
    //-----------------------------------------------------------
    /** Gets the String name of this array
     * @return The String name of this array */
    //-----------------------------------------------------------
    public String GetName()
    {
        return Name;
    }
    //-----------------------------------------------------------
    /** Gets the alias String name of this array.
     *  Solvers can temporarily rename an array
     *  it is saved as a result.
     * @return The String name of this array */
    //-----------------------------------------------------------
    public String GetAlias()
    {
        return Alias;
    }
    //-----------------------------------------------------------
    /** Puts the alias String name for this array.
     *  Used by solvers to temporarily rename an array before
     *  it is saved as a result.  The alias name change only
     *  affects calls to GetAlias 
     * @return true if the Alias was previously null */
    //-----------------------------------------------------------
    public boolean PutAlias(String alias)
    {
        boolean result = (Alias==null);
        Alias=alias;
        return result;
    }
        
    //-----------------------------------------------------------
    /** Gets the current size of this array
     * @return The current size of the array.  This value will change
               to accomodate accesses beyond its bounds. */
    //-----------------------------------------------------------
    public int GetSize()
    {
        return Size;
    }
    //-----------------------------------------------------------
    /** Fills this array with the given integer value
     * @param value   The integer value to fill the array with */
    //-----------------------------------------------------------
    public void Fill(int value)
    {
        int i;
        Integer intval = new Integer(value);
        Fill(intval);
    }
    //-----------------------------------------------------------
    /** Fills this array with the given double value
     * @param value   The double value to fill the array with */
    //-----------------------------------------------------------
    public void Fill(double value)
    {
        int i;
        Double objval = new Double(value);
        Fill(objval);
    }
    
    //-----------------------------------------------------------
    /** Fills this array with the given Object
     * @param obj  The Object to fill the array with */
    //-----------------------------------------------------------
    public void Fill(Object obj)
    {
        int i;

        FillObject=obj;
        for(i=0;i<Size;i++)
            Values[i]=FillObject;
        if(obj instanceof Integer)
            FillType=INTEGER_TYPE;
        if(obj instanceof Float)
            FillType=FLOAT_TYPE;
        if(obj instanceof Double)
            FillType=DOUBLE_TYPE;
        if(obj instanceof String)
            FillType=STRING_TYPE;
        if(obj instanceof Node)
            FillType=NODE_TYPE;
        if(obj instanceof Edge)
            FillType=EDGE_TYPE;
    }
    void FillObject(Index indexobject)
    {
        PutObj(indexobject,FillObject);
    }
    /*-----------------------------------------------------------
    | Ensures the Index is within the array bounds
    -----------------------------------------------------------*/
    void EnsureSize(int index)
    {
		Object oldarray[];
		int    oldsize;
        int i,n;
        if(index>=Size)
        {
            System.out.println("Resize "+this.Name);
        	oldarray=Values;
        	oldsize=Size;

        	Size=index+Size/2;
        	Values=new Object[Size];

        	for(i=0;i<oldsize;i++)
        		Values[i]=oldarray[i];

        	for(i=oldsize;i<Size;i++)
        		Values[i]=FillObject;

        }
    }
    //-----------------------------------------------------------
    /** Gets the Object using the given Index object
     * @param indexobject  The Index object for this array
     * @return The Object at the given Index in this array*/
    //-----------------------------------------------------------
    public Object GetObj(Index indexobject)
    {
        int index=indexobject.GetIndex();
        EnsureSize(index);
        return Values[index];
    }
    //-----------------------------------------------------------
    /** Puts the Object using the given Index object
     * @param indexobject  The Index object for this array
     * @param obj        The Object to store in the array */
    //-----------------------------------------------------------
    public void PutObj(Index indexobject, Object obj)
    {
        int index=indexobject.GetIndex();
        EnsureSize(index);
        Values[index]=obj;
    }
    //-----------------------------------------------------------
    /** Puts the integer using the given Index object
     * @param indexobject  The object containing an array index
     * @param value  The integer value to store in this array */
    //-----------------------------------------------------------
    public void PutObj(Index indexobject, int value)
    {
        PutObj(indexobject,new Integer(value));
    }
    //-----------------------------------------------------------
    /** Puts the double using the given Index object
     * @param indexobject  The object containing an array index
     * @param value  The double value to store in this array */
    //-----------------------------------------------------------
    public void PutObj(Index indexobject, double value)
    {
        PutObj(indexobject,new Double(value));
    }
    

    //-----------------------------------------------------------
    /** Gets the integer using the given Index object
     * @param indexobject  The object containing an array index
     * @return The integer at the given index in this array */
    //-----------------------------------------------------------
    public int GetInt(Index indexobject)
    {
         return ((Integer)GetObj(indexobject)).intValue();
    }
    
    //-----------------------------------------------------------
    /** Gets the double using the given Index object
     * @param indexobject  The object containing an array index
     * @return The double at the given index in this array */
    //-----------------------------------------------------------
    public double GetDouble(Index indexobject)
    {
         return ((Double)GetObj(indexobject)).doubleValue();
    }
    //-----------------------------------------------------------
    /** Gets the string using the given Index object
     * @param indexobject  The object containing an array index
     * @return The string at the given index in this array */
    //-----------------------------------------------------------
    public String GetString(Index indexobject)
    {
         return (String)GetObj(indexobject);
    }
    
    //-----------------------------------------------------------
    /** Gets the Node using the given Indexo bject
     * @param indexobject  The object containing an array index
     * @return The Node at the given index in this array*/
    //-----------------------------------------------------------
    public Node GetNode(Index indexobject)
    {
        return (Node)GetObj(indexobject);
    }
    //-----------------------------------------------------------
    /** Gets the Edge using the given indexobject
     * @param indexobject  The object containing an array index
     * @return The Edge at the given index in this array*/
    //-----------------------------------------------------------
    public Edge GetEdge(Index indexobject)
    {
        return (Edge)GetObj(indexobject);
    }
    //-----------------------------------------------------------
    /** Gets the "DrawNode" using the given indexobject
     * @param indexobject  The object containing an array index
     * @return The Color at the given index */
    //-----------------------------------------------------------
    public UiNode GetUiNode(Index indexobject)
    {
        return (UiNode)GetObj(indexobject);
    }
    //-----------------------------------------------------------
    /** Gets the "DrawEdge" using the given indexobject
     * @param indexobject  The object containing an array index
     * @return The Color at the given index */
    //-----------------------------------------------------------
    public UiEdge GetUiEdge(Index indexobject)
    {
        return (UiEdge)GetObj(indexobject);
    }
    
}

