package giden.GRDS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
import java.lang.CloneNotSupportedException;

/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * LinkList is a base class that is inherited by
 * the SingleLinkList and DoubleLinkList classes.  Although the
 * current LinkList class implements a double link list, the
 * future design is to implement the single link and double link
 * functionality in the above named classes.  To ensure future
 * compatibility, users should create SingleLinkList or
 * DoubleLinkList objects; these objects can then
 * be manipulated as LinkList objects.
 * <p>
 * A LinkList is comprised of ListEntry
 * objects.  Each ListEntry object contains link information
 * for the list and the list-item of interest to the application.
 * The LinkList class only provides methods for iterating with ListEntry
 * objects; the user can access the underlying list-items by
 * manipulating the ListEntry objects.
 * The ListIter class is an
 * easier and recommended way of iterating and accessing the list-items.
 * Any method returning
 * a ListEntry will return null if accesses are attempted on
 * an empty list or beyond the boundaries of the list.
 * <p>
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 *
 * @see Network
 * @see DoubleLinkList
 * @see ListEntry
 * @see ListIter
 * @see SingleLinkList
 *
 * @version 	4.0a 2004 08 27
 * @author 	    D. S. Dilworth
 */

/*===============================================================
===============================================================*/
public class LinkList implements Cloneable, java.io.Serializable
{
    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    private ListEntry Head;
    private ListEntry Tail;
    private int Count;

    //-----------------------------------------------------------
    /** Creates an empty link list. This is a list with no
     *  ListEntry objects.  For an empty list, the head and
     * tail items are null.  */
    //-----------------------------------------------------------
    public LinkList()
    {
        Head=Tail=null;
        Count=0;
    }
    /** Returns a clone of this list */
    public Object clone()
    {
        LinkList new_list;
        
        try
        {
            new_list = (LinkList)super.clone();
            new_list.Head=new_list.Tail=null;
            new_list.Count=0;
            ListIter iter = new ListIter(this);
            Object item;
            for(item=iter.GetFirstItem();item!=null;item=iter.GetNextItem())
            {
                new_list.AddItem(item);
            }            
        }
        catch(CloneNotSupportedException e)
        {
            new_list=null;
        }
                     
        return new_list;        
    }

    //-----------------------------------------------------------
    /** Gets the current number of items in this list
     * @return The number of entries in this list    */
    //-----------------------------------------------------------
    public int GetSize()
    {   return Count;  }

    //-----------------------------------------------------------
    /** Adds an item to this list
     * @param item   the item to be added to this list
     * @return       The ListEntry containing the item.*/
    //-----------------------------------------------------------
    public ListEntry AddItem(Object item)
    {
        Count++;
        ListEntry entry = new ListEntry(item);
        return AppendEntry(entry);
    }

    //-----------------------------------------------------------
    /** Appends the given list to this list
     * @param list      The list to append to this list object.*/
    //-----------------------------------------------------------
    // OWENJ:  JoinList() method added by me 051397
    // DILWORTH:  Ongoing maintenance responsibility 98-Feb-02
    public void JoinList(LinkList list)
    {
        Count += list.GetSize();
        ListEntry entry = list.GetHead();

        if (list.GetHead() == null) return;

        if (Head == null)
        {
            Head = entry;
            Tail = list.GetTail();
        }
        else
        {
            Tail.PutNext(entry);
            entry.PutPrev(Tail);
            Tail = list.GetTail();
        }
    }

    //-----------------------------------------------------------
    /** Gets the head of this list
     * @return   The ListEntry at the head of this list.*/
    //-----------------------------------------------------------
    public ListEntry GetHead()
    {   return Head;    }

    //-----------------------------------------------------------
    /** Gets the tail of this list
     * @return The ListEntry at the tail of this list */
    //-----------------------------------------------------------
    public ListEntry GetTail()
    {   return Tail;    }

    //-----------------------------------------------------------
    /** Gets the first entry in this list
     * @return The first ListEntry in this list */
    //-----------------------------------------------------------
    public ListEntry GetFirstEntry()
    {   return GetHead();   }

    //-----------------------------------------------------------
    /** Gets the next entry in this list
     * @param   entry the current position in this list
     * @return The next ListEntry in this list. */
    //-----------------------------------------------------------
    public ListEntry GetNextEntry(ListEntry entry)
    {   return entry.GetNext();     }

    //-----------------------------------------------------------
    /** Gets the previous entry in this list
     * @param entry the current position in this list
     * @return The previous ListEntry in this list */
    //-----------------------------------------------------------
    public ListEntry GetPrevEntry(ListEntry entry)
    {   return entry.GetPrev();     }

    //-----------------------------------------------------------
    /** Gets the last entry in this list
     * @return The last ListEntry in this list */
    //-----------------------------------------------------------
    public ListEntry GetLastEntry()
    {   return GetTail();   }

    //-----------------------------------------------------------
    /** Deletes the list entry
     * @param entry   the entry to delete from this list */
    // !! should ensure the entry is in this list?
    //    - added test for null
    //-----------------------------------------------------------
    public void DeleteEntry(ListEntry entry)
    {

        if(entry==null)
            return;
            
        ListEntry next = entry.GetNext();
        ListEntry prev = entry.GetPrev();
        entry.PutNext(null);
        entry.PutPrev(null);

        if(prev==null && next==null)    // H <=> E <=> T
        {
            Head=null;
            Tail=null;
            entry=null;
        }
        if(prev!=null && next!=null)    // Ep <=> E <=> En
        {
            next.PutPrev(prev);
            prev.PutNext(next);
            entry=null;
        }
        if(prev==null && next!=null)    // H <=> E <=> En
        {
            Head = next;
            next.PutPrev(null);
            entry=null;
        }
        if(prev!=null && next==null)    // Ep <=> E <=> T
        {
            Tail = prev;
            prev.PutNext(null);
            entry=null;
        }
        Count--;
    }

    //-----------------------------------------------------------
    /** Inserts the second entry after the first entry.
     *  Assumes first is in list and second is not
     *  Should not delete or create ListEntries
     */
   //-----------------------------------------------------------
   /* rework from InsertBefore
    public void InsertAfter(ListEntry first, ListEntry second)
    {
        ListEntry first_next = first.GetNext();
        ListEntry second_prev = second.GetPrev();
        ListEntry second_next = second.GetNext();
        
        second_prev.PutNext(second_next);   // link around second
        second_next.PutPrev(second_prev);

        first.PutNext(second);              // link first forward to second
        second.PutPrev(first);              // link second back to first
        second.PutNext(first_next);         // link second forward
    }
    *
    //-----------------------------------------------------------
    /**
     * Inserts the second entry before the first entry.
     * Assumes first is in list and second is not;
     * second's links must be null.
     * Does not delete or create a ListEntry
     * @return true if successful, false if second was linked into a list
     */
    //-----------------------------------------------------------
    public boolean InsertBefore(ListEntry first, ListEntry second)
    {
        if(second.GetPrev()!=null || second.GetNext()!=null)
            return false;
        if(Head==first)
        {
            Head=second;                        // Head=>second
            second.PutPrev(null);               // link second-prev to null
        }
        else
        {
            ListEntry first_prev = first.GetPrev();
            first_prev.PutNext(second);
            second.PutPrev(first_prev);         // link second-prev to first-prev
        }
        second.PutNext(first);              // link second fwd to first
        first.PutPrev(second);              // link first rev to second
        return true;
    }
    

    //-----------------------------------------------------------
    /** Deletes all entries in this list */
    //  !! fix to step through list normally
    //-----------------------------------------------------------
    public void DeleteContents()
    {
        ListEntry current;
        ListEntry next;

        current = GetFirstEntry();
        while(current!=null)
        {
            next = GetNextEntry(current);
            DeleteEntry(current);
            current=next;
        }
    }

    //-----------------------------------------------------------
    //-----------------------------------------------------------
    private ListEntry AppendEntry(ListEntry entry)
    {
        if(Head==null)
        {
            Head=entry;
            Tail=entry;
            entry.PutPrev(null);
            entry.PutNext(null);
        }
        else
        {
            Tail.PutNext(entry);
            entry.PutNext(null);
            entry.PutPrev(Tail);
            Tail=entry;
        }
        return entry;
    }
}
