package giden.GRDS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * ListIter provides methods for accessing LinkList, SingleLinkList,
 * and DoubleLinkList objects, and is the recommended way of
 * manipulating these link list objects.  The ListIter methods
 * include forward and backward iteration through the list and retrieval
 * of the list-item.  It does not provide methods for altering the
 * list contents; those methods are provided by the link list
 * classes.
 *
 * A newly created ListIter is initialized to null.
 * At some point prior to using a newly created ListIter to access the list,
 * GetFirstItem() or GetLastItem() must be called.
 * <p>
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @see DoubleLinkList
 * @see LinkList
 * @see ListEntry
 * @see Network
 * @see SingleLinkList
 *
 * @version 	4.0a 2004 08 27
 * @author 	    D. S. Dilworth
 */

//import LinkList;

/*===============================================================
===============================================================*/
public class ListIter
{
    //-----------------------------------------------------------
    //-----------------------------------------------------------
    /** Internal data member */
    public LinkList List;
    /** Internal data member */
    public ListEntry Entry;

    //-----------------------------------------------------------
    /** Creates a ListIter object for the given list and initializes
     * it to null.  In order to use this ListIter to access items in
     * the list, GetFirstItem() or GetLastItem must initially be called.
     * @param list   The LinkList to be accessed. */
    //-----------------------------------------------------------
    public ListIter(LinkList list)
    {
        List=list;
        Entry=null;
    }

    //-----------------------------------------------------------
    /** Gets a count of items in the list.  NODE ADJACENCY LIST
     * WARNING: Although this count is correct for the total
     * number of edges adjacent to a node, it is not correct for
     * the incoming and outgoing adjacency lists.  A node only
     * stores one adjacency list, and both the IncListIter and OutListIter
     * classes use that same list to test which of those
     * edges are incoming or outgoing edges.
     * @return The number of entries in the list    */
    //-----------------------------------------------------------
    public int GetSize()
    {
        return List.GetSize();
    }


    //-----------------------------------------------------------
    /** Gets the list-item at the current list location
     * @return   The list-item stored in the current ListEntry */
    //-----------------------------------------------------------
    public Object GetItem()
    {
        return Entry.GetItem();
    }

    //-----------------------------------------------------------
    /** Moves this ListIter to the ListEntry at the head of the
     * list and gets that ListEntry's item.
     * @ returns The first list-item stored in the link list. */
    //-----------------------------------------------------------
    public Object GetFirstItem()
    {
        return (Entry=List.GetFirstEntry())==null ? null : GetItem();
    }

    //-----------------------------------------------------------
    /** Moves this ListIter to the next ListEntry in the list
     * and gets that ListEntry's item.
     * @ returns The next list-item stored in the link list. */
    //-----------------------------------------------------------
    public Object GetNextItem()
    {
        return (Entry=List.GetNextEntry(Entry))==null ? null : GetItem();
    }

    //-----------------------------------------------------------
    /** Moves this ListIter to the previous ListEntry in the list
     * and gets that ListEntry's item.
     * @ returns The previous list-item stored in the link list. */
        //-----------------------------------------------------------
    public Object GetPrevItem()
    {
        return (Entry=List.GetPrevEntry(Entry))==null ? null : GetItem();
    }

    //-----------------------------------------------------------
    /** Moves this ListIter to the last ListEntry in the list
     * and gets that ListEntry's item.
     * @ returns The last list-item stored in the link list. */
    //-----------------------------------------------------------
    public Object GetLastItem()
    {
        return (Entry=List.GetLastEntry())==null ? null : GetItem();
    }
}

