package  giden.GRDS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
import giden.userSOLVERS.*;

/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * Network inherits the Graph class and provides methods
 * for creating and retrieving NodeArray and EdgeArray objects.
 * The current version can hold at most 10 NodeArray and 10 EdgeArray
 * objects.  Arrays are created with a String name, and those names
 * are displayed by the GUI and saved with the network.  The user
 * can also create NodeArray and EdgeArray objects independent of
 * a Network object.  Network creates NodeId, EdgeId, and
 * EdgeLength arrays when it is created.  Network maintains the
 * NodeId and EdgeId values, the UserInterface is responsible for
 * maintaining the EdgeLength values.
 * <p>
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * <p>
 * Following are all the documented classes for the
 * developer release of GIDEN.
 *
 * @see BoundedQueue
 * @see DoubleLinkList
 * @see Edge
 * @see EdgeArray
 * @see EdgeListIter
 * @see Graph
 * @see InListIter
 * @see Index
 * @see IndexArray
 * @see LinkList
 * @see ListEntry
 * @see ListIter
 * @see Network
 * @see Node
 * @see NodeArray
 * @see NodeListIter
 * @see OutListIter
 * @see PQList
 * @see Queue
 * @see SingleLinkList
 * @see SolverMenu
 * @see SolverMenuBase
 * @see UserIO
 *
 * @version 	4.0a 2004 08 27
 * @author 	    D. S. Dilworth
 */

 //@see Edge
 //@see Node
 //@see Index
 //@see EdgeListIter
 //@see NodeListIter
 //@see GraphState
 //@see IncListIter
 //@see OutListIter


//===============================================================
//===============================================================
public class Network extends Graph
{
    /** Internal data member */
    public LinkList UiNetComments;
    /** Internal data member */
    public LinkList UiNetNodesComments;
    /** Internal data member */
    public LinkList UiNetEdgesComments;
    
    NodeArray NodeArrays[];
    EdgeArray EdgeArrays[];
    int EdgeListIndex;
    int NodeListIndex;
    LinkList NamedObjectList;
    
    //-----------------------------------------------------------
    /** Restores this graph to its prior state
     * @param state  The state of the graph previously recorded
     * @see GraphState*/
    // don't call ui with deletes for now
    // delete edges first to ensure we clear edge arrays
    //-----------------------------------------------------------
    void Restore(GraphState state)
    {
        int index;
        Edge edge,nextedge;
        Node node,nextnode;
        UiEdge uiedge;
        UiNode uinode;
        
        
        index = state.GetEdgeIndex();
        EdgeListIter eit = GetEdgeListIter();
        edge=eit.GetFirstEdge();
        while(edge!=null)
        {
            nextedge=eit.GetNextEdge();
            if(edge.GetIndex() >= index)
            {
                uiedge=UiNet.Edge2UiEdge(edge);
                if(uiedge!=null)
                    UiNet.GUI.draw_canvas.deleteEdge((GUIEdge) uiedge);
                else
                {
                    DeleteEdge(edge,false);
                }
                FillEdge(edge);
                
            }
            edge=nextedge;
        }
        EdgeIndex=index;
        

        index=state.GetNodeIndex();
        NodeListIter nit = GetNodeListIter();
        node=nit.GetFirstNode();
        while(node!=null)
        {
            nextnode=nit.GetNextNode();
            if(node.GetIndex() >= index)
            {
                uinode=UiNet.Node2UiNode(node);
                if(uinode!=null)
                    UiNet.GUI.draw_canvas.deleteNode((GUINode) uinode);
                else
                {
                    DeleteNode(node,false);
                }
                FillNode(node);
            }
            node=nextnode;
        }
        NodeIndex=index;

    }
    /** Adds the name and its value to the NamedObject list.
        If the name already exists, the old Object will be discarded */
    public NamedObject AddNamedObject(String name, Object value)
    {
        ListIter lit = new ListIter(NamedObjectList);
        NamedObject name_object;
        String text;
        for(name_object = (NamedObject)lit.GetFirstItem();name_object!=null;name_object=(NamedObject)lit.GetNextItem())
        {
            if(name.equals(name_object.GetName()))
            {
                name_object.PutObject(value);
                break;
            }
        }
        if(name_object==null)
        {
            name_object = new NamedObject(name,value);
            NamedObjectList.AddItem(name_object);
        }
        return name_object;
    }
    /** Gets and interator for the NamedObject list */
    public ListIter GetNamedObjectIter()
    {
        return new ListIter(NamedObjectList);
    }
    /** returns the first NamedObject item with the given name */
    public NamedObject FindNamedObject(String name)
    {
        NamedObject result=null;
        ListIter lit = GetNamedObjectIter();
        Object obj;
        for(obj=lit.GetFirstItem();obj!=null;obj=lit.GetNextItem())
        {
            if(((NamedObject)obj).GetName().equals(name))
                result=(NamedObject)obj;
        }
        return result;
    }
    void FillNode(Node node)
    {
        int i;
        for(i=0;i<NodeListIndex;i++)
        {
            if(NodeArrays[i]!=null)
            {
                NodeArrays[i].FillObject(node);
            }
        }
    }
    void FillEdge(Edge edge)
    {
        int i;
        for(i=0;i<EdgeListIndex;i++)
        if(EdgeArrays[i]!=null)
        {
            EdgeArrays[i].FillObject(edge);
        }
    }
    
    

    void PutNetworkName(String name)
    {
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    void UpdateNodeId(Node node)
    {
        if(UiNet!=null)
            UiNet.UpdateNodeId(node);
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    void UpdateEdgeId(Edge edge)
    {
        if(UiNet!=null)
            UiNet.UpdateEdgeId(edge);
    }


    //-----------------------------------------------------------
    /** Creates an empty Network (a Network with no Nodes and no
     *  Edges) */
    //-----------------------------------------------------------
    public Network()
    {
        DeleteContents();
    }
    //-----------------------------------------------------------
    /** Deletes all Nodes and Edges and NodeArrays and EdgeArrays of
     *  this Network.  */
    //-----------------------------------------------------------
    public void DeleteContents()
    {
        NamedObjectList = new DoubleLinkList();
        
        NodeArrays = new NodeArray[20];
        EdgeArrays = new EdgeArray[20];
        //NodeArrayList = new ArrayList();
        //EdgeArrayList = new ArrayList();

        EdgeListIndex=0;
        NodeListIndex=0;
        
        UiNet=null;

        DeleteGraph();
        
    }
    /** Internal method */
    public void PutUiNetwork(UiNetwork uinet)
    {
        UiNet=uinet;
    }
    

    //-----------------------------------------------------------
    /** Gets the name of the first NodeArray
     * @return  The first NodeArray in the network. */
    //-----------------------------------------------------------
    public String GetFirstNodeArrayName()
    {
        NodeListIndex=-1;
        return GetNextNodeArrayName();
    }
    //-----------------------------------------------------------
    /** Gets the name of the next NodeArray
     * @return  The next NodeArray name in the network */
    //-----------------------------------------------------------
    public String GetNextNodeArrayName()
    {
        String name;
        NodeArray array;
        while(NodeListIndex < NodeArrays.length-1)
        {
            NodeListIndex++;
            array = GetNodeArray();
            if(array!=null)
            {
                if(array.GetName()!=null)
                {
                    return NodeArrays[NodeListIndex].GetName();
                }
            }
        }
        return null;
    }

    //-----------------------------------------------------------
    /** Gets the name of the first EdgeArray
     * @return  The first EdgeArray name in the network.   */
    //-----------------------------------------------------------
    public String GetFirstEdgeArrayName()
    {
        EdgeListIndex=-1;
        return GetNextEdgeArrayName();
    }
    //-----------------------------------------------------------
    /** Gets the name of the next EdgeArray
     * @return  The next EdgeArray name in the network  */
    //-----------------------------------------------------------
    public String GetNextEdgeArrayName()
    {
        String name;
        EdgeArray array;
        while(EdgeListIndex < EdgeArrays.length-1)
        {
            EdgeListIndex++;
            array = GetEdgeArray();
            if(array!=null)
            {
                if(array.GetName()!=null)
                {
                    return EdgeArrays[EdgeListIndex].GetName();
                }
            }
        }
        return null;
    }
    //-----------------------------------------------------------
    //-----------------------------------------------------------
    NodeArray GetNodeArray(int index)
    {
        return NodeArrays[index];
    }
    //-----------------------------------------------------------
    //-----------------------------------------------------------
    NodeArray GetNodeArray()
    {
        if(NodeListIndex<NodeArrays.length)
            return NodeArrays[NodeListIndex];
        else
            return null;
    }
    
    //-----------------------------------------------------------
    /** Gets the node array with the given name from this Network
    * @return  The node array or null if it doesn't exist */
    //-----------------------------------------------------------
    public NodeArray GetNodeArray(String name)
    {
        String arrayname;
        arrayname=GetFirstNodeArrayName();
        while(name.equals(arrayname)==false)
        {
            arrayname=GetNextNodeArrayName();
            if(arrayname==null)
                break;
        }
        return GetNodeArray();
    }
    //-----------------------------------------------------------
    /** Internal method */
    //-----------------------------------------------------------
    public void DeleteNodeArray(String name)
    {
        if(GetNodeArray(name)!=null)
            NodeArrays[NodeListIndex]=null;
    }
    
    //-----------------------------------------------------------
    /** Creates a new NodeArray with the given name.
     * NodeArrays created in this way are also stored by the
     * Network object and can be later retrieved through the name.
     * @return  A NodeArray
     * @see NodeArray
     */
    //-----------------------------------------------------------
    public NodeArray CreateNodeArray(String name)
    {
        DeleteNodeArray(name);
        int index;
        for(index=0;index < NodeArrays.length;index++)
        {
            if(NodeArrays[index]==null)
            {
                return NodeArrays[index]=new NodeArray(GetCreatedNodesCount(),name);
            }
        }
        return null;
    }

    //-----------------------------------------------------------
    //-----------------------------------------------------------
    EdgeArray GetEdgeArray()
    {
        return GetEdgeArray(EdgeListIndex);
    }
    //-----------------------------------------------------------
    //-----------------------------------------------------------
    EdgeArray GetEdgeArray(int index)
    {
        return EdgeArrays[index];
    }
    //-----------------------------------------------------------
    /** Gets the edge array with the given name from this Network
    * @return  The edge array or null if it doesn't exist */
    //-----------------------------------------------------------
    public EdgeArray GetEdgeArray(String name)
    {
        String arrayname;
        arrayname=GetFirstEdgeArrayName();
        while(arrayname!=null && name.equals(arrayname)==false)
        {
            arrayname=GetNextEdgeArrayName();
            if(arrayname==null)
                break;
        }
        return GetEdgeArray();
    }
    //-----------------------------------------------------------
    /** Internal method */
    //-----------------------------------------------------------
    public void DeleteEdgeArray(String name)
    {
        if(GetEdgeArray(name)!=null)
            EdgeArrays[EdgeListIndex]=null;
    }
    
    
    //-----------------------------------------------------------
    /** Creates a new EdgeArray with the given name.
     * EdgeArrays created in this way are also stored by the
     * Network object and can be later retrieved through the name.
     * @return  A new EdgeArray.
     * @see EdgeArray */
    //-----------------------------------------------------------
    public EdgeArray CreateEdgeArray(String name)
    {
        DeleteEdgeArray(name);
        int index;
        for(index=0;index < EdgeArrays.length;index++)
        {
            if(EdgeArrays[index]==null)
            {
                return EdgeArrays[index]=new EdgeArray(GetCreatedEdgesCount(),name);
            }
        }
        return null;
    }
    /** Internal method */
    public LinkList GetNodeArrayStrings()
    {
        return GetNodeArrayStrings(IndexArray.ANY_TYPE);
    }
    /** Internal method */
    public LinkList GetNodeArrayStrings(int type)
    {
        int i;
        LinkList names = new DoubleLinkList();
        for(i=0;i<NodeArrays.length;i++)
        {
            if(NodeArrays[i]!=null)
            {
                if((NodeArrays[i].FillType&type)!=0)
                {
                    names.AddItem(NodeArrays[i].GetName());
                }
            }
        }
        return names;
    }
    /** Internal method */
    public LinkList GetEdgeArrayStrings()
    {
        return GetEdgeArrayStrings(IndexArray.ANY_TYPE);
    }
    /** Internal method */
    public LinkList GetEdgeArrayStrings(int type)
    {
        int i;
        LinkList names = new DoubleLinkList();
        for(i=0;i<EdgeArrays.length;i++)
        {
            if(EdgeArrays[i]!=null)
            {
                if((EdgeArrays[i].FillType&type)!=0)
                {
                    names.AddItem(EdgeArrays[i].GetName());
                }
            }
        }
        return names;
    }
    /** Internal method */
    public LinkList GetNodeIndexStrings()
    {
        Node node;
        LinkList names = new DoubleLinkList();
        NodeListIter nit = GetNodeListIter();
        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
        {
            names.AddItem("N"+node.GetIndex());
        }
        return names;
    }
    /** Internal method */
    public Node GetNode(String index_string)
    {
        Node result=null;
        if(index_string!=null)
        {
            if(index_string.startsWith("N"))
                index_string=index_string.substring(1);
            int index=new Integer(index_string).intValue();
        
            Node node;
            LinkList names = new DoubleLinkList();
            NodeListIter nit = GetNodeListIter();
            for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
            {
                if(index==node.GetIndex())
                {
                    result=node;
                    break;
                }
            }
        }
        return result;
    }
    /** Internal method */
    public LinkList GetEdgeIndexStrings()
    {
        Edge edge;
        LinkList names = new DoubleLinkList();
        EdgeListIter eit = GetEdgeListIter();
        for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
        {
            names.AddItem("E"+edge.GetIndex());
        }
        return names;
    }
    /** Internal method */
    public Edge GetEdge(String index_string)
    {
        Edge result=null;
        if(index_string!=null)
        {
            if(index_string.startsWith("E"))
                index_string=index_string.substring(1);
            int index=new Integer(index_string).intValue();
            
            Edge edge;
            EdgeListIter eit = GetEdgeListIter();
            for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
            {
                if(index==edge.GetIndex())
                {
                    result=edge;
                    break;
                }
            }
        }
        return result;
    }

    
}
