package giden.GUI;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.*;
import java.awt.event.*;


public class GUIDrawCanvas extends Panel implements MouseListener,
                                             MouseMotionListener {
    
    GUINetwork UiNet;
    
    private Image backBuffer;
    Graphics backGraphics;
    
    public GUICanvasGrid canvas_grid = null;
    
    public GUIController gui_controller = null;
    private GUINetworkWindow net_window = null;
    
    public boolean isFocusTraversable() {return true;}
    
    private boolean selection_box_enabled = false;
    private Point selection_box_start;
    private Point selection_box_end;
    
    
    private int num_items_selected = 0;
    private GUINode[] selected_nodes = new GUINode[0];
    private int selected_nodecount = 0;
    private GUIEdge[] selected_edges = new GUIEdge[0];
    private int selected_edgecount = 0;
    
    public final static Color defaultItemColor = new Color(184,175,167); 
    private final Color selectedItemColor = new Color(48,175,48); //new Color(48,188,48);
    

    private int canvas_width = 600;
    private int canvas_height = 350;
    /*
    public Dimension getPreferredSize() {
        return new Dimension(canvas_width,canvas_height);
    }
    */
    /*
    public Dimension getPreferredSize() {
        int width  = 0;
        int height = 0;

        Rectangle node_bds;

        GUINode node;
        GUINodeListIter nit = UiNet.GetNodeListIter();
        for (node=nit.GetFirstNode(); node!=null ; node=nit.GetNextNode()) {
            node_bds = node.getBounds();
            width  = Math.max(width, (node_bds.x + node_bds.width));
            height = Math.max(height, (node_bds.y + node_bds.height));
        }
        return new Dimension(width,height);
    }
    */
    
    public UiNodeInfo getDefaultUiNodeInfo()
    {
        UiNodeInfo info = new UiNodeInfo();
        info.color=defaultItemColor;
        info.label="";
        info.visible=true;
        info.radius=18;
        info.x=0;
        info.y=0;
        return info;
        /*
            node = new GUINode(gui_controller,this,18,true,
                               new Font("DialogInput",Font.PLAIN,12),
                               "",defaultItemColor,p.x,p.y);
        */        
    }
    public UiEdgeInfo getDefaultUiEdgeInfo()
    {
        UiEdgeInfo info = new UiEdgeInfo();
        info.color=defaultItemColor;
        info.label="";
        info.visible=true;
        return info;
        /*
            edge = new GUIEdge(gui_controller,this,directed_edges,true,
                               new Font("DialogInput",Font.PLAIN,12),
                               "",defaultItemColor,3);
        */        
    }


    public void pasteSelectedToCanvas() {
        deselectGUINodesAndGUIEdges();
        
        GUINodeInfo[] node_info = gui_controller.getClipboardGUINodeInfo();
        GUINode node;
        for (int i=0 ; i < node_info.length ; i++) {
            node_info[i].canvas = this;
            node_info[i].in_group_selection = false;
            node_info[i].color = defaultItemColor;
            
            node = new GUINode(node_info[i]);
            UiNet.CreateNodeY(node);
            this.add(node,0);
            
            selectGUINode(node);
        }
        
        GUIEdgeInfo[] edge_info = gui_controller.getClipboardGUIEdgeInfo();
        GUIEdge edge;
        
        UTILITIES.Repaint(this,"pasteSelectedToCanvas");
    }
    
    public void deleteSelectedItems() {
        
        // Delete edges first!  If we delete nodes first, then the
        // selected edge list may contain invalid edges.
        
        // delete selected edges
        GUIEdge edge;
        for (int i=0 ; i < selected_edgecount ; i++)
            if (selected_edges[i] != null) {
                edge = selected_edges[i];
                deselectGUIEdge(edge);
                //                UiNet.DeleteEdge(edge);
                //                removeGUIEdgeFromCanvas(edge);
                deleteEdge(edge);
            }
        
        // delete selected nodes
        GUINode node;
        for (int i=0 ; i < selected_nodecount ; i++)
            if(selected_nodes[i] != null) {
                node = selected_nodes[i];
                deselectGUINode(node);
                //                UiNet.DeleteNode(node);
                //                removeGUINodeFromCanvas(node);
                deleteNode(node);
            }
    }
    
    public void copySelectedToClipboard() {
        GUINodeInfo[] node_info = new GUINodeInfo[selected_nodecount];
        for (int i=0 ; i < selected_nodecount ; i++) {
            if (selected_nodes[i] != null)
                node_info[i] = selected_nodes[i].GetInfo();
            else
                node_info[i] = null;
        }
        
        GUIEdgeInfo[] edge_info = new GUIEdgeInfo[selected_edgecount];
        for (int i=0 ; i < selected_edgecount ; i++) {
            if (selected_edges[i] != null)
                edge_info[i] = selected_edges[i].GetInfo();
            else
                edge_info[i] = null;
        }
        
        gui_controller.setClipboardItems(node_info,selected_nodecount,
                                         edge_info,selected_edgecount);
    }
    
    synchronized public void moveSelectedItems(int dx, int dy) {
        // move spline points
        for (int i=0 ; i < selected_edgecount ; i++)
            if (selected_edges[i] != null) {
                selected_edges[i].shiftSpline(dx,dy);
            }

        // move nodes
        for (int i=0 ; i < selected_nodecount ; i++)
            if (selected_nodes[i] != null) {
                selected_nodes[i].shiftCenter(dx,dy);
            }
        
        UTILITIES.Repaint(this,"moveSelectedItems");
    }

    private int deselectGUINodesAndGUIEdges() {
        int deselect_count = 0;
        
        GUINodeInfo n_info;
        for (int i=0 ; i < selected_nodecount ; i++)
            if (selected_nodes[i] != null) {
                deselectGUINode(selected_nodes[i]);
                deselect_count++;
            }
        selected_nodecount = 0;
        
        GUIEdgeInfo e_info;
        for (int i=0 ; i < selected_edgecount ; i++)
            if (selected_edges[i] != null) {
                deselectGUIEdge(selected_edges[i]);
                deselect_count++;
            }
        selected_edgecount = 0;
        
        UTILITIES.Repaint(this,"deselectGUINodesAndGUIEdges");        
        num_items_selected = 0;
        return deselect_count;
    }
    
    private void toggleSelectGUINode(GUINode i) {
        GUINodeInfo info = i.GetInfo();
        if (info == null) return;
        
        if (info.in_group_selection) 
            deselectGUINode(i);
        else
            selectGUINode(i);
    }
    synchronized private void selectGUINode(GUINode i) {
        GUINodeInfo info = i.GetInfo();
        if (info == null) return;
        
        if (!info.in_group_selection) {
            info.in_group_selection = true;
            info.color = new Color(48,188,48);
            
            if (selected_nodes.length <= selected_nodecount)
                selected_nodes = resizeGUINodeArray(selected_nodes);
            selected_nodes[selected_nodecount] = i;
            selected_nodecount++;
            i.PutInfo(info);
            UTILITIES.Repaint(this,"selectGUINode");
            num_items_selected++;
            if (num_items_selected == 1) {
                net_window.enableEditModeMenu("Copy");
                net_window.enableEditModeMenu("Cut");
                net_window.enableEditModeMenu("Nudge Selected");
            }
        }
    }
    synchronized private void deselectGUINode(GUINode i) {
        GUINodeInfo info = i.GetInfo();
        if (info == null) return;
        
        if (info.in_group_selection) {
            info.in_group_selection = false;
            info.color = defaultItemColor;
            for (int j=0 ; j < selected_nodecount ; j++)
                if (selected_nodes[j] != null && selected_nodes[j].equals(i)) {
                    selected_nodes[j] = null;
                    break;
                }
            i.PutInfo(info);
            UTILITIES.Repaint(this,"deselectGUINode");
            num_items_selected--;
            if (num_items_selected == 0) {
                net_window.disableEditModeMenu("Copy");
                net_window.disableEditModeMenu("Cut");
                net_window.disableEditModeMenu("Nudge Selected");
            }
        }      
    }
    
    private void toggleSelectGUIEdge(GUIEdge e) {
        GUIEdgeInfo info = e.GetInfo();
        if (info == null) return;
        
        if (info.in_group_selection)
            deselectGUIEdge(e);
        else
            selectGUIEdge(e);
    }
    synchronized private void selectGUIEdge(GUIEdge e) {
        GUIEdgeInfo info = e.GetInfo();
        if (info == null) return;
        
        if (!info.in_group_selection) {
            info.in_group_selection = true;
            info.color = new Color(48,188,48);
            
            if (selected_edges.length <= selected_edgecount)
                selected_edges = resizeGUIEdgeArray(selected_edges);
            selected_edges[selected_edgecount] = e;
            selected_edgecount++;
            e.PutInfo(info);
            UTILITIES.Repaint(this,"selectGUIEdge");
            num_items_selected++;
            if (num_items_selected == 1) {
                net_window.enableEditModeMenu("Copy");
                net_window.enableEditModeMenu("Cut");
                net_window.enableEditModeMenu("Nudge Selected");
            }
        }
    }
    synchronized private void deselectGUIEdge(GUIEdge e) {
        GUIEdgeInfo info = e.GetInfo();
        if (info == null) return;
        
        if (info.in_group_selection) {      
            info.in_group_selection = false;
            info.color = defaultItemColor;
            for (int j=0 ; j < selected_edgecount ; j++)
                if (selected_edges[j] != null && selected_edges[j].equals(e)) {
                    selected_edges[j] = null;
                    break;
                }
            e.PutInfo(info);
            UTILITIES.Repaint(this,"deselectGUIEdge");
            num_items_selected--;
            if (num_items_selected == 0) {
                net_window.disableEditModeMenu("Copy");
                net_window.disableEditModeMenu("Cut");
                net_window.disableEditModeMenu("Nudge Selected");
            }
        }
    }
    
    
    
    private GUINode[] resizeGUINodeArray(GUINode[] old_nodes) {
        GUINode new_nodes[] = new GUINode[old_nodes.length+10];
        for (int i=0 ; i < old_nodes.length ; i++)
            new_nodes[i] = old_nodes[i];
        
        return new_nodes;
    }
    private GUIEdge[] resizeGUIEdgeArray(GUIEdge[] old_edges) {
        GUIEdge new_edges[] = new GUIEdge[old_edges.length+10];
        for (int i=0 ; i < old_edges.length ; i++)
            new_edges[i] = old_edges[i];
        
        return new_edges;
    }
    
    GUIDrawCanvas(GUIController c, GUINetworkWindow nw, GUINetwork net) {
        super();
        
        UiNet = net;

        this.node_data_name = new String(UiNetwork.NodeIdString);
        this.node_data_array = UiNet.GetNodeArray(node_data_name);
        this.edge_data_name = new String(UiNetwork.EdgeIdString);
        this.edge_data_array = UiNet.GetEdgeArray(edge_data_name);
        
        GUIEdge edge;
        GUIEdgeInfo e_info;
        GUIEdgeListIter eit = UiNet.GetEdgeListIter();
        for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge()) {
            e_info = edge.GetInfo();
            if (e_info == null) {
                GIDEN.out.println("Could not get GUIEdge info");
                System.exit(1);
            }
            e_info.gui_controller = c;
            e_info.canvas = this;
            e_info.label = new String(edge_data_array.GetObj(edge).toString());
            edge.PutInfo(e_info);
            this.add(edge,0);
        }
        GUINode node;
        GUINodeInfo n_info;
        GUINodeListIter nit = UiNet.GetNodeListIter();
        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode()) {
            n_info = node.GetInfo();
            if (n_info == null) {
                GIDEN.out.println("Could not get GUINode info");
                System.exit(1);
            }
            n_info.gui_controller = c;
            n_info.canvas = this;
            n_info.label = new String(node_data_array.GetObj(node).toString());
            node.PutInfo(n_info);
            this.add(node,0);
        }
        

        // extrapolate the direction state of the network from the
        // first edge, or set to directed by default:
        /*
        if (eit.GetFirstEdge() != null) {
            e_info = eit.GetFirstEdge().GetInfo();
            directed_edges = e_info.directed;
        }
        else
            directed_edges = true;
        */
        
        
        this.net_window = nw;
        this.gui_controller = c;
        updateTool(c.getCurrentTool());
        
        this.canvas_grid = new GUICanvasGrid(this);
        this.canvas_grid.setVisible(false);
        this.canvas_grid.setEnabled(false);
        
        addMouseListener(this);
        addMouseMotionListener(this);
        
        this.setLayout(null);
        
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
    
    synchronized public void clearNetwork() {
        selectAllGUINodesAndGUIEdges();
        deleteSelectedItems();
        reset();
    }
    
    private void reset() {
        action_node = null;
        action_node_specified = false;
    }
    
    synchronized public void selectAllGUINodesAndGUIEdges() {
        selectAllGUINodes();
        selectAllGUIEdges();
    }

    synchronized public void clearSelections() {
        deselectGUINodesAndGUIEdges();
    }

    synchronized public void selectAllGUIEdges() {
        GUIEdge edge;
        GUIEdgeListIter eit = UiNet.GetEdgeListIter();
        for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
            selectGUIEdge(edge);
    }

    synchronized public void selectAllGUINodes() {
        GUINode node;
        GUINodeListIter nit = UiNet.GetNodeListIter();
        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
            selectGUINode(node);
    }
    
    
    synchronized private void removeGUINodesAndGUIEdgesFromCanvas() {
        GUINode node;
        GUINodeListIter nit = UiNet.GetNodeListIter();
        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
            this.remove(node);
        
        GUIEdge edge;
        GUIEdgeListIter eit = UiNet.GetEdgeListIter();
        for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
            this.remove(edge);
    }
    synchronized private void addGUINodesAndGUIEdgesToCanvas() {
        GUINode node;
        GUINodeListIter nit = UiNet.GetNodeListIter();
        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
            this.add(node,0);
        
        GUIEdge edge;
        GUIEdgeListIter eit = UiNet.GetEdgeListIter();
        for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
            this.add(edge,0);
    }
    
    public void removeGUINodeFromCanvas(GUINode i) {
        if (this.isAncestorOf(i))
            this.remove(i);
    }
    
    public void removeGUIEdgeFromCanvas(GUIEdge e) {
        if (this.isAncestorOf(e))
            this.remove(e);
    }
    
    
    
    
    public void setEdgeDataField(GUIEdge edge, 
                                 String value, 
                                 GUIEdgeInfo info) {
        Object data = edge_data_array.GetObj(edge);
        if (data instanceof Integer) {
            int fill_int;
            try {
                fill_int = (Integer.decode(value)).intValue();
            } catch (NumberFormatException e) {
                String err = "Ignored edit --- specified value (\""
                    + value + "\") was out of range or not a valid integer.";
                net_window.notifyOfSolverError(err);

                /*
                net_window.putStatusLineInfo("Ignored edit --- specified value"
                                 +" (\""+info.textfield.getText()+"\") "
                                 +"was out of range or not a valid integer.");
                */
                return;
            }
            edge_data_array.Put(edge,new Integer(fill_int));
            info.label = new String(edge_data_array.GetObj(edge).toString());
            return;
        }
        else if (data instanceof String) {
            edge_data_array.Put(edge,value);
            info.label = new String(edge_data_array.GetObj(edge).toString());
            return;
        }
        else {
            String err = "Ignored edit --- data field is not of "
                + "\"String\" or \"Integer\" type";
            net_window.notifyOfSolverError(err);
        }
    }
    
    public void setNodeDataField(GUINode node, 
                                 String value,
                                 GUINodeInfo info) {
        Object data = node_data_array.GetObj(node);
        if (data instanceof Integer) {
            int fill_int;
            try {
                fill_int = (Integer.decode(value)).intValue();
            } catch (NumberFormatException e) {
                String err = "Ignored edit --- specified value (\""
                    + value + "\") was out of range or not a valid integer.";
                net_window.notifyOfSolverError(err);

                return;
            }
            node_data_array.Put(node,new Integer(fill_int));
            info.label = new String(node_data_array.GetObj(node).toString());
            return;
        }
        else if (data instanceof String) {
            node_data_array.Put(node,value);    // dsd 010317 and next line per jon 2/27
            //node_data_array.Put(node,new String(info.textfield.getText()));
            info.label = new String(node_data_array.GetObj(node).toString());
            return;
        }
        else {
            String err = "Ignored edit --- data field is not of "
                + "\"String\" or \"Integer\" type";
            net_window.notifyOfSolverError(err);
        }
    }
    
    private String node_data_name;
    private UiNodeArray node_data_array;
    private boolean node_data_editable = false;
    public final boolean displayedNodeDataIsEditable() {
        return node_data_editable;
    }
    synchronized public void updateNodeData(String name,
                                            boolean editable) {
        node_data_editable = editable;
        node_data_name = new String(name);
        node_data_array = UiNet.GetNodeArray(node_data_name);
        if (node_data_array == null) {
            GIDEN.out.println("Array \""+node_data_name+"\" is null");
            return;
        }
        
        GUINode node;
        Object data;
        GUINodeInfo info;
        GUINodeListIter nit = UiNet.GetNodeListIter();
        for (node=nit.GetFirstNode();node!=null;node=nit.GetNextNode()) {
            info = node.GetInfo();
            
            data = node_data_array.GetObj(node);
            if (data == null)
                info.label = new String("?");
            else
                info.label = new String(data.toString());
            node.PutInfo(info);
        }
        UTILITIES.Repaint(this,"updateNodeData");
    }
    
    
    private String edge_data_name;
    private UiEdgeArray edge_data_array;
    private boolean edge_data_editable = false;
    public final boolean displayedEdgeDataIsEditable() {
        return edge_data_editable;
    }
    synchronized public void updateEdgeData(String name,
                                            boolean editable) {
        edge_data_editable = editable;
        edge_data_name = new String(name);
        edge_data_array = UiNet.GetEdgeArray(edge_data_name);
        if (edge_data_array == null) {
            GIDEN.out.println("Array \""+edge_data_name+"\" is null");
            return;
        }
        
        GUIEdge edge;
        Object data;
        GUIEdgeInfo info;
        GUIEdgeListIter eit = UiNet.GetEdgeListIter();
        for (edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge()) {
            info = edge.GetInfo();
            
            data = edge_data_array.GetObj(edge);
            if (data == null)
                info.label = new String("?");
            else
                info.label = new String(data.toString());
            edge.PutInfo(info);
        }
        UTILITIES.Repaint(this,"updateEdgeData");
    }
    
    
    synchronized public void resetState() {
        if (current_tool == GUIController.SELECT_ITEM_TOOL) {
            deselectGUINodesAndGUIEdges();
            net_window.disableEditModeMenu("Select All");
            net_window.disableEditModeMenu("Select All Nodes");
        }
        reset();
        UTILITIES.Repaint(this,"resetState");
    }
    
    private int current_tool;
    synchronized public void updateTool(int tool) {
        if (current_tool == GUIController.SELECT_ITEM_TOOL &&
            tool != GUIController.SELECT_ITEM_TOOL) {
            deselectGUINodesAndGUIEdges();
            net_window.disableEditModeMenu("Select All");
            net_window.disableEditModeMenu("Select All Nodes");
        }
        
        this.current_tool = tool;
        
        GUINode node;
        GUINodeListIter nit = UiNet.GetNodeListIter();
        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
            node.updateTool(tool);
        
        GUIEdge edge;
        GUIEdgeListIter eit = UiNet.GetEdgeListIter();
        for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
            edge.updateTool(tool);
        
        
        reset();
        
        if (tool == GUIController.SELECT_ITEM_TOOL) {
            net_window.enableEditModeMenu("Select All");
            net_window.enableEditModeMenu("Select All Nodes");
        }
        
        UTILITIES.Repaint(this,"updateTool");
    }
    public int getTool() {return current_tool;}
    
    public boolean inSolverMode() {return net_window.inSolverMode();}
    //  synchronized public void registerSolverNodeSelection(UiNode i, 
    //                                                   int modifiers) {
    //    net_window.registerSolverNodeSelection(i,modifiers);
    //  }
    //  synchronized public void registerSolverEdgeSelection(UiEdge e, 
    //                                                   int modifiers) {
    //    net_window.registerSolverEdgeSelection(e,modifiers);
    //  }
    synchronized public void registerSolverModeMouseSelection(Object obj,
                                                              int modifiers) {
        net_window.registerSolverModeMouseSelection(obj,modifiers);
    }
    

    private boolean directed_edges;
    //    public boolean isDirected() {return directed_edges;}
    synchronized public void setDirected(boolean directed) {
        this.directed_edges = directed;

        GUIEdge edge;
        GUIEdgeListIter eit = UiNet.GetEdgeListIter();
        for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
            edge.setDirected(directed_edges);

        UTILITIES.Repaint(this,"setDirected");
    }
    
    
    private GUINode action_node = null;
    private boolean action_node_specified = false;
    public void registerNodeSelection(GUINode i, int selection_modifiers) {
        if (current_tool == GUIController.NEW_EDGE_TOOL) {
            if (action_node_specified && !action_node.equals(i)) {
                createEdge(action_node,i);
                UTILITIES.Repaint(this,"registerNodeSelection (1)");
                
                action_node = null;
                action_node_specified = false;
            }
            else {
                action_node = i;
                action_node_specified = true;
            }
        }
        else if (current_tool == GUIController.TRAIL_EDGE_TOOL) {
            if (action_node_specified && !action_node.equals(i)) {
                createEdge(action_node,i);
                UTILITIES.Repaint(this,"registerNodeSelection (2)");
                
                action_node = i;
            }
            else {
                action_node = i;
                action_node_specified = true;
            }
        }
        else if (current_tool == GUIController.STAR_EDGE_TOOL) {
            if (action_node_specified && !action_node.equals(i)) {
                createEdge(action_node,i);
                UTILITIES.Repaint(this,"registerNodeSelection (3)");
            }
            else {
                action_node = i;
                action_node_specified = true;
            }
        }
        else if (current_tool == GUIController.DELETE_ITEM_TOOL) {
            //            UiNet.DeleteNode(i);
            //            removeGUINodeFromCanvas(i);
            deleteNode(i);
            UTILITIES.Repaint(this,"registerNodeSelection (4)");
        }
        else if (current_tool == GUIController.SELECT_ITEM_TOOL) {
            if ((selection_modifiers & InputEvent.SHIFT_MASK) != 0) 
                toggleSelectGUINode(i);
            else {
                GUINodeInfo info = i.GetInfo();
                if (info == null) return;
                
                if (info.in_group_selection) {
                    if (deselectGUINodesAndGUIEdges() > 1)
                        selectGUINode(i);
                }
                else {
                    deselectGUINodesAndGUIEdges();
                    selectGUINode(i);
                }
            }
        }
    }
    
    
    public void registerEdgeSelection(GUIEdge e, int selection_modifiers) {
        if (current_tool == GUIController.DELETE_ITEM_TOOL) {
            //            UiNet.DeleteEdge(e);
            //            removeGUIEdgeFromCanvas(e);
            deleteEdge(e);
            UTILITIES.Repaint(this,"registerEdgeSelection (1)");
        }
        else if (current_tool == GUIController.REVERSE_EDGE_TOOL) {
            UiNet.ReverseEdge(e);
            UTILITIES.Repaint(this,"registerEdgeSelection (2)");
        }
        else if (current_tool == GUIController.SELECT_ITEM_TOOL) {
            if ((selection_modifiers & InputEvent.SHIFT_MASK) != 0)
                toggleSelectGUIEdge(e);
            else {
                GUIEdgeInfo info = e.GetInfo();
                if (info == null) return;
                
                if (info.in_group_selection) {
                    if (deselectGUINodesAndGUIEdges() > 1)
                        selectGUIEdge(e);
                }
                else {
                    deselectGUINodesAndGUIEdges();
                    selectGUIEdge(e);
                }
            }
        }
    }
    
    
    public void registerNodeMove(GUINode n) {
        GUIEdge e;

        /* -- don't have GUINode edge list iters
        GUIEdgeListIter eit = n.GetAdjListIter();
        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
            setEuclideanLength(e);
        */
        GUIEdgeListIter eit = UiNet.GetEdgeListIter();
        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
            if (UiNet.GetSource(e) == n || UiNet.GetTarget(e) == n)
                setEuclideanLength(e);

        UiEdgeArray EL = UiNet.GetEdgeArray(UiNetwork.EuclideanLengthString);
        if (edge_data_array.equals(EL)) {
            Object data;
            GUIEdgeInfo info;

            // need to update the edge labels
            /* -- don't have GUINode edge list iters
            eit = n.GetAdjListIter();
            for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
                info = e.GetInfo();
                data = edge_data_array.GetObj(e);
                if (data == null)
                    info.label = new String("?");
                else
                    info.label = new String(data.toString());
                e.PutInfo(info);
            }
            */
            eit = UiNet.GetEdgeListIter();
            for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
                if (UiNet.GetSource(e) == n || UiNet.GetTarget(e) == n) {
                    info = e.GetInfo();
                    data = edge_data_array.GetObj(e);
                    if (data == null)
                        info.label = new String("?");
                    else
                        info.label = new String(data.toString());
                    e.PutInfo(info);
                }
            }

            UTILITIES.Repaint(this,"registerNodeMove");
        }
    }

    /*
    public static int getEuclideanDistance(GUINode i, GUINode j) {
        // calculate euclidean distance between nodes
        Point c1 = i.getCenterPoint();
        Point c2 = j.getCenterPoint();
        return (int) Math.sqrt((c1.x-c2.x)*(c1.x-c2.x) +
                                       (c1.y-c2.y)*(c1.y-c2.y));
    }
    */
    public void setEuclideanLength(GUIEdge e) {
        UiEdgeArray EL = UiNet.GetEdgeArray(UiNetwork.EuclideanLengthString);
        if (EL != null) {
            // set "euclidean length" of edge
            EL.Put(e,GUIUserUtil.getEuclideanDistance(UiNet.GetSource(e),
                                                      UiNet.GetTarget(e)));
        }
    }
    public void deleteEdge(GUIEdge edge) {
        UiNet.DeleteEdge(edge);
        removeGUIEdgeFromCanvas(edge);
    }
    
    
    public GUIEdge createEdge(GUINode e1, GUINode e2) {
        Edge net_edge = UiNet.GetNet().CreateEdge(e1.GetNode(),e2.GetNode());
        return createEdge(net_edge,GUIEdge.DEFAULT_EDGE_STYLE);
    }
    public GUIEdge createEdge(Edge net_edge,int style) {

        GUIEdge edge;
        switch(style) {
        case GUIEdge.THIN_EDGE_STYLE:
            edge = new GUIEdge(gui_controller,this,directed_edges,true,
                               new Font("DialogInput",Font.PLAIN,12),
                               "",defaultItemColor,1);
            break;

        default:
        case GUIEdge.DEFAULT_EDGE_STYLE:
            edge = new GUIEdge(gui_controller,this,directed_edges,true,
                               new Font("DialogInput",Font.PLAIN,12),
                               "",defaultItemColor,3);
            break;
        }
        
        //UiNet.CreateEdge(edge,e1,e2);
        UiNet.CreateEdgeX(net_edge,edge);
        
        // calculate "euclidean length" of edge
        setEuclideanLength(edge);

        // update the label to use specified data field
        GUIEdgeInfo info = edge.GetInfo();
        Object data = edge_data_array.GetObj(edge);
        if (data == null)
            info.label = new String("?");
        else
            info.label = new String(data.toString());
        edge.PutInfo(info);
        
        
        this.add(edge,0);
        
        return edge;
    }
    
    
    /*-----------------------------------------------------------
    | 001203 modified by dsd
    | # support for AddNode()
    | - the net node is created before the uinode
    | - the net node index sets the ID
    -----------------------------------------------------------*/
    public GUIEdge createEdge(Edge net_edge, UiEdgeInfo inp_info) {

        
        boolean visible=inp_info.visible;
        Font font=new Font("DialogInput",Font.PLAIN,12);
        String label=inp_info.label;
        Color color=inp_info.color;
        
        GUIEdge edge = new GUIEdge(gui_controller,this,directed_edges,visible,
                                 font,label,color,3);
        edge.PutUiInfo(inp_info);
        
        //UiNet.CreateEdge(edge,e1,e2);
        UiNet.CreateEdgeX(net_edge,edge);
        
        // calculate "euclidean length" of edge
        setEuclideanLength(edge);

        // update the label to use specified data field
        //if(label==null)
        /*
        {
            GUIEdgeInfo info = edge.GetInfo();
            Object data = edge_data_array.GetObj(edge);
            if (data == null)
                info.label = new String("?");
            else
                info.label = new String(data.toString());
            edge.PutInfo(info);
        }
        */
        
        
        this.add(edge,0);
        
        return edge;
        
        
    }
    
    
    
   

    public void deleteNode(GUINode node) {
        UiNet.DeleteNode(node);
        //removeGUINodeFromCanvas(node);
    }
    
    public GUINode createNode(Point p) {
        return createNode(p,GUINode.DEFAULT_NODE_STYLE);
    }
    
    /*-----------------------------------------------------------
    | 001203 modified by dsd
    | # support for AddNode()
    | - the net node is passed from outside
    -----------------------------------------------------------*/
    public GUINode createNode(Node node, Point p) {
        return createNode(node,p,GUINode.DEFAULT_NODE_STYLE);
    }
    /*-----------------------------------------------------------
    | 001203 modified by dsd
    | # support for AddNode()
    | - the net node is created and passed to createNode()
    -----------------------------------------------------------*/
    public GUINode createNode(Point p, int style) {
        Node net_node = UiNet.GetNet().CreateNode();
        return createNode(net_node,p,style);
    }
    
    /*-----------------------------------------------------------
    | 001203 modified by dsd
    | # support for AddNode()
    | - the net node is created before the uinode
    | - the net node index sets the ID
    -----------------------------------------------------------*/
    public GUINode createNode(Node net_node, Point p, int style) {
        GUINode node;
        
        // this could be a problem for solver-generated nodes!
        if (canvas_grid.isEnabled())
            p = canvas_grid.getNearestGridPoint(p);


        switch(style) {
        case GUINode.IMAGE_NODE_STYLE:
            // just for fun, let's try creating nodes using images...
            node = new GUINode(gui_controller,this,
                               new GUINodeImageComponent("image/nodes/3-D-node.gif"),
                               true, new Font("DialogInput",Font.PLAIN,12),
                               "",defaultItemColor,p.x,p.y);
            break;

        case GUINode.SMALL_NODE_STYLE:
            node = new GUINode(gui_controller,this,6,true,
                               new Font("DialogInput",Font.PLAIN,12),
                               "",defaultItemColor,p.x,p.y);
            break;

        case GUINode.STAR_NODE_STYLE:
            node = new GUINode(gui_controller,this,
                               GUIPolygonUtil.getStarPolygon(20),true,
                               new Font("DialogInput",Font.PLAIN,12),
                               "",defaultItemColor,p.x,p.y);
            break;

        default:
        case GUINode.DEFAULT_NODE_STYLE:
            // this is the real code used to create nodes...
            node = new GUINode(gui_controller,this,18,true,
                               new Font("DialogInput",Font.PLAIN,12),
                               "",defaultItemColor,p.x,p.y);
            break;
        }

        UiNet.CreateNodeX(net_node,node);    // 001203 dsd, added net_node param
        
        // update the label to use specified data field
        GUINodeInfo info = node.GetInfo();
        Object data = node_data_array.GetObj(node);
        if (data == null)
            info.label = new String("?");
        else
            info.label = new String(data.toString());
        node.PutInfo(info);
                
        this.add(node,0);
        return node;
        
        // old code for creating different shape nodes.  we won't allow
        // users to do this anymore...
        /* 
           if ((event_modifiers & InputEvent.SHIFT_MASK) != 0)
               node = new GUINode(gui_controller,this,
                                  GUIPolygonUtil.getStarPolygon(20),true,
                                  new Font("DialogInput",Font.PLAIN,12),
                                  "2",defaultItemColor,p.x,p.y);
           else if ((event_modifiers & InputEvent.CTRL_MASK) != 0) 
               node = new GUINode(gui_controller,this,
                                  GUIPolygonUtil.getHousePolygon(15),true,
                                  new Font("DialogInput",Font.PLAIN,12),
                                  "3", defaultItemColor,p.x,p.y);
           else if ((event_modifiers & InputEvent.ALT_MASK) != 0)
               node = new GUINode(gui_controller,this,
                                  new GUINodeImageComponent("image/nodes/test.gif"),
                                  true, new Font("DialogInput",Font.PLAIN,12),
                                  "4",defaultItemColor,p.x,p.y);
           else
               node = new GUINode(gui_controller,this,15,true,
                                  new Font("DialogInput",Font.PLAIN,12),
                                  "1",defaultItemColor,p.x,p.y);
        */
    }
    
    
    /*-----------------------------------------------------------
    | 001203 modified by dsd
    | # support for AddNode()
    | - the net node is created before the uinode
    | - the net node index sets the ID
    -----------------------------------------------------------*/
    public GUINode createNode(Node net_node, UiNodeInfo inp_info) {
        GUINode node;
        
        // this is the real code used to create nodes...
        int radius=inp_info.radius;
        boolean visible=inp_info.visible;
        Font font=new Font("DialogInput",Font.PLAIN,12);
        String label=inp_info.label;
        Color color=inp_info.color;
        int x=inp_info.x;
        int y=inp_info.y;
        node = new GUINode(gui_controller,this,radius,visible,
                               font,label,color,x,y);
        UiNet.CreateNodeX(net_node,node);    // 001203 dsd, added net_node param
                
        this.add(node,0);
        return node;
        
    }
    
    
    

    public void print(Graphics g) {
        // The following is needed for high resolution printing.  It
        // is disabled because there seems to be a Win32 bug with
        // printing graphics and text together that yields poor
        // results.  As a temporary solution (until Java is fixed), we
        // will simply call update() instead...
        //
        //        direct_draw(g,getBounds().width,getBounds().height);

        // delete this line when Java bug is fixed, and we will have hi-res
        //        update(g);

        // new temp fix: we use hi-res printing for linux, and low-res
        // for all else (including Win32)
        /*
        String os_name = System.getProperty("os.name","Unknown").toLowerCase();
        if (os_name.lastIndexOf("linux") != -1)
            direct_draw(g,getBounds().width,getBounds().height); // Linux
        else
            update(g);
        */

        // new temp fix: we use hi-res printing for jdk1.1, and low-res
        // for all else
        String java_version = System.getProperty("java.version").toLowerCase();
        if (java_version.indexOf("1.1") == 0)
            direct_draw(g,getBounds().width,getBounds().height); // jdk-1.1
        else
            update(g); // all other versions
    }
    public void printAll(Graphics g) {print(g);}
    public void printComponents(Graphics g) {print(g);}
    private void direct_draw(Graphics g, int width, int height) {
        g.setColor(Color.white);
        g.fillRect(0,0,width,height);

        // draw grid
        canvas_grid.draw(g);

        // draw edges
        GUIEdge edge;
        GUIEdgeListIter eit = UiNet.GetEdgeListIter();
        for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
            edge.draw(g);

        // draw nodes
        GUINode node;
        GUINodeListIter nit = UiNet.GetNodeListIter();
        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
            node.draw(g);

        // draw canvas selection box
        if (selection_box_enabled) {
            int rectX, rectY;
            int rectW = selection_box_end.x - selection_box_start.x;
            int rectH = selection_box_end.y - selection_box_start.y;
            
            if (rectW >= 0)
                rectX = selection_box_start.x;
            else {
                rectX = selection_box_end.x;
                rectW = -rectW;
            }
            
            if (rectH >= 0)
                rectY = selection_box_start.y;
            else {
                rectY = selection_box_end.y;
                rectH = -rectH;
            }

            //      g.setColor(new Color(48,188,48));
            g.setColor(Color.blue);
            g.drawRect(rectX,rectY,rectW,rectH);
            //      g.draw3DRect(rectX,rectY,rectW,rectH,true);
        }
    }


    public boolean repaint_enabled = true;
    public void paint(Graphics g) {update(g);}
    public void update(Graphics g) {
        if (!repaint_enabled) return;
        
        int width = Math.max(getBounds().width,1);
        int height = Math.max(getBounds().height,1);
        
        if (backBuffer == null ||
            backBuffer.getWidth(null) != width ||
            backBuffer.getHeight(null) != height) {
            backBuffer = createImage(width,height);
            if (backBuffer != null) {
                if (backGraphics != null)
                    backGraphics.dispose();
                backGraphics = backBuffer.getGraphics();
            }
        }
        if (backBuffer != null) {

            // use "direct_draw" to draw on backGraphics
            direct_draw(backGraphics, width, height);
            
            // draw buffered image to canvas
            g.drawImage(backBuffer,0,0,null);
        }
    }
    
    
    // Event Handlers
    public void mouseClicked(MouseEvent evt) {}
    public void mouseEntered(MouseEvent evt) {
        requestFocus();
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
    public void mouseExited(MouseEvent evt) {}
    public void mousePressed(MouseEvent evt) {
        if (inSolverMode()) {
            registerSolverModeMouseSelection(null,evt.getModifiers());
            return;
        }

        if (gui_controller == null) return;
        
        int selX = evt.getX();
        int selY = evt.getY();

        if (current_tool == GUIController.NEW_NODE_TOOL) {
            
            // check proximity to existing nodes
            GUINode node;
            GUINodeListIter nit = UiNet.GetNodeListIter();
            for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode()) {
                if (node != null && node.contains(selX,selY)) {
                    GIDEN.out.println("Shouldn't be here ....");
                    return;
                }
            }
            
            createNode(evt.getPoint());
            repaint_enabled = true;
            UTILITIES.Repaint(this,"mousePressed (1)");
        }
        else if (current_tool == GUIController.TRAIL_EDGE_TOOL) {
            GUINode i = createNode(evt.getPoint());
            if (action_node_specified)
                createEdge(action_node,i);
            else
                action_node_specified = true;
            
            action_node = i;
            repaint_enabled = true;
            UTILITIES.Repaint(this,"mousePressed (2)");
        }
        else if (current_tool == GUIController.STAR_EDGE_TOOL) {
            GUINode i = createNode(evt.getPoint());
            if (action_node_specified)
                createEdge(action_node,i);
            else {
                action_node_specified = true;
                action_node = i;
            }
            
            repaint_enabled = true;
            UTILITIES.Repaint(this,"mousePressed (3)");
        }
        else if (current_tool == GUIController.SELECT_ITEM_TOOL) {
            // temporarily remove nodes and edged from canvas parent
            // heirarchy to avoid them intercepting drag events
            removeGUINodesAndGUIEdgesFromCanvas();
            
            // start rubber-band box
            selection_box_enabled = true;
            selection_box_start = evt.getPoint();
            selection_box_end = evt.getPoint();
            repaint_enabled = true;
            UTILITIES.Repaint(this,"mousePressed (4)");
        }
    }
    public void mouseReleased(MouseEvent evt) {
        if (gui_controller == null) return;
        
        if (current_tool == GUIController.MOVE_SELECTED_TOOL) {
        }
        else if (current_tool == GUIController.SELECT_ITEM_TOOL) {
            // restore nodes and edges as components of canvas
            addGUINodesAndGUIEdgesToCanvas();
            
            if (selection_box_enabled) {
                // end rubber-band box 
                selection_box_end = evt.getPoint();
                repaint_enabled = true;
                UTILITIES.Repaint(this,"mouseReleased");
                
                
                // create rectangle for selection box
                int rectX, rectY;
                int rectW = selection_box_end.x - selection_box_start.x;
                int rectH = selection_box_end.y - selection_box_start.y;
                
                if (rectW >= 0)
                    rectX = selection_box_start.x;
                else {
                    rectX = selection_box_end.x;
                    rectW = -rectW;
                }
                
                if (rectH >= 0)
                    rectY = selection_box_start.y;
                else {
                    rectY = selection_box_end.y;
                    rectH = -rectH;
                }
                Rectangle selection_rectangle = new Rectangle(rectX,rectY,rectW,rectH);
                
                // select components in box:
                //-- if we don't have shift down, then deselect all first
                if ((evt.getModifiers() & InputEvent.SHIFT_MASK) == 0)
                    deselectGUINodesAndGUIEdges();
                
                //-- go ahead and select items in box
                GUINode i;
                GUINodeListIter nit = UiNet.GetNodeListIter();
                for (i=nit.GetFirstNode() ; i != null ; i=nit.GetNextNode())
                    if (selection_rectangle.contains(i.getCenterPoint()))
                        selectGUINode(i);
                
                GUIEdge e;
                GUIEdgeListIter eit = UiNet.GetEdgeListIter();
                for (e=eit.GetFirstEdge() ; e != null ; e=eit.GetNextEdge())
                    if (selection_rectangle.contains(e.getSplinePoint()))
                        selectGUIEdge(e);
                
                
                selection_box_enabled = false;
            }
        }
    }
    
    public void mouseMoved(MouseEvent evt) {}
    public void mouseDragged(MouseEvent evt) {
        
        int selX = evt.getX();
        int selY = evt.getY();
        
        Dimension canvas_size = this.getSize();
        
        if (selX < 0) selX = 0;
        if (selY < 0) selY = 0;
        if (selX >= canvas_size.width) selX = canvas_size.width-1;
        if (selY >= canvas_size.height) selY = canvas_size.height-1;
        
        if (current_tool == GUIController.SELECT_ITEM_TOOL) {
            if (selection_box_enabled) {
                // resize rubber-band box
                selection_box_end = new Point(selX,selY);
                repaint_enabled = true;
                UTILITIES.Repaint(this,"mouseDragged");
            }
        }
    }
}
