package giden.GUI;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.io.*;
import java.awt.*;
import java.awt.event.*;


public class GUIModalMessageDialog extends Dialog implements ActionListener, 
                                                      KeyListener,
                                                      FocusListener,
                                                      ComponentListener,
                                                      MouseListener,
                                                      WindowListener
{
    Button dismiss;
    
    public static final String WARNING = new String("Warning!");
    public static final String ERROR   = new String("Error!");
    public static final String NOTICE  = new String("Notice...");
    public static final String NEUTRAL = new String("");

    public static final int LABEL_DISPLAY = 0;
    public static final int AREA_DISPLAY  = 1;

    public static final int CENTER_ON_PARENT = 0;
    public static final int CENTER_ON_WINDOW = 1;

    public GUIModalMessageDialog(Frame parent, String title, String message) {
        this(parent,title,message,LABEL_DISPLAY);
    }

    public GUIModalMessageDialog(Frame parent, 
                                 String title, 
                                 String message,
                                 int display_type) {
        this(parent,title,message,display_type,CENTER_ON_PARENT);
    }

    public GUIModalMessageDialog(Frame parent, 
                                 String title, 
                                 String message,
                                 int display_type,
                                 int position_type) {
        super(parent, title, true/*modal*/);
        setBackground(Color.lightGray);
        addWindowListener(this);
        
        GridBagLayout gbl  = new GridBagLayout();
        setLayout(gbl);
        
        // setup default data for components of the gridbag
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridwidth  = 1;
        gbc.gridheight = 1;
        gbc.ipadx      = 3;
        gbc.ipady      = 3;
        gbc.insets     = new Insets(3,3,3,3);    // (top,left,bottom,right)
        
        // setup constraints for the message
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.gridx      = 0;
        gbc.gridy      = 0;
        gbc.weightx    = 0;
        gbc.weighty    = 0;
        if (display_type == LABEL_DISPLAY) {
            Label l = new Label(message);
            gbl.setConstraints(l,gbc);
            add(l);
        }
        else if (display_type == AREA_DISPLAY) {
            int ta_width = 40;
            int ta_height = (int)Math.ceil(message.length()/(float)(ta_width));
            TextArea ta = new TextArea("",ta_height,ta_width,
                                       TextArea.SCROLLBARS_VERTICAL_ONLY);
            ta.setEditable(false);
            ta.setText(message);
            gbl.setConstraints(ta,gbc);
            add(ta);
        }
        
        // setup constraints for the dismiss button
        dismiss = new Button("OK");
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.gridx      = 0;
        gbc.gridy      = 1;
        gbc.weightx    = 0;
        gbc.weighty    = 0;
        gbl.setConstraints(dismiss,gbc);
        dismiss.addActionListener(this);
        dismiss.addKeyListener(this);
        add(dismiss);
        
        addFocusListener(this);
        addMouseListener(this);
        addComponentListener(this);
        pack();
        
        if (position_type == CENTER_ON_WINDOW) {
            Dimension screen_size = Toolkit.getDefaultToolkit().getScreenSize();
            Dimension my_size = getSize();
            setLocation(new Point((screen_size.width - my_size.width)/2,
                                  (screen_size.height - my_size.height)/2));
        }
        else {
            Rectangle parent_bounds = parent.getBounds();
            Rectangle my_bounds = getBounds();
            setLocation(parent_bounds.x 
                              + (parent_bounds.width-my_bounds.width)/2,
                        parent_bounds.y 
                              + (parent_bounds.height-my_bounds.height)/2);
        }
        
        setVisible(true);
        dismiss.requestFocus();
    }
    

    /**********************************************************************/
    // key listener methods
    /**********************************************************************/
    public void keyTyped(KeyEvent evt) {}
    public void keyReleased(KeyEvent evt) {}
    public void keyPressed(KeyEvent evt) {
        int key_code = evt.getKeyCode();
        Object src = evt.getSource();

        if (key_code == KeyEvent.VK_ENTER) {
            if (src.equals(dismiss))
                dispose();
        }
    }

    /**********************************************************************/
    // focus listener methods
    /**********************************************************************/
    public void focusGained(FocusEvent evt) {dismiss.requestFocus();}
    public void focusLost(FocusEvent evt) {}

    /**********************************************************************/
    // mouse listener methods
    /**********************************************************************/
    public void mouseClicked(MouseEvent evt) {}
    public void mousePressed(MouseEvent evt) {}
    public void mouseReleased(MouseEvent evt) {}
    public void mouseEntered(MouseEvent evt) {dismiss.requestFocus();}
    public void mouseExited(MouseEvent evt) {}

    
    /**********************************************************************/
    // action listener method
    /**********************************************************************/
    public void actionPerformed(ActionEvent evt) {
        Object source = evt.getSource();
        
        if (source.equals(dismiss))
            dispose();
    }
    
    
    /**********************************************************************/
    // component listener methods
    /**********************************************************************/
    public void componentResized(ComponentEvent evt) {
        pack(); 
        UTILITIES.Repaint(this,"componentResized");
    }
    public void componentMoved(ComponentEvent evt)   {
        pack(); 
        UTILITIES.Repaint(this,"componentMoved");
    }
    public void componentShown(ComponentEvent evt)   {
        pack(); 
        UTILITIES.Repaint(this,"componentShown");
    }
    public void componentHidden(ComponentEvent evt)  {
        pack(); 
        UTILITIES.Repaint(this,"componentHidden");
    }
    
    /**********************************************************************/
    // window listener methods
    /**********************************************************************/
    public void windowActivated(WindowEvent evt)   {}
    public void windowClosed(WindowEvent evt)      {}
    public void windowClosing(WindowEvent evt)     {dispose();}
    public void windowDeactivated(WindowEvent evt) {}
    public void windowDeiconified(WindowEvent evt) {}
    public void windowIconified(WindowEvent evt)   {}
    public void windowOpened(WindowEvent evt)      {}
}
