package giden.GUI;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.*;
import java.awt.event.*;

public class GUINode extends Component implements UiNode,
                                           ActionListener,
                                           // KeyListener,
                                           MouseListener,
                                           MouseMotionListener {

    public final static int DEFAULT_NODE_STYLE = 0;
    public final static int IMAGE_NODE_STYLE   = 1;
    public final static int SMALL_NODE_STYLE   = 2;
    public final static int STAR_NODE_STYLE    = 3;

    public GUINodeInfo gui_node_info;  // dropped "private", dsd 000506
    
    /***********************************************************************/
    // Constructor methods
    /***********************************************************************/
    GUINode() {
        //    addKeyListener(this);
        addMouseListener(this);
        addMouseMotionListener(this);
        setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    }
    
    GUINode(GUINodeInfo info) {
        this();
        PutInfo(info);
    }
    
    GUINode(GUIController gui_controller,   // called by normal create - dsd
            GUIDrawCanvas canvas,
            int           radius,
            boolean       visible,
            Font          font,
            String        label,
            Color         color,
            int           x,
            int           y) {
        this();
        
        PutInfo(new GUINodeInfo(gui_controller,canvas,radius,
                                visible,font,label,color,x,y));
        if (canvas != null)
            updateTool(canvas.getTool());
    }
    
    GUINode(GUIController gui_controller,
            GUIDrawCanvas canvas,
            GUIImageComponent image_component,
            boolean       visible,
            Font          font,
            String        label,
            Color         color,
            int           x,
            int           y) {
        this();
        
        PutInfo(new GUINodeInfo(gui_controller,canvas,image_component,
                                visible,font,label,color,x,y));
        if (canvas != null)
            updateTool(canvas.getTool());
    }
    
    GUINode(GUIController gui_controller,
            GUIDrawCanvas canvas,
            Polygon       p,
            boolean       visible,
            Font          font,
            String        label,
            Color         color,
            int           x,
            int           y) {
        this();
        
        p.translate(x,y);
        PutInfo(new GUINodeInfo(gui_controller,canvas,p,
                                visible,font,label,color,x,y));
        if (canvas != null)
            updateTool(canvas.getTool());
    }
    
    
    /***********************************************************************/
    // Info management methods
    /***********************************************************************/
    synchronized public void PutInfo(GUINodeInfo info) {
        GUINodeInfo temp = info.Clone();
        if (temp != null)
            this.gui_node_info = temp;
    }
    
    synchronized public GUINodeInfo GetInfo() {
        return gui_node_info.Clone();
    }
    
    synchronized public void PutUiInfo(UiNodeInfo info)
    {
        GUINodeInfo gui = GetInfo();
        gui.color=info.color;
        gui.label=info.label;
        gui.visible=info.visible;
        gui.radius=info.radius;
        gui.x=info.x;
        gui.y=info.y;
        PutInfo(gui);
    }
    
    synchronized public UiNodeInfo GetUiInfo() // dsd 990314
    {
        return ((UiNodeInfo)gui_node_info).UiClone();
    }
    
    
    /***********************************************************************/
    /***********************************************************************/
    synchronized public void updateTool(int tool) {
        GUINodeInfo info = GetInfo();
        info.current_tool = tool;
        PutInfo(info);
    }
    
    synchronized public Point getCenter() {return getCenterPoint();}
    synchronized public Point getCenterPoint() {
        GUINodeInfo info = GetInfo();
        if (info == null) return null;
        
        return new Point(info.x, info.y);
    }
    
    
    /***********************************************************************/
    /***********************************************************************/
    synchronized public Node GetNode() {return getNode(gui_node_info);}
    synchronized private Node getNode(GUINodeInfo info) {
        return info.node;
    }
    
    synchronized public void PutNode(Node node) {
        GUINodeInfo info = GetInfo();
        if (info == null) return;
        
        PutNode(node,info);
        PutInfo(info);
    }
    synchronized private void PutNode(Node node, GUINodeInfo info) {    // 990314 dsd, was setNode()
        info.node = node;
    }
    
    
    /***********************************************************************/
    synchronized public Polygon getPolygon() {return getPolygon(gui_node_info);}
    synchronized private Polygon getPolygon(GUINodeInfo info) {
        return info.p;
    }
    
    synchronized public void setPolygon(Polygon p) {
        GUINodeInfo info = GetInfo();
        if (info == null) return;
        
        setPolygon(p,info);
        PutInfo(info);
    }
    synchronized private void setPolygon(Polygon p, GUINodeInfo info) {
        info.p = p;
    }
    
    /***********************************************************************/
    synchronized public boolean isVisible() {return isVisible(gui_node_info);}
    synchronized private boolean isVisible(GUINodeInfo info) {
        return info.visible;
    }
    
    synchronized public void setVisible(boolean visible) {
        GUINodeInfo info = GetInfo();
        if (info == null) return;
        
        setVisible(visible,info);
        PutInfo(info);
    }
    synchronized private void setVisible(boolean visible, GUINodeInfo info) {
        info.visible = visible;
    }
    
    /***********************************************************************/
    synchronized public Font getFont() {return getFont(gui_node_info);}
    synchronized private Font getFont(GUINodeInfo info) {
        return info.font;
    }
    
    synchronized public void setFont(Font font) {
        GUINodeInfo info = GetInfo();
        if (info == null) return;
        
        setFont(font,info);
        PutInfo(info);
    }
    synchronized private void setFont(Font font, GUINodeInfo info) {
        info.font = font;
    }
    
    /***********************************************************************/
    synchronized public String getLabel() {return getLabel(gui_node_info);}
    synchronized private String getLabel(GUINodeInfo info) {
        return new String(info.label);
    }
    
    synchronized public void setLabel(String label) {
        GUINodeInfo info = GetInfo();
        if (info == null) return;
        
        setLabel(label,info);
        PutInfo(info);
    }
    synchronized private void setLabel(String label, GUINodeInfo info) {
        info.label = new String(label);
    }
    
    /***********************************************************************/
    synchronized public Color getColor() {return getColor(gui_node_info);}
    synchronized private Color getColor(GUINodeInfo info) {
        return info.color;
    }
    
    synchronized public void setColor(Color color) {
        GUINodeInfo info = GetInfo();
        if (info == null) return;
        
        setColor(color,info);
        PutInfo(info);
    }
    synchronized private void setColor(Color color, GUINodeInfo info) {
        info.color = color;
    }
    
    
    /***********************************************************************/
    /***********************************************************************/
    synchronized public void draw(Graphics g) {
        GUINodeInfo info = GetInfo();
        if (info == null) return;
        if(info.visible==false) return; // dsd 000705
        
        switch (info.shape) {
        case GUINodeInfo.POLYGON_SHAPE:
            g.setColor(info.color);
            g.fillPolygon(info.p);
            g.setColor(Color.black);
            g.drawPolygon(info.p);
            break;
            
        case GUINodeInfo.CIRCLE_SHAPE:
            g.setColor(info.color);
            g.fillOval(info.x-info.radius,
                       info.y-info.radius,
                       2*info.radius,
                       2*info.radius);
            g.setColor(Color.black);
            g.drawOval(info.x-info.radius,
                       info.y-info.radius,
                       2*info.radius,
                       2*info.radius);
            break;
            
            
        case GUINodeInfo.IMAGE_SHAPE:
            int height = info.image_component.getHeight();
            int width = info.image_component.getWidth();
            if (info.image_component instanceof GUINodeImageComponent)
                g.drawImage(((GUINodeImageComponent) info.image_component).getImage(info.color), info.x-width/2, info.y-height/2, this);
            else // instanceof GUIImageCanvas
                g.drawImage(info.image_component.getImage(), info.x-width/2, info.y-height/2, this);
            break;
            
        case GUINodeInfo.UNDEFINED_SHAPE:
        default:
            return;
        }
        
        if (info.textfield_panel != null)
            drawLabelEditPanel(g,info);
        else
            drawLabel(g,info);
    }
    
    /***********************************************************************/
    /***********************************************************************/
    public Rectangle getBounds() {
        GUINodeInfo info = GetInfo();

        if (info == null) return new Rectangle(-1,-1,-1,-1);

        switch (info.shape) {

        case GUINodeInfo.POLYGON_SHAPE:
            return info.p.getBounds();

        case GUINodeInfo.CIRCLE_SHAPE:
            return new Rectangle(info.x-info.radius,
                                 info.y-info.radius,
                                 2*info.radius,2*info.radius);

        case GUINodeInfo.IMAGE_SHAPE:
            int height = info.image_component.getHeight();
            int width = info.image_component.getWidth();
            return new Rectangle(info.x-width/2,
                                 info.y-height/2,
                                 width,height);

        case GUINodeInfo.UNDEFINED_SHAPE:
        default:
            return new Rectangle(1,1,1,1);
        }
    }

    /***********************************************************************/
    /***********************************************************************/
    public boolean contains(Point pt) {return contains(pt.x,pt.y);}
    public boolean contains(int x, int y) {
        GUINodeInfo info = GetInfo();
        
        if (info == null) return false;
        
        switch (info.shape) {
            
        case GUINodeInfo.POLYGON_SHAPE:
            return info.p.contains(x,y);
            
        case GUINodeInfo.CIRCLE_SHAPE:
            int dx = info.x - x;
            int dy = info.y - y;
            double distance = Math.sqrt((dx*dx)+(dy*dy));
            if (distance <= info.radius)
                return true;
            else
                return false;
            
        case GUINodeInfo.IMAGE_SHAPE:
            
            return info.image_component.contains(x,y,info.x,info.y);
            
        case GUINodeInfo.UNDEFINED_SHAPE:
        default:
            return false;
        }
    }
    
    /***********************************************************************/
    /***********************************************************************/
    
    public void shiftCenter(int dx, int dy) {
        GUINodeInfo info = GetInfo();
        if (info == null) return;
        
        if (info.shape == GUINodeInfo.POLYGON_SHAPE)
            info.p.translate(dx, dy);
        
        info.x += dx;
        info.y += dy;
        
        PutInfo(info);
    }
    
    synchronized private void setCenterPoint(int x, int y) {setCenter(x,y);}
    synchronized private void setCenter(int x, int y) {
        GUINodeInfo info = GetInfo();
        if (info == null) return;
        
        if (info.shape == GUINodeInfo.POLYGON_SHAPE)
            info.p.translate(x-info.x, y-info.y);
        
        info.x = x;
        info.y = y;
        
        PutInfo(info);
    }
    
    /***********************************************************************/
    private void processLabelEditPanel() {
        GUINodeInfo info = GetInfo();
        if (info == null) return;

        // save text value
        String value = info.textfield.getText();

        // tidy up textfield first (to avoid problems from synchronization)
        info.canvas.remove(info.textfield_panel);
        info.textfield = null;
        info.textfield_panel = null;
        PutInfo(info);

        // update data and (subsequently) the info
        info.canvas.setNodeDataField(this, value, info);
        PutInfo(info);

        UTILITIES.Repaint(info.canvas, " -- GUINode:processLabelEditPanel");
        // info.canvas.repaint();
    }
    synchronized private void buildLabelEditPanel(GUINodeInfo info) {
        if (info == null) return;
        
        info.textfield_panel = new Panel();
        info.textfield_panel.setLayout(new GridLayout(1,1));
        info.textfield = new TextField(info.label,
                                       (info.label.length() > 0)?info.label.length():2);
        info.textfield.addActionListener(this);
        info.textfield_panel.add(info.textfield);
        info.canvas.add(info.textfield_panel);

        PutInfo(info);
        
        UTILITIES.Repaint(info.canvas, " -- GUINode:buildLabelEditPanel");
        //info.canvas.repaint();
    }
    
    /***********************************************************************/
    /***********************************************************************/
    private synchronized void drawLabel(Graphics g, GUINodeInfo info) {
        g.setColor(Color.black);
        g.setFont(info.font); // don't save the original font... this may be bad
        FontMetrics fm = g.getFontMetrics();
        g.drawString(info.label,
                     info.x-(fm.stringWidth(info.label)/2),
                     info.y+(fm.getAscent()/2));
    }
    
    private synchronized void drawLabelEditPanel(Graphics g, 
                                                 GUINodeInfo info) {
        
        Color def_color = g.getColor();
        g.setColor(Color.black);
        info.textfield_panel.setFont(info.font);
        info.textfield_panel.doLayout();
        
        Dimension tfp_size = info.textfield_panel.getPreferredSize();
        info.textfield_panel.setSize(tfp_size);
        info.textfield_panel.setLocation(info.x-(tfp_size.width/2),
                                         info.y-(tfp_size.height/2));
        info.textfield.setCaretPosition(0);
        info.textfield.selectAll();
        info.textfield.requestFocus();
        g.setColor(def_color);
    }
    
    
    /***********************************************************************/
    /***********************************************************************/
    public boolean isFocusTraversable() {return false;}
    
    public void actionPerformed(ActionEvent evt) {
        if (evt.getSource().equals(gui_node_info.textfield) && 
            gui_node_info.textfield_panel != null)
            processLabelEditPanel();
    }
    
    public void mouseClicked(MouseEvent evt) {}
    public void mouseEntered(MouseEvent evt) {}
    public void mouseExited(MouseEvent evt) {
        if (!(this.contains(evt.getX(),evt.getY())) && 
            gui_node_info.textfield_panel != null)
            processLabelEditPanel();
    }
    public void mousePressed(MouseEvent evt) {
        GUINodeInfo info = GetInfo();
        if (info == null) return;
        
        if (info.canvas.inSolverMode()) {
            info.canvas.registerSolverModeMouseSelection(this,evt.getModifiers());
            //      info.canvas.registerSolverNodeSelection(this,evt.getModifiers());
            return;
        }
        
        if (info.current_tool == GUIController.UNKNOWN_TOOL) return;
        
        int selX = evt.getX();
        int selY = evt.getY();
        
        // obsolete; from when we had a "move node tool..."
        //    if (info.current_tool == GUIController.MOVE_NODE_TOOL ||
        //        info.current_tool == GUIController.MOVE_SELECTED_TOOL) {
        if (info.current_tool == GUIController.MOVE_SELECTED_TOOL) {
            info.selected = true;
            info.sel_offset_X = selX-info.x;
            info.sel_offset_Y = selY-info.y;
            PutInfo(info);
        }
        else if (info.current_tool == GUIController.NEW_EDGE_TOOL ||
                 info.current_tool == GUIController.TRAIL_EDGE_TOOL ||
                 info.current_tool == GUIController.STAR_EDGE_TOOL ||
                 info.current_tool == GUIController.DELETE_ITEM_TOOL) {
            info.canvas.registerNodeSelection(this,evt.getModifiers());
        }
        else if (info.current_tool == GUIController.EDIT_VALUES_TOOL) {
            if (info.canvas.displayedNodeDataIsEditable())
                buildLabelEditPanel(info);
            else
                GIDEN.out.println("Displayed node data is not editable!");
        }
        else if (info.current_tool == GUIController.SELECT_ITEM_TOOL) {
            info.sel_offset_X = selX-info.x;
            info.sel_offset_Y = selY-info.y;
            PutInfo(info);
            
            if ((evt.getModifiers() & InputEvent.BUTTON2_MASK) == 0 &&
                (evt.getModifiers() & InputEvent.CTRL_MASK) == 0)
                info.canvas.registerNodeSelection(this,evt.getModifiers());
        }
    }
    
    public void mouseReleased(MouseEvent evt) {
        GUINodeInfo info = GetInfo();
        if (info == null) return;
        
        if (info.current_tool == GUIController.UNKNOWN_TOOL) return;
        
        // obsolete; from when we had a "move node tool..."
        //    if (info.current_tool == GUIController.MOVE_NODE_TOOL ||
        //        info.current_tool == GUIController.MOVE_SELECTED_TOOL) {
        if (info.current_tool == GUIController.MOVE_SELECTED_TOOL) {
            info.selected = false;
            PutInfo(info);
            info.canvas.registerNodeMove(this);
        }
    }
    
    
    public void mouseMoved(MouseEvent evt) {}
    public void mouseDragged(MouseEvent evt) {
        GUINodeInfo info = GetInfo();
        if (info == null) return;
        
        // obsolete; from when we had a "move node tool..."
        //    if (info.current_tool == GUIController.MOVE_NODE_TOOL ||
        //        info.current_tool == GUIController.MOVE_SELECTED_TOOL) {
        if (info.current_tool == GUIController.MOVE_SELECTED_TOOL) {
            if (info.selected) {
                
                Point new_center = new Point(evt.getX()-info.sel_offset_X,
                                             evt.getY()-info.sel_offset_Y);
                /*
                  if ((evt.getModifiers() & InputEvent.SHIFT_MASK) != 0 &&
                  info.canvas.canvas_grid.isVisible())
                */
                if (info.canvas.canvas_grid.isEnabled())
                    new_center = info.canvas.canvas_grid.getNearestGridPoint(new_center);
                
                setCenter(new_center.x, new_center.y);
                /*
                  setCenter(evt.getX()-info.sel_offset_X,
                  evt.getY()-info.sel_offset_Y);
                */
                
                UTILITIES.Repaint(info.canvas, "GUINode:mouseDragged");
                // info.canvas.repaint();
            }
        }
        else if (info.current_tool == GUIController.SELECT_ITEM_TOOL &&
                 info.in_group_selection &&
                 ((evt.getModifiers() & InputEvent.BUTTON2_MASK) != 0 ||
                  (evt.getModifiers() & InputEvent.CTRL_MASK) != 0)) {
            /*
            int dx = (evt.getX()-info.sel_offset_X) - info.x;
            int dy = (evt.getY()-info.sel_offset_Y) - info.y;
            */
            //***************** replacement code starts here...
            Point new_center = new Point(evt.getX()-info.sel_offset_X,
                                         evt.getY()-info.sel_offset_Y);

            if (info.canvas.canvas_grid.isEnabled())
                new_center = info.canvas.canvas_grid.getNearestGridPoint(new_center);
            int dx = new_center.x-info.x;
            int dy = new_center.y-info.y;
            //***************** ... and ends here

            info.canvas.moveSelectedItems(dx,dy);
        }
    }
}
