package giden.GUI;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.Date;


public class GUISystemInfoDialog extends Dialog implements ActionListener, 
                                                    ComponentListener,
                                                    WindowListener
{
    private RegistrationCode rc = null;  // textfield for entering registration

    private final String fontname = "Dialog";
    private final int    fontsize = 12;
    private GridBagLayout gbl = null;

    String user_name;
    String os_name, os_arch, os_version;
    String java_version, java_vendor;
    String giden_version, giden_build_date, giden_build_host, giden_regcode;
    
    Button dismiss;

    public GUISystemInfoDialog(Frame parent) {

        super(parent,"System Information",true/*modal*/);

        gbl = new GridBagLayout();
        setLayout(gbl);
        
        user_name     = System.getProperty("user.name","Unknown");
        os_arch       = System.getProperty("os.arch","Unknown");
        os_name       = System.getProperty("os.name","Unknown");
        os_version    = System.getProperty("os.version","unknown version");
        java_version  = System.getProperty("java.version","Unknown");
        java_vendor   = System.getProperty("java.vendor","unknown vendor");

        giden_version = System.getProperty("giden.version","Unknown");
        giden_regcode = System.getProperty("giden.registration",GIDEN.defreg);
        giden_build_date = System.getProperty("giden.build.date",
                                              "unknown build date");
        giden_build_host = System.getProperty("giden.build.host",
                                              "unknown build host");


        GridBagConstraints gbc = new GridBagConstraints();
        
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.WEST;
        buildGBConstraints(gbc,0,0,1,1,0,10);
        Label l = new Label("  User Name:",Label.LEFT);
        gbl.setConstraints(l,gbc);
        add(l);
        //
        buildGBConstraints(gbc,1,0,1,1,0,0);
        l = new Label(user_name,Label.LEFT);
        l.setFont(new Font(fontname,Font.BOLD|Font.ITALIC,fontsize));
        gbl.setConstraints(l,gbc);
        add(l);
        
        
        buildGBConstraints(gbc,0,1,1,1,0,10);
        l = new Label("  Platform:",Label.LEFT);
        gbl.setConstraints(l,gbc);
        add(l);
        //
        buildGBConstraints(gbc,1,1,1,1,0,0);
        l = new Label(os_arch,Label.LEFT);
        l.setFont(new Font(fontname,Font.BOLD|Font.ITALIC,fontsize));
        gbl.setConstraints(l,gbc);
        add(l);
        
        
        buildGBConstraints(gbc,0,2,1,1,0,10);
        l = new Label("  Operating System:",Label.LEFT);
        gbl.setConstraints(l,gbc);
        add(l);
        //
        buildGBConstraints(gbc,1,2,1,1,0,0);
        String os_label;
        if (os_name.equals("Unknown"))
            os_label = os_name;
        else
            os_label = new String(os_name+" ("+os_version+")");
        l = new Label(os_label,Label.LEFT);
        l.setFont(new Font(fontname,Font.BOLD|Font.ITALIC,fontsize));
        gbl.setConstraints(l,gbc);
        add(l);
        
        
        buildGBConstraints(gbc,0,3,1,1,0,10);
        l = new Label("  Java Version:",Label.LEFT);
        gbl.setConstraints(l,gbc);
        add(l);
        //
        buildGBConstraints(gbc,1,3,1,1,0,0);
        String java_label;
        if (java_version.equals("Unknown"))
            java_label = java_version;
        else
            java_label = new String(java_version+" ("+java_vendor+")");
        l = new Label(java_label,Label.LEFT);
        l.setFont(new Font(fontname,Font.BOLD|Font.ITALIC,fontsize));
        gbl.setConstraints(l,gbc);
        add(l);
        

        buildGBConstraints(gbc,0,4,1,1,0,10);
        l = new Label("  GIDEN Version:",Label.LEFT);
        gbl.setConstraints(l,gbc);
        add(l);
        //
        buildGBConstraints(gbc,1,4,1,1,0,0);
        l = new Label(giden_version,Label.LEFT);
        l.setFont(new Font(fontname,Font.BOLD|Font.ITALIC,fontsize));
        gbl.setConstraints(l,gbc);
        add(l);


        buildGBConstraints(gbc,0,5,1,1,0,10);
        l = new Label("  GIDEN Build:",Label.LEFT);
        gbl.setConstraints(l,gbc);
        add(l);
        //
        buildGBConstraints(gbc,1,5,1,1,0,0);
        l = new Label(giden_build_date+"  ("+giden_build_host+")",Label.LEFT);
        l.setFont(new Font(fontname,Font.BOLD|Font.ITALIC,fontsize));
        gbl.setConstraints(l,gbc);
        add(l);


        buildGBConstraints(gbc,0,6,1,1,0,10);
        l = new Label("  GIDEN Registration:",Label.LEFT);
        gbl.setConstraints(l,gbc);
        add(l);
        //
        buildGBConstraints(gbc,1,6,1,1,0,0);
        if (giden_regcode.equals(GIDEN.defreg)) {
            rc = new RegistrationCode(25,"<enter code here>");
            gbl.setConstraints(rc,gbc);
            add(rc);
        }
        else {
            l = new Label(giden_regcode,Label.LEFT);
            l.setFont(new Font(fontname,Font.BOLD|Font.ITALIC,fontsize));
            gbl.setConstraints(l,gbc);
            add(l);
        }


        dismiss = new Button("  Dismiss  ");
        dismiss.addActionListener(this);
        Panel p = new Panel();
        p.add(dismiss);
        //
        gbc.fill = GridBagConstraints.BOTH;
        buildGBConstraints(gbc,0,7,2,1,0,10);
        gbl.setConstraints(p,gbc);
        add(p);
        
        // pack window
        pack();
        
        // need to set location --- center on screen
        Dimension screen_size = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension my_size = getSize();
        setLocation(new Point((screen_size.width - my_size.width)/2,
                              (screen_size.height - my_size.height)/2));
        
        setResizable(false);


        // add listeners for window
        addWindowListener(this);
        addComponentListener(this);

        // finally, make visible
        //        setVisible();
        show();
    }
    
    private void buildGBConstraints(GridBagConstraints gbc, int gx, int gy,
                                    int gw, int gh, int wx, int wy) {
        gbc.gridx = gx;
        gbc.gridy = gy;
        gbc.gridwidth = gw;
        gbc.gridheight = gh;
        gbc.weightx = wx;
        gbc.weighty = wy;
    }
        

    // method to be called after entering valid registration key
    //    private void updateRegistrationLabel(RegistrationCode rf) {
    private void updateRegistrationLabel() {

        // update label for registration code
        
        giden_regcode = System.getProperty("giden.registration",GIDEN.defreg);
        GridBagConstraints gbc = gbl.getConstraints(rc);
        remove(rc); rc = null;

        Label l = new Label(giden_regcode,Label.LEFT);
        l.setFont(new Font(fontname,Font.BOLD|Font.ITALIC,fontsize));
        gbl.setConstraints(l,gbc);
        add(l);

        //        pack();
        l.invalidate();
        validate();
    }

    
    /**********************************************************************/
    // action listener method
    /**********************************************************************/
     public void actionPerformed(ActionEvent evt) {
        Object source = evt.getSource();
        
        if (source.equals(dismiss)) {
            if (rc != null) {
                rc.keyPressed(new KeyEvent(this,
                                           KeyEvent.KEY_PRESSED,
                                           System.currentTimeMillis(),
                                           0,KeyEvent.VK_ENTER,
                                           KeyEvent.CHAR_UNDEFINED));
            }
            dispose();
        }
    }

    /**********************************************************************/
    // component listener methods
    /**********************************************************************/
    public void componentResized(ComponentEvent evt) {
        if (isShowing()) {
            pack();
            UTILITIES.Repaint(this,"componentResized");
        }
    }
    public void componentMoved(ComponentEvent evt)   {
        //        pack();
        //        UTILITIES.Repaint(this,"componentMoved");
    }
    public void componentShown(ComponentEvent evt)   {
        //        pack();
        //        UTILITIES.Repaint(this,"componentShown");
    }
    public void componentHidden(ComponentEvent evt)  {
        //        pack();
        //        UTILITIES.Repaint(this,"componentHidden");
    }
    
    
    /**********************************************************************/
    // window listener methods
    /**********************************************************************/
    public void windowActivated(WindowEvent evt)   {}
    public void windowClosed(WindowEvent evt)      {}
    public void windowClosing(WindowEvent evt)     {dispose();}
    public void windowDeactivated(WindowEvent evt) {}
    public void windowDeiconified(WindowEvent evt) {}
    public void windowIconified(WindowEvent evt)   {}
    public void windowOpened(WindowEvent evt)      {}


    // Inner class for registration code
    class RegistrationCode extends TextField implements KeyListener, 
                                                    MouseListener {
        RegistrationCode(int size, String deflabel) {
            super(size);

            setText(deflabel);
            selectAll();

            addKeyListener(this);
            addMouseListener(this);
        }
        
        public void keyTyped(KeyEvent evt) {}
        public void keyReleased(KeyEvent evt) {}
        public void keyPressed(KeyEvent evt) {
            char c = evt.getKeyChar();
            int key_code = evt.getKeyCode();
            
            if (key_code == KeyEvent.VK_ENTER) {
                String code = getText();
                if (GIDEN.isValidRegistrationCode(code)) {
                    if (GIDEN.saveRegistrationKey(code))
                        updateRegistrationLabel();
                        // updateRegistrationLabel(this);
                    else
                        setText("<Error saving valid code!>");
                }
                else {
                    setText("<invalid code>");
                    selectAll();
                }
            }

            if (!Character.isDigit(c) && !Character.isLetter(c) &&
                c != '-' && 
                key_code != KeyEvent.VK_LEFT && 
                key_code != KeyEvent.VK_RIGHT &&
                key_code != KeyEvent.VK_BACK_SPACE && 
                key_code != KeyEvent.VK_DELETE &&
                key_code != KeyEvent.VK_ENTER)
                evt.consume();
        }

        /******************************************************************/
        // mouse listener methods
        /******************************************************************/
        public void mouseClicked(MouseEvent evt)  {}
        public void mousePressed(MouseEvent evt)  {}
        public void mouseReleased(MouseEvent evt) {}
        public void mouseEntered(MouseEvent evt)  {requestFocus();}
        public void mouseExited(MouseEvent evt)   {}
    }
}
