package giden.GUI;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

// Viewer for GHTML -- "GIDEN's HyperText Markup Language"

public class GUIViewerGHTML extends Panel {
    
    private String fontname = "Serif";
    private String fw_fontname = "Monospaced";
    private int fontsize = 12;
    private int large_fontsize = fontsize+6;
    private Font current_font;
    private FontMetrics current_fm;
    
    GUIViewerGHTML() {
        super();
        setLayout(null);
        
        this.input = getInputString("help/test.html");
        initializeFonts();
        
        initializeTokenVector();
    }
    
    GUIViewerGHTML(String filename) {
        super();
        setLayout(null);
        
        this.input = getInputString(filename);
        initializeFonts();
        
        initializeTokenVector();
    }
    
    GUIViewerGHTML(String filename, String fontname, 
                   String fw_fontname, int fontsize) {
        super();
        setLayout(null);
        
        this.input       = getInputString(filename);
        this.fontname    = new String(fontname);
        this.fw_fontname = new String(fw_fontname);
        this.fontsize    = fontsize;
        
        large_fontsize = fontsize+6;
        initializeFonts();
        
        initializeTokenVector();
    }
    
    private String getInputString(String filename) {
        int count;
        char[] buf = new char[512];
        FileReader topic_file;
        String current = "";
        
        try {
            topic_file = new FileReader(filename);
            current = new String();
            while ((count = topic_file.read(buf)) >= 0)
                current = current.concat(new String(buf,0,count));
            topic_file.close();
        } catch(FileNotFoundException e) {
            GIDEN.out.println(e);
        } catch (IOException e) {
            GIDEN.out.println(e);
        }
        
        return current;
    }
    
    private boolean new_input_string = true;
    synchronized public void setInputString(String str) {
        this.input       = new String(str);
        
        new_input_string = true;
        initializeTokenVector();
        UTILITIES.Repaint(this,"setInputString");
    }
    
    private boolean first_item;
    private String  input;
    private String  buffer;
    private int     bufp;
    private Color   text_color = Color.black;
    
    
    private Font plain_font;
    private Font fw_plain_font;
    private FontMetrics plain_fm;
    private FontMetrics fw_plain_fm;
    
    private void initializeFonts() {
        plain_font      = new Font(fontname,Font.PLAIN,fontsize);
        plain_fm      = getFontMetrics(plain_font);
        
        fw_plain_font      = new Font(fw_fontname,Font.PLAIN,fontsize);
        fw_plain_fm      = getFontMetrics(fw_plain_font);
        
        current_font = plain_font;
        current_fm = plain_fm;
    }
    
    
    
    private String getNextField() {
        if (whiteSpaceDetected()) return " ";
        
        int start = bufp;
        char c;
        try {
            // look for special characters
            if (buffer.charAt(bufp)=='&') {
                do {
                    c = buffer.charAt(bufp++);
                } while (c != ';');
                
                String special_char = buffer.substring(start+1,bufp);
                if (special_char.startsWith("lt")) return "<";
                if (special_char.startsWith("gt")) return ">";
                if (special_char.startsWith("quot")) return "\"";
                if (special_char.startsWith("amp")) return "&";
            }
            else if (buffer.charAt(bufp)=='<') {
                do {
                    c = buffer.charAt(bufp++);
                } while(c != '>');
                
                // handle comments      
                if (bufp-start >=4 && //  begins with token string "<!--"
                    buffer.charAt(start+1) == '!' &&
                    buffer.charAt(start+2) == '-' &&
                    buffer.charAt(start+3) == '-') {
                    bufp--;
                    do {
                        c = buffer.charAt(bufp++);
                    } while (c != '>' ||
                             buffer.charAt(bufp-2) != '-' ||
                             buffer.charAt(bufp-3) != '-');
                }
                bufp++;
            }
            else {
                do {
                    c = buffer.charAt(bufp++);
                } while (c!=' ' && c!='\t' && c!='\n' && c!='\r' && c!='<' && c!='&');
            }
            bufp--;
            return buffer.substring(start,bufp);
        } catch(StringIndexOutOfBoundsException e) {
            return null;
        }
    }
    
    private boolean whiteSpaceDetected() {
        char c;
        boolean is_space = false;
        
        try {
            c = buffer.charAt(bufp++);
            while (c==' ' || c=='\t' || c=='\n' || c=='\r') {
                if (c == ' ' || c == '\t')
                    is_space = true;
                c = buffer.charAt(bufp++);
            }
        } catch (StringIndexOutOfBoundsException e) {}
        bufp--;
        
        return is_space;
    }
    
    public void paint(Graphics g) {
        update(g);
    }
    
    
    
    
    private Image backBuffer;
    private Graphics backGraphics;
    private int resize_threshold = 100;
    private int preferredWidth = -resize_threshold;
    private int preferredHeight = -1;
    
    public void update(Graphics g) {
        if (input == null) return;
        
        int width = getBounds().width;
        int height = getBounds().height;    
        
        if (getParent() != null && getParent() instanceof ScrollPane)
            width = ((ScrollPane) getParent()).getViewportSize().width;
        
        
        if (backBuffer == null ||
            preferredWidth != width || 
            new_input_string) {
            setupBackGraphics(width,height);
        }
        if (backBuffer != null && (preferredWidth != width || new_input_string)) {
            drawOnBackGraphics();
            new_input_string = false;
            
            // if the width or height is out of whack, we should reallocate
            // a buffer of the appropriate size, and redo the layout...
            if (preferredWidth != width || preferredHeight > height) {
                // reallocate buffer and redraw
                setupBackGraphics(preferredWidth,Math.max(preferredHeight,height));
                if (backBuffer != null) {
                    drawOnBackGraphics();
                    
                    if (getParent() != null) {
                        invalidate();
                        getParent().validate();
                    }
                }
            }
            else {
                if (getParent() != null) {
                    invalidate();
                    getParent().validate();
                }
            }
        }
        if (backBuffer != null) {
            // draw buffered image to canvas
            g.drawImage(backBuffer,0,0,null);
        }
    }
    
    private void setupBackGraphics(int width, int height) {
        backBuffer = createImage(width,height);
        if (backBuffer != null) {
            if (backGraphics != null)
                backGraphics.dispose();
            backGraphics = backBuffer.getGraphics();
        }
    }
    
    private void drawOnBackGraphics() {
        // clear the drawing buffer's background 
        clearBufferBackground();
        
        // draw on backGraphics; this also sets preferred width and height
        drawFromTokenGHTML(backGraphics,backBuffer.getWidth(null));
    }
    
    private void clearBufferBackground() {
        int w = Math.max(backBuffer.getWidth(null), getSize().width);
        int h = Math.max(backBuffer.getHeight(null), getSize().height);
        
        backGraphics.setColor(Color.white);
        backGraphics.fillRect(0,0,w,h);
    }
    
    
    public Dimension getPreferredSize() {
        return new Dimension(preferredWidth, preferredHeight);
    }
    
    public Insets getInsets() {
        int inset_offset = 10;
        
        Insets insets= (Insets)(super.getInsets()).clone();
        insets.top += inset_offset;
        insets.bottom += inset_offset;
        insets.left += inset_offset;
        insets.right += inset_offset;
        
        return insets;
    }
    
    
    Font start_font;
    Font f;
    FontMetrics fm;
    int startx;
    int xpos;
    int ypos;
    private void drawFromTokenGHTML(Graphics g, 
                                    int canvas_width) {
        preferredWidth = canvas_width;
        
        // track how wide the line goes...
        int max_width = 0;
        
        Insets insets = getInsets();
        canvas_width -= (insets.left+insets.right);
        
        f = plain_font;
        start_font = f;
        fm = g.getFontMetrics(f);
        
        g.setFont(f);
        g.setColor(text_color);
        
        startx = insets.left;
        xpos = startx;
        ypos = 0;
        int start=0;
        int ht=fm.getMaxAscent()+fm.getLeading();
        int base = fm.getMaxAscent();
        
        for (int i=0 ; i < tokens.size() ; i++) {
            Object ct = tokens.elementAt(i);
            if (ct instanceof Font) {
                f = (Font)ct;
                g.setFont(f);
                fm = g.getFontMetrics(f);
            }
            else if (ct instanceof String) {
                xpos += fm.stringWidth((String) ct);
                if (xpos > canvas_width) {
                    if (start < i) {
                        paintLine(g,start,i,ht,base);
                        start = i--;
                    }
                    else {
                        // must violate canvas width...
                        paintLine(g,start,i+1,ht,base);
                        
                        // save max width
                        max_width = Math.max(xpos,max_width);
                    }
                    start_font = f;
                    ypos+=ht;
                    xpos = startx;
                    ht = base = 0;
                }
                else {
                    if (ypos == 0) {
                        ht = Math.max(ht, fm.getMaxAscent() + fm.getMaxDescent());
                        base = Math.max(base,fm.getMaxAscent());
                    }
                    else {
                        ht = Math.max(ht, fm.getHeight());
                        base = Math.max(base,fm.getAscent()+fm.getLeading());
                    }
                }
            }
            else if (ct instanceof Integer) {
                switch(((Integer) ct).intValue()) {
                case START_NEW_PARAGRAPH:
                    // write current line
                    paintLine(g,start,i,ht,base);
                    start_font = f; start = i; ypos+=ht; xpos = startx; ht = base = 0;
                    
                    // skip forward
                    ypos+=(g.getFontMetrics(plain_font)).getHeight();
                    
                    break;
                    
                case START_NEW_LINE:
                    if (ht == 0) {
                        // skip a "line"
                        ypos+=(g.getFontMetrics(plain_font)).getHeight();
                    }
                    else {
                        // write current line
                        paintLine(g,start,i,ht,base);
                        start_font = f; start = i; ypos+=ht;
                        xpos = startx;
                        ht = base = 0;
                    }
                    break;
                    
                case ADD_HORIZONTAL_RULE:
                    // write current line
                    paintLine(g,start,i,ht,base);
                    
                    // draw line
                    ypos+=3;
                    //g.setColor(new Color(20,191,241));
                    g.setColor(Color.darkGray);
                    g.drawLine(insets.left+10,ypos,insets.left+canvas_width-10,ypos);
                    g.setColor(Color.lightGray);
                    g.drawLine(insets.left+10,ypos+1,insets.left+canvas_width-10,ypos+1);
                    g.setColor(text_color);
                    ypos+=3;
                    
                    // update cursor position and reset other info
                    start_font = f; start = i; ypos+=ht;
                    xpos = startx;
                    ht = base = 0;
                    
                    break;
                    
                case LIST_INDENT:
                    // write current line
                    paintLine(g,start,i,ht,base);
                    
                    // indent
                    startx += 4*fw_plain_fm.charWidth(' ');
                    
                    // update cursor position and reset other info
                    start_font = f; start = i; ypos+=ht;
                    xpos = startx;
                    ht = base = 0;
                    
                    break;
                    
                case LIST_UNINDENT:
                    // write current line
                    paintLine(g,start,i,ht,base);
                    
                    // indent
                    startx -= 4*fw_plain_fm.charWidth(' ');
                    
                    // update cursor position and reset other info
                    start_font = f; start = i; ypos+=ht;
                    xpos = startx;
                    ht = base = 0;
                    
                    break;
                    
                case LISTITEM_INDENT:
                    // write current line
                    paintLine(g,start,i,ht,base);
                    
                    // indent
                    startx += 2*fw_plain_fm.charWidth(' ');
                    
                    // update cursor position and reset other info
                    start_font = f; start = i; ypos+=ht;
                    xpos = startx;
                    ht = base = 0;
                    
                    break;
                    
                case LISTITEM_UNINDENT:
                    // write current line
                    paintLine(g,start,i,ht,base);
                    
                    // indent
                    startx -= 2*fw_plain_fm.charWidth(' ');
                    
                    // update cursor position and reset other info
                    start_font = f; start = i; ypos+=ht;
                    xpos = startx;
                    ht = base = 0;
                    
                    break;
                    
                case DRAW_BULLET:
                    break;
                }
            }
            else
                GIDEN.out.println("Error with token vector.");
        }
        paintLine(g,start,tokens.size(),ht,base);
        preferredWidth = Math.max(max_width,preferredWidth);
        preferredHeight = ypos+ht+insets.top+insets.bottom;
    }
    
    private void paintLine(Graphics g, int start, int end, 
                           int ht, int ac) {
        xpos = startx;
        //    ypos += getInsets().top;
        
        Font f = start_font;
        FontMetrics fm = g.getFontMetrics(f);
        g.setFont(f);
        
        for (int i=start ; i<end ; i++) {
            Object ct = tokens.elementAt(i);
            if (ct instanceof Font) {
                g.setFont(f = (Font) ct);
                fm = g.getFontMetrics(f);
            }
            else if (ct instanceof String) {
                g.drawString((String) ct, xpos, ypos+ac);
                xpos += fm.stringWidth((String) ct);
            }
            else if (ct instanceof Integer) {
                switch(((Integer) ct).intValue()) {
                case DRAW_BULLET:
                    int size = Math.min(fw_plain_fm.charWidth(' '),
                                        fw_plain_fm.getHeight());
                    
                    xpos -= 2*fw_plain_fm.charWidth(' ');
                    g.fillOval(xpos,ypos+ac-size,size,size);
                    xpos += 2*fw_plain_fm.charWidth(' ');
                    break;
                    
                default:
                    break;
                }
            }
        }
    }
    
    
    private final int START_NEW_PARAGRAPH = 0;
    private final int START_NEW_LINE      = 1;
    private final int ADD_HORIZONTAL_RULE = 2;
    private final int LIST_INDENT         = 3;
    private final int LIST_UNINDENT       = 4;
    private final int LISTITEM_INDENT     = 5;
    private final int LISTITEM_UNINDENT   = 6;
    private final int DRAW_BULLET         = 7;
    
    Vector tokens = null;
    private void initializeTokenVector() {
        //    if (tokens != null) tokens.removeAllElements();
        tokens  = new Vector();
        
        // initialize current line and reset buffer
        buffer = input;
        bufp = 0;
        
        Font f = plain_font;
        int style;
        
        String word;
        String line = "";
        while ((word = getNextField()) != null) {
            
            if (!word.startsWith("<")) {
                if (!word.equals(" "))
                    tokens.addElement(word+" ");
            }
            else {
                switch(GUIInfoGHTML.getToken(word)) {
                case GUIInfoGHTML.BOLD_FONT_BEGIN:
                    if (!f.isBold()) {
                        f = new Font(f.getName(), f.getStyle()|Font.BOLD, f.getSize());
                        tokens.addElement(f);
                    }
                    break;
                    
                case GUIInfoGHTML.BOLD_FONT_END:
                    if (f.isBold()) {
                        style = f.getStyle()^Font.BOLD;
                        f = new Font(f.getName(), style, f.getSize());
                        tokens.addElement(f);
                    }
                    break;
                    
                case GUIInfoGHTML.ITALIC_FONT_BEGIN:
                    if (!f.isItalic()) {
                        f = new Font(f.getName(), f.getStyle()|Font.ITALIC, f.getSize());
                        tokens.addElement(f);
                    }
                    break;
                    
                case GUIInfoGHTML.ITALIC_FONT_END:
                    if (f.isItalic()) {
                        style = f.getStyle()^Font.ITALIC;
                        f = new Font(f.getName(), style, f.getSize());
                        tokens.addElement(f);
                    }
                    break;
                    
                case GUIInfoGHTML.BOLDITALIC_FONT_BEGIN:
                    if (!f.isBold() || !f.isItalic()) {
                        f = new Font(f.getName(), f.getStyle()|Font.ITALIC|Font.BOLD, 
                                     f.getSize());
                        tokens.addElement(f);
                    }
                    break;
                    
                case GUIInfoGHTML.BOLDITALIC_FONT_END:
                    if (f.isBold() || f.isItalic()) {
                        style = f.getStyle();
                        if (f.isBold())
                            style ^= Font.BOLD;
                        if (f.isItalic())
                            style ^= Font.ITALIC;
                        f = new Font(f.getName(), style, f.getSize());
                        tokens.addElement(f);
                    }
                    break;
                    
                case GUIInfoGHTML.FIXED_WIDTH_FONT_BEGIN:
                    if (f.getName().equals(fontname)) {
                        f = new Font(fw_fontname,f.getStyle(),f.getSize());
                        tokens.addElement(f);
                    }
                    break;
                    
                case GUIInfoGHTML.FIXED_WIDTH_FONT_END:
                    if (f.getName().equals(fw_fontname)) {
                        f = new Font(fontname,f.getStyle(),f.getSize());
                        tokens.addElement(f);
                    }
                    break;
                    
                case GUIInfoGHTML.HEAD1_BEGIN:
                case GUIInfoGHTML.HEAD2_BEGIN:
                    // start a new line and set font to bold with a larger size
                    tokens.addElement(new Integer(START_NEW_PARAGRAPH));
                    if (f.getSize() != large_fontsize) {
                        f = new Font(f.getName(), f.getStyle(), large_fontsize);
                        tokens.addElement(f);
                    }
                    break;
                    
                case GUIInfoGHTML.HEAD1_END:
                case GUIInfoGHTML.HEAD2_END:
                    if (f.getSize() != fontsize) {
                        f = new Font(f.getName(), f.getStyle(), fontsize);
                        tokens.addElement(f);
                    }
                    tokens.addElement(new Integer(START_NEW_PARAGRAPH));
                    break;
                    
                case GUIInfoGHTML.PARAGRAPH:
                case GUIInfoGHTML.PARAGRAPH_BEGIN:
                    tokens.addElement(new Integer(START_NEW_PARAGRAPH));
                    break;
                    
                case GUIInfoGHTML.PARAGRAPH_END:
                    break;
                    
                case GUIInfoGHTML.BREAK:
                    tokens.addElement(new Integer(START_NEW_LINE));
                    break;
                    
                case GUIInfoGHTML.RULE:
                    tokens.addElement(new Integer(ADD_HORIZONTAL_RULE));
                    break;
                    
                case GUIInfoGHTML.UNORDERED_LIST_BEGIN:
                case GUIInfoGHTML.ORDERED_LIST_BEGIN:
                    //      tokens.addElement(new Integer(START_NEW_PARAGRAPH));
                    tokens.addElement(new Integer(LIST_INDENT));
                    first_item = true;
                    break;
                    
                case GUIInfoGHTML.UNORDERED_LIST_END:
                case GUIInfoGHTML.ORDERED_LIST_END:
                    if (!first_item)
                        tokens.addElement(new Integer(LISTITEM_UNINDENT));
                    tokens.addElement(new Integer(LIST_UNINDENT));
                    //      tokens.addElement(new Integer(START_NEW_PARAGRAPH));
                    break;
                    
                case GUIInfoGHTML.LIST_ITEM:
                    if (!first_item)
                        tokens.addElement(new Integer(LISTITEM_UNINDENT));
                    
                    tokens.addElement(new Integer(LISTITEM_INDENT));
                    tokens.addElement(new Integer(DRAW_BULLET));
                    first_item = false;
                    break;
                    
                case GUIInfoGHTML.UNKNOWN_GHTML_TOKEN:
                default:
                    if (word.startsWith("<") && word.length() > 1) {}
                    else
                        tokens.addElement(word);
                }
            }
        }
    }
}
