package giden.contrib.NU_IEMS_452_W04;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;
import giden.gidenSOLVERS.*;

/*
 * Copyright (c) 1993-1999 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * WE MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. WE SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY USER AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <p>
 * Test Solver is an example program that shows some simple animation
 * and network features.
 * <p>
 *
 * @see Network
 * @see SolverMenu
 * @see UserIO
 *
 * @version 	2.2 18 Jan 03
 * @author 	    D. S. Dilworth
 */


public class BipartiteSolver extends ExecBase
{
    UserBase UI;                  // user interactions
    Network Net;                  // access to network

    //--------------------
    // Animation variables
    //--------------------
    Color Set1Color;             // colors for trace sets
    Color Set2Color;
    Color EdgeColor;
    Color InitialColor;
    Color CircuitColor;

    AnimationSet Set1;           // trace sets
    AnimationSet Set2;
    AnimationSet EdgeSet;
    AnimationSet InitialSet;
    AnimationSet CircuitSet;
    AnimationSet MatchSet;

    NodeArray NodeSet;       // records the state of each node (set 1 or set 2)
    NodeArray PreEdgeSet;		// records the preceding edge
    NodeArray xcor;
    NodeArray ycor;
    EdgeArray Flow;		//records the flow value return from the MaxFlow algorithm
    EdgeArray Capacity;		//records the capacity assigned to each edge for implementing the MaxFlow algorithm
    
	int MaxMatch;

    boolean oddcircuit;

    /** Initializes invariant objects */
    public BipartiteSolver (UserBase userinfo)
    {
        super(userinfo);        // mandatory call to the base class
        UI =  userinfo;         // local access for user interactions

        // create the start and end Sets
        Set1 = UI.CreateAnimationSet("Set 1",Color.red);
        Set2 = UI.CreateAnimationSet("Set 2",Color.blue);
        EdgeSet = UI.CreateAnimationSet("",Color.lightGray);
        InitialSet = UI.CreateAnimationSet("",Color.green);
        CircuitSet = UI.CreateAnimationSet("Circuit",Color.orange);
        MatchSet = UI.CreateAnimationSet("Matching",Color.black);
        UI.CreateAnimationButtons();
    }

    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
        Net=net;                        // local access to the network
        NodeSet = new NodeArray(Net.GetCreatedNodesCount(),"NodeSet");
        NodeSet.Fill(0);

        PreEdgeSet = new NodeArray(Net.GetCreatedNodesCount(),"PreEdgeSet");

        xcor = new NodeArray(Net.GetCreatedNodesCount(),"xcor");

        ycor = new NodeArray(Net.GetCreatedNodesCount(),"ycor");

        Flow = new EdgeArray(Net.GetCreatedEdgesCount(),"Flow");
	    Flow.Fill(0);

        Capacity = new EdgeArray(Net.GetCreatedEdgesCount(),"Capacity");
	    Capacity.Fill(1000000000);


        UI.StatusLine("Setup");         // status line
        UI.MoveAll(InitialSet);           // nodes/edges -> StartSet
        return true;
    }

    /** Executes the solver */
    public boolean Execute()
    {
        UI.StatusLine("Executing");         // status line
        Net.PutDirected(false);	//remove the direction on all edges

        ExecuteBipartiteSolver();                // execute the solver to find the partition

	if(oddcircuit)
	{
        	UI.StatusLine("Graph is not bipartite and an odd-number circuit is highlighted in orange");
	}else{
		UI.StatusLine("The number of matches = "+MaxMatch);
	}

        UI.ShowPseudoCodeText("400");       // pseudocode window
        return true;
    }

    /** Identify the partition of the graph, otherwise, returns the error to main program */
    void ExecuteBipartiteSolver()
    {
        	Queue CkList = new Queue();		//Create check list
		NodeListIter nit = Net.GetNodeListIter();    //node list iterator
		EdgeListIter eit;		//edge list iterator
		Node node;
		Node Onode;
		Edge edge;
		int CurrentSet;
        	int set1nodes = 0;

		eit = Net.GetEdgeListIter();				//get the list of its adj edges
		for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())	//loop through the adj edges
		{
			UI.PutLabel(edge,"");
		}

        	//UI.ShowPseudoCodeText("10"); // pseudocode window
        	UI.ShowPseudoCodeText("30"); // pseudocode window
	        CkList.Append(node = nit.GetFirstNode());
      	 	 NodeSet.Put(node, 1);

	     	UI.ShowPseudoCodeText("40"); // pseudocode window
		UI.SingleStep(node, Set1);

		++set1nodes;

		oddcircuit = false;

		while(!CkList.IsEmpty())
		{
			node = (Node)CkList.Pop();	//take out the first node in the node queue
            		UI.StatusLine("Node poped up from the Queue: >>> "+ node.GetIndex()+ " <<<");
           		UI.ShowPseudoCodeText("90"); // pseudocode window
            		UI.SingleStep();
			CurrentSet = NodeSet.GetInt(node);		//assign the set value to variable "CurrentSet"
			eit = node.GetAdjListIter();				//get the list of its adj edges

			for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())	//loop through the adj edges
			{
				Onode = edge.GetOpposite(node);		//identify the opposite node
                		UI.ShowPseudoCodeText("120"); // pseudocode window
				UI.SingleStep(edge, EdgeSet);	//move connecting edge to the EdgeSet

				if(NodeSet.GetInt(Onode) != 0)
				{  
					// UI.ShowPseudoCodeText("140"); // pseudocode window
					if(NodeSet.GetInt(Onode) == CurrentSet)	//opposite node violates bipartite
					{
						FindCircuit(node, Onode, edge); 	//calls program to find the odd-numbercircuit
		                        	UI.ShowPseudoCodeText("220"); // pseudocode window
         					oddcircuit = true;
						return;
					}
				} else {					//opposite node has not been labelled
					PreEdgeSet.Put(Onode, edge);	//record its preceding edge
		                  	UI.ShowPseudoCodeText("160"); // pseudocode window
					if(CurrentSet == 1)
					{
						NodeSet.Put(Onode, 2);		//assign to the opposite set
						UI.SingleStep(Onode, Set2);
					} else {
						NodeSet.Put(Onode, 1);
						UI.SingleStep(Onode, Set1);
						++set1nodes;
					}
                    		UI.ShowPseudoCodeText("180"); // pseudocode window
                    		UI.SingleStep();
                    		CkList.Append(Onode);		//add to future check list
				}
			}
		}



		for (node=nit.GetFirstNode(); node!=null; node=nit.GetNextNode())
		{
			UiNodeInfo thisNode = UI.GetNodeInfo(node);
			xcor.Put(node, thisNode.x);
			ycor.Put(node, thisNode.y);
		}

		GraphState SaveGraph = new GraphState(Net);	//saving the original layout/format of graph

	  	UseMaxFlow(set1nodes);		//calling UseMaxFlow method

        	SaveGraph.Restore();		//restore the original layout/format of graph
		
		for (node=nit.GetFirstNode(); node!=null; node=nit.GetNextNode())
		{
			UiNodeInfo thisNode = UI.GetNodeInfo(node);
			thisNode.x = xcor.GetInt(node);
			thisNode.y = ycor.GetInt(node);
		}

		UI.SingleStep();


	}


	// Rearrange node locations to 2 set, add source and sink node, apply Maximum flow algorithm
	void UseMaxFlow(int set1nodes)
	{
        UI.StatusLine("Executing Max-Flow Algorithm");
		NodeListIter nit = Net.GetNodeListIter();    // node list iterator
		Node node;
		Edge edge;

		/*Rearrange nodes*/

		UI.MoveAll(EdgeSet);		//move all edges to the EdgeSet
		int dist = 60;			// the distance between two nodes in the same partition
		int x1 = 150;			// the x coordinate of the first partition
		int x2 = x1+300;		// the x coordinate of the second partition
		int y1 = 60;			// the y coordinate of the first node of the first partition
		int y2 = 60;			// the y coordinate of the first node of the second partition
		int centerline;			// the y coordinate of the line that passes through the center of the partitioned graph

		//define the location of the first node in Set 1(x1, y1) & Set 2(x2, y2)
		if (2*set1nodes > Net.GetCreatedNodesCount())
		{
			y2 = y1 + (set1nodes*dist - (Net.GetCreatedNodesCount()-1-set1nodes)*dist)/2;
		}
		else
		{
			y1 = y2 + (-set1nodes*dist + (Net.GetCreatedNodesCount()-1-set1nodes)*dist)/2;
		}
		centerline = (set1nodes*dist + y1)/2;

		//move the nodes to the 2 parallel sets
		for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())	//loop through nodes
		{
			UiNodeInfo thisNode = UI.GetNodeInfo(node);
			if (NodeSet.GetInt(node) == 1)
			{
				thisNode.x = x1;
				thisNode.y = y1;
                		UI.PutNodeInfo(node,thisNode);
                		UI.MoveToSet(node, Set1);
                		y1 += dist;

                		EdgeListIter eit = node.GetAdjListIter();
                		for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())	//loop through edges
                		{
                    		if (node.equals(edge.GetTarget()))
                        	Net.ReverseEdge(edge);
                		}
			} else {
				thisNode.x = x2;
				thisNode.y = y2;
				UI.PutNodeInfo(node,thisNode);
				UI.MoveToSet(node, Set2);
				y2 += dist;
			}
		}
        
		UI.ShowPseudoCodeText("290"); // pseudocode window
		UI.SingleStep();		//show animation

		/* Create supersink and supersource nodes and corresponding edges*/

		Node asource = Net.CreateNode();
		UI.AddNode(asource,x1/2,centerline);
		UI.PutLabel(asource,"r");

		Node asink = Net.CreateNode();
		UI.AddNode(asink,x2+x1/2,centerline);
		UI.PutLabel(asink,"s");
        	UI.ShowPseudoCodeText("300"); // pseudocode window
        	UI.SingleStep();

        	EdgeListIter eit = Net.GetEdgeListIter();

		for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())	//loop through nodes
		{
			if (!node.equals(asource) && !node.equals(asink))
			{
				if ( NodeSet.GetInt(node) == 1)
				{
					edge = Net.CreateEdge(asource,node);
					UI.AddEdge(edge);
                    		Capacity.Put(edge, 1);
                    		UI.PutLabel(edge,1);
				}
				else
				{
					edge = Net.CreateEdge(node,asink);
					UI.AddEdge(edge);
                    		Capacity.Put(edge, 1);
                    		UI.PutLabel(edge,1);
				}
			}
		}

        	UI.ShowPseudoCodeText("310"); // pseudocode window
		UI.SingleStep();		//show animation

		/* call max flow algorithm */
		//XXX
		UserBase nullbase = new UserBase(UI.Services);
	        MaxFlow solver = new PreFlowPush(nullbase);

		solver.Setup(Net);
		solver.Source = asource;
		solver.Sink = asink;
		solver.Capacity = Capacity;

		solver.Execute();
		System.out.println("The number of matches = "+solver.FlowValue);
		UI.StatusLine("The number of matches = "+solver.FlowValue);
		MaxMatch = solver.FlowValue;

         	eit = Net.GetEdgeListIter();
         	for (edge=eit.GetFirstEdge() ; edge!=null ; edge=eit.GetNextEdge()) 
		{
             		Flow.Put(edge, solver.Flow.GetInt(edge));
               		if (Flow.GetInt(edge) == 1)
			{
                    		UI.MoveToSet(edge,MatchSet);
			
			}
	         }
	      	UI.ShowPseudoCodeText("370"); // pseudocode window
        	UI.SingleStep();
	}


    /** Identify odd-number circuit by backtracking*/
    void FindCircuit(Node node, Node onode, Edge initedge)
    {
       	UI.StatusLine("Finding odd-number circuit");
		UI.SingleStep(initedge, CircuitSet);	//update the 1st edge in the circuit
        	UI.ShowPseudoCodeText("500"); // pseudocode window
		//Identify the circuit by backtracking
		//Starting from boths ends of "initedge" (an input)
		//Which create two paths along the circuit
		//Stops until the two paths meet at one node
		while (!onode.equals(node))
		{
			Edge edge = PreEdgeSet.GetEdge(node);
            		UI.ShowPseudoCodeText("560"); // pseudocode window
			UI.SingleStep(edge, CircuitSet);	//update the 1st edge in the circuit
			node = edge.GetOpposite(node);

			edge = PreEdgeSet.GetEdge(onode);
            		UI.ShowPseudoCodeText("570"); // pseudocode window
			UI.SingleStep(edge, CircuitSet);	//update the 1st edge in the circuit
			onode = edge.GetOpposite(onode);
		}
	}

}
