package giden.contrib.TSP;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;

/*
 * WE MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. WE SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY USER AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * TspSolverExhaustive is TSP solver that uses a recursive exhaustive search to find the optimal tour.
 * Copyright (c) 2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @see TspSolverBase
 * @see TspSolverNearestNeighbor
 *
 * @version 	1.0a 2004 Feb 01
 * @author 	    D. S. Dilworth
 */


public class TspSolverExhaustive extends TspSolverBase
{ 
    Node StartNode;
    NodeArray Busy;
    AnimationSet TrialSet;        // StartSet data member
    
    /** Initializes invariant objects */
    public TspSolverExhaustive (UserBase userinfo)
    {
        super(userinfo);        // mandatory call to the base class       
        TrialSet = UI.CreateAnimationSet("Trial",Color.orange);
        UI.CreateAnimationButtons();
    }
    
    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
        if(super.Setup(net)==false)
            return false;
        StartNode=null;
        UI.StatusLine("TspSolverExhaustive Setup");         // status line
        Busy = new NodeArray(NodeCount,"");
        Busy.Fill(0);
        return true;
    }
     
       
    /** Executes the solver */
    
    public boolean Shutdown()
    {
        super.Shutdown();
        return true;
    }

    
  
    /** Individually moves nodes and edges into the EndSet */
    public boolean ExecuteTspSolver()
    {
        StartNode = Net.GetNodeListIter().GetFirstNode();
        if(StartNode==null)
            return true;
        Busy.Put(StartNode,1);
        Order[0]=StartNode;
        UI.SingleStep(StartNode,TrialSet);
        UI.Trace(TrialSet);        // trace the set             
        Exhaustive(0, StartNode);
        
        SaveResultsToNetwork("TspSolverExhaustive");
        /*
        // save results
        String results = new Integer(TspLength).toString();
        int i;
        for(i=0;i<NodeCount;i++)
            results=results+",N"+Order[i].GetIndex();
        
        Net.AddNamedObject("TspSolverExhaustive",results);
        */
        return true;
    }
    
    void Exhaustive(int level, Node previous)
    {
        int length;
        if(level==NodeCount-1)
        {
            Edge edge = NodesToEdge.GetEdge(previous,StartNode);
            SingleStepVisible(true,edge,TrialSet);
            length=Evaluate();
            UI.StatusLine("Current="+length+"  Best="+TspLength);
            for(int j=0;j<NodeCount;j++)
            {
                int index = Order[j].GetIndex();
                System.out.print(" N"+index);
            }
            System.out.println(" = "+length);
            UI.Trace(TrialSet);        // trace the set
            UI.Update();
            SingleStepVisible(false,edge,InitialSet);
            return;
        }
        else
        {
            level++;
            NodeListIter nit = Net.GetNodeListIter();    // node list iterator
            for(Node node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
            {
                if(Busy.GetInt(node)==0)
                {
                    Edge edge = NodesToEdge.GetEdge(previous,node);
                    SingleStepVisible(true,edge,TrialSet);
                    UI.SingleStep(node,TrialSet);
                    
                    Busy.Put(node,1);
                    Order[level]=node;
                    Exhaustive(level,node);
                    Busy.Put(node,0);
                    
                    UI.SingleStep(node,InitialSet);
                    SingleStepVisible(false,edge,InitialSet);
                    
                }
            }
        }
    }
}

