package giden.contrib.TSP;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;

/*
 * WE MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. WE SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY USER AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * TspSolverNearestNeighbor is TSP solver that uses the nearest neigbor algorithm to find a TSP tour.
 * Copyright (c) 2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @see TspSolverBase
 * @see TspSolverExhaustive
 *
 * @version 	1.0a 2004 Feb 01
 * @author 	    D. S. Dilworth
 */


public class TspSolverNearestNeighbor extends TspSolverBase
{ 
    public Node StartNode;             // starting node from user
    NodeArray Busy;             // state of nodes (in or out of tour)
    AnimationSet AcquiredSet;   // nodes in the tour
    
    /** Initializes invariant objects */
    public TspSolverNearestNeighbor (UserBase userinfo)
    {
        super(userinfo);        // mandatory call to the base class       
        AcquiredSet = UI.CreateAnimationSet("Acquired",Color.green);
        UI.CreateAnimationButtons();
        StartNode=null;
    }
    
    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
        if(super.Setup(net)==false) // was Setup() successful?
            return false;
        UI.StatusLine("TspSolverNearest Setup");
        if(StartNode==null)
            StartNode = UI.UserMouseNodeInput("Click on Start Node");
        if(StartNode!=null)      // if the user clicked on a start node
        {
            Busy = new NodeArray(NodeCount,""); // create busy array
            Busy.Fill(0);                       // fill with 0s
            UI.PutLabel(StartNode,"s");     // Draw StartNode with "s"
            UI.SingleStep(StartNode,AcquiredSet);
            UI.Trace(AcquiredSet);          // trace the set        
        }
        else
        {
            UI.StatusLine("User Canceled"); // no StartNode
        }
        return true;
    }
       
    /** Executes the solver */
    public boolean Shutdown()
    {
        super.Shutdown();
        return true;
    }

    /** Solve TSP */
    public boolean ExecuteTspSolver()
    {
        return NearestNeighbor(StartNode);        
    }
    
    /** Solve TSP using the Nearest Neighbor algorithm */
    boolean NearestNeighbor(Node start)
    {
        boolean result=true;    // set solver result to default
        int index=0;            // index for tour order
        Busy.Put(start,1);      // start is busy
        Order[index++]=start;   // start is first in tour
        Node node=start;        // node is moved through the tour
        for(int i=0;i<NodeCount-1;i++) // we already have first node
        {
            int min_dist=Integer.MAX_VALUE; // preset min distance
            Node min_node=null;             // preset min node
            NodeListIter nit = Net.GetNodeListIter();
            for(Node n2=nit.GetFirstNode();n2!=null;n2=nit.GetNextNode())
            {
                if(Busy.GetInt(n2)==0)  // skip if node is busy
                {
                    int dist = NodesToDistance.GetInt(node,n2);
                    if(dist<min_dist)   // is this min distance?
                    {
                        min_dist=dist;  // yes, record distance
                        min_node=n2;    // record node
                    }
                }
            }
            if(min_node!=null)          // did we get a min?
            {
                Busy.Put(min_node,1);       // this node is busy
                Order[index++]=min_node;    // put it in the tour
                Edge edge = NodesToEdge.GetEdge(node, min_node);
                node = min_node;
                // update the animation
                UI.SingleStep(min_node,AcquiredSet);
                SingleStepVisible(true,edge,AcquiredSet);
                UI.Trace(AcquiredSet);
                UI.Update();
            }
            else                        // illegal state
                result=false;
        }  
        return result;
    }
}
