package giden.contrib.samples;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;

/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * DialogSolver is a simple GIDEN solver that demonstrates some features
 * of the RowDialogMain and its related classes.
 * <p>
 *
 * @see RowDialogChoice
 * @see RowDialogEdge
 * @see RowDialogEntry
 * @see RowDialogFilename
 * @see RowDialogMain
 * @see RowDialogNode
 * @see RowDialogText
 *
 * @version 	4.0a 2004 08 29
 * @author 	    D. S. Dilworth
 */

public class DialogSolver extends ExecBase
{
    /** Used for most user interactions */
    UserBase UI;
    /** The network the solver is processing */
    Network Net;
    /** Use for high level access to the GUI */
    SolverServices Services;
    
    /** The set all nodes/edges are moved to in the Setup method */
    AnimationSet StartSet;
    /** The selected nodes and edges are moved to this set in the ExecuteDialogSolver method */
    AnimationSet SelectSet;
    
    /* A choice the user selects from a dialog box */
    String TestChoice;
    /* A filenam the user selects from a dialog box */
    String TestFilename;
    /* A node the user selects from a dialog box */
    Node TestNode;
    /* An edge the user selects from a dialog box */
    Edge TestEdge;
    /* A text string the user enters into a dialog box */
    String TestText;
    
    
    /** Initializes invariant objects */
    public DialogSolver (UserBase userinfo)
    {
        super(userinfo);        // mandatory call to the base class
        UI =  userinfo;         // local access for user interactions
        Services=UI.Services;   // local access for some GUI ops
                
        // create the start and end Sets
        StartSet = UI.CreateAnimationSet("",Color.red);
        SelectSet   = UI.CreateAnimationSet("Select",Color.green);    
        UI.CreateAnimationButtons();
    }
    
    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
        Net=net;                           // local access to the network
        Services.AddSolverMenuInfo();      // enable FeatureExample.html as the Help->Info file       
        UI.StatusLine("Setup (Samples)");  // status line
        UI.MoveAll(StartSet);              // nodes/edges -> StartSet
        UI.ShowPseudoCodeText("start");    // pseudocode window
        return true;
    }
    
    /** Executes the solver */
    public boolean Execute()
    {
        UI.StatusLine("Executing");         // status line
        UI.ShowPseudoCodeText("execute");   // pseudocode window
        
        if(InteractWithUser()==true)
        {
            ExecuteDialogSolver();              // execute the solver
        }        
        UI.StatusLine("Completed");         // status line
        UI.ShowPseudoCodeText("end");       // pseudocode window
        return true;
    }
    
    /** Creates the dialog box and interacts with the user.
     * Fills the TestText, TestChoice, TestFilename, TestNode, TestEdge
     * @return true if the OK button was pushed */
    boolean InteractWithUser()
    {
        // create an empty dialog box
        RowDialogMain Dialog = UI.CreateRowDialog("Dialog Solver");
        
        // add the filename row 
        RowDialogText TextRow = Dialog.AddRowText("Test Text","default text");          
        
        // add the filename row 
        RowDialogFilename FilenameRow = Dialog.AddRowFilename("Test Filename","default.csv",false); 
        
        // add the choice row
        String Choices[] = new String[3];
        Choices[0]="Choice A";
        Choices[1]="Choice B";
        Choices[2]="Choice C";
        RowDialogChoice ChoiceRow = Dialog.AddRowChoice("Test Choice",Choices);
        
        // add the edge row
        RowDialogEdge EdgeRow = Dialog.AddRowEdge("Test Edge",Net.GetEdgeIndexStrings());
        EdgeRow.PutHelpFile("DialogSolverEdge.html"); // add a help button for edges
        
        // add the node row
        RowDialogNode NodeRow = Dialog.AddRowNode("Test Node",Net.GetNodeIndexStrings());

        // add Cancel and OK buttons
        Dialog.AddButton("Cancel");
        Dialog.AddButton("OK");
        
        // interact with the user        
        String pushed_button = Dialog.Interact(); 
        
        // retrieve results
        TestText = TextRow.GetText();
        TestFilename = FilenameRow.GetFilename();
        TestChoice = ChoiceRow.GetChoice();
        TestNode = NodeRow.GetNode();
        TestEdge = EdgeRow.GetEdge();
        
        // dispose of the dialog box
        Dialog.dispose();
        
        // returns true if the OK button was pushed
        return pushed_button.equals("OK");
    }    
    
    /** Displays result from the dialog box.
     *  Prints the TestText, TestChoice, and TestFilename on the status line
     *  Moves the TestNode and TestEdge to the SelectSet */
    void ExecuteDialogSolver()
    {     
        String status = "Text="+TestText+"  |  Choice="+TestChoice+"  |  Filename="+TestFilename;
        UI.StatusLine(status);
        if(TestNode!=null)
            UI.SingleStep(TestNode,SelectSet);
        if(TestEdge!=null)
            UI.SingleStep(TestEdge,SelectSet);
        UI.Trace(SelectSet);        // trace the SelectSet
        UI.Update();                // display animation            
    }
}
