package giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;

/**********************************************************************/
// Generic Augmenting Path (Ford-Fulkerson) class
/**********************************************************************/
public class GenAugPath extends MaxFlow {

    private AnimationSet Trial;
    private AnimationSet Acquired;
    private AnimationSet Discarded;

    private final int BFS_SEARCH = 1;
    private final int DFS_SEARCH = 2;

    private int SelectedSearchMethod;

    /****************************************************************/
    // GenAugPath constructor
    /****************************************************************/
    public GenAugPath(UserBase userinfo) {
        super(userinfo);

        Trial     = UI.CreateAnimationSet("Trial",Color.red);
        Acquired  = UI.CreateAnimationSet("Acquired",new Color(48,175,48));
        Discarded = UI.CreateAnimationSet("Discarded",Color.yellow);
    }

    public String getSolverName() {return "Generic Augmenting Path";}

    /****************************************************************/
    // Setup
    //   - calls MaxFlow base class Setup()
    //   - initializes solver specific information
    /****************************************************************/
    public boolean Setup(Network net) {
        if(super.Setup(net)==false)
            return false;

        return true;
    }

    /****************************************************************/
    // GenAugPath Execute
    /****************************************************************/
    public boolean Execute() {
        SelectedSearchMethod = GetSearchMethod();

        Node Source = GetSourceNode();
        Node Sink = GetSinkNode();
        //        EdgeArray Capacity = GetCapacityArray();
        EdgeArray Flow = GetFlowArray();
        NodeArray Reachable = GetReachableArray();

        Network Net = GetNetwork();

        int SearchMethod = SelectedSearchMethod;

        FordFulkSolver(Net,Source,Sink,Capacity,Flow,Reachable,SearchMethod);
        DisplayMinCut(Net,Reachable,Capacity);

        return true;
    }

    /****************************************************************/
    private int GetSearchMethod() {
        String temp;

        do {
            temp = UI.UserTextInput("BFS or DFS search? ","BFS");
            if (temp == null) temp = new String("bfs");

            temp = temp.toUpperCase();
        } while (temp.compareTo("BFS") != 0 && temp.compareTo("DFS") != 0);

        if (temp.compareTo("BFS") == 0)
            return BFS_SEARCH;
        else
            return DFS_SEARCH;
    }

    /********************************************************************/
    // GenAugPath FordFulkSolver
    /********************************************************************/
    public void FordFulkSolver(Network   Net,
                               Node      Source,
                               Node      Sink,
                               EdgeArray Capacity,
                               EdgeArray Flow,
                               NodeArray Reached,
                               int       SearchMethod) {
        UI.StatusLine("Generic Augmenting Path Solver begins...");
        UI.ShowPseudoCodeText("start");
        UI.StatusLine("");

        UI.ShowPseudoCodeText("init flow");

        NodeArray Pred = new NodeArray(Net.GetNodeIndex(),"Pred");
        String temp;

        if (SearchMethod == BFS_SEARCH) {
            while (PathExists_BFS(Net,Capacity,Flow,Source,Sink,Pred,Reached)) {
                AugmentFlow(Capacity,Flow,Source,Sink,Pred);
                
                UI.MoveAll(InitialSet);
            }
        }
        else {
            while (PathExists_DFS(Capacity,Flow,Source,Sink,Pred,Reached)) {
                AugmentFlow(Capacity,Flow,Source,Sink,Pred);

                UI.MoveAll(InitialSet);
            }
        }
        UI.StatusLine("Generic Augmenting Path Solver finished.");
        UI.ShowPseudoCodeText("end");

    }

    /****************************************************************/
    /****************************************************************/
    private boolean PathExists_DFS(EdgeArray Capacity,
                                   EdgeArray Flow,
                                   Node      Source,
                                   Node      Sink,
                                   NodeArray Pred,
                                   NodeArray Reached) {
        Node v;

        Pred.Put(Sink,null);
        Reached.Fill(NOT_REACHED);

        UI.ShowPseudoCodeText("main loop");

        Reached.Put(Source,REACHED);
        UI.SingleStep(Source,Trial);
        UI.Trace(Trial);
        UI.Update();

        LoopDFS(Source,Sink,Pred,Reached,Capacity,Flow);
        
        if (Pred.GetEdge(Sink) == null)
            return false;
        else
            return true;
    }

    /****************************************************************/
    private void LoopDFS(Node      v,
                         Node      Sink,
                         NodeArray Pred,
                         NodeArray Reached,
                         EdgeArray Capacity,
                         EdgeArray Flow) {
        Node w;
        Edge e;

        EdgeListIter eit;

        eit = v.GetAdjListIter();
        e = eit.GetFirstEdge();

        while (e != null && Reached.GetInt(Sink) == NOT_REACHED) {
            if (e.GetSource() == v) {
                w = e.GetTarget();

                if (Reached.GetInt(w) == NOT_REACHED
                    && Flow.GetInt(e) < Capacity.GetInt(e)) {
                    Reached.Put(w,REACHED);
                    Pred.Put(w,e);
                    UI.SingleStep(e,Trial);
                    UI.SingleStep(w,Trial);
                    UI.Trace(Trial);
                    UI.Update();
                    LoopDFS(w,Sink,Pred,Reached,Capacity,Flow);
                }
                else if (Reached.GetInt(w) == REACHED && e != Pred.GetEdge(v)) {
                    UI.SingleStep(e,Discarded);
                    UI.Trace(Discarded);
                    UI.Update();
                }
            }
            else {
                w = e.GetSource();
                if (Reached.GetInt(w) == NOT_REACHED && Flow.GetInt(e) > 0) {
                    Reached.Put(w,REACHED);
                    Pred.Put(w,e);
                    UI.SingleStep(e,Trial);
                    UI.SingleStep(w,Trial);
                    UI.Trace(Trial);
                    UI.Update();
                    LoopDFS(w,Sink,Pred,Reached,Capacity,Flow);
                }
                else if (Reached.GetInt(w) == REACHED && e != Pred.GetEdge(v)) {
                    UI.SingleStep(e,Discarded);
                    UI.Trace(Discarded);
                    UI.Update();
                }
            }
            e = eit.GetNextEdge();
        }
    }

    /****************************************************************/
    /****************************************************************/
    private boolean PathExists_BFS(Network   Net,
                                   EdgeArray Capacity,
                                   EdgeArray Flow,
                                   Node      Source,
                                   Node      Sink,
                                   NodeArray Pred,
                                   NodeArray Reached) {
        BoundedQueue BQ = new BoundedQueue(Net.GetNodeIndex());
        // static BoundedQueue BQ = new BoundedQueue(Net.GetNodeIndex());
        
        Node i;
        Node j;
        Edge e;

        EdgeListIter eit;

        Reached.Fill(NOT_REACHED);

        UI.ShowPseudoCodeText("main loop");

        BQ.Append(Source);
        Reached.Put(Source,REACHED);
        UI.SingleStep(Source,Trial);
        UI.Trace(Trial);
        UI.Update();

        while (BQ.GetSize() > 0 && Reached.GetInt(Sink) == NOT_REACHED) {

            i = (Node) BQ.Pop();

            eit = i.GetAdjListIter();

            e = eit.GetFirstEdge();
            while (e != null && Reached.GetInt(Sink) == NOT_REACHED) {

                // forward edge
                if (e.GetSource() == i) {
                    j = e.GetTarget();
                    
                    if (Reached.GetInt(j) == NOT_REACHED &&
                        Capacity.GetInt(e) > Flow.GetInt(e)) {

                        Reached.Put(j,REACHED);
                        Pred.Put(j,e);
                        BQ.Append(j);
                        UI.SingleStep(e,Trial);
                        UI.SingleStep(j,Trial);
                        UI.Trace(Trial);
                        UI.Update();
                    }
                    else if (Reached.GetInt(j) == REACHED && e != Pred.GetEdge(i)) {
                        UI.SingleStep(e,Discarded);
                        UI.Trace(Discarded);
                        UI.Update();
                    }
                }
                else {  // reverse edge
                    j = e.GetSource();

                    if (Reached.GetInt(j) == NOT_REACHED && Flow.GetInt(e) > 0) {
                        Reached.Put(j,REACHED);
                        Pred.Put(j,e);
                        BQ.Append(j);
                        UI.SingleStep(e,Trial);
                        UI.SingleStep(j,Trial);
                        UI.Trace(Trial);
                        UI.Update();
                    }
                    else if (Reached.GetInt(j) == REACHED && e != Pred.GetEdge(i)) {
                        UI.SingleStep(e,Discarded);
                        UI.Trace(Discarded);
                        UI.Update();
                    }
                }

                e = eit.GetNextEdge();
            }

        }

        BQ.DeleteContents(); // make sure that BQ is empty for next time

        if (Reached.GetInt(Sink) == NOT_REACHED)
            return false;
        else
            return true;
    }

    /****************************************************************/
    /****************************************************************/
    private void AugmentFlow(EdgeArray Capacity,
                             EdgeArray Flow,
                             Node      Source,
                             Node      Sink,
                             NodeArray Pred) {
        int delta;
        Edge e;
        Node v;


        UI.ShowPseudoCodeText("id cycle");

        //if (Pred.Get(Source) != null) Abort();

        v = Sink;
        e = Pred.GetEdge(Sink);
        UI.SingleStep(v,Acquired);
        UI.SingleStep(e,Acquired);
        
        // we know that Pred[sink] is defined (?)
        if (e.GetTarget() == v) {
            v = e.GetSource();
            delta = Capacity.GetInt(e) - Flow.GetInt(e);
        }
        else {
            v = e.GetTarget();
            delta = Flow.GetInt(e);
        }
        e = Pred.GetEdge(v);
        UI.SingleStep(v,Acquired);
        
        while (e != null) {
            UI.SingleStep(e,Acquired);
            
            if (e.GetTarget() == v) {
                v = e.GetSource();
                if (delta > Capacity.GetInt(e) - Flow.GetInt(e))
                    delta = Capacity.GetInt(e) - Flow.GetInt(e);
            }
            else {
                v = e.GetTarget();
                if (delta > Flow.GetInt(e))
                    delta = Flow.GetInt(e);
            }
            e = Pred.GetEdge(v);
            UI.SingleStep(v,Acquired);
        }
        
        UI.StatusLine("Residual capacity delta = "+delta+" units.");
        UI.ShowPseudoCodeText("calc delta");
        UI.StatusLine("");
        
        UI.StatusLine("Augmenting path found; residual capacity delta = "
                            +delta+" units.");
        UI.Trace(Acquired);
        UI.Update();
        UI.StatusLine("");
        
        // should check that v == source
        
        // DELTA is computed -- now we actually update the flow values
        
        v = Sink;
        e = Pred.GetEdge(v);
        
        while (e != null) {
            if (e.GetTarget() == v) {
                v = e.GetSource();
                Flow.Put(e,Flow.GetInt(e) + delta);
            }
            else {
                v = e.GetTarget();
                Flow.Put(e,Flow.GetInt(e) - delta);
            }
            PutEdgeLabel(e);
            e = Pred.GetEdge(v);
        }
        
        // owenj ---- no method available 12-Apr-1999        UI.UpdateLabels();
        FlowValue += delta;
        
        UI.StatusLine("Augmented flow by "+delta+" units.  Flow value = "
                      +FlowValue+" units.");
        UI.ShowPseudoCodeText("update flow");
        UI.StatusLine("");
    }

}
