package giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;


/**********************************************************************/
// Kruskal solver class
/**********************************************************************/
public class Kruskal extends MinSpan {

    private AnimationSet Trial;
    private AnimationSet Acquired;
    private AnimationSet Discarded;

    /******************************************************************/
    // Kruskal constructor
    /******************************************************************/
    public Kruskal(UserBase userinfo) {
        super(userinfo);

        Trial     = UI.CreateAnimationSet("Trial",Color.red);
        Acquired  = UI.CreateAnimationSet("Acquired",new Color(48,175,48));
        Discarded = UI.CreateAnimationSet("Discarded",Color.yellow);        
    }

    public String getSolverName() {return "Kruskal";}

    /******************************************************************/
    // Setup
    //   - calls MinSpan base class Setup()
    //   - initializes solver specific information
    /******************************************************************/
    public boolean Setup(Network net) {
        if(super.Setup(net)==false)
            return false;

        return true;
    }

    /******************************************************************/
    // Kruskal Execute
    /******************************************************************/
    public boolean Execute() {
        LinkList Forest = GetForestList();
        Network Net = GetNetwork();
        // owenj 09-Apr-1999       Length = GetLengthArray();

        KruskalSolver(Net,Length,Forest);
        DisplayMinSpanForest(Net,Forest,Length);

        return true;
    }

    /********************************************************************/
    // Kruskal Shutdown
    /********************************************************************/
    public boolean Shutdown() {
        return super.Shutdown();
    }



    /******************************************************************/
    // Kruskal KruskalSolver
    /******************************************************************/
    private void KruskalSolver(Network   Net,
                               EdgeArray Length,
                               LinkList  Forest) {

        UI.StatusLine("Kruskal Solver begins...");
        UI.ShowPseudoCodeText("start");
        UI.StatusLine("");

        Node i,j;
        SingleLinkList iList,jList;
        SingleLinkList Slist,Llist;

        Edge e;
        int NumAcquiredEdges = 0;
        int MaxAcquiredEdges = Net.GetNodeCount() - 1;

        PQList SortedEdgeList = CreateSortedEdgeList(Net,Length);
        NodeArray TreeLists = CreateTreeLists(Net);
        UI.ShowPseudoCodeText("edge sort");
        UI.ShowPseudoCodeText("init list");

        while (NumAcquiredEdges < MaxAcquiredEdges 
               && SortedEdgeList.NotEmpty()) {
            UI.ShowPseudoCodeText("main loop");

            e = SortedEdgeList.DeleteMinEdge();
            UI.SingleStep(e,Trial);
            UI.ShowPseudoCodeText("select edge");
            UI.Trace(Trial);
            UI.Update();

            // undirected network, so "source" and "target" are arb. endpoints
            i = e.GetSource();
            j = e.GetTarget();

            // set iList and jList to correct values for edge e
            iList = (SingleLinkList) TreeLists.GetObj(i);
            jList = (SingleLinkList) TreeLists.GetObj(j);

            UI.ShowPseudoCodeText("cycle check");
            if (iList.GetHead() != jList.GetHead()) {
                // add edge to forest
                Forest.AddItem(e);
                ForestLength += Length.GetInt(e);
                NumAcquiredEdges++;

                MergeLists(iList,jList,TreeLists);

                UI.SingleStep(i,Acquired);
                UI.SingleStep(j,Acquired);
                UI.SingleStep(e,Acquired);
                UI.ShowPseudoCodeText("add edge");
                UI.Trace(Acquired);
                UI.Update();
            }
            else {
                UI.SingleStep(e,Discarded);
                UI.ShowPseudoCodeText("discard edge");
                UI.Trace(Discarded);
                UI.Update();
            }
        }
        UI.ShowPseudoCodeText("main loop");

        // owenj (05-aug-1999); moved to "DisplayMinSpanForest" in MinSpan
        UI.ShowPseudoCodeText("end");

        UI.StatusLine("Kruskal Solver finished.  "+
                            "Length of minimum spanning forest = "+
                            ForestLength +" units.");
    }
    
    /********************************************************************/
    // Kruskal CreateSortedEdgeList
    /********************************************************************/
    private PQList CreateSortedEdgeList(Network   Net,
                                        EdgeArray Length) {
        Edge e;
        EdgeListIter eit = Net.GetEdgeListIter();
        PQList EdgeList = new PQList();

        for (e=eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge())
            EdgeList.Insert(e,Length.GetInt(e));

        return EdgeList;
    }


    /********************************************************************/
    // Kruskal CreateTreeLists
    /********************************************************************/
    private NodeArray CreateTreeLists(Network   Net) {
        NodeArray TreeLists = new NodeArray(Net.GetNodeIndex(),"TreeList");
        SingleLinkList c_tree;

        Node i;
        NodeListIter nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i != null ; i=nit.GetNextNode()) {
            c_tree = new SingleLinkList();
            c_tree.AddItem(i);
            TreeLists.Put(i,c_tree);
        }

        return TreeLists;
    }

    /********************************************************************/
    // Kruskal MergeLists
    /********************************************************************/
    private void MergeLists(SingleLinkList iList,
                            SingleLinkList jList,
                            NodeArray      TreeLists) {
        Node k;
        SingleLinkList small,large;

        if (iList.GetSize() >= jList.GetSize()) {
            large = iList;
            small = jList;
        }
        else {
            large = jList;
            small = iList;
        }

        large.JoinList(small);
        ListEntry entry = small.GetFirstEntry();
        while (entry != null) {
            k = (Node) entry.GetItem();
            TreeLists.Put(k,large);
            entry = entry.GetNext();
        }
    }
}
