package giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;


/**********************************************************************/
// Maximum Flow Problem base class
/**********************************************************************/
abstract public class MaxFlow extends ExecBase {

    protected GidenSolverInput input;

    protected UserBase UI;

    // problem data
    public Node Source;
    public Node Sink;
    public EdgeArray Capacity = null;
    public int FlowValue;
    public EdgeArray Flow;
    //    private EdgeArray Flow;
    private NodeArray Reachable;

    private Network Net;

    protected ColorSet InitialSet;
    private FinalSet MinCutEdges;
    private FinalSet ReachableNodes;


    // TRY TO REMOVE THESE VARIABLES LATER!!!
    final protected int NOT_REACHED = 1;
    final protected int REACHED = 2;


    public boolean ValidateInput() {
        // extract capacity array
        Object data = input.getItemData("Capacity");
        if (!(data instanceof EdgeArray))
            return false;

        // check capacity data (to ensure >= 0)
        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
            if (((EdgeArray)data).GetInt(e) < 0) {
                UI.NotifyOfSolverError("Negative values exist in selected \"Capacity\" array.");
                return false;
            }

        return true;
    }


    /********************************************************************/
    // MaxFlow constructor
    /********************************************************************/
    MaxFlow(UserBase userinfo) {
        super(userinfo);
        UI = userinfo;

        InitialSet     = UI.CreateColorSet(Color.lightGray);
        MinCutEdges    = UI.CreateFinalSet("Minimum Capacity Cut",Color.darkGray);
        ReachableNodes = UI.CreateFinalSet("Reachable Nodes",Color.orange);

        input = new GidenSolverInput();
        input.addItem(SolverInputItem.INTEGER_EDGE_ARRAY,"Capacity","Capacity");
    }

    abstract public String getSolverName();

    /********************************************************************/
    // Setup
    /********************************************************************/
    public boolean Setup(Network net) {
        if(net==null)
            return false;

        super.Setup(net);

        Net = net;
        UI.PutDirected(true);

        if (!UI.NullBase) {
            UI.CreateSolverInputDialog(getSolverName(), this, net, input);
            if (!input.isValidated())
                return false;

            // extract input info
            Object data = input.getItemData("Capacity");
            if (data instanceof EdgeArray)
                Capacity = (EdgeArray) data;
        }

        if (Capacity == null)
            Capacity = Net.GetEdgeArray("Capacity");
        // This is just a kluge -- we want to insist on a "capacity array"
        if (Capacity == null)
            Capacity = Net.GetEdgeArray("Euclidean Length");

        Flow = new EdgeArray(Net.GetEdgeIndex(),"Flow");
        Flow.Fill(0);
        FlowValue = 0;
        Reachable = new NodeArray(Net.GetNodeIndex(),"Reachable");
        Reachable.Fill(NOT_REACHED);

        InitializeEdgeLabels();
        // owenj 12-Apr-1999        GetSource();
        // owenj 12-Apr-1999        GetSink();
        InitializeNodeLabels();

        UI.MoveAll(InitialSet);

        return true;
    }

    /********************************************************************/
    // Shutdown
    /********************************************************************/
    public boolean Shutdown() {
        Source = null;
        Sink = null;
        FlowValue = 0;

        return super.Shutdown();
    }

    /********************************************************************/
    /********************************************************************/
    protected Node GetSinkNode() {
        if (Sink == null) 
            GetSink();

        return Sink;
    }
    protected Node GetSourceNode() {
        if (Source == null) 
            GetSource();

        return Source;
    }
    protected EdgeArray GetFlowArray() {return Flow;}
    protected EdgeArray GetCapacityArray() {return Capacity;}
    protected NodeArray GetReachableArray() {return Reachable;}
    protected Network GetNetwork() {return Net;}

    /********************************************************************/
    private void GetSource() {
        Source = null;
        while (Source == null)
            Source = UI.UserMouseNodeInput("Click on Source Node");

        PutNodeLabel(Source);
        GIDEN.out.println("Source Node = "+Source.GetIndex());
    }

    /********************************************************************/
    private void GetSink() {
        Sink = null;
        while (Sink == null)
            Sink = UI.UserMouseNodeInput("Click on Sink Node");

        PutNodeLabel(Sink);
        GIDEN.out.println("Sink Node = "+Sink.GetIndex());
    }

    /********************************************************************/
    protected void PutEdgeLabel(Edge e) {
        int residual = Capacity.GetInt(e) - Flow.GetInt(e);
        String label = "(" + Flow.GetInt(e) + "," + residual + ")";

        UI.PutLabel(e,label);
    }

    /********************************************************************/
    protected void PutNodeLabel(Node i) {
        if (i == Source)
            UI.PutLabel(i,"s");
        else if (i == Sink)
            UI.PutLabel(i,"t");
        else
            UI.PutLabel(i,"");
    }

    /********************************************************************/
    protected void InitializeEdgeLabels() {
        // initialize edge labels
        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge())
            PutEdgeLabel(e);
    }

    /********************************************************************/
    protected void InitializeNodeLabels() {
        // initialize node labels
        NodeListIter nit = Net.GetNodeListIter();
        for (Node i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode())
            PutNodeLabel(i);
    }

    /********************************************************************/
    protected void FindSourceReachableNodes(Network   Net,
                                            Node      Source,
                                            NodeArray Reached,
                                            EdgeArray Flow,
                                            EdgeArray Capacity) {
        Reached.Fill(NOT_REACHED);
        BoundedQueue BQ = new BoundedQueue(Net.GetNodeIndex());
        Node v,w;
        Edge e;
        EdgeListIter eit;

        Reached.Put(Source,REACHED);
        BQ.Append(Source);

        while (!BQ.IsEmpty()) {
            v = (Node) BQ.Pop();
            eit = v.GetAdjListIter();
            for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
                if (e.GetSource() == v) {
                    w = e.GetTarget();
                    if (Reached.GetInt(w) == NOT_REACHED &&
                        Capacity.GetInt(e) > Flow.GetInt(e)) {
                        Reached.Put(w,REACHED);
                        BQ.Append(w);
                    }
                }
                else {
                    w = e.GetSource();
                    if (Reached.GetInt(w) == NOT_REACHED && Flow.GetInt(e) > 0) {
                        Reached.Put(w,REACHED);
                        BQ.Append(w);
                    }
                }
            }
        }
    }

    /********************************************************************/
    protected void DisplayMinCut(Network   Net,
                                 NodeArray Reached,
                                 EdgeArray Capacity) {
        Node i;
        Edge e;
        int CutCapacity = 0;

        String temp;
        
        /*
        do {
            temp = UI.UserTextInput("Show minimum cut? ","Yes");
            if (temp == null) temp = new String("yes");

            temp = temp.toUpperCase();
        } while (temp.compareTo("Y")   != 0 && temp.compareTo("N")  != 0
                 && temp.compareTo("YES") != 0 && temp.compareTo("NO") != 0);

        if (temp.compareTo("N") == 0 || temp.compareTo("NO") == 0)
            return;
        */

        NodeListIter nit = Net.GetNodeListIter();
        for (i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode()) {
            if (Reached.GetInt(i) == REACHED)
                UI.SingleStep(i,ReachableNodes);
            else
                UI.SingleStep(i,InitialSet);
        }
        
        EdgeListIter eit = Net.GetEdgeListIter();
        for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
            if (Reached.GetInt(e.GetSource()) == REACHED
                && Reached.GetInt(e.GetTarget()) == NOT_REACHED) {
                UI.SingleStep(e,MinCutEdges);
                CutCapacity += Capacity.GetInt(e);
            }
            else if (Reached.GetInt(e.GetSource()) == NOT_REACHED
                     && Reached.GetInt(e.GetTarget()) == REACHED)
                UI.SingleStep(e,MinCutEdges);
            else
                UI.SingleStep(e,InitialSet);
        }
        UI.Trace(MinCutEdges);
        UI.Trace(ReachableNodes);
        UI.Update();

        if (CutCapacity == FlowValue)
            UI.StatusLine("Cut Capacity = Flow Value = "+CutCapacity+" units.");
        else
            UI.StatusLine("Error:  Cut Capacity = "+CutCapacity
                                +", Flow Value = "+FlowValue);
    }
}
