package giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;

// 010110 dsd changed GetNodeIndex to GetCreatedNodesCount for array allocation


/**********************************************************************/
// Miniumu Cost Flow Problem base class
/**********************************************************************/
abstract class MinCostFlow extends ExecBase {

    protected GidenSolverInput input;

    protected UserBase UI;

    // problem data
    protected EdgeArray Cost = null;
    protected EdgeArray Capacity = null;
    protected NodeArray Supply = null;
    protected int FlowValue;
    protected int FlowCost;
    protected EdgeArray Flow;
    //    private EdgeArray Flow;
    private NodeArray Potential;

    private Network Net;

    protected ColorSet InitialSet;

    //    protected FinalSet SupplyNodes;
    //    protected FinalSet DemandNodes;
    protected FinalSet EdgesAtCapacity;
    protected FinalSet EdgesAtZero;
    protected FinalSet EdgesWithResidual;

    protected FinalSet ReachableNodes;
    protected FinalSet InfeasibleCut;

    public boolean ValidateInput() {
        // extract supply array
        Object data = input.getItemData("Supply");
        if (!(data instanceof NodeArray))
            return false;

        // check supply data (to make sure that sum to zero)
        int supply = 0;
        NodeListIter nit = Net.GetNodeListIter();
        for (Node i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode())
            supply += ((NodeArray)data).GetInt(i);

        if (supply > 0) {
            UI.NotifyOfSolverError("Total supply exceeds demand in selected \"Supply\" array.");
            return false;
        }
        else if (supply < 0) {
            UI.NotifyOfSolverError("Total demand exceeds supply in selected \"Supply\" array.");
            return false;
        }


        // extract capacity array
        data = input.getItemData("Capacity");
        if (!(data instanceof EdgeArray))
            return false;

        // check capacity data (to ensure >= 0)
        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
            if (((EdgeArray)data).GetInt(e) < 0) {
                UI.NotifyOfSolverError("Negative values exist in selected \"Capacity\" array.");
                return false;
            }


        return true;
    }


    /********************************************************************/
    // MinCostFlow constructor
    /********************************************************************/
    MinCostFlow(UserBase userinfo) {
        super(userinfo);
        UI = userinfo;

        InitialSet   = UI.CreateColorSet(Color.lightGray);

        //        SupplyNodes  = UI.CreateFinalSet("",Color.orange);
        //        DemandNodes  = UI.CreateFinalSet("",Color.white);
        EdgesAtZero       = UI.CreateFinalSet("",Color.lightGray);
        EdgesWithResidual = UI.CreateFinalSet("",Color.orange);
        EdgesAtCapacity   = UI.CreateFinalSet("",Color.red);

        ReachableNodes = UI.CreateFinalSet("",Color.orange);
        InfeasibleCut  = UI.CreateFinalSet("",Color.darkGray);


        input = new GidenSolverInput();
        input.addItem(SolverInputItem.INTEGER_EDGE_ARRAY,"Capacity","Capacity");
        input.addItem(SolverInputItem.INTEGER_EDGE_ARRAY,"Cost","Cost");
        input.addItem(SolverInputItem.INTEGER_NODE_ARRAY,"Supply","Supply");
    }

    abstract public String getSolverName();

    /********************************************************************/
    // Setup
    /********************************************************************/
    public boolean Setup(Network net) {
        GIDEN.out.println("** MinCostFlow Setup()");
        if(net==null)
            return false;

        super.Setup(net);

        Net = net;
        UI.PutDirected(true);

        Flow = new EdgeArray(Net.GetCreatedEdgesCount(),"Flow");
        Flow.Fill(0);
        FlowValue = 0;
        FlowCost = 0;

        Potential = new NodeArray(Net.GetCreatedNodesCount(),"Potential");
        Potential.Fill(0);

        if (!UI.NullBase) {
            UI.CreateSolverInputDialog(getSolverName(), this, net, input);
            if (!input.isValidated())
                return false;

            // extract input info
            Object data;
            data = input.getItemData("Cost");
            if (data instanceof EdgeArray)
                Cost = (EdgeArray) data;

            data = input.getItemData("Capacity");
            if (data instanceof EdgeArray)
                Capacity = (EdgeArray) data;

            data = input.getItemData("Supply");
            if (data instanceof NodeArray)
                Supply = (NodeArray) data;
        }

        if (Cost == null) Cost = Net.GetEdgeArray("Cost");
        if (Capacity == null) Capacity = Net.GetEdgeArray("Capacity");
        if (Supply == null) Supply = Net.GetNodeArray("Supply");

        //        InitializeNetworkLabels();

        UI.MoveAll(InitialSet);

        return true;
    }

    /********************************************************************/
    // Shutdown
    /********************************************************************/
    public boolean Shutdown() {
        GIDEN.out.println("** MinCostFlow Shutdown()");
        FlowValue = 0;
        FlowCost = 0;

        return super.Shutdown();
    }

    /********************************************************************/
    /********************************************************************/
    protected Network GetNetwork() {return Net;}
    protected EdgeArray GetFlowArray() {return Flow;}
    protected EdgeArray GetCapacityArray() {return Capacity;}
    protected EdgeArray GetCostArray() {return Cost;}
    protected NodeArray GetSupplyArray() {return Supply;}
    protected NodeArray GetPotentialArray() {return Potential;}

    /********************************************************************/
    /********************************************************************/
    protected void PutEdgeLabel(Edge      e,
                                EdgeArray Flow,
                                EdgeArray Capacity,
                                EdgeArray Cost) {
        // when called, can pass other arrays -- e.g., "RedCost" for "Cost"
        int residual = Capacity.GetInt(e) - Flow.GetInt(e);
        String label = "(" + Flow.GetInt(e) + "," + residual
            + ",$" + Cost.GetInt(e) +")";

        UI.PutLabel(e,label);
    }

    /********************************************************************/
    protected void PutNodeLabel(Node      i,
                                NodeArray Supply) {
        String label = ""+Supply.GetInt(i);

        UI.PutLabel(i,label);
    }

    /********************************************************************/
    protected void PutNodeLabel(Node      i,
                                NodeArray Supply,
                                NodeArray Potential) {
        String label = ""+Supply.GetInt(i)+","+Potential.GetInt(i);

        UI.PutLabel(i,label);
    }

    /********************************************************************/
    protected void InitializeNetworkLabels(boolean using_node_potentials) {
        // initialize edge labels
        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge())
            PutEdgeLabel(e,Flow,Capacity,Cost);

        // initialize node labels
        NodeListIter nit = Net.GetNodeListIter();
        for (Node i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode()) {
            if (using_node_potentials)
                PutNodeLabel(i,Supply,Potential);
            else
                PutNodeLabel(i,Supply);
        }
        
    }

    /********************************************************************/
    /********************************************************************/
    protected void DisplayFinalSolution(Network   Net,
                                        EdgeArray Cost,
                                        EdgeArray Flow,
                                        EdgeArray Capacity,
                                        NodeArray Potential,
                                        NodeArray Supply) {
        //        SupplyNodes.PutLabel("Supply Nodes");
        //        DemandNodes.PutLabel("Demand Nodes");
        // EdgesAtZero.PutLabel("Unused Edges");
        EdgesAtZero.PutLabel("");
        EdgesWithResidual.PutLabel("Flow between Bounds");
        EdgesAtCapacity.PutLabel("Flow at Capacity");

        ReachableNodes.PutLabel("");
        InfeasibleCut.PutLabel("");

        UI.MoveAll(InitialSet);
        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
            PutEdgeLabel(e,Flow,Capacity,Cost);
            if (Flow.GetInt(e) == 0)
                UI.SingleStep(e,EdgesAtZero);
            else if (Flow.GetInt(e) == Capacity.GetInt(e))
                UI.SingleStep(e,EdgesAtCapacity);
            else if (0 < Flow.GetInt(e) && Flow.GetInt(e) < Capacity.GetInt(e))
                UI.SingleStep(e,EdgesWithResidual);
        }

        /*
        NodeListIter nit = Net.GetNodeListIter();
        for (Node i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode()) {
            PutNodeLabel(i,Supply,Potential);
            if (Supply.GetInt(i) > 0)
                UI.SingleStep(i,SupplyNodes);
            else if (Supply.GetInt(i) < 0)
                UI.SingleStep(i,DemandNodes);
        }
        */

        UI.StatusLine("Min-Cost Flow cost = $"+FlowCost+".");

        //        UI.Trace(SupplyNodes);
        //        UI.Trace(DemandNodes);        
        UI.Trace(EdgesAtZero);
        UI.Trace(EdgesWithResidual);
        UI.Trace(EdgesAtCapacity);
        UI.Update();
    }

    /********************************************************************/
    /********************************************************************/
    private int NetOutFlow(Node i, EdgeArray Flow) {
        int outflow = 0;

        Edge e;
        EdgeListIter eit = i.GetOutListIter();
        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
            outflow += Flow.GetInt(e);

        eit = i.GetInListIter();
        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
            outflow -= Flow.GetInt(e);

        return outflow;
    }
    /********************************************************************/
    /* Assumes that "Flow" contains a maximal flow, and determines which
     * nodes are reachable in the residual network from the supply nodes
     */
    protected void ReportMaxFlowInfeasibility(Network   Net,
                                              EdgeArray Flow,
                                              EdgeArray Capacity,
                                              NodeArray Supply) {
        //        SupplyNodes.PutLabel("");
        //        DemandNodes.PutLabel("");
        EdgesAtZero.PutLabel("");
        EdgesWithResidual.PutLabel("");
        EdgesAtCapacity.PutLabel("");

        ReachableNodes.PutLabel("Excess Reachable Nodes");
        InfeasibleCut.PutLabel("Infeasibility Cut");


        int NOT_REACHED = 1;
        int REACHED = 2;

        NodeArray Reached = new NodeArray(Net.GetCreatedNodesCount(),"Reached");
        BoundedQueue BQ = new BoundedQueue(Net.GetCreatedNodesCount());
        Node i,j;
        Edge e;
        EdgeListIter eit;
        NodeListIter nit;

        Reached.Fill(NOT_REACHED);

        nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode()) {
            PutNodeLabel(i,Supply);
            if (Supply.GetInt(i)>0 && NetOutFlow(i,Flow) < Supply.GetInt(i)) {
                BQ.Append(i);
                Reached.Put(i,REACHED);
            }
        }

        while (!BQ.IsEmpty()) {
            i = (Node) BQ.Pop();
            eit = i.GetAdjListIter();
            for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
                if (e.GetSource() == i) {
                    j = e.GetTarget();
                    if (Reached.GetInt(j) == NOT_REACHED &&
                        Capacity.GetInt(e) > Flow.GetInt(e)) {
                        Reached.Put(j,REACHED);
                        BQ.Append(j);
                    }
                }
                else {
                    j = e.GetSource();
                    if (Reached.GetInt(j)==NOT_REACHED && Flow.GetInt(e) > 0) {
                        Reached.Put(j,REACHED);
                        BQ.Append(j);
                    }
                }
            }
        }

        // display the reachable nodes
        int CutCapacity = 0;

        nit = Net.GetNodeListIter();
        for (i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode()) {
            if (Reached.GetInt(i) == REACHED)
                UI.SingleStep(i,ReachableNodes);
            else
                UI.SingleStep(i,InitialSet);
        }

        boolean cut_visible = false;
        eit = Net.GetEdgeListIter();
        for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
            if (Reached.GetInt(e.GetSource()) == REACHED
                && Reached.GetInt(e.GetTarget()) == NOT_REACHED) {
                UI.SingleStep(e,InfeasibleCut);
                CutCapacity += Capacity.GetInt(e);
                cut_visible = true;
            }
            else if (Reached.GetInt(e.GetSource()) == NOT_REACHED
                     && Reached.GetInt(e.GetTarget()) == REACHED) {
                UI.SingleStep(e,InfeasibleCut);
                cut_visible = true;
            }
            else
                UI.SingleStep(e,InitialSet);
        }
        int NetSupply = 0;
        nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode())
            if (Reached.GetInt(i) == REACHED)
                NetSupply += Supply.GetInt(i);

        if (!cut_visible) {
            InfeasibleCut.PutLabel("");
            UI.StatusLine("Problem infeasible: "
                          +"Net supply of highlighted nodes = "
                          +NetSupply+" > 0 = (empty) cut capacity.");
        }
        else
            UI.StatusLine("Problem infeasible: "
                          +"Net supply of highlighted nodes = "
                          +NetSupply+" > "+CutCapacity+
                          " = cut capacity.");

        UI.Trace(InfeasibleCut);
        UI.Trace(ReachableNodes);
    }
}
