package  giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;


/**********************************************************************/
// Out-of-Kilter class
/**********************************************************************/
public class OutOfKilter extends MinCostFlow {

    private AnimationSet OffKilter;
    private AnimationSet Cycle;

    private boolean feasible;

    private final int INFINITY = Integer.MAX_VALUE;

    /****************************************************************/
    // constructor
    /****************************************************************/
    public OutOfKilter(UserBase userinfo) {
        super(userinfo);

        OffKilter = UI.CreateAnimationSet("Kilter Edge",Color.red);
        Cycle     = UI.CreateAnimationSet("Cycle",Color.blue);
    }

    public String getSolverName() {return "Out-of-Kilter";}

    /****************************************************************/
    // setup
    /****************************************************************/
    public boolean Setup(Network net) {
        if(super.Setup(net)==false)
            return false;

        InitializeNetworkLabels(true);
        feasible = true;

        return true;
    }

    /****************************************************************/
    // shutdown
    /****************************************************************/
    public boolean Shutdown() {
        UI.StatusLine("");
        return super.Shutdown();
    }

    /****************************************************************/
    // execute
    /****************************************************************/
    public boolean Execute() {
        //        EdgeArray Cost = GetCostArray();
        //        EdgeArray Capacity = GetCapacityArray();
        //        NodeArray Supply = GetSupplyArray();
        EdgeArray Flow = GetFlowArray();
        NodeArray Potential = GetPotentialArray();

        Network Net = GetNetwork();

        EdgeArray RedCost = new EdgeArray(Net.GetEdgeIndex(),"RedCost");

        OutOfKilterSolver(Net,Cost,Flow,Capacity,Potential,Supply,RedCost);
        UI.ShowPseudoCodeText("end");
        if (feasible)
            DisplayFinalSolution(Net,Cost,Flow,Capacity,Potential,Supply);
        else
            ReportMaxFlowInfeasibility(Net,Flow,Capacity,Supply);

        return true;
    }

    /****************************************************************/
    // OutOfKilterSolver
    /****************************************************************/
    private void OutOfKilterSolver(Network   Net,
                                   EdgeArray Cost,
                                   EdgeArray Flow,
                                   EdgeArray Capacity,
                                   NodeArray Potential,
                                   NodeArray Supply,
                                   EdgeArray RedCost) {
        UI.StatusLine("Out-of-Kilter Solver begins...");
        UI.ShowPseudoCodeText("start");
        UI.StatusLine("");


        // Initialization
        Potential.Fill(0);
        UI.ShowPseudoCodeText("init potentials");

        GetInitialFlow(Net,Cost,Flow,Capacity,Supply);
        if (!feasible) return;

        BoundedQueue Kilter = new BoundedQueue(Net.GetEdgeIndex());
        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
            // set reduced cost (equals cost since potentials init to 0)
            RedCost.Put(e,Cost.GetInt(e));

            if (RedCost.GetInt(e) != 0 
		&& getKilterNumber(e,Flow,Capacity,RedCost) > 0)
	        Kilter.Append(e);
        }


        Edge current;
        Node p,q;
        NodeArray Pred = new NodeArray(Net.GetNodeIndex(),"Pred");
        NodeArray Dist = new NodeArray(Net.GetNodeIndex(),"Dist");
        Edge e;
        Node i;
        NodeListIter nit;
        int delta;
        int cycle_cost;

        UI.ShowPseudoCodeText("kilter loop");
        while (!Kilter.IsEmpty()) {
            current = (Edge) Kilter.GetTop();

            if (RedCost.GetInt(current) < 0) { 
                // original edge is out-of-kilter
                p = current.GetSource();
                q = current.GetTarget();
            }
            else {
                // reverse edge is out-of-kilter
                p = current.GetTarget();
                q = current.GetSource();
            }
            UI.SingleStep(current,OffKilter);
            //            UI.SingleStep(p,OffKilter);
            //            UI.SingleStep(q,OffKilter);
            UI.Trace(OffKilter);
            UI.StatusLine("Kilter number of edge = "
                          +getKilterNumber(current,Flow,Capacity,RedCost));
            UI.ShowPseudoCodeText("select edge");
            UI.Update();
            UI.StatusLine("");

            UI.ShowPseudoCodeText("define lengths");

            DetermineDistanceLabels(Net,p,q,current,Dist,Pred,RedCost);
            UI.ShowPseudoCodeText("distance labels");

	    // update node potentials
            UI.ShowPseudoCodeText("check pi (GIDEN)");
            if (Dist.GetInt(p) == INFINITY || Pred.GetObj(p) == null) {
 	        // node "p" is unreachable; to reduce the kilter
 	        // number of the current edge, we need to change the
 	        // node potentials by -c_pq(pi) for all reachable
 	        // nodes.  This fix by CC, and is needed b/c we don't
 	        // accept the AMO reachability assumption

	        int rc = RedCost.GetInt(current);
	        if (p == current.GetTarget())
		    rc = -rc;

                nit = Net.GetNodeListIter();
	        for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode())
		    if (Dist.GetInt(i) < INFINITY) {
		        Potential.Put(i,Potential.GetInt(i)-rc);
			PutNodeLabel(i,Supply,Potential);
		    }
		UI.ShowPseudoCodeText("update pi (GIDEN)");
            }
	    else {
	        // update node potentials as described in AMO
	        nit = Net.GetNodeListIter();
		for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode())
		    if (Dist.GetInt(i) < INFINITY) {
		        Potential.Put(i,Potential.GetInt(i)-Dist.GetInt(i));
			PutNodeLabel(i,Supply,Potential);
		    }
		UI.ShowPseudoCodeText("update pi (AMO)");
	    }

            // update edge reduced costs
            eit = Net.GetEdgeListIter();
            for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
                RedCost.Put(e,Cost.GetInt(e)-Potential.GetInt(e.GetSource())
                                            +Potential.GetInt(e.GetTarget()));

            // check reduced cost and augment
            UI.ShowPseudoCodeText("check red-cost");
            if ((RedCost.GetInt(current) < 0 && p == current.GetSource()) ||
                (RedCost.GetInt(current) > 0 && p == current.GetTarget())) {

                // calculate cost and capacity of cycle
                if (p == current.GetSource()) { // forward edge
                    delta = Capacity.GetInt(current)-Flow.GetInt(current);
                    cycle_cost = Cost.GetInt(current);
                }
                else { // reverse edge
                    delta = Flow.GetInt(current);
                    cycle_cost = -Cost.GetInt(current);
                }

                i = p;
                while (Pred.GetEdge(i) != null) {
                    e = Pred.GetEdge(i);
                    if (i == e.GetTarget()) {
                        delta = Math.min(delta,
                                         Capacity.GetInt(e)-Flow.GetInt(e));
                        cycle_cost += Cost.GetInt(e);
                    }
                    else {
                        delta = Math.min(delta,Flow.GetInt(e));
                        cycle_cost -= Cost.GetInt(e);
                    }

                    UI.SingleStep(e,Cycle);
                    i = e.GetOpposite(i);
                }
                UI.Trace(Cycle);
                UI.ShowPseudoCodeText("cycle");
                UI.StatusLine("Cycle residual capacity = "+delta+".  "+
                              "Cycle cost = "+cycle_cost);
                UI.ShowPseudoCodeText("cycle capacity");
                UI.Update();
                UI.StatusLine("");



                // augment flow
                if (p == current.GetSource())  // forward edge
                    Flow.Put(current,Flow.GetInt(current)+delta);
                else // reverse edge
                    Flow.Put(current,Flow.GetInt(current)-delta);
                PutEdgeLabel(current,Flow,Capacity,Cost);

                i=p;
                while (Pred.GetEdge(i) != null) {
                    e = Pred.GetEdge(i);
                    if (i == e.GetTarget())
                        Flow.Put(e,Flow.GetInt(e)+delta);
                    else
                        Flow.Put(e,Flow.GetInt(e)-delta);

                    PutEdgeLabel(e,Flow,Capacity,Cost);
                    i = e.GetOpposite(i);
                }
                FlowCost += delta*cycle_cost;
                UI.StatusLine("Flow updated.  New flow cost = $"+FlowCost);
                UI.ShowPseudoCodeText("augment flow");
                UI.ShowPseudoCodeText("update all");
            }

            while (!Kilter.IsEmpty() 
                   && getKilterNumber((Edge)Kilter.GetTop(),
                                      Flow,Capacity,RedCost) == 0)
                Kilter.Pop();
            //**************
            //
            //            int k_e; // the kilter number of edge e 
            //            do {
            //                e = (Edge) Kilter.GetTop();
            //
            //                if (RedCost.GetInt(e) < 0)
            //                    k_e = Capacity.GetInt(e)-Flow.GetInt(e);
            //                else if (RedCost.GetInt(e) > 0)
            //                    k_e = Flow.GetInt(e);
            //                else
            //                    k_e = 0;
            //
            //                if (k_e == 0)
            //                    Kilter.Pop();
            //            } while (!Kilter.IsEmpty() && k_e == 0);
            //**************
            //
            //            while (!Kilter.IsEmpty() && 
            //                   RedCost.GetInt((Edge)Kilter.GetTop()) == 0)
            //                Kilter.Pop();

            UI.MoveAll(InitialSet);

            UI.ShowPseudoCodeText("kilter loop");
        }
    }

    /****************************************************************/
    // determine distance labels
    /****************************************************************/
    private void DetermineDistanceLabels(Network   Net,
                                         Node      p,
                                         Node      q,
                                         Edge      e_pq,
                                         NodeArray Dist,
                                         NodeArray Pred,
                                         EdgeArray RedCost) {
        PQList PQ = new PQList();

        Pred.Fill(null);
        Dist.Fill(INFINITY);
        Dist.Put(q,0);

        NodeListIter nit = Net.GetNodeListIter();
        for (Node i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode())
            PQ.Insert(i,Dist.GetInt(i));

        Edge e;
        Node i,j;
        EdgeListIter eit;
        int temp_cost;
        while (!PQ.IsEmpty()) {
            i = PQ.DeleteMinNode();

            if (Dist.GetInt(i) == INFINITY) 
                return;  // remaining PQ nodes are unreachable from q

            eit = i.GetAdjListIter();
            for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
                if (e != e_pq) {
                    temp_cost = INFINITY;
                    j = e.GetOpposite(i);
                    if (j == e.GetTarget() 
                        && Capacity.GetInt(e)-Flow.GetInt(e) > 0) {
                        temp_cost = Dist.GetInt(i)
                                   +Math.max(0,RedCost.GetInt(e));
                    }
                    else if (j == e.GetSource() && Flow.GetInt(e) > 0) {
                        temp_cost = Dist.GetInt(i)
                                   +Math.max(0,-RedCost.GetInt(e));
                    }

                    if (temp_cost < Dist.GetInt(j)) {
                        Dist.Put(j,temp_cost);
                        Pred.Put(j,e);
                        PQ.DecreaseValue(j,temp_cost);
                    }
                }
            }
        }

    }

    /****************************************************************/
    // get initial flow
    /****************************************************************/
    private void GetInitialFlow(Network   Net,
                                EdgeArray Cost,
                                EdgeArray Flow,
                                EdgeArray Capacity,
                                NodeArray Supply) {

        GraphState SaveGraph = new GraphState(Net);

        Node asource = Net.CreateNode();
        Node asink = Net.CreateNode();

        int TotalSupply = 0;

        NodeListIter nit = Net.GetNodeListIter();
        Node n;
        Edge e;
        for (n=nit.GetFirstNode() ; n!=null ; n=nit.GetNextNode()) {
            if (Supply.GetInt(n) > 0) {
                e = Net.CreateEdge(asource,n);
                Capacity.Put(e,Supply.GetInt(n));
                TotalSupply += Supply.GetInt(n);
            }
            else if (Supply.GetInt(n) < 0) {
                e = Net.CreateEdge(n,asink);
                Capacity.Put(e,-Supply.GetInt(n));
            }
        }
        
        // call a max-flow solver to get an initial feasible flow
        UserBase nullbase = new UserBase(UI.Services);
        MaxFlow solver = new PreFlowPush(nullbase);
        
        solver.Setup(Net);

        solver.Source = asource;
        solver.Sink = asink;
        solver.Capacity = Capacity;

        solver.Execute();
        GIDEN.out.println("Flow Value= "+solver.FlowValue);
        
        // restore network and save flow values
        SaveGraph.Restore();
        EdgeListIter eit = Net.GetEdgeListIter();
        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
            Flow.Put(e, solver.Flow.GetInt(e));
            PutEdgeLabel(e,Flow,Capacity,Cost);
            FlowCost += Flow.GetInt(e)*Cost.GetInt(e);
        }
        
        // check feasibility; is flow value equal to sum of supplies (demands)?
        feasible = (solver.FlowValue == TotalSupply);

        solver.Shutdown();

        if (!feasible)
            UI.StatusLine("Could not find an initial feasible flow!");
        else
            UI.StatusLine("Initial flow generated using a max-flow solver.  "+
                          "Flow cost = $"+FlowCost+".");
        UI.ShowPseudoCodeText("init flow");
        UI.StatusLine("");
    }
    
    /****************************************************************/
    // calculate the kilter number of an edge
    /****************************************************************/
    private int getKilterNumber(Edge e,
                                EdgeArray Flow,
                                EdgeArray Capacity,
                                EdgeArray RedCost) {
        if (RedCost.GetInt(e) < 0)
            return Capacity.GetInt(e)-Flow.GetInt(e);
        else if (RedCost.GetInt(e) > 0)
            return Flow.GetInt(e);
        else
            return 0;
    }
}
