package  giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;

/**********************************************************************/
// Pre-Flow Push (Goldberg-Tarjan) class
/**********************************************************************/
public class PreFlowPush extends MaxFlow {

    private ColorSet Discarded;
    private AnimationSet Current;
    private AnimationSet Active;

    private int RelabelCount;

    private final int NOT_ACTIVE = 1;
    private final int ACTIVE = 2;


    // control parameters
    protected Boolean AdvancedTerminationFlag = null;


    /****************************************************************/
    // PreFlowPush constructor
    /****************************************************************/
    public PreFlowPush(UserBase userinfo) {
        super(userinfo);

        Current   = UI.CreateAnimationSet("Current",Color.red);
        Active    = UI.CreateAnimationSet("Active",Color.blue);
        Discarded = UI.CreateColorSet(Color.lightGray);
    }

    public String getSolverName() {return "Pre-Flow Push";}

    /****************************************************************/
    // Setup
    //   - calls MaxFlow base class Setup()
    //   - initializes solver specific information
    /****************************************************************/
    public boolean Setup(Network net) {
        if(super.Setup(net)==false)
            return false;

        return true;
    }

    /****************************************************************/
    // PreFlowPush Execute
    /****************************************************************/
    public boolean Execute() {
        RelabelCount = 0;

        Node Source = GetSourceNode();
        Node Sink = GetSinkNode();
        //        EdgeArray Capacity = GetCapacityArray();
        EdgeArray Flow = GetFlowArray();
        NodeArray Reachable = GetReachableArray();

        Network Net = GetNetwork();


        int[] CntDist = new int[2*Net.GetNodeIndex()+1];
        NodeArray Dist = new NodeArray(Net.GetNodeIndex(),"Dist");
        NodeArray Excess = new NodeArray(Net.GetNodeIndex(),"Excess");
        NodeArray NodeStatus = new NodeArray(Net.GetNodeIndex(),"NodeStatus");
        NodeArray CurrentEIT = new NodeArray(Net.GetNodeIndex(),"CurrentEIT");

        NodeListIter nit = Net.GetNodeListIter();
        for (Node i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode())
            InitializeAdjEdgeIter(i,CurrentEIT);

        NodeStatus.Fill(NOT_ACTIVE);
        Excess.Fill(0);

        if (AdvancedTerminationFlag == null)
            AdvancedTerminationFlag = GetAdvancedTerminationFlag();

        PreFlowPushSolver(Net,Source,Sink,Capacity,Flow,Reachable,CntDist,Dist,
                          NodeStatus,CurrentEIT,Excess,AdvancedTerminationFlag.booleanValue());
        FindSourceReachableNodes(Net,Source,Reachable,Flow,Capacity);
        DisplayMinCut(Net,Reachable,Capacity);

        return true;
    }

    /****************************************************************/
    private Boolean GetAdvancedTerminationFlag() {
        String temp;
        do {
            temp = UI.UserTextInput("Advanced termination?","Yes");
            if (temp == null) temp = new String("yes");

            temp = temp.toUpperCase();
        } while (temp.compareTo("YES") != 0 && temp.compareTo("NO") != 0);
        return new Boolean(temp.compareTo("YES") == 0);
    }

    /****************************************************************/
    // PreFlowPush PreFlowPushSolver
    /****************************************************************/
    public void PreFlowPushSolver(Network   Net,
                                  Node      Source,
                                  Node      Sink,
                                  EdgeArray Capacity,
                                  EdgeArray Flow,
                                  NodeArray Reached,
                                  int[]     CntDist,
                                  NodeArray Dist,
                                  NodeArray NodeStatus,
                                  NodeArray CurrentEIT,
                                  NodeArray Excess,
                                  boolean   AdvancedTermination) {
        UI.StatusLine("Preflow-Push Solver begins...");
        UI.ShowPseudoCodeText("start");
        UI.StatusLine("");

        PQList PQ = new PQList();
        Node i;
        Edge e;

        UI.ShowPseudoCodeText("call preprocess");
        // Preprocess
        UI.ShowPseudoCodeText("proc preprocess");
        UI.ShowPseudoCodeText("init flow");
        InitDistanceLabels(Net,Sink,Reached,Dist,CntDist);
        UI.ShowPseudoCodeText("init dist");
        SaturateOutgoingArcs(Source,Dist,CntDist,
                             PQ,NodeStatus,Excess,Flow,Capacity);
        UI.ShowPseudoCodeText("saturate source");
        CntDist[Dist.GetInt(Source)]--;
        Dist.Put(Source,Net.GetNodeIndex());
        CntDist[Net.GetNodeIndex()]++;
        PutNodeLabel(Source,Dist);
        UI.ShowPseudoCodeText("dist(s)");

        i = GetCandidateNode(Net,PQ,Dist,AdvancedTermination);

        while (ActiveNodeExists(i)) {
            UI.SingleStep(i,Current);
            UI.Trace(Current);
            UI.Update();
            UI.ShowPseudoCodeText("select node");
            
            // Push/Relabel
            UI.ShowPseudoCodeText("call push/relabel");
            UI.ShowPseudoCodeText("proc push/relabel");
            e = AdmissibleAdjArc(i,Capacity,Flow,Dist,CurrentEIT);
            UI.ShowPseudoCodeText("admissible?");
            if (e != null)
                Push(i,e,PQ,NodeStatus,Excess,Dist,Flow,Capacity,Source,Sink);
            else {
                CntDist[Dist.GetInt(i)]--;
                if (AdvancedTermination && CntDist[Dist.GetInt(i)] == 0)
                    AdvRelabel(Net,Dist.GetInt(i),Dist,CntDist);
                else {
                    Relabel(Net,i,Capacity,Flow,Dist);
                    if (AdvancedTermination && Dist.GetInt(i) > Net.GetNodeIndex())
                        Dist.Put(i,Net.GetNodeIndex());
                    PutNodeLabel(i,Dist);
                    PQ.DecreaseValue(i,-Dist.GetInt(i));
                    CntDist[Dist.GetInt(i)]++;
                    InitializeAdjEdgeIter(i,CurrentEIT);
                    if (Dist.GetInt(i) >= Net.GetNodeIndex() && AdvancedTermination) {
                        UI.SingleStep(i,Discarded);
                        UI.Trace(Discarded);
                        UI.ShowPseudoCodeText("relabel");
                        UI.Update();
                        UI.StatusLine("Advanced Termination ==> d(i) := number of nodes");
                    }
                    else
                        UI.ShowPseudoCodeText("relabel");
                }
            }
            
            if (Excess.GetInt(i) == 0) {
                PQ.DeleteMinNode();
                NodeStatus.Put(i,NOT_ACTIVE);
                UI.SingleStep(i,Discarded);
            }
            UI.Trace(Discarded);
            UI.Trace(Active);
            UI.Update();

            i = GetCandidateNode(Net,PQ,Dist,AdvancedTermination);
        }
        
        if (AdvancedTermination)
            Preflow2Flow(Net,Source,Sink,Flow,Capacity,Reached,
                         Dist,PQ,NodeStatus,CurrentEIT,Excess);

        UI.StatusLine("Preflow-Push Solver finished.");
        UI.ShowPseudoCodeText("end");
    }


    /****************************************************************/
    //   Put Edge label for edge e
    /****************************************************************/
    private void PutEdgeLabel(Edge      e,
                              EdgeArray Flow,
                              EdgeArray Capacity) {
        int residual = Capacity.GetInt(e) - Flow.GetInt(e);
        String label = "(" + Flow.GetInt(e) + "," + residual + ")";
        
        UI.PutLabel(e,label);
    }


    /****************************************************************/
    // Put Node label for i
    /****************************************************************/
    private void PutNodeLabel(Node      i,
                              NodeArray Dist) {
        UI.PutLabel(i,Dist.GetInt(i));
    }


    /****************************************************************/
    //   Resolve the current (maximal) preflow to a valid flow.  The
    //   algorithm used is a variant of highest-label preflow-push that
    //   sends the preflow excess back to the source node.
    /****************************************************************/
    private void Preflow2Flow(Network Net,
                              Node    Source,
                              Node    Sink,
                              EdgeArray Flow,
                              EdgeArray Capacity,
                              NodeArray Reached,
                              NodeArray Dist,
                              PQList    PQ,
                              NodeArray NodeStatus,
                              NodeArray CurrentEIT,
                              NodeArray Excess) {
        int num_active = 0;
        Node i,j;
        Edge e;

        boolean excess_resolved;

        // init arrays, distance labels, and active node PQ
        num_active = Setup_P2F(Net,Source,Sink,Flow,Reached,Dist,
                               Excess,NodeStatus,PQ,CurrentEIT);

        i = GetCandidateNode_P2F(PQ);
        while (ActiveNodeExists(i)) {
            NodeStatus.Put(i,NOT_ACTIVE);
            UI.SingleStep(i,Current);
            UI.Trace(Current);
            UI.Update();

            excess_resolved = false;
            while (!excess_resolved) {
                e = AdmissibleIncArc(i,Flow,Dist,CurrentEIT);
                //-JHO 18aug1998        j = e.GetSource();

                if (e != null) {
                    Push_P2F(i,e,PQ,NodeStatus,Dist,Excess,Flow,Source);
                    PutEdgeLabel(e,Flow,Capacity);

                    if (Excess.GetInt(i) == 0) {
                        excess_resolved = true;
                        UI.SingleStep(i,Discarded);
                    }
                    UI.Trace(Discarded);
                    UI.Update();
                }
                else {
                    Relabel_P2F(Net.GetNodeIndex(),i,Flow,Dist);
                    PutNodeLabel(i,Dist);
                    InitializeIncEdgeIter(i,CurrentEIT);
                }
            }

            i = GetCandidateNode_P2F(PQ);
        }
    }


    /****************************************************************/
    // Return an admissible incoming arc if one exists or null otherwise
    /****************************************************************/
    private Edge AdmissibleIncArc(Node      i,
                                  EdgeArray Flow,
                                  NodeArray Dist,
                                  NodeArray CurrentEIT) {
        Node j;
        Edge e = CurrentEdge(i,CurrentEIT);

        while (e != null) {
            j = e.GetSource();

            if (Dist.GetInt(i) == Dist.GetInt(j) + 1 && Flow.GetInt(e) > 0)
                return e;
            else {
                UI.SingleStep(e,Discarded);
                UI.Trace(Discarded);
                UI.Update();
                e = GetNextEdge(i,CurrentEIT);
            }
        }
        return ((Edge) null);
    }


    /****************************************************************/
    //   If the PQ is non-empty, delete and return the top element,
    //   otherwise return null.
    /****************************************************************/
    private Node GetCandidateNode_P2F(PQList    PQ) {
        if (PQ.NotEmpty())
            return (PQ.DeleteMinNode());
        else
            return ((Node) null);
    }


    /****************************************************************/
    //   Update the distance label of the current active node so that
    //   excess can later be pushed along an arc with residual capacity
    /****************************************************************/
    private void Relabel_P2F(int       n,
                             Node      i,
                             EdgeArray Flow,
                             NodeArray Dist) {
        Node j;
        Edge e;
        int residual;
        int current_min_dist = n;

        EdgeListIter eit = i.GetInListIter();
        for (e=eit.GetFirstEdge(); e != null ; e = eit.GetNextEdge()) {
            j = e.GetSource();
            residual = Flow.GetInt(e);

            if (residual > 0 && Dist.GetInt(j) < current_min_dist)
                current_min_dist = Dist.GetInt(j);
        }

        Dist.Put(i,current_min_dist + 1);
        RelabelCount++;
    }


    /****************************************************************/
    //   Send flow along arc e to resolve excess at i
    /****************************************************************/
    private void Push_P2F(Node      i,
                          Edge      e,
                          PQList    PQ,
                          NodeArray NodeStatus,
                          NodeArray Dist,
                          NodeArray Excess,
                          EdgeArray Flow,
                          Node      Source) {
        Node j = e.GetSource();
        int delta = Flow.GetInt(e);

        if (delta >= Excess.GetInt(i))
            delta = Excess.GetInt(i);

        Flow.Put(e, Flow.GetInt(e) - delta);
        Excess.Put(i, Excess.GetInt(i) - delta);
        Excess.Put(j, Excess.GetInt(j) + delta);

        if (j != Source && NodeStatus.GetInt(j) == NOT_ACTIVE) {
            PQ.Insert(j,-Dist.GetInt(j));
            NodeStatus.Put(j,ACTIVE);

            UI.SingleStep(j,Active);
        }
        UI.SingleStep(e,Current); // owenj-change 060497
        UI.Trace(Active);
        UI.Update();

        UI.SingleStep(e,Discarded);
    }


    /****************************************************************/
    //   Initialization for the Preflow2Flow method.
    //    - initialize misc arrays
    //    - initialize node distance labels
    //    - place active nodes in the PQ
    /****************************************************************/
    private int Setup_P2F(Network   Net,
                          Node      Source,
                          Node      Sink,
                          EdgeArray Flow,
                          NodeArray Reached,
                          NodeArray Dist,
                          NodeArray Excess,
                          NodeArray NodeStatus,
                          PQList    PQ,
                          NodeArray CurrentEIT) {
        Node i,j;
        Edge e;
        EdgeListIter eit;
        int num_added_to_pq = 0;

        // initialize arrays
        Dist.Fill(Net.GetNodeIndex());
        Reached.Fill(NOT_REACHED);
        NodeStatus.Fill(NOT_ACTIVE);

        // initialize distance labels from source node and add active nodes to PQ
        BoundedQueue BQ = new BoundedQueue(Net.GetNodeIndex());
        Dist.Put(Source,0);
        UI.PutLabel(Source,"s");
        Reached.Put(Source,REACHED);
        BQ.Append(Source);

        while (!BQ.IsEmpty()) {
            i = (Node) BQ.Pop();

            eit = i.GetOutListIter();
            for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
                j = e.GetTarget();
                if (Flow.GetInt(e) > 0 && Reached.GetInt(j) != REACHED && j != Source){
                    BQ.Append(j);
                    Reached.Put(j,REACHED);
                    Dist.Put(j,Dist.GetInt(i) + 1);
                    PutNodeLabel(j,Dist);
                    if (Excess.GetInt(j) > 0 && j != Sink) {
                        PQ.Insert(j,-Dist.GetInt(j));
                        NodeStatus.Put(j,ACTIVE);
                        UI.SingleStep(j,Active);
                        num_added_to_pq++;
                    }
                }
            }
        }

        // initialize the current edgelist iterator for each node
        // and put distances and labels on NOT_REACHED nodes
        NodeListIter nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i != null ; i = nit.GetNextNode()) {
            InitializeIncEdgeIter(i,CurrentEIT);
            if (Reached.GetInt(i) == NOT_REACHED) {
                Dist.Put(i,2*Net.GetNodeIndex());
                UI.PutLabel(i,"--");
            }
        }

        // write graphics updates to screen
        UI.Trace(Active);
        UI.Update();

        // return the number of active nodes in the PQ
        return num_added_to_pq;
    }


    /****************************************************************/
    //   Set edge iterator for the current node's first incoming edge
    /****************************************************************/
    private void InitializeIncEdgeIter(Node i, NodeArray CurrentEIT) {
        EdgeListIter eit = i.GetInListIter();
        eit.GetFirstEdge();
        CurrentEIT.Put(i,eit);
    }


    /****************************************************************/
    //   Relabels all nodes with Distance labels >= k to GetNodeIndex().
    //   (This is only called when using advanced termination.)
    /****************************************************************/
    private void AdvRelabel(Network   Net,
                            int       k,
                            NodeArray Dist,
                            int[]     CntDist) {
        NodeListIter nit = Net.GetNodeListIter();
        for (Node i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode()) {
            if (Dist.GetInt(i) >= k) {
                Dist.Put(i,Net.GetNodeIndex());
                PutNodeLabel(i,Dist);
                UI.SingleStep(i,Discarded);
                UI.Trace(Discarded);
                UI.Update();
            }
        }
        for (int j=k ; j < Net.GetNodeIndex() ; j++)
            CntDist[j] = 0;
    }


    /****************************************************************/
    //   If advanced termination is selected, we return the highest label
    //   element with distance <= num_nodes if one exists, otherwise null.
    //
    //   For basic termination, if the PQ is non-empty then we delete and
    //   return the top element,otherwise return null.
    /****************************************************************/
    private Node GetCandidateNode(Network   Net,
                                  PQList    PQ,
                                  NodeArray Dist,
                                  boolean   AdvancedTermination) {
        if (AdvancedTermination) {
            Node i;

            while (PQ.NotEmpty()) {
                i = PQ.GetMinNode();
                if (Dist.GetInt(i) < Net.GetNodeIndex())
                    return i;

                PQ.DeleteMinNode();
            }
            return ((Node) null);
        }

        // Basic Termination
        if (PQ.NotEmpty())
            return (PQ.GetMinNode());
        else
            return ((Node) null);
    }


    /****************************************************************/
    /****************************************************************/
    private boolean ActiveNodeExists(Node i) {
        UI.ShowPseudoCodeText("main loop");
        return (i != null);
    }


    /****************************************************************/
    //   Return an admissible adjacent edge if one exists, null otherwise
    /****************************************************************/
    private Edge AdmissibleAdjArc(Node      i,
                                  EdgeArray Capacity,
                                  EdgeArray Flow,
                                  NodeArray Dist,
                                  NodeArray CurrentEIT) {
        Node j;
        Edge e = CurrentEdge(i,CurrentEIT);
        int residual;

        while (e != null) {
            j = e.GetOpposite(i);

            if (e.GetSource() == i)
                residual = Capacity.GetInt(e) - Flow.GetInt(e);
            else
                residual = Flow.GetInt(e);

            if (Dist.GetInt(i) == Dist.GetInt(j) + 1 && residual > 0)
                return e;
            else {
                UI.SingleStep(e,Discarded);
                UI.Trace(Discarded);
                UI.Update();
                e = GetNextEdge(i,CurrentEIT);
            }
        }
        return ((Edge) null);
    }

    /****************************************************************/
    //   Set edge iterator for the current node's first adjacent edge
    /****************************************************************/
    private void InitializeAdjEdgeIter(Node i, NodeArray CurrentEIT) {
        EdgeListIter eit = i.GetAdjListIter();
        eit.GetFirstEdge();
        CurrentEIT.Put(i,eit);
    }

    /****************************************************************/
    //   Return the edge pointed to by the current edge iterator
    /****************************************************************/
    private Edge CurrentEdge(Node i, NodeArray CurrentEIT) {
        EdgeListIter eit = (EdgeListIter) CurrentEIT.GetObj(i);
        return ((Edge) eit.GetItem());
    }

    /****************************************************************/
    //   Increment the edge iterator and return the next edge in the list
    /****************************************************************/
    private Edge GetNextEdge(Node i, NodeArray CurrentEIT) {
        EdgeListIter eit = (EdgeListIter) CurrentEIT.GetObj(i);
        Edge nextedge = eit.GetNextEdge();
        CurrentEIT.Put(i,eit);
        return nextedge;
    }

    /****************************************************************/
    //   Send flow along arc e to resolve excess at i
    /****************************************************************/
    private void Push(Node      i,
                      Edge      e,
                      PQList    PQ,
                      NodeArray NodeStatus,
                      NodeArray Excess,
                      NodeArray Dist,
                      EdgeArray Flow,
                      EdgeArray Capacity,
                      Node      Source,
                      Node      Sink) {
        int delta;
        Node j = e.GetOpposite(i);

        UI.SingleStep(e,Current); // owenj change 064097

        if (e.GetSource() == i)
            delta = Capacity.GetInt(e) - Flow.GetInt(e);
        else
            delta = Flow.GetInt(e);

        if (Excess.GetInt(i) < delta)
            delta = Excess.GetInt(i);
        UI.StatusLine("Residual capacity = "+delta+" units.");
        UI.ShowPseudoCodeText("update delta");
        UI.StatusLine("");

        
        if (e.GetSource() == i)
            Flow.Put(e,Flow.GetInt(e)+delta);
        else
            Flow.Put(e,Flow.GetInt(e)-delta);

        if (j == Sink)
            FlowValue += delta;

        Excess.Put(i,Excess.GetInt(i) - delta);
        Excess.Put(j,Excess.GetInt(j) + delta);


        if (j != Sink && j != Source && NodeStatus.GetInt(j) == NOT_ACTIVE) {
            PQ.Insert(j,-Dist.GetInt(j));
            NodeStatus.Put(j,ACTIVE);

            UI.SingleStep(j,Active);
        }
        UI.Trace(Active);
        UI.Update();
        PutEdgeLabel(e,Flow,Capacity);

        UI.SingleStep(e,Discarded);
        UI.ShowPseudoCodeText("push delta");
    }


    /****************************************************************/
    //   Update the distance label of the current active node so that
    //   excess can later be pushed along an arc with residual capacity
    /****************************************************************/
    private void Relabel(Network   Net,
                         Node      i,
                         EdgeArray Capacity,
                         EdgeArray Flow,
                         NodeArray Dist) {
        Node j;
        Edge e;
        int residual;
        int current_min_dist = 2 * Net.GetNodeIndex();

        EdgeListIter eit = i.GetAdjListIter();
        for (e=eit.GetFirstEdge(); e != null ; e = eit.GetNextEdge()) {
            if (e.GetSource() == i) {
                j = e.GetTarget();
                residual = Capacity.GetInt(e) - Flow.GetInt(e);
            }
            else {
                j = e.GetSource();
                residual = Flow.GetInt(e);
            }

            if (residual > 0 && Dist.GetInt(j) < current_min_dist)
                current_min_dist = Dist.GetInt(j);
        }
        
        Dist.Put(i,current_min_dist + 1);
        RelabelCount++;
    }


    /****************************************************************/
    //   Send all flow possible along outgoing arcs from the source node
    /****************************************************************/
    private void SaturateOutgoingArcs(Node      Source,
                                      NodeArray Dist,
                                      int[]     CntDist,
                                      PQList    PQ,
                                      NodeArray NodeStatus,
                                      NodeArray Excess,
                                      EdgeArray Flow,
                                      EdgeArray Capacity) {
        Edge e;

        UI.SingleStep(Source,Current);
        UI.Trace(Current);
        UI.Update();

        EdgeListIter eit = Source.GetOutListIter();
        for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
            UI.SingleStep(e,Current); // owenj change 060497

            Node j = e.GetTarget();
            Flow.Put(e,Capacity.GetInt(e));
            Excess.Put(j, Excess.GetInt(j) + Capacity.GetInt(e));
            if (NodeStatus.GetInt(j) == NOT_ACTIVE) {
                PQ.Insert(j,-Dist.GetInt(j));
                NodeStatus.Put(j,ACTIVE);

                UI.SingleStep(j,Active);
            }

            UI.Trace(Active);
            UI.Update();

            PutEdgeLabel(e,Flow,Capacity);
            UI.SingleStep(e,Discarded);
            UI.Trace(Discarded);
            UI.Update();
        }
        UI.SingleStep(Source,Discarded);
        UI.Trace(Discarded);
        UI.Update();
    }


    /****************************************************************/
    //   Initialize the node distance labels via a BFS from the Sink node
    /****************************************************************/
    private void InitDistanceLabels(Network   Net,
                                    Node      Sink,
                                    NodeArray Reached,
                                    NodeArray Dist,
                                    int[]     CntDist) {
        Reached.Fill(NOT_REACHED);
        BoundedQueue BQ = new BoundedQueue(Net.GetNodeIndex());

        Node v;
        Node w;
        Edge e;

        EdgeListIter eit;

        Dist.Put(Sink,0);
        CntDist[0] = 1;
        Reached.Put(Sink,REACHED);
        BQ.Append(Sink);

        while (!BQ.IsEmpty()) {
            v = (Node) BQ.Pop();

            eit = v.GetInListIter();
            for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
                w =e.GetSource();

                if (Reached.GetInt(w) == NOT_REACHED) {
                    Reached.Put(w,REACHED);
                    Dist.Put(w,Dist.GetInt(v) + 1);
                    PutNodeLabel(w,Dist);
                    CntDist[Dist.GetInt(w)]++;
                    BQ.Append(w);
                }
            }
        }

        NodeListIter nit = Net.GetNodeListIter();
        for (v = nit.GetFirstNode() ; v != null ; v = nit.GetNextNode()) {
            if (Reached.GetInt(v) == NOT_REACHED) {
                Dist.Put(v,2*Net.GetNodeIndex());
                UI.PutLabel(v,"--");
            }
        }


        // DECISION:  WE SHOLD HANDLE "if (!reached[Source)"
    }
}
