package  giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;

/**********************************************************************/
// Successive Shortest Path class
/**********************************************************************/
public class SuccShortPath extends MinCostFlow {

    private AnimationSet Path;
    private AnimationSet Bottleneck;

    private boolean feasible;

    private final int INFINITY = Integer.MAX_VALUE;
    private final int PERM_LABELED = 1;
    private final int NOT_PERM_LABELED = 2;

    /****************************************************************/
    // SuccShortPath constructor
    /****************************************************************/
    public SuccShortPath(UserBase userinfo) {
        super(userinfo);

        Path       = UI.CreateAnimationSet("Path",Color.blue);
        Bottleneck = UI.CreateAnimationSet("Path Bottleneck",Color.red);
    }

    public String getSolverName() {return "Successive Shortest Paths";}

    /****************************************************************/
    // Setup
    //   - calls MinCostFlow base class Setup()
    //   - initializes solver specific information
    /****************************************************************/
    public boolean Setup(Network net) {
        if(super.Setup(net)==false)
            return false;

        InitializeNetworkLabels(true);
        feasible = true;

        return true;
    }

    /****************************************************************/
    // SuccShortPath Execute
    /****************************************************************/
    public boolean Execute() {

        //        EdgeArray Cost = GetCostArray();
        //        EdgeArray Capacity = GetCapacityArray();
        //        NodeArray Supply = GetSupplyArray();
        EdgeArray Flow = GetFlowArray();
        NodeArray Potential = GetPotentialArray();

        Network Net = GetNetwork();

        NodeArray Excess = new NodeArray(Net.GetNodeIndex(),"Excess");
        EdgeArray RedCost = new EdgeArray(Net.GetEdgeIndex(),"RedCost");

        SuccShortPathSolver(Net,Cost,Flow,Capacity,Potential,Supply,
                            Excess,RedCost);
        if (feasible) {
            DisplayFinalSolution(Net,Cost,Flow,Capacity,Potential,Supply);
            UI.Trace(Path);
            UI.Update();
        }

        return true;
    }

    /****************************************************************/
    // SuccShortPath SuccShortPathSolver
    /****************************************************************/
    private void SuccShortPathSolver(Network   Net,
                                     EdgeArray Cost,
                                     EdgeArray Flow,
                                     EdgeArray Capacity,
                                     NodeArray Potential,
                                     NodeArray Supply,
                                     NodeArray Excess,
                                     EdgeArray RedCost) {
        UI.StatusLine("Successive Shortest Path Solver begins...");
        UI.ShowPseudoCodeText("start");
        UI.StatusLine("");

        Node s,t;
        BoundedQueue E = new BoundedQueue(Net.GetNodeIndex());
        NodeArray Dist = new NodeArray(Net.GetNodeIndex(),"Dist");
        NodeArray Pred = new NodeArray(Net.GetNodeIndex(),"Pred");
        NodeArray NodeStatus = new NodeArray(Net.GetNodeIndex(),"NodeStatus");
        

        Preprocess(Net,Flow,Capacity,Potential,Excess,Supply,E,Cost,RedCost);
        while (!E.IsEmpty()) {
            UI.ShowPseudoCodeText("main loop");
            s = (Node) E.GetTop();
	    UI.SingleStep(s,Path); // color the chosen excess node immediately
            UI.ShowPseudoCodeText("excess node");

            t = GetDeficitNode(Net,s,Dist,Excess,Pred,NodeStatus,
                               Flow,Capacity,RedCost);

            // check to be sure that s != t --- feasibility check
            if (s == t) {
                UI.StatusLine("Infeasible problem --- no path to a deficit node.");
                UI.ShowPseudoCodeText("calc dist");
                UI.StatusLine("");

  	        UI.SingleStep(s,InitialSet); // un-color chosen excess node
                UI.ShowPseudoCodeText("end");

                ReportInfeasibility(Net,NodeStatus,Supply,Capacity);
                return;
            }
            else
                UI.ShowPseudoCodeText("calc dist");

            ShowPath(s,t,Pred);
            UI.ShowPseudoCodeText("show path");

            UpdatePotentialsAndReducedCosts(Net,t,Dist,Excess,Potential,NodeStatus,
                                            Cost,Flow,Capacity,RedCost);
            UI.ShowPseudoCodeText("update pi");
            
            AugmentFlow(s,t,Excess,Pred,Potential,Cost,RedCost,Flow,Capacity);
            UI.ShowPseudoCodeText("augment");
            if (Excess.GetInt(s) == 0)
                E.Pop();
            
            UI.ShowPseudoCodeText("update all");
        }
        UI.ShowPseudoCodeText("main loop");

        UI.StatusLine("Successive Shortest Path Solver finished.");
        UI.ShowPseudoCodeText("end");
    }


    /****************************************************************/
    // update reduced costs on edges
    /****************************************************************/
    private void UpdatePotentialsAndReducedCosts(Network   Net,
                                                 Node      t,
                                                 NodeArray Dist,
                                                 NodeArray Excess,
                                                 NodeArray Potential,
                                                 NodeArray NodeStatus,
                                                 EdgeArray Cost,
                                                 EdgeArray Flow,
                                                 EdgeArray Capacity,
                                                 EdgeArray RedCost) {
        NodeListIter nit = Net.GetNodeListIter();
        for (Node i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode()) {
            if (NodeStatus.GetInt(i) == PERM_LABELED)
                Potential.Put(i,Potential.GetInt(i)+Dist.GetInt(t)-Dist.GetInt(i));
            PutNodeLabel(i,Excess,Potential);
        }

        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
            RedCost.Put(e,Cost.GetInt(e)
                        -Potential.GetInt(e.GetSource())
                        +Potential.GetInt(e.GetTarget()));
            //            PutEdgeLabel(e,Flow,Capacity,RedCost);
            PutEdgeLabel(e,Flow,Capacity,Cost);
        }
    }

    /****************************************************************/
    // Graphically display the path between excess and deficit nodes
    /****************************************************************/
    private void ShowPath(Node s, Node t, NodeArray Pred) {
        Edge e;
        Node i = t;
        while (i != s) {
            UI.SingleStep(i,Path);
            e = Pred.GetEdge(i);
            UI.SingleStep(e,Path);
            i = e.GetOpposite(i);
        }
	//        UI.SingleStep(i,Path);
        UI.Trace(Path);
        UI.Update();
    }

    /****************************************************************/
    // AugmentFlow -- send flow along s-t path
    /****************************************************************/
    private void AugmentFlow(Node      s,
                             Node      t,
                             NodeArray Excess,
                             NodeArray Pred,
                             NodeArray Potential,
                             EdgeArray Cost,
                             EdgeArray RedCost,
                             EdgeArray Flow,
                             EdgeArray Capacity) {
        int delta;
        Edge e;
        Node i;

        if (Excess.GetInt(s) <= -Excess.GetInt(t))
            delta = Excess.GetInt(s);
        else
            delta = -Excess.GetInt(t);
        
        i = t;
        while (i != s) {
            e = Pred.GetEdge(i);

            if (e.GetSource() == i && Flow.GetInt(e) < delta)
                delta = Flow.GetInt(e);
            else if (e.GetTarget() == i && Capacity.GetInt(e)-Flow.GetInt(e) < delta)
                delta = Capacity.GetInt(e)-Flow.GetInt(e);
            
            i = e.GetOpposite(i);
        }
	UI.StatusLine("Residual capacity \"delta\" = "+delta+".");
	UI.ShowPseudoCodeText("calc delta");
	UI.StatusLine("");

        i = t;
        while (i != s) {
            e = Pred.GetEdge(i);

            if (e.GetSource() == i) {
                if (Flow.GetInt(e) == delta
                    && Excess.GetInt(s) != delta && -Excess.GetInt(t) != delta)
                    UI.SingleStep(e,Bottleneck);
                FlowCost -= delta*Cost.GetInt(e);
                Flow.Put(e, Flow.GetInt(e)-delta);
                //      if (Flow.GetInt(e) == 0)
                //        SingleStep(e,Bottleneck);
            }
            else {
                if (delta == Capacity.GetInt(e)-Flow.GetInt(e)
                    &&  Excess.GetInt(s) != delta && -Excess.GetInt(t) != delta)
                    UI.SingleStep(e,Bottleneck);
                FlowCost += delta*Cost.GetInt(e);
                Flow.Put(e, Flow.GetInt(e)+delta);
                //      if (Flow.GetInt(e) == Capacity.GetInt(e))
                //        SingleStep(e,Bottleneck);
            }

            i = e.GetOpposite(i);
        }

        UI.Trace(Bottleneck);  // makes no sense to trace an "invisible set"
        UI.Trace(Path);
        UI.Update();

        Excess.Put(s,Excess.GetInt(s)-delta);
        Excess.Put(t,Excess.GetInt(t)+delta);

        UI.MoveAll(InitialSet);
        PutNodeLabel(s,Excess,Potential);
        PutNodeLabel(t,Excess,Potential);
        i = t;
        while (i != s) {
            e = Pred.GetEdge(i);
            //            PutEdgeLabel(e,Flow,Capacity,RedCost);
            PutEdgeLabel(e,Flow,Capacity,Cost);
            i = e.GetOpposite(i);
        }
        UI.Trace(Path);
        UI.Update();

        FlowValue += delta;
    }
    

    /****************************************************************/
    // GetDeficitNode -- returns a deficit node
    /****************************************************************/
    private Node GetDeficitNode(Network   Net,
                                Node      s,
                                NodeArray Dist,
                                NodeArray Excess,
                                NodeArray Pred,
                                NodeArray NodeStatus,
                                EdgeArray Flow,
                                EdgeArray Capacity,
                                EdgeArray RedCost) {
        PQList PQ = new PQList();
        int temp_cost;
        Node i,j;
        Edge e;
        EdgeListIter eit;

        NodeStatus.Fill(NOT_PERM_LABELED);
        Pred.Fill(null);
        Dist.Fill(INFINITY);

        NodeListIter nit = Net.GetNodeListIter();
        for (i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode())
            PQ.Insert(i,Dist.GetInt(i));

        Dist.Put(s,0);
        PQ.DecreaseValue(s,0);

        while (!PQ.IsEmpty()) {
            i = PQ.DeleteMinNode();

            if (Dist.GetInt(i) == INFINITY) // infeasible problem
                continue;

            NodeStatus.Put(i,PERM_LABELED);

            if (Excess.GetInt(i) < 0 && Dist.GetInt(i) != INFINITY)
                return i;

            eit = i.GetAdjListIter();
            for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
                temp_cost = INFINITY;

                if (e.GetSource() == i && Capacity.GetInt(e)-Flow.GetInt(e) > 0)
                    temp_cost = Dist.GetInt(i) + RedCost.GetInt(e);
                else if (e.GetTarget() == i && Flow.GetInt(e) > 0)
                    temp_cost = Dist.GetInt(i) - RedCost.GetInt(e);

                j = e.GetOpposite(i);
                if (temp_cost < Dist.GetInt(j)) {
                    Dist.Put(j,temp_cost);
                    Pred.Put(j,e);
                    PQ.DecreaseValue(j,temp_cost);
                }
            }
        }

        // we only exit here if the problem is infeasible;
        return s;
    }

    /****************************************************************/
    // ReportInfeasibility
    /****************************************************************/
    private void ReportInfeasibility(Network   Net,
                                     NodeArray NodeStatus,
                                     NodeArray Supply,
                                     EdgeArray Capacity) {
        //        SupplyNodes.PutLabel("");
        //        DemandNodes.PutLabel("");
        EdgesAtZero.PutLabel("");
        EdgesWithResidual.PutLabel("");
        EdgesAtCapacity.PutLabel("");

        ReachableNodes.PutLabel("Excess Reachable Nodes");
        InfeasibleCut.PutLabel("Infeasibility Cut");

        feasible = false;

        Node i;
        Edge e;
        int NetSupply = 0;  // net amount of supply for reachable node set
        int CutCapacity = 0;

        UI.MoveAll(InitialSet);

        boolean cut_visible = false;

        NodeListIter nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode()) {
            PutNodeLabel(i,Supply);
            if (NodeStatus.GetInt(i) == PERM_LABELED) {
                UI.SingleStep(i,ReachableNodes);
                NetSupply += Supply.GetInt(i);

                EdgeListIter eit = i.GetOutListIter();
                for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
                    if (NodeStatus.GetInt(e.GetTarget()) != PERM_LABELED) {
                        UI.SingleStep(e,InfeasibleCut);
                        CutCapacity += Capacity.GetInt(e);
                        cut_visible = true;
                    }

                eit = i.GetInListIter();
                for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
                    if (NodeStatus.GetInt(e.GetSource()) != PERM_LABELED) {
                        UI.SingleStep(e,InfeasibleCut);
                        cut_visible = true;
                    }
            }
        }

        if (!cut_visible) {
            InfeasibleCut.PutLabel("");
            UI.StatusLine("Problem infeasible: "
                          +"Net supply of highlighted nodes = "
                          +NetSupply+" > 0 = (empty) cut capacity.");
        }
        else
            UI.StatusLine("Problem infeasible: "
                          +"Net supply of highlighted nodes = "
                          +NetSupply+" > "+CutCapacity+
                          " = cut capacity.");

        UI.Trace(InfeasibleCut);
        UI.Trace(ReachableNodes);
    }

    /****************************************************************/
    // Preprocess
    /****************************************************************/
    private void Preprocess(Network      Net,
                            EdgeArray    Flow,
                            EdgeArray    Capacity,
                            NodeArray    Potential,
                            NodeArray    Excess,
                            NodeArray    Supply,
                            BoundedQueue E,
                            EdgeArray    Cost,
                            EdgeArray    RedCost) {
        Flow.Fill(0);
        Potential.Fill(0);
        UI.ShowPseudoCodeText("init flow");
        UI.ShowPseudoCodeText("init excess");

        NodeListIter nit = Net.GetNodeListIter();
        for (Node i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode()) {
            Excess.Put(i,Supply.GetInt(i));

            if (Excess.GetInt(i) > 0)
                E.Append(i);
        }

        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
            RedCost.Put(e,Cost.GetInt(e)); // since Potential.Fill(0);
        }
        UI.ShowPseudoCodeText("init sets");
    }

}
