package giden.userSOLVERS;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;
import giden.contrib.TSP.*;

/*
 * WE MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. WE SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY USER AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * TspSolverNearestNeighbor is TSP solver that uses the nearest neigbor algorithm to find a TSP tour.
 * Copyright (c) 2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @see TspSolverBase
 * @see TspSolverExhaustive
 *
 * @version 	1.0a 2004 Feb 01
 * @author 	    D. S. Dilworth
 */


public class TspSolverTwoSwap extends TspSolverBase
{ 
    NodeArray Busy;             // state of nodes (in or out of tour)
    AnimationSet AcquiredSet;   // nodes in the tour
    TspSolverBase Existing;
    
    /** Initializes invariant objects */
    public TspSolverTwoSwap (UserBase userinfo)
    {
        super(userinfo);        // mandatory call to the base class       
        AcquiredSet = UI.CreateAnimationSet("Acquired",Color.green);
        UI.CreateAnimationButtons();
    }
    
    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
        if(super.Setup(net)==false) // was Setup() successful?
            return false;
        UI.StatusLine("TspSolverNearest Setup");
        
        Busy = new NodeArray(NodeCount,""); // create busy array
        Busy.Fill(0);                       // fill with 0s
        Existing=null;
        return true;
    }
    
    public void PutExistingTour(TspSolverBase existing)
    {
        Existing = existing;
    }
       
    /** Executes the solver */
    public boolean Shutdown()
    {
        super.Shutdown();
        return true;
    }

    /** Solve TSP */
    public boolean ExecuteTspSolver()
    {
        return TwoSwap();        
    }
    
    /** Solve TSP using the Nearest Neighbor algorithm */
    boolean TwoSwap()
    {
        NodeListIter nit;
        boolean result=true;    // set solver result to default
        if(Existing==null)
            result=false;
        else
        {
            /*
            Edge e11,e12,e21,e22;
            Node n1,n2,n3,n4;
            
            int i=0;
            LinkList exist_list = Existing.GetTour();
            nit = new NodeListIter(exist_list);
            n1 = nit.GetFirstNode();
            n2 = nit.GetNextNode();
            n3 = nit.GetNextNode();
            n4 = nit.GetNextNode();
            */
            
            int i=0;
            LinkList exist_list = Existing.GetTour();
            nit = new NodeListIter(exist_list);
            Node node;
            for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
            {
                Order[i++]=node;
            }
            Evaluate();
            Edge e1=null;
            Edge e2=null;
        }
        return result;
    }
}
