package giden.userSOLVERS;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;

/*
 * Copyright (c) 1993-1999 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * WE MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. WE SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY USER AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <p>
 * <p>
 *
 * @see Network
 * @see SolverMenu
 * @see UserIO
 *
 * @version 	4.0a 2004 08 29
 * @author 	    D. S. Dilworth
 */


public class CallAnimatedSolver extends ExecBase
{
    UserBase UI;                  // user interactions
    Network Net;                  // access to network
    
    AnimationSet StartSet;        // StartSet data member
    AnimationSet EndSet;          // EndSet data member
    SolverServices Services;
    
    
    /** Initializes invariant objects */
    public CallAnimatedSolver (UserBase userinfo)
    {
        super(userinfo);        // mandatory call to the base class
        UI =  userinfo;         // local access for user interactions
        Services=UI.Services;
        
        // create the start and end Sets
        StartSet = UI.CreateAnimationSet("",Color.red);
        EndSet   = UI.CreateAnimationSet("End",Color.green);    
        UI.CreateAnimationButtons();
    }
    
    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
        Net=net;                        // local access to the network
        UI.StatusLine("Setup (User)");         // status line
        UI.MoveAll(StartSet);           // nodes/edges -> StartSet
        UI.ShowPseudoCodeText("start"); // pseudocode window
        return true;
    }
    
    /** Executes the solver */
    public boolean Execute()
    {
        
        GUISolver aux_window = (GUISolver) Services.GUI.parent.createSolverWindowDsd("Test", new GUINetwork(Net)); 
        aux_window.setupNetworkMode();
        //aux_window.pack();
        //aux_window.repaint();
        SolverServices aux_services = (SolverServices)aux_window.SolverControl;
        
        /*
        UserBase aux_user = new UserIO(aux_services,aux_services.GUI);
                
        Node node;
        Edge edge;
        NodeListIter nit;
        EdgeListIter eit;
        UiNodeInfo nodeinfo;
        UiEdgeInfo edgeinfo;

        //-------------------------------------
        // clear labels for each node/edge
        //-------------------------------------
        nit = Net.GetNodeListIter();    // node list iterator
        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
        {
            nodeinfo = UI.GetNodeInfo(node);
            aux_user.AddNode(node,nodeinfo);
        } 
        
        eit = Net.GetEdgeListIter();    // edge list iterator
        for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
        {
            edgeinfo = UI.GetEdgeInfo(edge);
            aux_user.AddEdge(edge,edgeinfo);
            
        } 
        */
        
        //aux_services.EventNotify("FeatureExample");
        
        UserBase aux_user = new UserIO(aux_services,aux_services.GUI);
        
        FeatureExample callsolver = new FeatureExample(aux_user);
        aux_services.Solver=callsolver;
        aux_window.setupSolverMode("test");
        //aux_window.pack();
        //aux_window.repaint();
        
        
        aux_services.RemoveUserBases();
        Services.SolverSetupExecute=true;                    // start setup/exec
        callsolver.PreSetup(Net);             // low level setup
        aux_services.AddUserBase(aux_user);  // add this solver
        aux_services.GUI.UiNet.UserInfo=aux_user;     // wire GUI->solver
        
        
        callsolver.Setup(Net);                                 // setup Kruskal
        callsolver.Execute();                                  // run Kruskal
        callsolver.Shutdown();       
        
        UI.StatusLine("Executing");         // status line
        UI.ShowPseudoCodeText("execute");   // pseudocode window
        
        ExecuteTestSolver();                // execute the solver
        
        UI.StatusLine("Completed");         // status line
        UI.ShowPseudoCodeText("end");       // pseudocode window
        return true;
    }
    
    /** Individually moves nodes and edges into the EndSet */
    void ExecuteTestSolver()
    {
        NodeListIter nit = Net.GetNodeListIter();    // node list iterator
        for(Node node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
        {
            UI.SingleStep(node,EndSet); // move node into EndSet
        }                
        EdgeListIter eit = Net.GetEdgeListIter();    // edge list iterator
        for(Edge edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
        {
            UI.SingleStep(edge,EndSet); // move edge into EndSet
        }    
        UI.Trace(EndSet);               // trace the EndSet
        UI.Update();                    // display animation            
    }
}
