package giden.CORE;
//import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.io.FileNotFoundException;
import java.net.MalformedURLException;
import java.io.IOException;
import java.net.URL;
import java.io.*;
import java.util.Properties;


public class GdnCsvInput
{
    double test;
    /** URL to load */
    URL UrlName;
    /** reader object */
    BufferedReader Reader;
    /** stream object */
    BufferedInputStream Stream;
    /** last line read and available for processing */
    public String LastLine;
    /** count of the number of commas */
    int Count;
    /** array of commas indices */
    int Commas[];
    /** true if valid connection to URL was made */
    boolean ValidUrl;
    /** true if the last read was valid */
    boolean ValidRead;
    
    
    public GdnCsvInput(String path, String filename)
    {
        String filepath;
        
        if(path==null)
        {
            Properties prop = System.getProperties();
            String user_dir = prop.getProperty("user.dir");
            filepath = "file:///"+user_dir+"/"+filename;
        }
        else
        {
            filepath = "file:///"+path+filename;
        }
        UrlName=null;
        try
        {
            UrlName = new URL(filepath);
       	}
       	catch( MalformedURLException e )
   	    {
   	        System.out.println(e.toString());
  	    }
  	    OpenUrl();
    }   
    
  
    /** constructs object, opens file, clears data members */
    public GdnCsvInput(URL url)
    {
        UrlName=url;
        OpenUrl();
    }
    
    void OpenUrl()
    {
        Reader=null;
        Stream=null;
        LastLine=null;
        Count=0;
        Commas=null;
        ValidUrl=false;
        ValidRead=false;
        
        try
        {
    	    InputStream stream =  UrlName.openStream();
            Stream = (new BufferedInputStream(stream));
            Reader = new BufferedReader( new InputStreamReader( Stream ) );
            ValidUrl=true;
        }
        catch(IOException error)
        {
            error.printStackTrace();            
        }
    }
    
    /** Gets the current string */
    public String toString()
    {
        return LastLine;
    }
    /** Reads a line from the file */
    public String readLine()
    {
        LastLine=null;
        Count=0;
        Commas=null;
        ValidRead=false;
        try
        {
            if(Reader!=null)
            {
                LastLine = Reader.readLine();
                ValidRead=true;
            }
        }
        catch(IOException error)
        {
            error.printStackTrace();            
        }
        return LastLine;
    }
    /** Gets the variable string from a Variable=Value line */
    public String getVariable()
    {
        String result=null;
        int index=LastLine.indexOf('=',0);
        if(index>0)
        {
            result=LastLine.substring(0,index).trim();
        }
        return result;
    }
    /** Gets the value string from a Variable=Value line */
    public String getValue()
    {
        String result=null;
        int index=LastLine.indexOf('=',0);
        if(index>0)
        {
            result=LastLine.substring(index+1).trim();
        }
        return result;
   }
    /** Parses the current line by commas */
    public int parseCommas()
    {
        Count=0;
        Commas=null;
        if(LastLine!=null)
        {
            int fromindex=0;
            int index;
            while( true )
            {
                index = LastLine.indexOf(',',fromindex);
                if(index==-1)
                    break;
                Count++;
                fromindex=index+1;
            }
            Commas = new int[Count];
            Count=0;
            fromindex=0;
            while( true )
            {
                index = LastLine.indexOf(',',fromindex);
                if(index==-1)
                    break;
                Commas[Count++]=index;
                fromindex=index+1;
            }
            Count++;
        }
        return Count;
    }
    /** Gets a count of the number of fields */
    public int getCount()
    {
        return Count;
    }
    
    /** Strips empty comma fields from the end of a line */
    public void NEWstripEmptyFields()
    {
        if(LastLine==null)
            return;
        int i = LastLine.length()-1;
        if(i==0)
            return;
        while(LastLine.charAt(i)==',')
        {
            i--;
        }
        LastLine=LastLine.substring(0,i+1);
    }
    
    /** Gets the string at the given field index */
    public String getString(int field)
    {
        int index;
        String result=null;
        if(field==0 && Count==1)
            result = LastLine;
        else if(field>=0 && field < Count)
        {
            if(field==0)
            {
                result = LastLine.substring(0,Commas[0]);
            }
            else if(field==Count-1)
            {
                index = Commas[field-1]+1;
                result = LastLine.substring(Commas[field-1]+1);
            }
            else
            {
                result = LastLine.substring(Commas[field-1]+1,Commas[field]);
            }
        }
        return result;
    }
    /** Converts the field into a double */
    public double getDouble(int field)
    {
        try
        {
            String string = getString(field).trim();
            return new Double(string).doubleValue();
        }
        catch(NumberFormatException error)
        {
            error.printStackTrace();            
        }
        return 0;
        
    }
    /** Converts the field into a int */
    public int getInt(int field)
    {
        try
        {
            String string = getString(field).trim();
            return (new Integer(string).intValue());
        }
        catch(NumberFormatException error)
        {
            error.printStackTrace();            
        }
        return 0;
        
    }
    /** Converts the field into a long */
    public long getLong(int field)
    {
        try
        {
            String string = getString(field).trim();
            return (new Long(string).longValue());
        }
        catch(NumberFormatException error)
        {
            error.printStackTrace();            
        }
        return 0;
        
    }
    
    /** Closes the file */    
    public void close()
    {
        try
        {
            if(Stream!=null)
            {
                Stream.close();
                Stream=null;
                if(Reader!=null)
                {
                    Reader.close();
                    Reader=null;
                }
            }
        }
        catch(IOException error)
        {
            error.printStackTrace();            
            
        }
    }
}
