package  giden.CORE;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.contrib.samples.*;
import giden.GRDS.*;
import giden.GUI.*;
import java.awt.*;
import java.awt.event.*;
/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 <p>
 RowDialogEntry is a base class for rows a RowDialogMain dialog box.
 Each row generally has four columns that are dynamically allocated:
 <ul>
 <li> The name of the row
 <li> The primary object being input (text, choice, node, edge, etc)
 <li> An optional object typically for a button
 <li> A help button the loads a specified HTML file or the info file for the solver
 </ul>
 See the example in DialogSolver.java and the documentation in RowDialogMain.java.
 * <p>
 * @see DialogSolver
 * @see RowDialogChoice
 * @see RowDialogEdge
 * @see RowDialogFilename
 * @see RowDialogMain
 * @see RowDialogNode
 * @see RowDialogText
 *
 * @version 	4.0a 2004 08 27
 * @author 	    D. S. Dilworth
 */

abstract class RowDialogEntry extends Panel implements ActionListener
{
    RowEntryLabel Name;
    RowEntryButton Help;
    RowDialogMain Dialog;
    GdnHtmlWindow HelpWindow;
    String HelpFile;
    
    RowDialogEntry(String name,RowDialogMain dialog)
    {
        Dialog=dialog;
        Name = new RowEntryLabel(name);
        Help = new RowEntryButton("Help");
        Help.addActionListener(this);
        HelpFile=null;
        HelpWindow=null;
    }
    /** Internal method */
    public void actionPerformed(ActionEvent event)
    {
       String helpfile=HelpFile;
        if(HelpFile==null)
            helpfile=Dialog.HelpFile;
        if(helpfile!=null)
        {
            HelpWindow = new GdnHtmlWindow(helpfile,"Solver Help",Dialog.GetBottomLeft(),400,200);
            HelpWindow.makeVisible(true);
        }
    }
    /** Internal method */
    public void Terminate()
    {
        if(HelpWindow!=null)
            HelpWindow.dispose();
    }
    /** Sets the name of the HTML help file.
     * Note this file is assumed to be in the same folder as the solver.
     * If no filename if specified, the HelpFile for the RowMainDialog is used*/
    public void PutHelpFile(String helpfile)
    {
        HelpFile = Dialog.Solver.CreatePathToFile(helpfile);
    }
    
    
    abstract void Build();
    abstract void PutSize(int height, int name, int info, int extra, int help);
    abstract Dimension GetExtraSize();
    abstract Dimension GetInfoSize();
}


