package  giden.CORE;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Point;

/*==============================================================
| UiNetwork class
|---------------------------------------------------------------
| Provides a Network object for a User Interface
|---------------------------------------------------------------
==============================================================*/
abstract public class UiNetwork implements java.io.Serializable
{
    Network Net;
    EdgeArray UiEdges;
    NodeArray UiNodes;
    GUINodeListIter NIT;
    GUIEdgeListIter EIT;
    
    public GUISolver GUI;
    public UserBase UserInfo;      // valid only when solver running
    
    UiNodeArray NodeId;
    UiEdgeArray EdgeId;
    UiEdgeArray EdgeLength;
    
    UiNodeEdgeIdManager IdManager; // owenj 22-Mar-1999

    public static String NodeIdString = "Node ID";
    public static String EdgeIdString = "Edge ID";
    public static String EuclideanLengthString = "Euclidean Length";
    
    //----------------------------------------------------------
    //----------------------------------------------------------
    UiNetwork()
    {
        Create(new Network());
    }
    UiNetwork(Network net)
    {
        Create(net);
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    void Create(Network net)
    {
        GUI=null;
        Net = net;
        Net.PutUiNetwork(this);
        UiEdges = new EdgeArray(100,"UiEdges");
        UiNodes = new NodeArray(100,"UiNodes");
        NIT=null;
        EIT=null;
        
        NodeId = CreateNodeArray(NodeIdString);
        EdgeId = CreateEdgeArray(EdgeIdString);
        EdgeLength = CreateEdgeArray(EuclideanLengthString);
        EdgeLength.Fill(0);

        IdManager = new UiNodeEdgeIdManager();

        int count;
        count=GetNodeArrayCount();
        count=GetEdgeArrayCount();
        UiIndexArray array;
        array = GetNodeArray(NodeIdString);
        array = GetEdgeArray(EdgeIdString);
        array = GetEdgeArray(EuclideanLengthString);
        UserInfo=null;
    }
    
    
    //----------------------------------------------------------
    //----------------------------------------------------------
    void PutGuiSolver(GUISolver gui)
    {
        GUI=gui;
    }
    
    public void DeleteAllNodes()
    {
        DeleteAllEdges();
        if (UserInfo != null)
        {
            ((UserIO)UserInfo).NodeSetIndex.Fill(null);
            IdManager.DsdClearNodes();
            Net.DeleteAllNodes();
        }
    }

    public void DeleteAllEdges()
    {
        if (UserInfo!= null)
        {
            ((UserIO)UserInfo).EdgeSetIndex.Fill(null);
            IdManager.DsdClearEdges();
            Net.DeleteAllEdges();
        }
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public boolean GetDirected()
    {
        return Net.GetDirected();
    }
    
    //----------------------------------------------------------
    //----------------------------------------------------------
    public void PutDirected(boolean directed)
    {
        Net.PutDirected(directed);
    }
    
    /*----------------------------------------------------------
    | methods called from the core giden side
    ----------------------------------------------------------*/
    //----------------------------------------------------------
    //----------------------------------------------------------
    public void UpdateNodeId(Node node)
    {
        // owenj 22-Mar-1999
        // NodeId.Put(Node2UiNode(node),node.GetIndex());
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public void UpdateEdgeId(Edge edge)
    {
        // owenj 22-Mar-1999
        // EdgeId.Put(Edge2UiEdge(edge),edge.GetIndex());
    }

    abstract public void DeleteNodeX(Node node);
    
    abstract public void DeleteEdgeX(Edge edge);
    
    /*----------------------------------------------------------
    | NETWORK MANIPULATION
    ----------------------------------------------------------*/
    //----------------------------------------------------------
    //----------------------------------------------------------
    public Network GetNet()
    {
        return Net;
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    void DeleteContents()
    {
        GetNet().DeleteContents();
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public void DeleteNodeArray(String name)
    {
        GetNet().DeleteNodeArray(name);
    }
    
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiNodeArray CreateNodeArray(String name)
    {
        NodeArray array = GetNet().CreateNodeArray(name);
        UiNodeArray uiarray = new UiNodeArray(this,array);
        array.PutUiIndexArray(uiarray);
        return uiarray;
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public int GetNodeArrayCount()
    {
        int count=0;
        String name;
        if((name=GetFirstNodeArrayName())!=null)
        {
            count++;
            while((name=GetNextNodeArrayName())!=null)
                count++;
        }
        return count;
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public int GetEdgeArrayCount()
    {
        int count=0;
        if(GetFirstEdgeArrayName()!=null)
        {
            count++;
            while(GetNextEdgeArrayName()!=null)
                count++;
        }
        return count;
    }
   
    
    //----------------------------------------------------------
    //----------------------------------------------------------
    public String GetFirstNodeArrayName()
    {
        return GetNet().GetFirstNodeArrayName();
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public String GetNextNodeArrayName()
    {
        return GetNet().GetNextNodeArrayName();
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiNodeArray GetNodeArray(String name)
    {
        UiNodeArray uiarray = null;
        NodeArray array = GetNet().GetNodeArray(name);
        if(array!=null)
            uiarray=(UiNodeArray)array.GetUiIndexArray();
        return uiarray;
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public void DeleteEdgeArray(String name)
    {
        GetNet().DeleteEdgeArray(name);
    }
    
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiEdgeArray CreateEdgeArray(String name)
    {
        EdgeArray array = GetNet().CreateEdgeArray(name);
        UiEdgeArray uiarray = new UiEdgeArray(this,array);
        array.PutUiIndexArray(uiarray);
        return uiarray;
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public String GetFirstEdgeArrayName()
    {
        return GetNet().GetFirstEdgeArrayName();
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public String GetNextEdgeArrayName()
    {
        return GetNet().GetNextEdgeArrayName();
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiEdgeArray GetEdgeArray(String name)
    {
        UiEdgeArray uiarray = null;
        EdgeArray array = GetNet().GetEdgeArray(name);
        if(array!=null)
            uiarray=(UiEdgeArray)array.GetUiIndexArray();
        return uiarray;
    }
    /*----------------------------------------------------------
    | GRAPH MANIPULATION
    ----------------------------------------------------------*/
    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    public void DeleteGraph()
    {
        Net.DeleteGraph();
    }

    /*----------------------------------------------------------
    | NODE MANIPULATION
    ----------------------------------------------------------*/
    //----------------------------------------------------------
    //----------------------------------------------------------
    public int GetNodeCount()
    {
        return Net.GetNodeCount();
    }
    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    public int GetCreatedNodesCount()
    {
        return Net.GetCreatedNodesCount();
    }
    
    /*----------------------------------------------------------
    | USED ONLY BY SYMMETRICTSP
    Node CreateNode(int x, int y, int param)
    {
        UiNode ui_node = GUI.draw_canvas.createNode(new Point(x,y),param);
        Node node = UiNode2Node(ui_node);
        if(UserInfo!=null)
            UserInfo.AddNetworkSolverNode(ui_node);
        return node;
    }
    ----------------------------------------------------------*/
    
    
    /*-----------------------------------------------------------
    | called from CORE.GdnNeworkReaderV3
    public boolean CreateNode(Node node, Point pt)
    {
        boolean result=false;
        if(node.IsMember(Net))
        {
            UiNode ui_node = GUI.draw_canvas.createNode(node,pt);
            if(UserInfo!=null)
                UserInfo.AddNetworkSolverNode(ui_node);
            result=true;
        }
        return result;
    }
    -----------------------------------------------------------*/
    
    
    /*-----------------------------------------------------------
    | called from CORE.UserIO
    -----------------------------------------------------------*/
    public boolean CreateNode(Node node, UiNodeInfo info)
    {
        boolean result=false;
        if(node.IsMember(Net))
        {
            UiNode ui_node = GUI.draw_canvas.createNode(node,info);
            if(UserInfo!=null)
                UserInfo.AddNetworkSolverNode(ui_node);
            result=true;
        }
        return result;
    }
   
    /*-----------------------------------------------------------
    | called from GUIDrawCanvas
    | - this->CreateNode(Node,UiNodeInfo)
    -----------------------------------------------------------*/
    public boolean CreateNodeX(Node node, UiNode uinode)
    {
        boolean result=false;
        if(node!=null)
        {
            result=true;
            IdManager.DsdPresetNodeId(node.GetIndex());
            uinode.PutNode(node);
            UiNodes.Put(node,uinode);
            NodeId.Put(uinode,IdManager.GenerateNodeId(uinode));
        }
        return result;
    }
    
    
    //----------------------------------------------------------
    // called from GUI.Version2Reader, maintain for now
    //----------------------------------------------------------
    public boolean CreateNodeY(UiNode uinode)
    {
        boolean result=false;
        Node node = Net.CreateNode();
        if(node!=null)
        {
            result=true;
            uinode.PutNode(node);
            UiNodes.Put(node,uinode);
            NodeId.Put(uinode,IdManager.GenerateNodeId(uinode));
        }
        return result;
    }
    //----------------------------------------------------------
    // Normal entry point to delete a node
    //----------------------------------------------------------
    public void DeleteNode(UiNode uinode)
    {
        Node node = UiNode2Node(uinode);
        if(node!=null)
            Net.DeleteNode(node);
    }
    
    //----------------------------------------------------------
    //----------------------------------------------------------
    // owenj 22-Mar-1999 (entire method, for changing identifier)
    public boolean PutNodeId(UiNode uinode, String id) {
        if (IdManager.GetNode(id) != null &&
            IdManager.GetNode(id) != uinode)
            return false; // identifier in use by another node

        String old_id = NodeId.GetObj(uinode).toString();
        IdManager.ClearNodeId(old_id);
        IdManager.PutNodeId(uinode,id);
        NodeId.Put(uinode,id);
        return true;
    }
    public UiNode GetUiNode(String id)
    {
        return IdManager.GetNode(id);
    }
    public Node GetNode(String id)
    {
        return UiNode2Node(IdManager.GetNode(id));
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    public int GetEdgeCount(UiNode uinode)
    {
        int result=0;
        Node node = UiNode2Node(uinode);
        if(node!=null)
        {
            result=node.GetEdgeCount();
        }
        return result;
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiNodeListIter GetUiNodeListIter()
    {
        return new UiNodeListIter(this);
    }
    
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiEdgeListIter GetAdjListIter(UiNode uinode)
    {
        UiEdgeListIter result=null;
        Node node = UiNode2Node(uinode);
        if(node!=null)
        {
            EdgeListIter eit = node.GetAdjListIter();
            if(eit!=null)
            {
                result = new UiEdgeListIter(this,eit);
            }
        }
        return result;
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiEdgeListIter GetInListIter(UiNode uinode)
    {
        UiEdgeListIter result=null;
        Node node = UiNode2Node(uinode);
        if(node!=null)
        {
            EdgeListIter eit = node.GetInListIter();
            if(eit!=null)
            {
                result = new UiEdgeListIter(this,eit);
            }
        }
        return result;
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiEdgeListIter GetOutListIter(UiNode uinode)
    {
        UiEdgeListIter result=null;
        Node node = UiNode2Node(uinode);
        if(node!=null)
        {
            EdgeListIter eit = node.GetOutListIter();
            if(eit!=null)
            {
                result = new UiEdgeListIter(this,eit);
            }
        }
        return result;
    }
    
    //----------------------------------------------------------
    //----------------------------------------------------------
    public int GetIndex(UiNode uinode)
    {
        Node node = UiNode2Node(uinode);
        if(node!=null)
            return node.GetIndex();
        else
            return -1;
    }
    
    
    
    /*----------------------------------------------------------
    | EDGE MANIPULATION
    ----------------------------------------------------------*/

    //----------------------------------------------------------
    //----------------------------------------------------------
    public int GetEdgeCount()
    {
        return Net.GetEdgeCount();
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public int GetCreatedEdgesCount()
    {
        return Net.GetCreatedEdgesCount();
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    /*
    public Edge CreateEdge(Node source, Node target)
    {
        // must be done by derived class
        return null;
    }
    */
    
    //----------------------------------------------------------
    // called from GUI.Version2aReader
    //----------------------------------------------------------
    public boolean CreateEdgeY(UiEdge uiedge, UiNode uisource, UiNode uitarget)
    {
        boolean result=false;
        Node source,target;
        Edge edge;

        source=uisource.GetNode();
        target=uitarget.GetNode();
        edge=Net.CreateEdge(source,target);
        if(edge!=null)
        {
            result=true;
            uiedge.PutEdge(edge);
            UiEdges.Put(edge,uiedge);
     
            // EdgeId.Put(uiedge,GetIndex(uiedge));
            EdgeId.Put(uiedge,IdManager.GenerateEdgeId(uiedge));
        }
        return result;
    }
    
    /*-----------------------------------------------------------
    | called from GUIDrawCanvas
    | - GUINetwork.CreateEdge
    | - this->CreateEdge(Edge,UiEdgeInfo)
    -----------------------------------------------------------*/
    public boolean CreateEdgeX(Edge edge, UiEdge uiedge)
    {
        boolean result=false;
        if(edge!=null)
        {
            result=true;
            IdManager.DsdPresetEdgeId(edge.GetIndex());
            uiedge.PutEdge(edge);
            UiEdges.Put(edge,uiedge);
            EdgeId.Put(uiedge,IdManager.GenerateEdgeId(uiedge));
        }
        return result;
    }

    //----------------------------------------------------------
    // internal call from UserInfo
    //----------------------------------------------------------
    public boolean CreateEdge(Edge edge, UiEdgeInfo info)
    {
        boolean result=false;
        if(edge.IsMember(Net))
        {
            UiEdge ui_edge = GUI.draw_canvas.createEdge(edge,info);
            if(UserInfo!=null)
                UserInfo.AddNetworkSolverEdge(ui_edge);
            result=true;
        }
        return result;
    }
  
    //----------------------------------------------------------
    //----------------------------------------------------------
    public void ReverseEdge(UiEdge uiedge)
    {
        Edge newedge;
        Edge oldedge = UiEdge2Edge(uiedge);
        if(oldedge!=null)
        {
            newedge=Net.ReverseEdge(oldedge);
        }
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public void DeleteEdge(UiEdge uiedge)
    {
        Edge edge = UiEdge2Edge(uiedge);
        if(edge!=null) {
            IdManager.ClearEdgeId(EdgeId.GetObj(uiedge).toString());
            Net.DeleteEdge(edge);
            //uiedge.PutEdge(null);       // dsd 20040808, added then removed
        }
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    // owenj 22-Mar-1999 (entire method, for changing identifier)
    public boolean PutEdgeId(UiEdge uiedge, String id) {
        if (IdManager.GetEdge(id) != null &&
            IdManager.GetEdge(id) != uiedge)
            return false; // identifier in use by another edge

        String old_id = EdgeId.GetObj(uiedge).toString();
        IdManager.ClearEdgeId(old_id);
        IdManager.PutEdgeId(uiedge,id);
        EdgeId.Put(uiedge,id);
        return true;
    }

    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiNode GetUiSource(UiEdge uiedge)
    {
        Edge edge = UiEdge2Edge(uiedge);
        Node source = edge.GetSource();
        return Node2UiNode(source);
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiNode GetUiTarget(UiEdge uiedge)
    {
        Edge edge = UiEdge2Edge(uiedge);
        Node target = edge.GetTarget();
        return Node2UiNode(target);
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiNode GetUiOpposite(UiEdge uiedge,UiNode uinode)
    {
        UiNode result=null;
        Edge edge = UiEdge2Edge(uiedge);
        if(edge!=null)
        {
            Node node = UiNode2Node(uinode);
            if(node!=null)
                result=Node2UiNode(edge.GetOpposite(node));
        }
        return result;
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public boolean IsSource(UiEdge uiedge,UiNode uinode)
    {
        boolean result=false;
        Edge edge = UiEdge2Edge(uiedge);
        if(edge!=null)
        {
            Node node = UiNode2Node(uinode);
            if(node!=null)
                result=edge.IsSource(node);
        }
        return result;
        
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public boolean IsTarget(UiEdge uiedge,UiNode uinode)
    {
        boolean result=false;
        Edge edge = UiEdge2Edge(uiedge);
        if(edge!=null)
        {
            Node node = UiNode2Node(uinode);
            if(node!=null)
                result=edge.IsTarget(node);
        }
        return result;
        
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public int GetIndex(UiEdge uiedge)
    {
        Edge edge = UiEdge2Edge(uiedge);
        if(edge!=null)
            return edge.GetIndex();
        else
            return -1;
    }
    
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiEdgeListIter GetUiEdgeListIter()
    {
        return new UiEdgeListIter(this);
    }
    

    /*==========================================================
    | INTERNAL UTILITIES
    ==========================================================*/
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiEdge Edge2UiEdge(Edge edge)
    {
        UiEdge uiedge=null;
        if(edge!=null)
        {
            uiedge=UiEdges.GetUiEdge(edge);
        }
        return uiedge;
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public UiNode Node2UiNode(Node node)
    {
        UiNode uinode=null;
        if(node!=null)
        {
            uinode=UiNodes.GetUiNode(node);
        }
        return uinode;
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public Node UiNode2Node(UiNode uinode)
    {
        Node node=null;
        if(uinode!=null)
        {
            node=uinode.GetNode();
        }
        return node;
    }
    //----------------------------------------------------------
    //----------------------------------------------------------
    public Edge UiEdge2Edge(UiEdge uiedge)
    {
        Edge edge=null;
        if(uiedge!=null)
        {
            edge=uiedge.GetEdge();
        }
        return edge;
    }
    
    
}
