package  giden.CORE;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.*;
import java.awt.event.*;

/**
 * UiSolverResult is a row-oriented dialog box that supports multiple
 * objects derived from UiSolverResultRow.  This object displays each
 * row, interacts with GIDEN, displays a subset of buttons defined by
 * the solver, and interacts with the user.  Solvers retrieve the
 * results of the interactions from the derived objects.
 *
 * 
 * @see UiSolverResultCheckBox
 * @see UiSolverResultChoice
 * @see UiSolverResultEdge
 * @see UiSolverResultNode
 * @see UiSolverResultRow
 * @see UiSolverResultText
 */
 

public class UiSolverResult extends Frame implements ActionListener, MouseMotionListener, MouseListener
    /*implements ActionListener,ComponentListener,FocusListener,MouseListener,KeyListener*/
{
    public static final String STRING_CANCEL="Cancel";
    public static final int BUTTON_NONE=0x0;
    public static final int BUTTON_ACCEPT=0x1;
    public static final int BUTTON_CANCEL=0x2;
    public static final int BUTTON_SAVE=0x4;
    public static final int BUTTON_BACK=0x8;
    public static final int BUTTON_INCREMENT=0x10;
    public static final int BUTTON_DECREMENT=0x20;
 
    int NameWidth;
    int InfoWidth;
    int ClickWidth;
    int HelpWidth;
    int Height;
    Button Cancel;
    Button Accept;
    Button Save;
    Button Back;
    Button Increment;
    Button Decrement;
    UiSolverResultRow Rows[];
    int RowCount;
    int RowMax;
    Panel ButtonPanel;
    UserBase UserInfo;
    UiSolverResultNode NodeClickResult;
    String NodeMessage;
    UiSolverResultEdge EdgeClickResult;
    String EdgeMessage;
    
    int ResultCode;
    int ButtonMode;
    Panel SolverPanel;
    
    public UiSolverResult(String name, UserBase user)
    {
        super(name);
        UserInfo=user;
        RowMax=10;
        Rows = new UiSolverResultRow[RowMax];
        RowCount=0;
        
        NameWidth=0;
        InfoWidth=0;
        ClickWidth=0;
        HelpWidth=0;
        Height=0;
        
        ResultCode=BUTTON_NONE;
        ButtonMode=0;
        AddButtons(BUTTON_CANCEL|BUTTON_ACCEPT);
                
        Cancel = new Button(STRING_CANCEL);
        Accept = new Button("Accept");
        Save = new Button("Save");
        Back = new Button("Back");
        Increment = new Button(" > ");
        Decrement = new Button(" < ");
        Cancel.addActionListener(this);
        Accept.addActionListener(this);
        Save.addActionListener(this);
        Back.addActionListener(this);
        Increment.addActionListener(this);
        Decrement.addActionListener(this);

        
        SolverPanel = new Panel();
        add(SolverPanel);
        SolverPanel.addMouseMotionListener(this);
        SolverPanel.addMouseListener(this);
        
        ButtonPanel = new Panel();
        ButtonPanel.addMouseListener(this);
        
        setLocation(user.GetTopRight());
        /*
        Rectangle bounds = user.GetGUI().getBounds();
        int x = bounds.x+bounds.width;
        int y = bounds.y;
        setLocation(x,y);
        */
        
        
        //addMouseMotionListener(this);
        //addMouseListener(this);
    }
    public UiSolverResult(String name, UserBase user, int buttons)
    {
        super(name);
        UserInfo=user;
        RowMax=10;
        Rows = new UiSolverResultRow[RowMax];
        RowCount=0;
        
        NameWidth=0;
        InfoWidth=0;
        ClickWidth=0;
        HelpWidth=0;
        Height=0;
        
        ResultCode=BUTTON_NONE;
        ButtonMode=0;
        AddButtons(buttons);
                
        Cancel = new Button("Cancel");
        Accept = new Button("Accept");
        Save = new Button("Save");
        Back = new Button("Back");
        Increment = new Button(" > ");
        Decrement = new Button(" < ");

        Cancel.addActionListener(this);
        Accept.addActionListener(this);
        Save.addActionListener(this);
        Back.addActionListener(this);
        Increment.addActionListener(this);
        Decrement.addActionListener(this);
        
        SolverPanel = new Panel();
        add(SolverPanel);
        SolverPanel.addMouseMotionListener(this);
        SolverPanel.addMouseListener(this);
        
        ButtonPanel = new Panel();
        ButtonPanel.addMouseListener(this);
        
        setLocation(user.GetTopRight());
        
        
        //addMouseMotionListener(this);
        //addMouseListener(this);
    }
    
    public void AddRow(UiSolverResultRow row)
    {
        row.Build();
        Rows[RowCount++]=row;
    }
    public void AddButtons(int buttons)
    {
        ButtonMode|=buttons;
    }
    public void RemoveButtons(int buttons)
    {
        ButtonMode&=~buttons;
    }
    public Point GetBottomLeft()
    {
        Rectangle bounds = getBounds();
        Point point = new Point();
        point.x = bounds.x;
        point.y = bounds.y+bounds.height;
        return point;
    }
    
    public int Interact()
    {
        boolean loop;
        String event;
                 
        SolverPanel.setBackground(Color.lightGray);
        SolverPanel.setLayout(new GridLayout(RowCount+1, 1));
        
        AddComponents();
        pack();
        FixSizes();
        SolverPanel.removeAll();
        pack();
        AddComponents();
        show();
        
        loop=true;
        while(loop)
        {
            loop=false;
            UserInfo.Services.DrawNetworks();
            event=UserInfo.Services.EventWait("Simple",0);
            if(event.equals("Node"))
            {
                Node node =UserInfo.UserMouseNodeInput(NodeMessage);
                NodeClickResult.PutNodeClick(node);
                loop=true;
            }
            if(event.equals("Edge"))
            {
                Edge edge =UserInfo.UserMouseEdgeInput(EdgeMessage);
                EdgeClickResult.PutEdgeClick(edge);
                loop=true;
            }
            if(event.equals("Null Edge Click"))
                loop=true;
            if(event.equals("Null Node Click"))
                loop=true;
        }
        int i;
        for(i=0;i<RowCount;i++)
            Rows[i].Terminate();
        setVisible(false);
        
        return ResultCode;
    }
    void AddComponents()
    {
        ButtonPanel.removeAll();
        if((ButtonMode&BUTTON_BACK)!=0)
            ButtonPanel.add(Back);
        if((ButtonMode&BUTTON_CANCEL)!=0)
            ButtonPanel.add(Cancel);
        if((ButtonMode&BUTTON_DECREMENT)!=0)
            ButtonPanel.add(Decrement);
        if((ButtonMode&BUTTON_INCREMENT)!=0)
            ButtonPanel.add(Increment);
        if((ButtonMode&BUTTON_ACCEPT)!=0)
            ButtonPanel.add(Accept);
        if((ButtonMode&BUTTON_SAVE)!=0)
            ButtonPanel.add(Save);
        
        int i;
        for(i=0;i<RowCount;i++)
            SolverPanel.add(Rows[i]);
        SolverPanel.add(ButtonPanel);
        pack();
        
    }
    void FixSizes()
    {
        int i;
        for(i=0;i<RowCount;i++)
            GetMaxSizes(Rows[i]);
        for(i=0;i<RowCount;i++)
            Rows[i].PutSize(Height,NameWidth,InfoWidth,ClickWidth,HelpWidth);
    }
    void GetMaxSizes(UiSolverResultRow row)
    {
        Dimension d;
        
        d=row.Name.getSize();
        if(d.width>NameWidth)
            NameWidth=d.width;
        if(d.height>Height)
            Height=d.height;
        
        d=row.GetInfoSize();
        if(d.width>InfoWidth)
            InfoWidth=d.width;
        if(d.height>Height)
            Height=d.height;
            
        d=row.GetExtraSize();
        if(d.width>ClickWidth)
            ClickWidth=d.width;
        if(d.height>Height)
            Height=d.height;
            
        d=row.Help.getSize();
        if(d.width>HelpWidth)
            HelpWidth=d.width;
        if(d.height>Height)
            Height=d.height;
            
    }
    public void actionPerformed(ActionEvent event)
    {
        boolean result;
        Object obj=event.getSource();
        if(obj==Cancel)
            ResultCode=BUTTON_CANCEL;
        if(obj==Accept)
            ResultCode=BUTTON_ACCEPT;
        if(obj==Back)
            ResultCode=BUTTON_BACK;
        if(obj==Decrement)
            ResultCode=BUTTON_DECREMENT;
        if(obj==Increment)
            ResultCode=BUTTON_INCREMENT;
        
        UserInfo.Services.EventNotify("Dialog");
    }
    void UserMouseNodeInput(String message,UiSolverResultNode who)
    {
        NodeClickResult=who;
        NodeMessage=message;
        UserInfo.Services.EventNotify("Node");
    }
    void UserMouseEdgeInput(String message,UiSolverResultEdge who)
    {
        EdgeClickResult=who;
        EdgeMessage=message;
        UserInfo.Services.EventNotify("Edge");
    }
    
    public void mouseDragged(MouseEvent event)
    {
        test();
    }
    public void mouseMoved(MouseEvent event)
    {
        test();
    }
    public void mouseClicked(MouseEvent event)
    {
        test();
    }
    public void mouseEntered(MouseEvent event)
    {
        test();
        requestFocus();
    }
    public void mouseExited(MouseEvent event)
    {
        test();
    }
    public void mousePressed(MouseEvent event)
    {
        test();
    }
    public void mouseReleased(MouseEvent event)
    {
        test();
    }
    int test()
    {
        int i=0;
        return i;
    }
}

