package giden.GRDS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * Edge is a class that is tightly coupled to the Graph class
 * and provides methods for interacting with edge objects.
 * An edge has a source and target node, and the unique edge index.
 * The index is an integer that is
 * assigned to the edge when it is created.  Methods
 * are provided to access the source and target nodes
 * and the index.  Note the index is accessed through
 * the Index base class using the GetIndex method.
 * Although edges are created from the Graph class
 * their source and target nodes and indexes are accessed through the
 * edge methods.
 * <p>
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @see EdgeArray
 * @see Index
 * @see Network
 * @see Node
 * @see Graph
 *
 * @version 	4.0a 2004 08 27
 * @author 	    D. S. Dilworth
 */



/*===============================================================
===============================================================*/
public class Edge extends Index
{
    private Node Source;
    private Node Target;
    private ListEntry NetEntry;
    private ListEntry SourceEntry;
    private ListEntry TargetEntry;
    private Graph Owner;

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    Edge(Graph graph,int index)
    {
        super(index);
        Owner=graph;
    }
    
    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    /** internal method */
    public boolean IsMember(Graph graph)
    {
        if(graph==Owner)
            return true;
        else
            return false;
    }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    void PutEntry(ListEntry entry)
    {   NetEntry=entry;     }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    ListEntry GetEntry()
    {   return NetEntry;    }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    void PutSource(Node node)
    {   Source = node;      }

    //-----------------------------------------------------------
    /** Gets the source node of this Edge
     * @return The source Node of this Edge */
    //-----------------------------------------------------------
    public Node GetSource()
    {   return Source;      }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    void PutTarget(Node node)
    {   Target = node;      }

    //-----------------------------------------------------------
    /** Gets the target node of this Edge
     * @return The target Node of this Edge */
    //-----------------------------------------------------------
    public Node GetTarget()
    {   return Target;      }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    void PutSourceEntry(ListEntry entry)
    {   SourceEntry = entry;        }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    ListEntry GetSourceEntry()
    {   return SourceEntry;     }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    void PutTargetEntry(ListEntry entry)
    {   TargetEntry = entry;    }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    ListEntry GetTargetEntry()
    {   return TargetEntry;     }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    void DetachNodes()
    {
        Node source,target;
        ListEntry entry;

        source=GetSource();
        if(source!=null)
        {
            entry=SourceEntry;
            if(entry!=null)
            {
                source.DeleteEdge(entry);
                SourceEntry=null;
            }
        }
        PutSource(null);

        target=GetTarget();
        if(target!=null)
        {
            entry=TargetEntry;
            if(entry!=null)
            {
                target.DeleteEdge(entry);
                TargetEntry=null;
            }
        }
        PutTarget(null);
    }

    //-----------------------------------------------------------
    /** Gets the Node incident to this Edge that is not the given
     * Node
     * @return The Node incident to this Edge that is not the
     * given Node */
    //-----------------------------------------------------------
    public Node GetOpposite(Node node)
    {
        if(IsTarget(node))
            return Source;
        if(IsSource(node))
            return Target;
        // error
        return null;
    }

    //-----------------------------------------------------------
    /** Tests if the given node is the source Node of this Edge
     * @return True if the given Node is the source Node of this Edge */
    //-----------------------------------------------------------
    public boolean IsSource(Node node)
    {
        return node==Source;
    }

    //-----------------------------------------------------------
    /** Tests if the given node is the target Node of this Edge
     * @return True if the given Node is the target Node of this Edge */
    //-----------------------------------------------------------
    public boolean IsTarget(Node node)
    {
        return node==Target;
    }
}

