package  giden.GRDS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * Node is a class that is tightly coupled to the Graph class
 * and provides methods for interacting with Node objects.
 * A Node has a list of incident edges (called
 * the "adjacency list") and a unique node Index.
 * The incident edges can be further partitioned into incoming
 * and outgoing edges.  The Index is an integer that is
 * assigned to the Node when it is created.  Methods
 * are provided to access the incident edges (all, incoming, or outgoing),
 * through iterators, and the Index.
 * Note the Index is accessed through
 * the Index base class using the GetIndex method.
 * Although Nodes (and Edges) are created from the Graph class,
 * their Edges and Indexes are accessed through the Node methods.
 * <p>
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @see Edge
 * @see EdgeListIter
 * @see Graph
 * @see InListIter
 * @see Index
 * @see Network
 * @see Node
 * @see NodeArray
 * @see OutListIter

 *
 * @version 	4.0a 2004 08 27
 * @author 	    D. S. Dilworth
 */



/*===============================================================
===============================================================*/
public class Node extends Index
{
    private LinkList AdjList;
    private ListEntry NetEntry;
    private Graph Owner;

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    Node(Graph graph,int index)
    {
        super(index);
        Owner = graph;
        AdjList = new SingleLinkList();
    }
    /** Internal method */
    public boolean IsMember(Graph graph)
    {
        if(graph==Owner)
            return true;
        else
            return false;
    }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    void PutEntry(ListEntry entry)
    {   NetEntry=entry;     }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    ListEntry GetEntry()
    {   return NetEntry;    }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    ListEntry PutEdge(Edge edge)
    {   return AdjList.AddItem(edge);       }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    void DeleteEdge(ListEntry entry)
    {   AdjList.DeleteEntry(entry); }

    //-----------------------------------------------------------
    /** Gets the count of adjacent edges for this node.  The
     * number of incoming and outgoing is not available without
     * sequencing through incoming or outgoing edge list.
     * @returns The number of edges in adjacent edge list */
    //-----------------------------------------------------------
    public int GetEdgeCount()
    {
        return AdjList.GetSize();
    }

    /*-----------------------------------------------------------
    public int GetIncEdgeCount()
    {
        return -1;                      // !! not implemented
    }
    -----------------------------------------------------------*/


    /*-----------------------------------------------------------
    public int GetOutEdgeCount()
    {
        return -1;                      // !! not implemented
    }
    -----------------------------------------------------------*/

    //-----------------------------------------------------------
    /** Gets the adjacency list iterator of this Node
     * @return An EdgeListIter of this nodes adjacent Edges */
    //-----------------------------------------------------------
    public EdgeListIter GetAdjListIter()
    {
        return new EdgeListIter(AdjList);
    }

    //-----------------------------------------------------------
    /** Gets the incoming edge list iterator of this Node
     * @return An incoming EdgeListIter of this nodes adjacent Edges */
    //-----------------------------------------------------------
    public EdgeListIter GetInListIter()
    {
        return new InListIter(this,AdjList);
    }

    //-----------------------------------------------------------
    /** Gets the outgoing edge list of this Node
     * @return An outgoing EdgeListIter of this Nodes adjacent Edges */
    //-----------------------------------------------------------
    public EdgeListIter GetOutListIter()
    {
        return new OutListIter(this,AdjList);
    }

    /*-----------------------------------------------------------
    -----------------------------------------------------------*/
    /** Internal method */
    public LinkList GetAdjList()
    {   return AdjList; }

}

