package  giden.GRDS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * PriorityQueue (PQ) is a class that stores objects with
 * associated integer values.  Objects can be retrieved
 * from the PQ sorted by the integer value (smallest
 * to largest).  The integer value associated with an
 * object can be decreased.
 * <p>
 * PriorityQueue is an abstract base class, and hence can not
 * be directly created.  Currently GIDEN only supports the PQList
 * priority queue that is derived from the PriorityQueue class.
 * Other types of PQ classes (e.g. PQHeap) may be developed
 * in the future and they will use a similar (if not identical)
 * set of interface methods and parameters.
 * <p>
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @see Network
 * @see Queue
 * @see BoundedQueue
 * @see LinkList
 * @see PQList
 *
 * @version 	4.0a 2004 08 27
 * @author 	    D. S. Dilworth
 */





/*===============================================================
===============================================================*/
abstract public class PriorityQueue
{
    LinkList List;
    //-----------------------------------------------------------
    /** Creates a Priority Queue (PQ) */
    //-----------------------------------------------------------
    public PriorityQueue()
    {
        List = new LinkList();
    }
    //-----------------------------------------------------------
    /** Inserts the object with its value into this PQ
     * @param item  The item to insert
     * @param value The value to assign to the item*/
    //-----------------------------------------------------------
    public void Insert(Object item, int value)
    {
        PQE pqe = new PQE(item,value);
        List.AddItem(pqe);
    }
    //-----------------------------------------------------------
    /** Gets the number of elements in this PQ
     * @return The size of this PQ */
    //-----------------------------------------------------------
    public int GetSize()
    {
        return List.GetSize();
    }
    //-----------------------------------------------------------
    /** Tests if this PQ is empty
     * @return True if the PQ is empty, else false
     * @see PQList#NotEmpty */
    //-----------------------------------------------------------
    public boolean IsEmpty()
    {
        return GetSize()==0;
    }
    //-----------------------------------------------------------
    /** Test if the PQ is not empty
     * @return True if the PQ is not empty, else false
     * @see PQList#IsEmpty */
    //-----------------------------------------------------------
    public boolean NotEmpty()
    {
        return GetSize()!=0;
    }
    //-----------------------------------------------------------
    /** Gets the node with the smallest value
     * @return The node with the smallest value.  This method
     *         does not delete the node
     * @see PQList#DeleteMinNode*/
     //-----------------------------------------------------------
    public Node GetMinNode()
    {
        return (Node) GetMinObject();
    }
    //-----------------------------------------------------------
    /** Deletes the node with the smallest value
     * @return The node with the smallest value.  This method
               deletes the node.
     * @see PQList#GetMinNode*/
    //-----------------------------------------------------------
    public Node DeleteMinNode()
    {
        return (Node) DeleteMinObject();
    }
    //-----------------------------------------------------------
    /** Gets the edge with the smallest value
     * @return The edge with the smallest value.  This method
     *         does not delete the edge
     * @see PQList#DeleteMinEdge*/
     //-----------------------------------------------------------
    public Edge GetMinEdge()
    {
        return (Edge) GetMinObject();
    }
    //-----------------------------------------------------------
    /** Deletes the edge with the smallest value
     * @return The edge with the smallest value.  This method
     *         deletes the edge.
     * @see PQList#GetMinEdge*/
    //-----------------------------------------------------------
    public Edge DeleteMinEdge()
    {
        return (Edge) DeleteMinObject();
    }
    //-----------------------------------------------------------
    //-----------------------------------------------------------
    Object DeleteMinObject()
    {
        PQE pqe;
        Object item;

        ListEntry minentry=FindMinEntry();
        if(minentry!=null)
        {
            pqe = (PQE)(minentry.GetItem());
            item= pqe.GetItem();
            List.DeleteEntry(minentry);
        }
        else
            item=null;
        return item;
    }
    //-----------------------------------------------------------
    //-----------------------------------------------------------
    Object GetMinObject()
    {
        PQE pqe;
        Object item;

        ListEntry minentry=FindMinEntry();
        if(minentry!=null)
        {
            pqe = (PQE)(minentry.GetItem());
            item= pqe.GetItem();
        }
        else
            item=null;
        return item;
    }
    //-----------------------------------------------------------
    /** Test if the object is a member of this PQ
     * @param object  The object to test
     * @return True if the object is a member of the PQ */
    //-----------------------------------------------------------
    public boolean IsMember(Object object)
    {
        ListEntry entry;
        PQE pqe;

        for(entry=List.GetFirstEntry();entry!=null;entry=List.GetNextEntry(entry))
        {
            pqe = (PQE)(entry.GetItem());
            if(pqe.GetItem()==object)
                return true;
        }
        return false;
    }
    //-----------------------------------------------------------
    //-----------------------------------------------------------
    ListEntry FindMinEntry()
    {
        int minval;
        ListEntry minentry;
        ListEntry entry;
        PQE pqe;
        Object obj,item;

        minentry=List.GetFirstEntry();
        pqe = (PQE)(minentry.GetItem());
        minval=pqe.GetValue();

        for(entry=List.GetFirstEntry();entry!=null;entry=List.GetNextEntry(entry))
        {
            pqe = (PQE)(entry.GetItem());

            if(pqe.GetValue()<minval)
            {
                minentry=entry;
                minval=pqe.GetValue();
            }
        }
        return minentry;
    }
    //-----------------------------------------------------------
    /** Decreases the value of the item
     * @param item   The item whose value is to be changed
     * @param value  The new value for the item */
    //-----------------------------------------------------------
    public void DecreaseValue(Object item, int value)
    {
        PQE pqe;
        Object obj;

        ListIter lit = new ListIter(List);
        for(obj=lit.GetFirstItem();obj!=null;obj=lit.GetNextItem())
        {
            pqe = (PQE) obj;
            if(pqe.GetItem()==item)
            {
                pqe.PutValue(value);
                break;
            }
        }
    }
}
