package giden.GUI;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.*;
import java.awt.event.*;

public class GUIAnimationControl extends Panel {
    private GUINetworkWindow parent;
    private GUISlider slider;
    private Button exit_button;
    private Button go_button;

    private int INFINITY = -1;

    GUIAnimationControl(GUINetworkWindow parent) {
        super(new FlowLayout());
        this.parent = parent;

        exit_button = new Button(GUINetworkWindow.EXIT);
        exit_button.addActionListener(parent);
        this.add(exit_button);

        go_button = new GoButton(GUINetworkWindow.TRACE);
        go_button.addActionListener(parent);
        this.add(go_button);

        slider = new GUISlider(this);
        slider.setValues(value,0,100);
        this.add(slider);
        this.doLayout();
    }

    public static final int UNDEFINIED_MODE = -1;
    public static final int FINAL_MODE      =  0;
    public static final int CONTINUOUS_MODE =  1;
    public static final int STOP_MODE       =  2;

    private boolean details = false;

    public void setAnimationDetail(boolean details) {
        this.details = details;
        updateMode();
    }

    private boolean enabled = true;
    public void setEnabled(boolean b) {
            this.enabled = b;

            slider.setLocked(!b);
            if (slider.isShowing())
                UTILITIES.Repaint(slider," -- GUIAnimationControl::setEnabled");
            if (!go_button.getLabel().equals(parent.GO_CONTINUOUS))
                go_button.setEnabled(b);
            else if (parent.getActionMode().equals(parent.PC_NEXT))
                go_button.setEnabled(b);

            if (enabled)
                go_button.requestFocus();

            updateMode();
    }

    public void executeReset() {
        // a kludge to keep "value == INFINITY" 
        setEnabled(true);
        value = INFINITY;
    }

    public void enableReset() {
        this.setEnabled(false);

        value = INFINITY;
        go_button.setLabel(parent.RESET_SOLVER);
        parent.setAnimationMode(parent.RESET_SOLVER);
        go_button.setEnabled(true);
        go_button.requestFocus();
    }


    // to be called *only* from the "setValues()" method of GUISlider;
    // otherwise we have a problem with the "go" button label
    // (especially in continuous mode)
    public void updateMode() {updateMode(slider.getMode());}
    public void updateMode(int mode) {
        //        int mode = slider.getMode();

        switch (mode) {
        case FINAL_MODE:
            value = 0;
            go_button.setLabel(parent.FINAL);
            parent.setAnimationMode(parent.FINAL);
            break;
        case STOP_MODE:
            value = INFINITY;
            if (details) {
                go_button.setLabel(parent.STEP);
                parent.setAnimationMode(parent.STEP);
            }
            else {
                go_button.setLabel(parent.TRACE);
                parent.setAnimationMode(parent.TRACE);
            }
            break;
        case CONTINUOUS_MODE:
            value = slider.getValue();
            if (enabled) {
                go_button.setLabel(parent.GO_CONTINUOUS);
                parent.setAnimationMode(parent.GO_CONTINUOUS);
            }
            else {
                go_button.setLabel(parent.PAUSE_CONTINUOUS);
                parent.setAnimationMode(parent.PAUSE_CONTINUOUS);
            }
            break;
        }        
    }

    public void setMode(int mode) {
        switch (mode) {
        case FINAL_MODE:
            slider.setValue(0);
            break;
        case STOP_MODE:
            slider.setValue(100); // max value
            break;
        case CONTINUOUS_MODE:
            slider.setValue(50);  // reasonable choice?
        }
    }


    private int value = INFINITY;
    public long getAnimationTimeStep() {
        if (value != INFINITY)
            return (long) value*50;
        else
            return (long) INFINITY;
    }

    class GoButton extends Button {
        private boolean initialized = false;
        private int width;

        GoButton(String s) {super(s);}

        public Dimension getPreferredSize() {
            if (initialized) 
                return new Dimension(width,super.getPreferredSize().height);

            // if not initialized, try to initialize
            Font bf = getFont();
            if (bf == null)
                return super.getPreferredSize();

            String label = getLabel();
            FontMetrics bfm = getFontMetrics(bf);
            if (getSize().width < bfm.stringWidth(label))
                return super.getPreferredSize();

            int w = bfm.stringWidth(GUINetworkWindow.TRACE);
            w = Math.max(w,bfm.stringWidth(GUINetworkWindow.FINAL));
            w = Math.max(w,bfm.stringWidth(GUINetworkWindow.STEP));
            w = Math.max(w,bfm.stringWidth(GUINetworkWindow.GO_CONTINUOUS));
            w = Math.max(w,bfm.stringWidth(GUINetworkWindow.PAUSE_CONTINUOUS));
            w = Math.max(w,bfm.stringWidth(GUINetworkWindow.RESET_SOLVER));

            width = getSize().width + (w-bfm.stringWidth(label));
            initialized = true;
            return new Dimension(width,super.getPreferredSize().height);
        }
    }
}
