package giden.GUI;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.text.SimpleDateFormat;
import java.net.URL;

public class GUIController extends Frame implements ActionListener,
                                             ComponentListener, 
                                             ItemListener,
                                             WindowListener 
{
    GdnHtmlWindow HelpWindow;   // dsd 020228
    
    private NetworkWindowNameManager nwnm;

    public static final int UNKNOWN_TOOL         = -1;
    public static final int TRAIL_EDGE_TOOL      =  0;
    public static final int STAR_EDGE_TOOL       =  1;
    public static final int NEW_NODE_TOOL        =  2;
    public static final int NEW_EDGE_TOOL        =  3;
    public static final int REVERSE_EDGE_TOOL    =  4;
    public static final int EDIT_VALUES_TOOL     =  5;
    public static final int SELECT_ITEM_TOOL     =  6;
    public static final int MOVE_SELECTED_TOOL   =  7;
    public static final int DELETE_ITEM_TOOL     =  8;
    
    private static final String toolnames[] = {"Trail Edge",
                                               "Star Edge",
                                               "New Node",
                                               "New Edge",
                                               "Reverse Edge",
                                               "Edit Values",
                                               "Select Item",
                                               "Move Item",
                                               "Delete Item"};
    
    
    private static final String filenames[] = {"Spanning Tree 1",
                                               "Spanning Tree 2",
                                               "Shortest Path 1",
                                               "Shortest Path 2",
                                               "Maximum Flow 1",
                                               "Maximum Flow 2",
                                               "Minimum-Cost Flow 1",
                                               "Minimum-Cost Flow 2",
                                               "Minimum-Cost Flow 2a",
                                               "User Guide Example"};
    
    private GUIHelpInfoDialog help = null;
    
    private GridLayout gl;
    
    private int network_count = 0;
    // dsd  private GUINetworkWindow networks[] = new GUINetworkWindow[0];
    private GUINetworkWindow networks[] = new GUISolver[0];
    
    private int current_tool = UNKNOWN_TOOL;
    private CheckboxMenuItem currentMenuTool;
    private CheckboxMenuItem showToolIcons;
    private GUIIconButton current_controller_button;
    private GUIIconButton controller_buttons[];
    private CheckboxMenuItem controller_menuitems[];
    private Choice controller_choice;
    
    /***********************************************************************/
    // constructor method
    /***********************************************************************/
    GUIController() {
        super("GIDEN");

        HelpWindow = null;
        /* (OWENJ-SPLASH SCREEN)
         */
        Date start = new Date();
        GUIAboutGIDEN splash = new GUIAboutGIDEN(this,false);


        addWindowListener(this);
        
        nwnm = new NetworkWindowNameManager();
        
        // setup choice menu for tool selection
        controller_choice = new Choice();
        for (int i=0 ; i < toolnames.length ; i++) 
            controller_choice.add(toolnames[i]);
        controller_choice.addItemListener(this);
        
        // setup menus
        this.setMenuBar(setupGUIControllerMenuBar());        
        
        // set initial layout layout
        gl = new GridLayout(1,1,0,0);
        setLayout(gl);
        
        // set background and icon image
        setBackground(Color.lightGray);
        // owenj-jar(2)        setIconImage(getToolkit().getImage("icons/giden.gif"));
        URL url = getClass().getResource("icons/giden.gif");
        setIconImage(getToolkit().getImage(url));
        
        Image icons[] = new Image[2*toolnames.length];
        MediaTracker tracker = new MediaTracker(this);
        for (int i=0 ; i < toolnames.length ; i++) {
            // owenj-jar(2)            icons[i] = getToolkit().getImage("icons/tools/active/"+toolnames[i]+".gif");
            url = getClass().getResource("icons/tools/active/"+toolnames[i]+".gif");
            icons[i] = getToolkit().getImage(url);
            tracker.addImage(icons[i],i);
            
            // owenj-jar(2)            icons[i+toolnames.length] = getToolkit().getImage("icons/tools/inactive/"+toolnames[i]+".gif");
            url = getClass().getResource("icons/tools/inactive/"+toolnames[i]+".gif");
            icons[i+toolnames.length] = getToolkit().getImage(url);
            tracker.addImage(icons[i+toolnames.length],i+toolnames.length);
        }
        try {
            tracker.waitForAll();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
        
        controller_buttons = new GUIIconButton[toolnames.length];
        for (int i=0 ; i < toolnames.length ; i++) {
            controller_buttons[i] = new GUIIconButton(icons[i],
                                                      icons[i+toolnames.length]);
            add(controller_buttons[i]);
        }
        
        //    current_tool = NEW_EDGE_TOOL;
        current_tool = TRAIL_EDGE_TOOL;
        controller_buttons[current_tool].select();
        currentMenuTool = controller_menuitems[current_tool];
        currentMenuTool.setState(true);
        controller_choice.select(current_tool);
        
        // OWENJ 05-Feb-1999
        current_controller_button = controller_buttons[current_tool];
        
        gl.setColumns(3);
        gl.setRows(3);
        
        
        addComponentListener(this);
        
        pack();
        /* (OWENJ-SPLASH SCREEN)
         */
        Date current = new Date();
        try {
            // make sure about info is displayed for at least five seconds
            Thread.sleep(Math.max((long)0, 3000-(current.getTime()-start.getTime())));
            // dsd changed 5000 to 1000
        } catch(InterruptedException e){}
        splash.close();


        setLocation(20,20);
        //        setVisible(true);
        show();
    }
    
    /***********************************************************************/
    // general use public methods
    /***********************************************************************/
    public int getCurrentTool() {return current_tool;}
    
    synchronized public void disposeGUIHelpInfoDialog() {
        if (help != null) {
            help.dispose();
            this.help = null;
        }
    }
    
    /***********************************************************************/
    // communication with GUINetworkWindow objects
    /***********************************************************************/
    public void registerFileameChange(GUINetworkWindow net,
                                      MenuItem mi,
                                      String oldfilename,
                                      String filename) {
        // remove nets menu item
        nets_menu.remove(mi);
        
        // update filename records
        nwnm.unregisterFilename(oldfilename);
        nwnm.registerFilename(filename);
        String netname = nwnm.getLatestNetname(filename);
        String windowname = nwnm.getLatestWindowname(filename);
        
        MenuItem new_mi = new MenuItem(netname);
        insertItemIntoNetsMenu(new_mi);
        
        // update info in network window
        net.updateNameRelatedInfo(netname,windowname,new_mi);
    }
    
    public void closeGUINetworkWindow(GUINetworkWindow nw,
                                      MenuItem mi,
                                      String filename) {
        for (int i = 0 ; i < network_count ; i++)
            if (nw.equals(networks[i])) {
                networks[i] = null;
                break;
            }
        
        // remove nets menu item
        nets_menu.remove(mi);
        if (nets_menu.getItemCount() == 0) {
            print_mi.setEnabled(false);
            nets_menu.setEnabled(false);
        }
        
        // update filename records
        nwnm.unregisterFilename(filename);
    }
    
    /***********************************************************************/
    // communication with GUIIconButton objects
    /***********************************************************************/
    public void selectGUIIconButton(GUIIconButton selected_button) {
        for (int i=0 ; i < controller_buttons.length ; i++)
            if (controller_buttons[i].equals(selected_button)) 
                selectTool(i);
    }
    
    /***********************************************************************/
    // generic private methods
    /***********************************************************************/
    private GUINetworkWindow[] resizeGUINetworkWindowArray(GUINetworkWindow nets[]) {
        GUINetworkWindow new_nets[] = new GUINetworkWindow[nets.length+10];
        for (int i=0 ; i < nets.length ; i++)
            new_nets[i] = nets[i];
        
        return new_nets;
    }
    
    private void insertItemIntoNetsMenu(MenuItem mi) {
        String label = mi.getLabel().toLowerCase();
        String current;
        
        // insert in sorted (unicode) order...
        int i;
        for (i=0 ; i < nets_menu.getItemCount() ; i++) {
            current = nets_menu.getItem(i).getLabel().toLowerCase();
            if (current.compareTo(label) > 0)
                break;
        }
        nets_menu.insert(mi,i);
    }
    
    private void selectTool(int tool_index) {
        if (tool_index == current_tool) return;

        // stop using the old tool ... 
        if (controller_buttons[current_tool].isSelected())
            controller_buttons[current_tool].deselect();
        currentMenuTool.setState(false);
        
        
        // ... and get ready to use the new tool
        current_tool = tool_index;
        controller_buttons[current_tool].select();
        currentMenuTool = controller_menuitems[current_tool];
        currentMenuTool.setState(true);
        controller_choice.select(current_tool);
        
        
        // OWENJ 05-Feb-1999
        // if not showing all icons, we need to update current icon displayed
        if (!showToolIcons.getState()) { 
            //      remove(current_controller_button);
            current_controller_button = controller_buttons[current_tool];
            
            removeAll();
            add(current_controller_button);
            add(controller_choice);
            /*
              add(current_controller_button,BorderLayout.WEST);
            */
            pack(); 
            //            UTILITIES.Repaint(this,"selectTool");
        }
        else
            current_controller_button = controller_buttons[current_tool];
        
        // update all of the existing networks 
        for (int i=0 ; i < network_count ; i++)
            if (networks[i] != null)
                networks[i].updateTool(current_tool);
    }
    
    
    /***********************************************************************/
    // clipboard data and methods
    /***********************************************************************/
    private boolean clipboard_empty = true;
    private GUINodeInfo[] clipboard_node_info = new GUINodeInfo[0];
    private GUIEdgeInfo[] clipboard_edge_info = new GUIEdgeInfo[0];
    
    public boolean isClipboardEmpty() {return clipboard_empty;}
    public GUINodeInfo[] getClipboardGUINodeInfo() {return clipboard_node_info;}
    public GUIEdgeInfo[] getClipboardGUIEdgeInfo() {return clipboard_edge_info;}
    public void setClipboardItems(GUINodeInfo[] node_info, int nodecount,
                                  GUIEdgeInfo[] edge_info, int edgecount) {
        // allocate arrays and copy node and edge info
        clipboard_node_info = new GUINodeInfo[nodecount];
        for (int i=0 ; i < nodecount ; i++)
            clipboard_node_info[i] = node_info[i];
        
        clipboard_edge_info = new GUIEdgeInfo[edgecount];
        for (int i=0 ; i < edgecount ; i++)
            clipboard_edge_info[i] = edge_info[i];
        
        // enable "paste" edit menuitem for each network window
        enableEditModeNetworkMenu("Paste");
        
        clipboard_empty = false;
    }
    
    public void enableEditModeNetworkMenu(String name) {
        for (int i = 0 ; i < network_count ; i++)
            if (networks[i] != null)
                networks[i].enableEditModeMenu(name);
    }
    public void disableEditModeNetworkMenu(String name) {
        for (int i = 0 ; i < network_count ; i++)
            if (networks[i] != null)
                networks[i].enableEditModeMenu(name);
    }
    
    
    /***********************************************************************/
    // menu bar data and methods
    /***********************************************************************/
    private Menu file_menu;
    private Menu file_open_menu;
    private Menu nets_menu;
    private Menu help_menu;
    private MenuItem print_mi;
    
    public static final String GIDEN_NEW_NETWORK        = "New";
    public static final String GIDEN_OPEN_NETWORK       = "Open...";
    public static final String GIDEN_PRINT_NETWORKS     = "Print Networks...";
    public static final String GIDEN_QUIT               = "Quit";
    public static final String GIDEN_HELP               = "Help...";
    public static final String GIDEN_ABOUT              = "About GIDEN...";
    public static final String GIDEN_SYSTEM_INFORMATION 
        = "System Information...";
    //    public static final String GIDEN_REGISTRATION_INFORMATION 
    //        = "Registration Information...";
    
    /***********************************************************************/
    private MenuBar setupGUIControllerMenuBar() {
        MenuBar mb = new MenuBar();
        
        file_menu = buildGUIControllerFileMenu();
        mb.add(file_menu);
        
        Menu tools_menu = buildGUIControllerToolsMenu();
        mb.add(tools_menu);
        
        nets_menu = new Menu("Nets"); nets_menu.setEnabled(false);
        nets_menu.addActionListener(this);
        mb.add(nets_menu);
        
        help_menu = buildGUIControllerHelpMenu();
        mb.setHelpMenu(help_menu);
        
        return mb;
    }
    
    /***********************************************************************/
    private Menu buildGUIControllerFileMenu() {
        Menu m;
        MenuItem mi;
        
        m = new Menu("File");
        mi = new MenuItem(GIDEN_NEW_NETWORK, new MenuShortcut(KeyEvent.VK_N));
        mi.setActionCommand(GIDEN_NEW_NETWORK); m.add(mi);
        
        file_open_menu = new Menu("Open",false);
        for (int i=0 ; i < filenames.length ; i++) {
            mi = new MenuItem(filenames[i]); 
            mi.setEnabled(true);
            file_open_menu.add(mi);
        }
        file_open_menu.addActionListener(this);
        m.add(file_open_menu);
        
        mi = new MenuItem(GIDEN_OPEN_NETWORK, new MenuShortcut(KeyEvent.VK_O));
        mi.setActionCommand(GIDEN_OPEN_NETWORK); mi.setEnabled(true);  m.add(mi);
        
        print_mi = new MenuItem(GIDEN_PRINT_NETWORKS); print_mi.setEnabled(false);
        m.add(print_mi);
        
        m.add("-");
        
        mi = new MenuItem(GIDEN_QUIT, new MenuShortcut(KeyEvent.VK_Q));
        mi.setActionCommand(GIDEN_QUIT); m.add(mi);
        
        m.addActionListener(this);
        
        return m;
    }
    
    /***********************************************************************/
    private Menu buildGUIControllerToolsMenu() {
        Menu m;
        MenuItem mi;
        
        m = new Menu("Tools");
        controller_menuitems = new CheckboxMenuItem[toolnames.length];
        for (int i=0 ; i < toolnames.length ; i++) {
            m.add(controller_menuitems[i] = new CheckboxMenuItem(toolnames[i]));
            controller_menuitems[i].addItemListener(this);
        }
        m.add("-");
        m.add(showToolIcons = new CheckboxMenuItem("Show Tool Icons",true));
        showToolIcons.addItemListener(this);
        
        return m;
    }
    
    /***********************************************************************/
    private Menu buildGUIControllerHelpMenu() {
        Menu m;
        MenuItem mi;
        
        m = new Menu("Help",true);
        
        mi = new MenuItem(GIDEN_HELP);
        mi.setEnabled(true);          // dsd 020228 made it true
        m.add(mi);
        m.add("-");
        mi = new MenuItem(GIDEN_ABOUT); m.add(mi);
        mi = new MenuItem(GIDEN_SYSTEM_INFORMATION); m.add(mi);
        
        // registration info 
        //        mi = new MenuItem(GIDEN_REGISTRATION_INFORMATION); 
        //        mi.setEnabled(false); 
        //        m.add(mi);
        
        m.addActionListener(this);
        
        return m;
    }
    
    
    /**********************************************************************/
    // component listener methods
    /**********************************************************************/
    public void componentMoved(ComponentEvent evt) {}
    public void componentShown(ComponentEvent evt) {}
    public void componentHidden(ComponentEvent evt) {}
    
    private Dimension oldsize = new Dimension(-1,-1);
    public void componentResized(ComponentEvent evt) {
        Dimension reqsize = evt.getComponent().getSize();
        if (reqsize.equals(oldsize)) {
            pack(); 
            if (isShowing())
                UTILITIES.Repaint(this,"componentResized (1)"); 
            return;
        }
        else {oldsize.width = reqsize.width; oldsize.height = reqsize.height;}
        
        Rectangle reqbounds = evt.getComponent().getBounds();
        
        // 9 components; each 72x66 in size; find best fit
        Insets inset = ((Container) evt.getComponent()).getInsets(); 
        if (reqsize.width >= (72*9)+inset.left+inset.right) {
            gl.setRows(1);
            gl.setColumns(9);
        }
        else if (reqsize.width >= (72*3)+inset.left+inset.right) {
            gl.setRows(3);
            gl.setColumns(3);
        }
        else { //make minimum number of cols to be 3
            gl.setRows(3);
            gl.setColumns(3);
        }
        
        setLocation(reqbounds.x,reqbounds.y);
        pack(); 
        if (isShowing())
            UTILITIES.Repaint(this,"componentResized (2)");
    }
    
    
    /***********************************************************************/
    // item listener method
    /***********************************************************************/
    public void itemStateChanged(ItemEvent evt) {
        Object source = evt.getSource();
        
        if (source.equals(controller_choice)) {
            selectTool(controller_choice.getSelectedIndex());
            return;
        }
        else if (source.equals(showToolIcons)) {
            this.remove(controller_choice);
            
            if (showToolIcons.getState()) {
                setLayout(gl);
                for (int i=0 ; i < controller_buttons.length ; i++)
                    add(controller_buttons[i]);
                
                gl.setRows(3); gl.setColumns(3);
            }
            else {
                for (int i=0 ; i < toolnames.length ; i++)
                    this.remove(controller_buttons[i]);
                
                /*
                  setLayout(new FlowLayout());
                  add(controller_choice);
                  add(current_controller_button);
                */
                setLayout(new FlowLayout());
                add(current_controller_button);
                add(controller_choice);
                /*
                  setLayout(new BorderLayout());
                  add(current_controller_button,BorderLayout.WEST);
                  add(controller_choice,BorderLayout.CENTER);
                */
            }
            pack(); 
            //            UTILITIES.Repaint(this,"itemStateChanged");
            return;
        }
        
        for (int i=0 ; i < controller_menuitems.length ; i++)
            if (controller_menuitems[i].equals(source)) {
                if (i == current_tool)
                    controller_menuitems[i].setState(true);
                else
                    selectTool(i);
            }
    }
    
    
    /***********************************************************************/
    // window listener methods
    /***********************************************************************/
    public void windowActivated(WindowEvent evt)   {}
    public void windowClosed(WindowEvent evt)      {}
    public void windowClosing(WindowEvent evt)     {System.exit(0);}
    public void windowOpened(WindowEvent evt)      {}
    public void windowDeactivated(WindowEvent evt) {}
    public void windowDeiconified(WindowEvent evt) {
        for (int i = 0 ; i < network_count ; i++)
            if (networks[i] != null)
                networks[i].unhide_net();
    }
    public void windowIconified(WindowEvent evt)   {
        for (int i = 0 ; i < network_count ; i++)
            if (networks[i] != null && networks[i].isVisible())
                networks[i].windowIconified(new WindowEvent(this,
                                                            WindowEvent.WINDOW_ICONIFIED));
    }
    
    
    /**********************************************************************/
    // public action listener method and private support methods (for menus)
    /**********************************************************************/
    public void actionPerformed(ActionEvent evt) {
        Object source = evt.getSource();
        
        if (source.equals(file_menu))
            processGUIControllerFileMenuSelections(evt);      
        else if (source.equals(file_open_menu))
            processGUIControllerFileOpenMenuSelections(evt);
        else if (source.equals(nets_menu))
            processGUIControllerNetsMenuSelections(evt);
        else if (source.equals(help_menu))
            processGUIControllerHelpMenuSelections(evt);
        else
            GIDEN.out.println("Unknown source (label = "
                               +evt.getActionCommand()+")");
    }
    
    /***********************************************************************/
    private void processGUIControllerHelpMenuSelections(ActionEvent evt) {
        
        String label = evt.getActionCommand();
        if (label.equals(GIDEN_HELP)) {
            if(HelpWindow==null) {
                Rectangle bounds = getBounds();
                Point point = new Point();
                point.x = bounds.x+bounds.width;
                point.y = bounds.y;
                HelpWindow = new GdnHtmlWindow("GidenHelp.html","GIDEN Help",point,400,400);
            }
            HelpWindow.makeVisible(true);
        }
        else if (label.equals(GIDEN_ABOUT))
            new GUIAboutGIDEN(this); 
        else if (label.equals(GIDEN_SYSTEM_INFORMATION))
            new GUISystemInfoDialog(this);
        
        
    /* dsd commented out and added HTML help
        String label = evt.getActionCommand();
        
        if (label.equals(GIDEN_HELP)) {
            if (help == null) {
                help = new GUIHelpInfoDialog(this); help.setVisible(true);
            }
            else {
                help.setVisible(true); help.toFront();
            }
        }
        else if (label.equals(GIDEN_ABOUT))
            new GUIAboutGIDEN(this); 
        else if (label.equals(GIDEN_SYSTEM_INFORMATION))
            new GUISystemInfoDialog(this);
    */
    }
    
    /***********************************************************************/
    private void processGUIControllerNetsMenuSelections(ActionEvent evt) {
        String label = evt.getActionCommand();
        
        for (int i=0 ; i < network_count ; i++)
            if (networks[i] != null && networks[i].getNetworkName().equals(label)) {
                networks[i].windowActivated(new WindowEvent(this,
                                              WindowEvent.WINDOW_ACTIVATED));
		//                networks[i].setVisible(true);
		//                networks[i].toFront();
		networks[i].unhide_net();
                break;
            }
    }
    
    /// !! DSD
    public GUINetworkWindow createSolverWindowDsd(String filename,GUINetwork net)
    {
        createSolverWindow(filename,net);
        return networks[network_count-1];
    }
    
    /***********************************************************************/
    private void createSolverWindow(String     filename,
                                    GUINetwork net) {
        createSolverWindow(filename, (File) null, net);
    }
    /***********************************************************************/
    private void createSolverWindow(String     filename,
                                    File       file,
                                    GUINetwork net) {
        String dirname;
        if (file != null)
            dirname = new String(file.getParent() + "/");
        else
            dirname = "./networks/";


        if (nets_menu.getItemCount() == 0) {
            print_mi.setEnabled(true);
            nets_menu.setEnabled(true);
        }

        nwnm.registerFilename(filename);
        String netname = nwnm.getLatestNetname(filename);
        String windowname = nwnm.getLatestWindowname(filename);

        MenuItem mi = new MenuItem(netname);
        insertItemIntoNetsMenu(mi);

        if (networks.length <= network_count)
            networks = resizeGUINetworkWindowArray(networks);

        networks[network_count] = new GUISolver(this, file,
                                                dirname, filename,
                                                netname, windowname,
                                                mi, net);
        networks[network_count].setVisible(true);
        network_count++;
    }

    
    /***********************************************************************/
    private void processGUIControllerFileOpenMenuSelections(ActionEvent evt) {
        String filename = new String(evt.getActionCommand() + ".gdn");
        InputStream is = getClass().getResourceAsStream("gidenNETS/"
                                                        +filename);
        if (is == null) {
            GIDEN.out.println("Error reading network ("+filename+").");
            return;
        }

        InputStreamReader reader = new InputStreamReader(is);
        GUINetwork net = (new GUINetworkReader()).readGUINetwork(reader);
        createSolverWindow(filename, net);
    }
    
    /***********************************************************************/
    private void processGUIControllerFileMenuSelections(ActionEvent evt) {
        String label = evt.getActionCommand();
        
        if (label.equals(GIDEN_QUIT))
            System.exit(0);
        else if (label.equals(GIDEN_PRINT_NETWORKS)) {
            PrintJob pjob = this.getToolkit().getPrintJob(this,GIDEN_PRINT_NETWORKS,null);
            if (pjob != null) {
                Graphics pg = pjob.getGraphics();
                
                /*
                // (TEMP-owenj) print controller window
                pg = pjob.getGraphics();
                if (pg != null) {
                    // translate to offset northwest corner by 0.5 inch
                    // int res = getToolkit().getScreenResolution();
                    // pg.translate((int) (.5*res), (int) (.5*res));
                    
                    printAll(pg);
                    pg.dispose(); // flush page
                }
                */

                // print networks
                for (int i=0 ; i < network_count ; i++) {
                    if (networks[i] != null) {
                        pg = pjob.getGraphics();
                        if (pg != null && networks[i].isVisible()) {
                            // translate to offset northwest corner by 0.5 inch
                            int res = getToolkit().getScreenResolution();
                            pg.translate((int) (.5*res), (int) (.5*res));

                            networks[i].printAll(pg);
                            pg.dispose(); // flush page
                        }
                    }
                }
                pjob.end();
            }
        }
        else if (label.equals(GIDEN_OPEN_NETWORK)) {
            FileDialog d = new FileDialog(this,"Open GIDEN Network...",FileDialog.LOAD);
            // next line doesn't work presently -- bug in JDK-1.1.3
            //    -see http://www.digitalfocus.com/faq/FI/forum_messages/180.html
            d.setFilenameFilter(new EndsWithFilter(".gdn"));
            d.setDirectory("..\\networks"); // dsd 021117, works on unix?
            d.show();
//          d.setVisible(true);
            
            if (d.getFile() != null) {
                
                String filename = d.getFile();
                                
                /*--- DSD 8/16/2000 START NEW NETWORK FILE ---*/
                GdnNetworkReaderV3 v3 = new GdnNetworkReaderV3(d.getDirectory(),filename);
                if(v3.getFileValid())
                {
                    GUINetwork net = new GUINetwork();
                    createSolverWindow(filename,net);
                    v3.readFile(net);
                }
                else
                /*=== DSD 8/16/2000  END NEW NETWORK FILE ===*/
                {
                    // ORIGINAL CODE 
                
                    if (!filename.toLowerCase().endsWith(".gdn"))
                        filename = new String(filename+".gdn");
                    
                
                    File file = new File(d.getDirectory(),filename);
                    if (!file.exists()) {
                        GIDEN.out.println("Could not open file \""+filename+"\"");
                        return;
                    }

                    // file exists
                    try {
                        FileReader reader = new FileReader(file);
                        GUINetwork net = (new GUINetworkReader()).readGUINetwork(reader);
                        createSolverWindow(filename,file,net);
                    } catch(FileNotFoundException e) {GIDEN.out.println(e);}
                }
            }
        }
        else if (label.equals(GIDEN_NEW_NETWORK))
            createSolverWindow("", new GUINetwork());
    }
    
}

