package giden.GUI;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.*;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.net.URL;

public class GUINetworkReader {

    private GUINetwork net;

    private final String FILE_VERSION_UNKNOWN = "unknown";
    private final String FILE_VERSION_2dev    = "0.0a"; // (development format)
    private final String FILE_VERSION_2alpha  = "2.0a"; // GIDEN-2.0

    GUINetworkReader(){}

    public GUINetwork readGUINetwork(InputStreamReader reader) {
        net = new GUINetwork();

        String text = getReaderText(reader);
        if (parseText(text))
            return net;
        else
            return new GUINetwork();
    }

    private void error(String msg) {
        GIDEN.out.println("Error:  Invalid network file format.\n"+
                           "        "+msg);
    }


    private boolean parseText(String text) {

        StringTokenizer line_reader = new StringTokenizer(text,"\r\n");


        String version = readVersionInfo(line_reader);

        // check for unknown file format
        if (version.equals(FILE_VERSION_UNKNOWN)) {
            // try to convert unknown text to a know format
            String newtext = OldGidenFiles.convertUnknownText(text);

            // re-initialize the line_reader with the converted text
            line_reader = new StringTokenizer(newtext,"\r\n");

            // read version converted text, and proceed as usual below
            version = readVersionInfo(line_reader);
        }

        if (version.equals(FILE_VERSION_2alpha) ||
            version.equals(FILE_VERSION_2dev)) {
            Version2aReader reader = new Version2aReader(line_reader);
            net = reader.getNetwork();
            return reader.dataParsed();
        }
        else {
            error("unrecognized file format");
            return false;
        }
    }

    private String getReaderText(InputStreamReader input) {
        int count;
        char[] buf = new char[512];
        String text = new String();

        try {
            while ((count = input.read(buf)) >= 0)
                text = text.concat(new String(buf,0,count));
            input.close();
        } catch(IOException e) {
            GIDEN.out.println(e);
        }

        return text;
    }


    private String readVersionInfo(StringTokenizer line_reader) {
        String token;
        StringTokenizer line_parser;

        while (line_reader.hasMoreTokens()) {
            line_parser = new StringTokenizer(line_reader.nextToken(),":",true);

            if (!line_parser.hasMoreTokens()) {
                // error("(missing line token)");
                return FILE_VERSION_UNKNOWN;
            }

            token = line_parser.nextToken();
            line_parser.nextToken();          // get ":" separator

            if ("v".equalsIgnoreCase(token) ||
                "version".equalsIgnoreCase(token)) {

                // reset line parser
                line_parser = new StringTokenizer(line_parser.nextToken("\r\n"),",");

                // return version info
                return line_parser.nextToken();
            }
            else if ("s".equalsIgnoreCase(token) ||
                     "system".equalsIgnoreCase(token)) {
                // ignore system lines for now...
            }
            else if (!"c".equalsIgnoreCase(token) &&
                     !"comment".equalsIgnoreCase(token)) {
                // error("(unexpected token \""+token+"\")");
                return FILE_VERSION_UNKNOWN;
            }
        }
        return FILE_VERSION_UNKNOWN;
    }

}

/*public*/ class OldGidenFiles {

    public static String convertUnknownText(String text) {
        // This *brute force* routine tries to convert unknown file
        // text into a format compatible with FILE_VERSION_2alpha.
        //
        // The converter is based on my own assumptions about the old
        // format from looking at a few samples --- no guarantees!
        //
        // This will (hopefully) enable reading network files
        // generated with the JDK-1.0.2 version of GIDEN, but will
        // give garbage for other files.

        StringTokenizer line_reader = new StringTokenizer(text,"\r\n");

        // skip first five lines; don't bother preserving the system
        // and user info in these lines
        for (int i=0 ; i < 5 ; i++)
            line_reader.nextToken();

        // get number of node data fields (other than index)
        int num_node_arrays = Integer.decode(line_reader.nextToken(" \t\r\n")).intValue();
        line_reader.nextToken("\r\n");

        // get array names
        String[] node_array_names = new String[num_node_arrays];
        for (int i=0 ; i < num_node_arrays ; i++)
            node_array_names[i] = line_reader.nextToken();

        // get number of node lines
        int num_node_lines =
            Integer.decode(line_reader.nextToken(" \t\r\n")).intValue();
        int num_nodes = num_node_lines;
        line_reader.nextToken("\r\n");



        // save node info and node data values
        String node_id;
        String x;
        String y;
        String node_info = new String("");
        String node_data_values = new String("");
        for (int i=0 ; i < num_node_lines ; i++) {
            node_id = line_reader.nextToken(" \t\r\n");
            x = line_reader.nextToken();
            y = line_reader.nextToken();

            if (Integer.decode(x).intValue() == -1) {
                // this is a deleted node, so we ignore it

                // decrement real node count
                num_nodes--;

                // pass by extra values (to finish the line)
                for (int j=0 ; j < num_node_arrays ; j++)
                    line_reader.nextToken();
            }
            else {
                node_info = node_info.concat("node:id="+node_id
                                             +",x="+x+",y="+y+"\n");

                if (num_node_arrays > 0) {
                    node_data_values = node_data_values.concat("ndv:"+node_id);

                    for (int j=0 ; j < num_node_arrays ; j++)
                        node_data_values =
                            node_data_values.concat(","+line_reader.nextToken());

                    node_data_values = node_data_values.concat("\n");
                }
            }
        }



        // get number of edge data fields (other than index)
        int num_edge_arrays = Integer.decode(line_reader.nextToken(" \t\r\n")).intValue();
        line_reader.nextToken("\r\n");


        // get array names
        GIDEN.out.println("number of edge arrays = "+num_edge_arrays);
        String[] edge_array_names = new String[num_edge_arrays];
        for (int i=0 ; i < num_edge_arrays ; i++)
            edge_array_names[i] = line_reader.nextToken();

        // get number of edge lines
        int num_edge_lines =
            Integer.decode(line_reader.nextToken(" \t\r\n")).intValue();
        int num_edges = num_edge_lines;
GIDEN.out.println("num edge lines = "+num_edge_lines);
        line_reader.nextToken("\r\n");


        // save edge info and edge data values
        String edge_id,e1,e2;
        String edge_info = new String("");
        String edge_data_values = new String("");
        for (int i=0 ; i < num_edge_lines ; i++) {
            edge_id = line_reader.nextToken(" \t\r\n");
            e1 = line_reader.nextToken();
            e2 = line_reader.nextToken();

            if (Integer.decode(e1).intValue() == -1) {
                // this is a deleted edge, so we ignore it

                // decrement real edge count
                num_edges--;

                // ignore edge label coordinates
                line_reader.nextToken();
                line_reader.nextToken();

                // pass by extra values (to finish the line)
                for (int j=0 ; j < num_edge_arrays ; j++)
                    line_reader.nextToken();
            }
            else {
                edge_info = edge_info.concat("edge:id="+edge_id
                                             +",e1="+e1+",e2="+e2+"\n");

                // ignore edge label coordinates
                line_reader.nextToken();
                line_reader.nextToken();

                if (num_edge_arrays > 0) {
                    edge_data_values = edge_data_values.concat("edv:"+edge_id);

                    for (int j=0 ; j < num_edge_arrays ; j++)
                        edge_data_values =
                            edge_data_values.concat(","+line_reader.nextToken());

                    edge_data_values = edge_data_values.concat("\n");
                }
            }
        }



        // generate string in FILE_VERSION_2alpha format
        String newtext = new String();

        // write version info
        SimpleDateFormat df =
            new SimpleDateFormat("E dd MMM yyyy H:mm:ss z",Locale.US);
        newtext = newtext.concat("v:2.0a,"+df.format(new Date())+"\n");

        // write graph info
        newtext = newtext.concat("graph:"+num_nodes+","
                                 +num_edges+",directed\n");

        // write node info
        newtext = newtext.concat(node_info);

        // write edge info
        newtext = newtext.concat(edge_info);

        // write node data labels (all integer)
        newtext = newtext.concat("node data labels:"+num_node_arrays);
        for (int i=0 ; i < num_node_arrays ; i++)
            newtext = newtext.concat(","+node_array_names[i]+",I");
        newtext = newtext.concat("\n");

        // write node data values
        newtext = newtext.concat(node_data_values);

        // write edge data labels (all integer)
        newtext = newtext.concat("edge data labels:"+num_edge_arrays);
        for (int i=0 ; i < num_edge_arrays ; i++)
            newtext = newtext.concat(","+edge_array_names[i]+",I");
        newtext = newtext.concat("\n");

        // write edge data values
        newtext = newtext.concat(edge_data_values);

        return newtext;
    }
}



/*public*/ class Version2aReader {

    private GUINetwork net = null;
    private boolean parsed = false;

    private Hashtable nodes = new Hashtable();
    private Hashtable edges = new Hashtable();

    private UiNodeArray[] nodedata = new UiNodeArray[0];
    private UiEdgeArray[] edgedata = new UiEdgeArray[0];

    private int num_nodes=0, num_edges=0;
    private boolean net_directed = true;

    Version2aReader(StringTokenizer line_reader) {
        net = new GUINetwork();
        parsed = parseVersion2a(line_reader);
    }

    synchronized public boolean dataParsed() {
        return parsed;
    }

    synchronized public GUINetwork getNetwork() {
        if (parsed)
            return net;
        else
            return new GUINetwork();
    }

    private void error(String msg) {
        GIDEN.out.println("Error:  Invalid network file format.\n"+
                          "        "+msg);
    }

    private synchronized boolean parseVersion2a(StringTokenizer line_reader) {
        if (!readGraphInfo(line_reader)) { 
            error("readGraphInfo");
            return false;
        }
        if (num_nodes > 0 && !readNodeInfo(line_reader)) {
            error("readNodeInfo");
            return false;
        }
        if (num_edges > 0 && !readEdgeInfo(line_reader)) {
            error("readEdgeInfo");
            return false;
        }
        if (!readNodeDataLabels(line_reader)) {
            error("readNodeDataLabels");
            return false;
        }
        if (nodedata.length > 0 && !readNodeDataValues(line_reader)) {
            error("readNodeDataValues");
            return false;
        }
        if (!readEdgeDataLabels(line_reader)) {
            error("readEdgeDataLabels");
            return false;
        }
        if (edgedata.length > 0 && !readEdgeDataValues(line_reader)) {
            error("readEdgeDataValues");
            return false;
        }
        if (!readRemnants(line_reader)) {
            error("readRemnants");
            return false;
        }

        return true;
    }

    private boolean readGraphInfo(StringTokenizer line_reader) {
        String token;
        StringTokenizer line_parser;

        int i;

        while (line_reader.hasMoreTokens()) {
            line_parser = new StringTokenizer(line_reader.nextToken(),":",true);

            if (!line_parser.hasMoreTokens()) {
                error("(missing line token)");
                return false;
            }

            token = line_parser.nextToken();
            line_parser.nextToken();          // get ":" separator
            
            if ("g".equalsIgnoreCase(token) ||
                "graph".equalsIgnoreCase(token)) {

                // reset line parser
                line_parser = new StringTokenizer(line_parser.nextToken("\r\n"),",");

                // graph line format:  "g:<num_nodes>,<num_edges>,<directed ('d'/'u'>"
                num_nodes = Integer.decode(line_parser.nextToken()).intValue();
                num_edges = Integer.decode(line_parser.nextToken()).intValue();

                // set directed flag
                token = line_parser.nextToken();
                if ("d".equalsIgnoreCase(token) ||
                    "directed".equalsIgnoreCase(token)) {
                    net_directed = true;
                }
                else if ("u".equalsIgnoreCase(token) ||
                         "undirected".equalsIgnoreCase(token)) {
                    net_directed = false;
                }
                else {
                    error("(bad \"directed\" indicator for network)");
                    return false;
                }

                return true;
            }
            else if ("s".equalsIgnoreCase(token) ||
                     "system".equalsIgnoreCase(token)) {
                // ignore system lines for now...
            }
            else if (!"c".equalsIgnoreCase(token) &&
                     !"comment".equalsIgnoreCase(token)) {
                error("(unexpected token \""+token+"\")");
                return false;
            }
        }
        return false;
    }


    private boolean readNodeInfo(StringTokenizer line_reader) {
        String token;
        StringTokenizer line_parser;

        int node_count = 0;
        GUINode node;
        GUINodeInfo node_info;

        int num_required_props = 3;
        String[] prop_id  = {"id","x","y"};
        //        Object[] prop_def = {"","10","10"};
        Object[] prop_def = {Boolean.FALSE, Boolean.FALSE, Boolean.FALSE};

        while (line_reader.hasMoreTokens()) {
            line_parser = new StringTokenizer(line_reader.nextToken(),":",true);

            if (!line_parser.hasMoreTokens()) {
                error("(missing line token)");
                return false;
            }

            token = line_parser.nextToken();
            line_parser.nextToken();          // get ":" separator
            
            if ("n".equalsIgnoreCase(token) ||
                "node".equalsIgnoreCase(token)) {

                // reset line parser
                line_parser = new StringTokenizer(line_parser.nextToken("\r\n"),",");

                // parse for property specifications
                Hashtable specs = getSpecs(line_parser,prop_id,prop_def,
                                           num_required_props);

                if (specs == null) return false; // error reported in getSpecs

                // extract info from specs
                String id = (String) specs.get("id");
                int x = Integer.decode((String)specs.get("x")).intValue();
                int y = Integer.decode((String)specs.get("y")).intValue();

                /*
                // create 3-D node
                node = new GUINode(null,null,
                                   new GUINodeImageComponent("image/nodes/3-D-node.gif"),
                                   // new GUINodeImageComponent("image/nodes/test.gif"),
                                   true,new Font("DialogInput",Font.PLAIN,12),"",
                                   GUIDrawCanvas.defaultItemColor,x,y);
                */

                // create plain node
                node = new GUINode(null,null,18,true,
                                   new Font("DialogInput",Font.PLAIN,12),
                                   "",GUIDrawCanvas.defaultItemColor,x,y);


                net.CreateNodeY(node);

                // update network's "node id" array
                if (!net.PutNodeId(node, id)) {
                    error("(could not associate specified node id)");
                    return false;
                }

                // save the node in the "nodes" hashtable
                if (nodes.containsKey(id)) {
                    error("(duplicate \"node id\")");
                    return false;
                }
                nodes.put(id,node);


                if (++node_count == num_nodes)
                    return true;
            }
            else if ("s".equalsIgnoreCase(token) ||
                     "system".equalsIgnoreCase(token)) {
                // ignore system lines for now...
            }
            else if (!"c".equalsIgnoreCase(token) &&
                     !"comment".equalsIgnoreCase(token)) {
                error("(unexpected token \""+token+"\")");
                return false;
            }
        }
        return false;
    }


    private boolean readEdgeInfo(StringTokenizer line_reader) {
        String token;
        StringTokenizer line_parser;

        int edge_count = 0;
        String edge_id;
        GUIEdge edge;
        GUIEdgeInfo info;

        int num_required_props = 3;
        String[] prop_id  = {"id","e1","e2",
                             "spline","x","y",
                             "lp"};
        Object[] prop_def = {Boolean.FALSE,Boolean.FALSE,Boolean.FALSE,
                             "false","0","0",
                             Integer.toString(GUIEdgeInfo.LABEL_NORTH)};

        while (line_reader.hasMoreTokens()) {
            line_parser = new StringTokenizer(line_reader.nextToken(),":",true);

            if (!line_parser.hasMoreTokens()) {
                error("(missing line token)");
                return false;
            }

            token = line_parser.nextToken();
            line_parser.nextToken();          // get ":" separator
            
            if ("e".equalsIgnoreCase(token) ||
                "edge".equalsIgnoreCase(token)) {

                // reset line parser
                line_parser = new StringTokenizer(line_parser.nextToken("\r\n"),",");
                // parse for property specifications
                Hashtable specs = getSpecs(line_parser,prop_id,prop_def,
                                           num_required_props);

                if (specs == null) return false; // error reported in getSpecs

                // extract info from specs
                String id = (String) specs.get("id");
                String e1 = (String) specs.get("e1");
                String e2 = (String) specs.get("e2");
                Boolean spline = new Boolean((String)specs.get("spline"));
                int x = Integer.decode((String)specs.get("x")).intValue();
                int y = Integer.decode((String)specs.get("y")).intValue();
                int lp = Integer.decode((String)specs.get("lp")).intValue();

                // create edge 
                edge = new GUIEdge(null,null,true,true,
                                   new Font("DialogInput",Font.PLAIN,12),
                                   "-",GUIDrawCanvas.defaultItemColor,3);

                if (!nodes.containsKey(e1) || 
                    !nodes.containsKey(e2)) {
                    error("(specified endpoints of edge do not exist)");
                    return false;
                }
                net.CreateEdgeY(edge, 
                               (GUINode) nodes.get(e1), 
                               (GUINode) nodes.get(e2));


                // update network's "edge id" array
                if (!net.PutEdgeId(edge, id)) {
                    error("(could not associate specified edge id)");
                    return false;
                }


                // set edge info
                info = edge.GetInfo();

                /* spline point position */
                info.spline_positioned = spline.booleanValue();
                if (info.spline_positioned)
                    info.setSplinePoint(new Point(x,y));

                /* set directed flag */
                info.directed = net_directed;
                
                /* euclidean length and edge id */
                Point c1 = net.GetSource(edge).getCenterPoint();
                Point c2 = net.GetTarget(edge).getCenterPoint();
                int e_length = (int) Math.sqrt((c1.x-c2.x)*(c1.x-c2.x) +
                                       (c1.y-c2.y)*(c1.y-c2.y));
                UiEdgeArray e_length_array = net.GetEdgeArray(UiNetwork.EuclideanLengthString);
                if (e_length_array != null)
                    e_length_array.Put(edge,e_length);
                else {
                    error("(could not open network's e-length array)");
                    return false;
                }

                /* set label position */
                info.label_position = lp;

                edge.PutInfo(info);


                // save the edge in the "edges" hashtable
                if (edges.containsKey(id)) {
                    error("(duplicate \"edge id\")");
                    return false;
                }
                edges.put(id,edge);

                if (++edge_count == num_edges)
                    return true;
            }
            else if ("s".equalsIgnoreCase(token) ||
                     "system".equalsIgnoreCase(token)) {
                // ignore system lines for now...
            }
            else if (!"c".equalsIgnoreCase(token) &&
                     !"comment".equalsIgnoreCase(token)) {
                error("(unexpected token \""+token+"\")");
                return false;
            }
        }
        return false;
    }


    private boolean readNodeDataLabels(StringTokenizer line_reader) {
        String token;
        StringTokenizer line_parser;

        while (line_reader.hasMoreTokens()) {
            line_parser = new StringTokenizer(line_reader.nextToken(),":",true);

            if (!line_parser.hasMoreTokens()) {
                error("(missing line token)");
                return false;
            }

            token = line_parser.nextToken();
            line_parser.nextToken();          // get ":" separator
            
            if ("ndl".equalsIgnoreCase(token) ||
                "node data labels".equalsIgnoreCase(token)) {

                // reset line parser
                line_parser = new StringTokenizer(line_parser.nextToken("\r\n"),",");

                // node data labels line format:   "ndl:<num labels>,<label 1>,<type 1 ('I'/'T')>,..."
                nodedata = new UiNodeArray[Integer.decode(line_parser.nextToken()).intValue()];
                for (int i=0 ; i < nodedata.length ; i++) {
                    nodedata[i] = net.CreateNodeArray(line_parser.nextToken());
                    token = line_parser.nextToken();
                    if ("i".equalsIgnoreCase(token) ||
                        "integer".equalsIgnoreCase(token)) {
                        nodedata[i].Fill(new Integer(0));
                    }
                    else if ("t".equalsIgnoreCase(token) ||
                             "text".equalsIgnoreCase(token)) {
                        nodedata[i].Fill(new String(""));
                    }
                    else {
                        error("(invalid node data type)");
                        return false;
                    }
                }

                return true;
            }
            else if ("s".equalsIgnoreCase(token) ||
                     "system".equalsIgnoreCase(token)) {
                // ignore system lines for now...
            }
            else if (!"c".equalsIgnoreCase(token) &&
                     !"comment".equalsIgnoreCase(token)) {
                error("(unexpected token \""+token+"\")");
                return false;
            }
        }
        return false;
    }


    private boolean readNodeDataValues(StringTokenizer line_reader) {
        String token;
        StringTokenizer line_parser;

        int node_count = 0;
        String node_id;
        GUINode node;
        Object data;

        while (line_reader.hasMoreTokens()) {
            line_parser = new StringTokenizer(line_reader.nextToken(),":",true);

            if (!line_parser.hasMoreTokens()) {
                error("(missing line token)");
                return false;
            }

            token = line_parser.nextToken();
            line_parser.nextToken();          // get ":" separator
            
            if ("ndv".equalsIgnoreCase(token) ||
                "node data values".equalsIgnoreCase(token)) {

                // reset line parser
                line_parser = new StringTokenizer(line_parser.nextToken("\r\n"),",");

                // node data values line format:  "ndv:<index>,<value 1>,..."
                node_id = line_parser.nextToken();
                if (!nodes.containsKey(node_id)) {
                    error("(invalid node id)");
                    return false;
                }
                node = (GUINode) nodes.get(node_id);
                for (int i=0 ; i < nodedata.length ; i++) {
                    data = nodedata[i].GetObj(node);
                    if (data instanceof Integer) {
                        int fill_int;
                        try {
                            fill_int = Integer.decode(line_parser.nextToken()).intValue();
                        } catch (NumberFormatException e) {
                            error("(node data value out of range or not a valid integer)");
                            return false;
                        }

                        nodedata[i].Put(node, new Integer(fill_int));
                    }
                    else if (data instanceof String) {
                        nodedata[i].Put(node, line_parser.nextToken());
                    }
                    else {
                        error("(unknown node data type)");
                        return false;
                    }
                }

                if (++node_count == num_nodes)
                    return true;
            }
            else if ("s".equalsIgnoreCase(token) ||
                     "system".equalsIgnoreCase(token)) {
                // ignore system lines for now...
            }
            else if (!"c".equalsIgnoreCase(token) &&
                     !"comment".equalsIgnoreCase(token)) {
                error("(unexpected token \""+token+"\")");
                return false;
            }
        }
        return false;
    }


    private boolean readEdgeDataLabels(StringTokenizer line_reader) {
        String token;
        StringTokenizer line_parser;

        while (line_reader.hasMoreTokens()) {
            line_parser = new StringTokenizer(line_reader.nextToken(),":",true);

            if (!line_parser.hasMoreTokens()) {
                error("(missing line token)");
                return false;
            }

            token = line_parser.nextToken();
            line_parser.nextToken();          // get ":" separator
            
            if ("edl".equalsIgnoreCase(token) ||
                "edge data labels".equalsIgnoreCase(token)) {

                // reset line parser
                line_parser = new StringTokenizer(line_parser.nextToken("\r\n"),",");

                // edge data labels line format:   "edl:<num labels>,<label 1>,<type 1 ('I'/'T')>,..."
                edgedata = new UiEdgeArray[Integer.decode(line_parser.nextToken()).intValue()];
                for (int i=0 ; i < edgedata.length ; i++) {
                    edgedata[i] = net.CreateEdgeArray(line_parser.nextToken());
                    token = line_parser.nextToken();
                    if ("i".equalsIgnoreCase(token) ||
                        "integer".equalsIgnoreCase(token)) {
                        edgedata[i].Fill(new Integer(0));
                    }
                    else if ("t".equalsIgnoreCase(token) ||
                             "text".equalsIgnoreCase(token)) {
                        edgedata[i].Fill(new String(""));
                    }
                    else {
                        error("(invalid edge data type)");
                        return false;
                    }
                }

                return true;
            }
            else if ("s".equalsIgnoreCase(token) ||
                     "system".equalsIgnoreCase(token)) {
                // ignore system lines for now...
            }
            else if (!"c".equalsIgnoreCase(token) &&
                     !"comment".equalsIgnoreCase(token)) {
                error("(unexpected token \""+token+"\")");
                return false;
            }
        }
        return false;
    }


    private boolean readEdgeDataValues(StringTokenizer line_reader) {
        String token;
        StringTokenizer line_parser;

        int edge_count = 0;
        GUIEdge edge;
        String edge_id;
        Object data;

        while (line_reader.hasMoreTokens()) {
            line_parser = new StringTokenizer(line_reader.nextToken(),":",true);

            if (!line_parser.hasMoreTokens()) {
                error("(missing line token)");
                return false;
            }

            token = line_parser.nextToken();
            line_parser.nextToken();          // get ":" separator
            
            if ("edv".equalsIgnoreCase(token) ||
                "edge data values".equalsIgnoreCase(token)) {

                // reset line parser
                line_parser = new StringTokenizer(line_parser.nextToken("\r\n"),",");

                // edge data values line format:  "edv:<index>,<value 1>,..."
                edge_id = line_parser.nextToken();
                if (!edges.containsKey(edge_id)) {
                    error("(invalid edge id)");
                    return false;
                }
                edge = (GUIEdge) edges.get(edge_id);

                for (int i=0 ; i < edgedata.length ; i++) {
                    data = edgedata[i].GetObj(edge);
                    if (data instanceof Integer) {
                        int fill_int;
                        try {
                            fill_int = Integer.decode(line_parser.nextToken()).intValue();
                        } catch (NumberFormatException e) {
                            error("(edge data value out of range or not a valid integer)");
                            return false;
                        }

                        edgedata[i].Put(edge, new Integer(fill_int));
                    }
                    else if (data instanceof String) {
                        edgedata[i].Put(edge, line_parser.nextToken());
                    }
                    else {
                        error("(unknown edge data type)");
                        return false;
                    }
                }

                if (++edge_count == num_edges)
                    return true;
            }
            else if ("s".equalsIgnoreCase(token) ||
                     "system".equalsIgnoreCase(token)) {
                // ignore system lines for now...
            }
            else if (!"c".equalsIgnoreCase(token) &&
                     !"comment".equalsIgnoreCase(token)) {
                error("(unexpected token \""+token+"\")");
                return false;
            }
        }
        return false;
    }



    private boolean readRemnants(StringTokenizer line_reader) {
        String token;
        StringTokenizer line_parser;

        while (line_reader.hasMoreTokens()) {
            line_parser = new StringTokenizer(line_reader.nextToken(),":",true);

            if (!line_parser.hasMoreTokens()) {
                error("(missing line token)");
                return false;
            }

            token = line_parser.nextToken();
            line_parser.nextToken();          // get ":" separator

            if ("s".equalsIgnoreCase(token) ||
                "system".equalsIgnoreCase(token)) {
                // ignore system lines for now...
            }
            else if (!"c".equalsIgnoreCase(token) &&
                     !"comment".equalsIgnoreCase(token)) {
                error("(unexpected token \""+token+"\")");
                return false;
            }
        }
        return true;
    }

    private Hashtable getSpecs(StringTokenizer line_parser,
                               String[] prop_id,
                               Object[] prop_def,
                               int num_required_props) {

        // the first (num_required_props) properties are required, and
        // it is assumed that the default values are of the type
        // Boolean; all read values will be saved in hashtable as
        // String objects, so this is how we make sure that the
        // required fields are satisfied

        Hashtable specs = new Hashtable();

        // set default node specs
        for (int i=0 ; i < prop_id.length ; i++)
            specs.put(prop_id[i], prop_def[i]);

        String id, value;
        StringTokenizer prop;
        while (line_parser.hasMoreTokens()) {
            prop = new StringTokenizer(line_parser.nextToken(","),"=");

            id = prop.nextToken();
            if (prop.hasMoreTokens())
                value = prop.nextToken();
            else
                value = new String("");

            specs.put(id,value);
        }

        // make sure that required fields are set
        for (int i=0 ; i < num_required_props ; i++)
            if (specs.get(prop_id[i]) instanceof Boolean) {
                error("(did not provide required property field)");
                return (Hashtable) null;
            }

        return specs;
    }
}
