package giden.GUI;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.*;
import java.io.*;
import java.io.FileReader;
import java.util.StringTokenizer;
import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.Date;

public class GUINetworkWriter {

    GUINetwork net;

    GUINetworkWriter(){}

    public void writeGUINetwork(String filename, GUINetwork net) {

        this.net = net;

        String file_text = new String();
        file_text = appendVersionInfo(file_text);
        file_text = appendGraphInfo(file_text);
        file_text = appendNodeInfo(file_text);
        file_text = appendEdgeInfo(file_text);
        file_text = appendNodeDataLabels(file_text);
        file_text = appendNodeDataValues(file_text);
        file_text = appendEdgeDatalabels(file_text);
        file_text = appendEdgeDataValues(file_text);

        try {
            FileWriter outfile = new FileWriter(filename);
            outfile.write(file_text);
            outfile.flush();
            outfile.close();
        } catch (IOException e) {GIDEN.err.println(e);}
    }

    private String appendVersionInfo(String str) {
        SimpleDateFormat df = 
            new SimpleDateFormat("E dd MMM yyyy H:mm:ss z",Locale.US);

        // write out version and date
        String fileversion = System.getProperty("giden.fileversion","Unknown");
        fileversion="2.0a"; // dsd 020609
        str = str.concat("v:"+fileversion+","+df.format(new Date())+"\n");

        return str;
    }

    private String appendGraphInfo(String str) {
        str = str.concat("graph:"+net.GetNodeCount()+","+net.GetEdgeCount());

        // extrapolate the direction state of the network from the
        // first edge, or set to directed by default:
        GUIEdge edge = net.GetEdgeListIter().GetFirstEdge();
        if (edge != null) {
            GUIEdgeInfo info = edge.GetInfo();
            if (info.directed)
                str = str.concat(",directed\n");
            else
                str = str.concat(",undirected\n");
        }
        else // no edges in network
            str = str.concat(",d\n");

        return str;
    }

    private String appendNodeInfo(String str) {
        String id;
        GUINode node;
        GUINodeInfo info;
        GUINodeListIter nit = net.GetNodeListIter();
        UiNodeArray node_id = net.GetNodeArray(UiNetwork.NodeIdString);

        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode()) {
            // get id and place in correct form
            id = node_id.GetObj(node).toString();
            id = id.replace(',','-');   // use commas only as field delimeters

            info = node.GetInfo();

            str = str.concat("node:id="+id+",x="+info.x+",y="+info.y+"\n");
        }

        return str;
    }

    private String appendEdgeInfo(String str) {
        Point sp;

        String e1_id, e2_id;
        UiNodeArray node_id = net.GetNodeArray(UiNetwork.NodeIdString);

        String id;
        GUIEdge edge;
        GUIEdgeInfo info;
        GUIEdgeListIter eit = net.GetEdgeListIter();
        UiEdgeArray edge_id = net.GetEdgeArray(UiNetwork.EdgeIdString);

        for (edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge()) {
            // get id and place in correct form
            id = edge_id.GetObj(edge).toString();
            id = id.replace(',','-');   // use commas only as field delimeters

            // get endpoint identifiers
            e1_id = node_id.GetObj(net.GetSource(edge)).toString();
            e2_id = node_id.GetObj(net.GetTarget(edge)).toString();
            e1_id = e1_id.replace(',','-');
            e2_id = e2_id.replace(',','-');

            str = str.concat("edge:id="+id
                             +",e1="+e1_id+",e2="+e2_id);

            info = edge.GetInfo();

            // write label position
            str = str.concat(",lp="+info.label_position);

            // get spline point info (set to "0,0" is not positioned)
            if (info.spline_positioned) {
                sp = info.getSplinePoint();
                str = str.concat(",spline=true,x="+sp.x+",y="+sp.y+"\n");
            }
            else
                str = str.concat(",spline=false\n");
        }

        return str;
    }

    private String appendNodeDataLabels(String str) {
        Object data;
        GUINode node = (net.GetNodeListIter()).GetFirstNode();

        // don't count "node id" array
        str = str.concat("node data labels:"+(net.GetNodeArrayCount()-1));

        for (String name = net.GetFirstNodeArrayName() ;
             name != null ; name = net.GetNextNodeArrayName()) {
            if (!name.equals(UiNetwork.NodeIdString)) {
                name = name.replace(',','-'); // use commas only as delimeters

                if (node != null) {
                    data = (net.GetNodeArray(name)).GetObj(node);
                    if (data instanceof Integer) 
                        str = str.concat(","+name+",i");
                    else if (data instanceof String)
                        str = str.concat(","+name+",t");
                    else {
                        GIDEN.out.println("Unknown node data type");
                    }
                }
            }
        }
        str = str.concat("\n");

        return str;
    }

    private String appendNodeDataValues(String str) {

        // no node data values to be written if the only node data is "node id"
        if (net.GetNodeArrayCount()-1 == 0)
            return str;
        
        Object data;
        GUINode node;
        GUINodeListIter nit = net.GetNodeListIter();
        UiNodeArray node_id = net.GetNodeArray(UiNetwork.NodeIdString);

        String name;

        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode()) {
            str = str.concat("node data values:"+node_id.GetObj(node).toString());
            for (name = net.GetFirstNodeArrayName() ;
                 name != null ; name = net.GetNextNodeArrayName()) {
                if (!name.equals(UiNetwork.NodeIdString)) {
                    data = (net.GetNodeArray(name)).GetObj(node);
                    if (data == null)
                        str = str.concat(",?");
                    else if ("".equals(data.toString()))
                        str = str.concat(", "); // include space for reader
                    else
                        str = str.concat(","+data.toString().replace(',','-'));
                }
            }
            str = str.concat("\n");
        }

        return str;
    }

    private String appendEdgeDatalabels(String str) {
        Object data;
        GUIEdge edge = (net.GetEdgeListIter()).GetFirstEdge();

        // don't count "edge id" or "euclidean length" arrays
        str = str.concat("edge data labels:"+(net.GetEdgeArrayCount()-2));

        for (String name = net.GetFirstEdgeArrayName() ;
             name != null ; name = net.GetNextEdgeArrayName()) {
            if (!name.equals(UiNetwork.EdgeIdString) &&
                !name.equals(UiNetwork.EuclideanLengthString)) {
                name = name.replace(',','-'); // use commas only as delimeters

                if (edge != null) {
                    data = (net.GetEdgeArray(name)).GetObj(edge);
                    if (data instanceof Integer) 
                        str = str.concat(","+name+",i");
                    else if (data instanceof String)
                        str = str.concat(","+name+",t");
                    else {
                        GIDEN.out.println("Unknown edge data type");
                    }
                }
            }
        }
        str = str.concat("\n");

        return str;
    }

    private String appendEdgeDataValues(String str) {

        // no edge data values to be written if the only edge data are
        // "edge id" and "euclidean length"
        if (net.GetEdgeArrayCount()-2 == 0)
            return str;

        Object data;
        GUIEdge edge;
        GUIEdgeListIter eit = net.GetEdgeListIter();
        UiEdgeArray edge_id = net.GetEdgeArray(UiNetwork.EdgeIdString);

        String name;

        for (edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge()) {
            str = str.concat("edge data values:"+edge_id.GetObj(edge).toString());
            for (name = net.GetFirstEdgeArrayName() ;
                 name != null ; name = net.GetNextEdgeArrayName()) {
                if (!name.equals(UiNetwork.EdgeIdString) &&
                    !name.equals(UiNetwork.EuclideanLengthString)) {
                    data = (net.GetEdgeArray(name)).GetObj(edge);
                    if (data == null)
                        str = str.concat(",?");
                    else if ("".equals(data.toString()))
                        str = str.concat(", "); // include space for reader
                    else
                        str = str.concat(","+data.toString().replace(',','-'));
                }
            }
            str = str.concat("\n");
        }

        return str;
    }
}
