package giden.GUI;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;


public class GUIPseudoCodeWindow extends Frame implements ActionListener,
                                                   ItemListener,
                                                   WindowListener
{
    public static final String PC_NEXT = " Next ";
    private Button next;

    private final String PC_SAVE = "Save PseudoCode...";
    private final String PC_PRINT = "Print PseudoCode...";


    private int def_size_index = 1; // ==> font_sizes[2] (= 14) pt default
    private int font_sizes[] = { 10, 14, 18 };
    private CheckboxMenuItem current_font_cmi;
    private CheckboxMenuItem font_cmi[];

    private GUIPseudoCodeScrollPane pc = null;
    
    private String pc_filename = null;


    public void setEnabled(boolean b) {
        next.setEnabled(b);
    }

    public void resetPCTextLine() {
        if (pc != null)
            pc.getPCText().resetPCTextLine();
    }
    public boolean showPCTextLine(int line) {
        if (pc != null)
            return pc.getPCText().showPCTextLine(line);
        else
            return false;
    }
    public boolean showPCTextLine(String token) {
        if (pc != null)
            return pc.getPCText().showPCTextLine(token);
        else
            return false;
    }

    public GUIPseudoCodeWindow(GUINetworkWindow  parent, 
                               String            filename,
                               InputStreamReader reader) {
                                
                               
        super("GIDEN PseudoCode for Solver \""+filename+"\""); 
//        GIDEN.out.println("setupSolverMode: #7a");
        
        addWindowListener(this);

        setBackground(Color.lightGray);
        
        this.pc_filename = new String(filename);

        GridBagLayout gbl = new GridBagLayout();
        setLayout(gbl);
        
        GridBagConstraints gbc = new GridBagConstraints();

        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.insets.left = 5;
        gbc.insets.right = 5;
        gbc.insets.top = 5;
        gbc.insets.bottom = 0;
        buildGBConstraints(gbc,0,0,2,1,100,75);
        pc = new GUIPseudoCodeScrollPane(reader);
        gbl.setConstraints(pc,gbc);
        add(pc);


        gbc.anchor = GridBagConstraints.CENTER;
        gbc.fill = GridBagConstraints.NONE;
        gbc.insets.left = 0;
        gbc.insets.right = 0;
        gbc.insets.top = 5;
        gbc.insets.bottom = 5;
        buildGBConstraints(gbc,0,1,2,1,100,0);
        next = new Button(parent.PC_NEXT);
        next.setEnabled(true);
        next.addActionListener(parent);
        gbl.setConstraints(next,gbc);
        add(next);


        // setup pseudocode window menu bar
        MenuBar mb = new MenuBar();

        // setup file menu (print and save)
        Menu m = new Menu("File");
        m.addActionListener(this);
        m.add(new MenuItem(PC_SAVE));
        m.add(new MenuItem(PC_PRINT));
        mb.add(m);

        m = new Menu("PC Font");
        font_cmi = new CheckboxMenuItem[font_sizes.length];
        for (int i=0 ; i < font_sizes.length ; i++) {
            font_cmi[i] = new CheckboxMenuItem("Monospaced "+font_sizes[i]+" pt");
            font_cmi[i].addItemListener(this);
            font_cmi[i].setState(false);
            m.add(font_cmi[i]);
        }
        pc.getPCText().setFontSize(font_sizes[def_size_index]);
        current_font_cmi = font_cmi[def_size_index];
        current_font_cmi.setState(true);
        mb.add(m);

        setMenuBar(mb);


        // set window location (at bottom of network or center of screen)
        Dimension screen_size = Toolkit.getDefaultToolkit().getScreenSize();
        setSize(parent.getSize().width, 250 /* default height */);
        Rectangle parent_bds = parent.getBounds();
        Dimension my_size = getSize();
        

        if (parent_bds.y+parent_bds.height+my_size.height > screen_size.height) {
            
            // place in center of screen
            setLocation(new Point((screen_size.width - my_size.width)/2,
                                  (screen_size.height - my_size.height)/2));
        }
        else {
            
            // place below network window
            setLocation(parent_bds.x, parent_bds.y+parent_bds.height);
            

            // show pseudocode by default
            parent.showPCwindow.setState(true);
//            GIDEN.out.println("setupSolverMode: #9f");
            
            setVisible(true);
//            GIDEN.out.println("setupSolverMode: #9g");
            
        }
//        GIDEN.out.println("setupSolverMode: #10a");
        
    }
    
    private void buildGBConstraints(GridBagConstraints gbc, int gx, int gy,
                                    int gw, int gh, int wx, int wy) {
        gbc.gridx = gx;
        gbc.gridy = gy;
        gbc.gridwidth = gw;
        gbc.gridheight = gh;
        gbc.weightx = wx;
        gbc.weighty = wy;
    }
    
    /***********************************************************************/
    // action listener method
    /***********************************************************************/
    public void actionPerformed(ActionEvent evt) {
        Object src = evt.getSource();
        String comm = evt.getActionCommand();

        if (comm.equals(PC_SAVE)) {
            pc.getPCText().savePCTextToFile();
        }
        else if (comm.equals(PC_PRINT)) {
            // PASS 1:  print directly after translating origin; maintains
            //          resolution, but loses pages for large jobs
            PrintJob pjob = this.getToolkit().getPrintJob(this,"Print PseudoCode...",null);
            if (pjob != null) {
                Graphics pg = pjob.getGraphics();
                if (pg != null && pc != null) {
                    // translate to offset northwest corner by 0.5 inch
                    int res = getToolkit().getScreenResolution();
                    pg.translate((int) (.5*res), (int) (.5*res));

                    pc.getPCText().print(pg);
                    pg.dispose(); // flush page
                }
                pjob.end();
            }

            /* 
            // PASS-0:  scale to one page; jaggies from Image loss
            PrintJob pjob = this.getToolkit().getPrintJob(this,"Print PseudoCode...",null);
            if (pjob != null && pc != null) {
                Dimension pcd = pc.getPCText().getPreferredSize();
                Image full_pc_image = createImage(pcd.width,pcd.height);
                Graphics full_pc_graphics = full_pc_image.getGraphics();
                if (full_pc_graphics == null) 
                    return;
                pc.getPCText().print(full_pc_graphics);


                // get page dimension
                Dimension page_size = pjob.getPageDimension();
                // getPageDimension doesn't return the correct
                // result; the following is a workaround:
                int res = getToolkit().getScreenResolution();
                page_size = new Dimension((int)(8.5*res), 11*res);

                // set margin of one inch on each boarder of page
                int page_margin = (1*res);

                int dx1,dy1,dx2,dy2;
                dx1 = page_margin;
                dy1 = page_margin;
                dx2 = Math.min(dx1+pcd.width, page_size.width-page_margin);
                dy2 = Math.min(dy1+pcd.height, page_size.height-page_margin);


                Graphics pg = pjob.getGraphics();
                if (pg != null) {
                    pg.drawImage(full_pc_image,dx1,dy1,dx2,dy2,
                                 0,0,pcd.width,pcd.height,null);
                    pg.dispose(); // flush page
                }
                pjob.end();
            }
            */
        }
    }

    /***********************************************************************/
    // item listener method
    /***********************************************************************/
    public void itemStateChanged(ItemEvent evt) {
        Object source = evt.getSource();

        // pseudocode font menu CheckboxMenuItem font_cmi
        for (int i=0 ; i <=font_sizes.length ; i++)
            if (font_cmi[i].equals(source)) {
                current_font_cmi.setState(false);
                font_cmi[i].setState(true);
                current_font_cmi = font_cmi[i];
                pc.getPCText().setFontSize(font_sizes[i]);

                break;
            }
    }
    
    /**********************************************************************/
    // window listener methods
    /**********************************************************************/
    public void windowActivated(WindowEvent evt)   {}
    public void windowClosed(WindowEvent evt)      {}
    public void windowClosing(WindowEvent evt)     {dispose();}
    public void windowDeactivated(WindowEvent evt) {}
    public void windowDeiconified(WindowEvent evt) {}
    public void windowIconified(WindowEvent evt)   {}
    public void windowOpened(WindowEvent evt)      {}
}
