package giden.GUI;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.*;
import java.awt.event.*;
import java.net.URL;

public class GUISlider extends Canvas implements Adjustable {
    GUIAnimationControl parent;

    Point locPt = new Point(0,0); // location of the slider

    // if non-null, the slider is being dragged and this field holds
    // the location of the cursor relative to the slider when the
    // mouse was pressed
    Point downPt;

    // image of the slider
    Image slider;
    Image f_slider; // "final" mode slider
    Image t_slider; // "trace" (and "step") mode slider

    // dimensions of the slider
    int sliderW, sliderH;

    // off-screen image is used for double buffering
    Image bbuf;

    public Dimension getPreferredSize() {
        return new Dimension((max-min), sliderH);
    }

    GUISlider(GUIAnimationControl parent) {
        this.parent = parent;

        URL url = getClass().getResource("image/slider.gif");
        slider = Toolkit.getDefaultToolkit().getImage(url);

        url = getClass().getResource("image/final-slider.gif");
        f_slider = Toolkit.getDefaultToolkit().getImage(url);

        url = getClass().getResource("image/trace-slider.gif");
        t_slider = Toolkit.getDefaultToolkit().getImage(url);


        // wait for the image to be loaded so that we can determine its size
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(slider,0);
        tracker.addImage(f_slider,1);
        tracker.addImage(t_slider,2);
        try {
            tracker.waitForAll();
        } catch(Exception e) {e.printStackTrace();}

        /*
        setSize(sliderW = slider.getWidth(this), 
                sliderH = slider.getHeight(this));
        */
        sliderW = slider.getWidth(this);
        sliderH = slider.getHeight(this);

        // register listeners
        addMouseListener(new MouseEventListener());
        addMouseMotionListener(new MouseMotionEventListener());
    }

    public void invalidate() {
        super.invalidate();

        // the bounds have changed, so recompute the slider location
        setValues(value, min, max);
    }

    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        int w = getSize().width;
        int h = getSize().height;

        // create of enlarge the double buffer if necessary
        if (bbuf==null || bbuf.getWidth(this)<w || bbuf.getHeight(this)<h) {
            if (bbuf != null) bbuf.flush();
            bbuf = createImage(w,h);
        }
        Graphics bbufG = bbuf.getGraphics();

        // clear the background
        bbufG.setColor(getBackground());
        bbufG.fillRect(0,0,w,h);

        // draw the track
        if (locked) {
            bbufG.setColor(Color.gray);
            bbufG.fillRoundRect(sliderW/2, h/2-1, w-sliderW, 3, 5, 5);
        }
        else {
            bbufG.setColor(new Color(116,161,180));
            //            bbufG.setColor(new Color(7,94,148));
            bbufG.fillRoundRect(sliderW/2, h/2-1, w-sliderW, 3, 5, 5);

            bbufG.setColor(Color.black);
            bbufG.drawRoundRect(sliderW/2, h/2-1, w-sliderW, 3, 5, 5);
        }

        // draw the slider
        if (value == min)
            bbufG.drawImage(f_slider,locPt.x, locPt.y, this);
        //        else if (locPt.x == max-sliderW)
        else if (value == max)
            bbufG.drawImage(t_slider,locPt.x, locPt.y, this);
        else
            bbufG.drawImage(slider,locPt.x, locPt.y, this);

        // paint double buffer on screen
        g.drawImage(bbuf,0,0,this);
        bbufG.dispose();
    }

    // returns -1 if left of slider
    //          0 if on slider
    //          1 if right of slider
    int onSlider(MouseEvent e) {
        int x = e.getX();

        if (x < locPt.x) 
            return -1;
        else if (x > locPt.x+sliderW) 
            return 1;
        else 
            return 0;
    }

    class MouseEventListener extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            switch(onSlider(evt)) {
            case -1:
                inc(AdjustmentEvent.BLOCK_DECREMENT, -blockInc);
                break;

            case 0:
                // enter dragging mode
                (downPt = evt.getPoint()).translate(-locPt.x, -locPt.y);
                break;

            case 1:
                inc(AdjustmentEvent.BLOCK_DECREMENT,blockInc);
                break;
            }
            requestFocus();
        }

        public void mouseReleased(MouseEvent evt) {
            downPt = null;
        }

        public void mouseEntered(MouseEvent evt) {
            if (onSlider(evt) == 0)
                setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        }

        public void mouseExited(MouseEvent evt) {
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
    }


    class MouseMotionEventListener extends MouseMotionAdapter {
        public void mouseDragged(MouseEvent evt) {
            if (downPt != null) {
                setValues(evt.getX() * (max-min)
                          / (getSize().width-sliderW) + min, min, max);
                inc(AdjustmentEvent.TRACK, 0);
            }
        }

        public void mouseMoved(MouseEvent evt) {
            if (onSlider(evt) == 0)
                setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        }
    }


    // adjustable get methods

    // properties
    int orientation = HORIZONTAL;
    int min = 0;  
    int max = 100;
    int value = 0;
    int unitInc = 1;
    int blockInc = 25;
    int visible = 0;
    private int mode = GUIAnimationControl.UNDEFINIED_MODE;


    public int getOrientation() {return orientation;}
    public int getValue() {return value;}
    public int getMinimum() {return min;}
    public int getMaximum() {return max;}
    public int getVisibleAmount() {return 0;}
    public int getUnitIncrement() {return unitInc;}
    public int getBlockIncrement() {return blockInc;}

    public int getMode() {return mode;}

    // adjustable set methods
    private boolean locked = false;
    public void setLocked(boolean b) {locked = b;}
    void setValues(int va, int mn, int mx) {
        if (locked) return;

        value = va;
        min = mn;
        max = mx;
        if (max <= min)
            max = min+1;

        if (value < blockInc/2) {
            value = min;
            if (mode != GUIAnimationControl.FINAL_MODE) {
                mode = GUIAnimationControl.FINAL_MODE;
                parent.updateMode(mode);
            }
        }
        else if (value > max-(blockInc/2)) {
            if (mode != GUIAnimationControl.STOP_MODE) {
                mode = GUIAnimationControl.STOP_MODE;
                parent.updateMode(mode);
            }
            value = max;
        }
        else if (mode != GUIAnimationControl.CONTINUOUS_MODE) {
            mode = GUIAnimationControl.CONTINUOUS_MODE;
            parent.updateMode(mode);
        }
        

        // update location of slider
        locPt.x = (value-min) * (getSize().width-sliderW) / (max-min);

        // generate an adjustment event
        inc(AdjustmentEvent.TRACK,0);
        if (isShowing())
            UTILITIES.Repaint(this,"setValues");
    }

    public void setValue(int v) {setValues(v,min,max);}
    public void setMinimum(int m) {setValues(value, m, max);}
    public void setMaximum(int m) {setValues(value, min, m);}
    public void setVisibleAmount(int v) {}
    public void setUnitIncrement(int u) {unitInc = u;}
    public void setBlockIncrement(int b) {blockInc = b;}


    // adjustable event methods
    protected void processEvent(AWTEvent evt) {
        if (evt instanceof AdjustmentEvent) {
            processAdjustmentEvent((AdjustmentEvent)evt);
            return;
        }
        super.processEvent(evt);
    }

    // deliver event to listeners
    protected void processAdjustmentEvent(AdjustmentEvent evt) {
        if (adjustmentListener != null) 
            adjustmentListener.adjustmentValueChanged(evt);
    }

    void inc(int type, int inc) {
        if (inc != 0)
            setValues(value+inc, min, max);
        if (adjustmentListener != null)
            processEvent(new AdjustmentEvent(this,
                   AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED, type, value));
    }

    // adjustable listener methods
    transient AdjustmentListener adjustmentListener;

    public void addAdjustmentListener(AdjustmentListener l) {
        adjustmentListener = AWTEventMulticaster.add(adjustmentListener,l);
    }
    public void removeAdjustmentListener(AdjustmentListener l) {
        adjustmentListener = AWTEventMulticaster.remove(adjustmentListener,l);
    }


}
