package giden.GUI;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.util.Enumeration;
import java.awt.*;
import java.awt.event.*;

public class GUISolverInputDialog extends Dialog implements ActionListener,
                                                     ComponentListener,
                                                     FocusListener,
                                                     MouseListener,
                                                     KeyListener {

    private String           solvername;
    private ExecBase         solver;
    //    private GUINetwork       net;
    private Network          net;
    private GidenSolverInput input;

    private Button accept;
    private Button cancel;

    private SolverInputItem[] items;
    private Choice[]          choices;
    private int item_count;

    public GUISolverInputDialog(Frame            parent,
                                String           sname,
                                ExecBase         solver,
                                //  GUINetwork       net,
                                Network          net,
                                GidenSolverInput input) {
        super(parent,"Input Data for \""+sname+"\"...",true/*modal*/);

        this.solvername = new String(sname);
        this.solver     = solver;
        this.net        = net;
        this.input      = input;

        setLayout(new GridLayout(0, 2));
        setBackground(Color.lightGray);

        item_count = 0;
        items = new SolverInputItem[input.size()];
        SolverInputItem current;
        choices = new Choice[items.length];
        for (int i=0 ; i < items.length ; i++) {
            current = input.getItem(i);

            switch (current.getType()) {
            case SolverInputItem.INTEGER_EDGE_ARRAY:
                if (addIntegerEdgeArraySelector(current))
                    items[item_count++] = current;
                break;
            case SolverInputItem.INTEGER_NODE_ARRAY:
                if (addIntegerNodeArraySelector(current))
                    items[item_count++] = current;
                break;
            default:
                GIDEN.out.println(" ** unknown input type --- item ignored");
            }
        }

        cancel = new Button(" Cancel ");
        cancel.addActionListener(this);
        cancel.addKeyListener(this);
        Panel p = new Panel(new BorderLayout());
        p.add(cancel, BorderLayout.EAST);
        add(p);

        accept = new Button(" Accept ");
        accept.addActionListener(this);
        accept.addKeyListener(this);
        accept.requestFocus();
        p = new Panel(new BorderLayout());
        p.add(accept, BorderLayout.WEST);
        add(p);

        // pack window and set location
        pack();
        Rectangle parent_bounds = parent.getBounds();
        Rectangle my_bounds = getBounds();
        setLocation(parent_bounds.x + (parent_bounds.width-my_bounds.width)/2,
                    parent_bounds.y + (parent_bounds.height-my_bounds.height)/2);

        setResizable(false);

        // add listeners for window
        addFocusListener(this);
        addMouseListener(this);
        addComponentListener(this);

        //        setVisible(true);
        show();
    }

    private boolean addIntegerEdgeArraySelector(SolverInputItem current) {
        String default_name = current.getName();

        Choice c = new Choice();
        String name;
        for (name = net.GetFirstEdgeArrayName() ; name != null ;
             name = net.GetNextEdgeArrayName()) {
            //            if (!name.equals(UiNetwork.EdgeIdString))
            //                c.add(name);
            if ((net.GetEdgeArray(name)).FillObject instanceof Integer) {
                c.add(name);
                if (name.equalsIgnoreCase(default_name))
                    c.select(name);
            }
        }

        add(new Label("\""+current.getDescription()+"\":"));
        if (c.getItemCount() > 0) {
            add(c);
            choices[item_count] = c;
            return true;
        }
        else {
            add(new Label("<no integer-type edge data available>"));
            return false;
        }
    }

    private boolean addIntegerNodeArraySelector(SolverInputItem current) {
        String default_name = current.getName();

        Choice c = new Choice();
        String name;
        for (name = net.GetFirstNodeArrayName() ; name != null ;
             name = net.GetNextNodeArrayName()) {
            //            if (!name.equals(UiNetwork.NodeIdString))
            //                c.add(name);
            if ((net.GetNodeArray(name)).FillObject instanceof Integer) {
                c.add(name);
                if (name.equalsIgnoreCase(default_name))
                    c.select(name);
            }
        }

        add(new Label("\""+current.getDescription()+"\":"));
        if (c.getItemCount() > 0) {
            add(c);
            choices[item_count] = c;
            return true;
        }
        else {
            add(new Label("<no integer-type node data available>"));
            return false;
        }
    }



    /**********************************************************************/
    // component listener methods
    /**********************************************************************/
    public void componentResized(ComponentEvent evt) {
        if (isShowing()) {
            pack();
            UTILITIES.Repaint(this,"componentResized");
        }
    }
    public void componentMoved(ComponentEvent evt)   {
        //        pack();
        //        UTILITIES.Repaint(this,"componentMoved");
    }
    public void componentShown(ComponentEvent evt)   {
        //        pack();
        //        UTILITIES.Repaint(this,"componentShown");
    }
    public void componentHidden(ComponentEvent evt)  {
        //        pack();
        //        UTILITIES.Repaint(this,"componentHidden");
    }


    /**********************************************************************/
    // mouse listener methods
    /**********************************************************************/
    public void mouseClicked(MouseEvent evt) {}
    public void mousePressed(MouseEvent evt) {}
    public void mouseReleased(MouseEvent evt) {}
    public void mouseEntered(MouseEvent evt) {accept.requestFocus();}
    public void mouseExited(MouseEvent evt) {}

    /**********************************************************************/
    // key listener methods
    /**********************************************************************/
    public void keyTyped(KeyEvent evt) {}
    public void keyReleased(KeyEvent evt) {}
    public void keyPressed(KeyEvent evt) {
        int key_code = evt.getKeyCode();
        Object src = evt.getSource();

        if (key_code == KeyEvent.VK_ENTER) {
            if (src.equals(accept))
                acceptCurrentInfo();
            if (src.equals(cancel))
                cancelCurrentInfo();
        }
    }

    /**********************************************************************/
    // action listener method
    /**********************************************************************/
    public void actionPerformed(ActionEvent evt) {
        Object source = evt.getSource();

        if (source.equals(cancel))
            cancelCurrentInfo();
        if (source.equals(accept))
            acceptCurrentInfo();
    }

    /**********************************************************************/
    // focus listener methods
    /**********************************************************************/
    public void focusGained(FocusEvent evt) {accept.requestFocus();}
    public void focusLost(FocusEvent evt) {}



    /**********************************************************************/
    private void cancelCurrentInfo() {
        input.setValidState(false);
        this.dispose();
    }


    private void acceptCurrentInfo() {
        this.setEnabled(false);

        Object obj;
        for (int i=0 ; i < item_count ; i++) {

            obj = null;
            switch (items[i].getType()) {
            case SolverInputItem.INTEGER_EDGE_ARRAY:
                obj = net.GetEdgeArray(choices[i].getSelectedItem());
                break;
            case SolverInputItem.INTEGER_NODE_ARRAY:
                obj = net.GetNodeArray(choices[i].getSelectedItem());
                break;
            }

            if (obj == null) 
                GIDEN.out.println(" ** Inconsistent state (input)");

            input.setItemData(items[i].getDescription(),
                              choices[i].getSelectedItem(), obj);

//GIDEN.out.println(" -- Description=\""+items[i].getDescription()+"\"  ;  "+
//                   "Selection=\""+choices[i].getSelectedItem()+"\"");
        }

        input.setValidState(solver.ValidateInput());
        if (input.isValidated())
            this.dispose();

        this.setEnabled(true);
    }


}
