package giden.contrib.NU_IEMS_452_W04;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;
import java.awt.Point;

/*
* talk about algorithm here
*/

public class MaxDemand extends ExecBase
{
	UserBase UI;
	Network Net;
	GraphState State;

	Color InitialColor;
	Color ElimColor;
	Color SelecColor;
	Color CoveredColor;
	Color UncoveredColor; 
	
	AnimationSet InitialSet;
	AnimationSet Eliminated;	//eliminated facility location
	AnimationSet Selected;		//selected facility location
	AnimationSet Covered;		//node covered by other nodes. 
	AnimationSet Uncovered;

	NodeArray2dInt DemandCover;  // DemandArray[n1,n2] is 0 if D[n1,n2]>CDist; = Demand[n2] otherwise. 
	NodeArray2dInt Coverage;
	NodeArray Candidates;
	
	int TotalDemandsCovered;   // the total number of demands covered. 
	int TotalDemands; 
	NodeArray NodeCoverage;    // the total demand each node can cover ; 

	//LinkList Facility;    // put all the facility nodes in it. 

	int NodeCount;
	int FacilityCount;
		
	int CDist;
	int P;

	boolean ValidNetwork;

	//******************************************
	public MaxDemand(UserBase userinfo)
	{
		super(userinfo);
		UI = userinfo;
	
		InitialColor = Color.cyan;
		ElimColor = Color.lightGray;
		SelecColor = Color.magenta;
		CoveredColor = Color.green;
		UncoveredColor = Color.yellow;

		InitialSet = UI.CreateAnimationSet("",InitialColor);
		Eliminated = UI.CreateAnimationSet("Eliminated",ElimColor);
		Selected = UI.CreateAnimationSet("Selected",SelecColor);
		Covered = UI.CreateAnimationSet("Covered",CoveredColor);
		Uncovered = UI.CreateAnimationSet("Uncovered",UncoveredColor);
		
		UI.CreateAnimationButtons();

	}


	//********************************************
	public boolean Setup(Network net)
	{
		Net = net;
		State = new GraphState(Net);

		NodeCount = Net.GetNodeCount();
		FacilityCount = 0 ; 

		if(NodeCount<2)
		{
			UI.UserMouseNodeInput("Solver needs two or more nodes (click anywhere)");
			ValidNetwork = false;
			return false;
		}
		if(Net.GetEdgeCount() != 0)
		{
			UI.UserMouseNodeInput("Network should have no edges (click anywhere)");
			ValidNetwork = false;
			return false;
		}

		UI.ShowPseudoCodeText("CDist");
		String temp1 = UI.UserTextInput("Enter Coverage Distance:","0");
		CDist = Integer.parseInt(temp1);

		UI.ShowPseudoCodeText("P");
		String temp2 = UI.UserTextInput("Enter the number of facilities to locate:","0");
		P = Integer.parseInt(temp2);

		EdgeArray lengths = Net.GetEdgeArray("Euclidean Length");
		NodeArray NodeDemand = Net.GetNodeArray("Demand");    // the demand at each node. 
		
		if(NodeDemand==null)
		{
			UI.UserMouseNodeInput("Exit and add node data: Demand ");
			ValidNetwork = false;
			return false;
		}

		FillEdgeArray(lengths, NodeDemand);

		Candidates = new NodeArray(NodeCount,"Candidates");
		Candidates.Fill(-1);
		

		
		//Facility = null; 
		TotalDemandsCovered = 0 ;
		FacilityCount = 0;
		
		UI.PutDirected(false);
		UI.MoveAll(InitialSet);
		
		ValidNetwork=true;
		return true;
	}

	//**********************************************
	public boolean Shutdown()
	{
		super.Shutdown();
		State.Restore();
		return true;
	}

	//**************************************************
	public boolean Execute()
	{
		if(ValidNetwork==false)
			return false;
		UI.StatusLine("Executing");
		MaxDemandSolver();

		UI.StatusLine("Final network - Maximum demand covered = "+TotalDemandsCovered);
		UI.ShowPseudoCodeText("end");
		
		//DrawFinalGraph();

		return true;
	}

	//***************************************************
	void FillEdgeArray(EdgeArray lengths, NodeArray NodeDemand)
	{
		UI.StatusLine("Network with edges less than " + CDist + " in length.");	
		UI.ShowPseudoCodeText("Coverage");

		Coverage = new NodeArray2dInt(Net) ;
		DemandCover = new NodeArray2dInt(Net) ;
		NodeCoverage = new NodeArray(NodeCount,"NodeCoverage"); 
		NodeCoverage.Fill(0);
		
		TotalDemands = 0;

		NodeListIter nit1 = Net.GetNodeListIter();
		for(Node node1=nit1.GetFirstNode(); node1!=null; node1=nit1.GetNextNode())
		{
			TotalDemands = TotalDemands + NodeDemand.GetInt(node1); 

			NodeListIter nit2 = Net.GetNodeListIter();
			for(Node node2=nit2.GetFirstNode(); node2!=null; node2=nit2.GetNextNode())
			{
				if(node1.GetIndex()<node2.GetIndex())
				{
					Edge edge=Net.CreateEdge(node1,node2);
					UI.AddEdge(edge);
					UI.PutEdgeVisibilty(edge,false);
					
					int dist=lengths.GetInt(edge);
					
					if (dist<=CDist)
					{
						UI.PutEdgeVisibilty(edge,true);  //display the edges less than CDist. 
						UI.PutLabel(edge,lengths.GetInt(edge));

						DemandCover.PutInt( node1,node2, NodeDemand.GetInt(node2) ); 
						DemandCover.PutInt( node2,node1, NodeDemand.GetInt(node1) ); 
						Coverage.PutInt(node1,node2,1);
						Coverage.PutInt(node2,node1,1);
						NodeCoverage.Put(node1, NodeCoverage.GetInt(node1)+NodeDemand.GetInt(node2));
						NodeCoverage.Put(node2, NodeCoverage.GetInt(node2)+NodeDemand.GetInt(node1));

					}
					else
					{
						DemandCover.PutInt( node1,node2, 0 ); 
						DemandCover.PutInt( node2,node1, 0 ); 
						Coverage.PutInt(node1,node2,0);
						Coverage.PutInt(node2,node1,0);
					}
				}
				else if(node1.GetIndex()==node2.GetIndex())
				{
					DemandCover.PutInt( node1,node2, NodeDemand.GetInt(node2) ); 
					Coverage.PutInt(node1,node2,1);
					NodeCoverage.Put(node1, NodeCoverage.GetInt(node1)+NodeDemand.GetInt(node2));
				}
			}
		}
		
		NodeListIter nit3 = Net.GetNodeListIter();
		for(Node node1=nit3.GetFirstNode(); node1!=null; node1=nit3.GetNextNode())
		{
			NodeListIter nit4 = Net.GetNodeListIter();
			for(Node node2=nit4.GetFirstNode(); node2!=null; node2=nit4.GetNextNode())
			{
				 System.out.print(Coverage.GetInt(node1,node2)+" ");
			}
			
			System.out.println();
		}

		NodeListIter nit5 = Net.GetNodeListIter();
		for(Node node1=nit5.GetFirstNode(); node1!=null; node1=nit5.GetNextNode())
		{
			NodeListIter nit6 = Net.GetNodeListIter();
			for(Node node2=nit6.GetFirstNode(); node2!=null; node2=nit6.GetNextNode())
			{
				 System.out.print(DemandCover.GetInt(node1,node2)+" ");
			}
			
			System.out.println();
		}

		NodeListIter nit7 = Net.GetNodeListIter();
		for(Node node1=nit7.GetFirstNode(); node1!=null; node1=nit7.GetNextNode())
		{
			System.out.print(NodeCoverage.GetInt(node1)+" ");
					
			System.out.println();
		}
		
		
	}

	

	//******************************************************
	public void MaxDemandSolver()
	{
			
		//column reduction. 
		UI.StatusLine("Eliminating inferior nodes from the facility candidates.");
		UI.ShowPseudoCodeText("Inferior");
		int s;
		NodeListIter nit1 = Net.GetNodeListIter();
		for(Node n1=nit1.GetFirstNode(); n1!=null; n1=nit1.GetNextNode())
		{
			if( Candidates.GetInt(n1)== (-1) )  // has not been checked yet; 
			{
				NodeListIter nit2 = Net.GetNodeListIter();
				for(Node n2=nit2.GetFirstNode(); n2!=null; n2=nit2.GetNextNode())
				{
					if(Candidates.GetInt(n2)== (-1) && n1.GetIndex()!=n2.GetIndex())
					{	
						s = 0;
						NodeListIter nit3 = Net.GetNodeListIter();
						for(Node r=nit3.GetFirstNode(); r!=null; r=nit3.GetNextNode())
						{
							if( Coverage.GetInt(r,n1)<=Coverage.GetInt(r,n2) )
								s = s + 1;
						}
						if(s==NodeCount)
						{	Candidates.Put(n1,0);      // Eliminate the node. 
							
							// Animation
							UI.SingleStep(n1,Eliminated);
							UI.Trace(Eliminated);
							
							EdgeListIter eit1 = n1.GetAdjListIter();
							for( Edge e=eit1.GetFirstEdge(); e!=null; e=eit1.GetNextEdge() )
							{
								UI.SingleStep(e,Eliminated);
								UI.Trace(Eliminated);
							}
							UI.Update();
							
						}
					}
				}
			}
		}
		
		// Make all the edges on the graph invisible. 
		
		UI.ShowPseudoCodeText("HideEdges");	
		EdgeListIter eit41 = Net.GetEdgeListIter();
		for( Edge e=eit41.GetFirstEdge(); e!=null; e=eit41.GetNextEdge() )
		{
			UI.SingleStep(e,Eliminated);
			UI.Trace(Eliminated);
			UI.PutEdgeVisibilty(e,false);
		}

		UI.ShowPseudoCodeText("GreedyAdding");
		UI.Update();				
		
				
		
		// Greedy Adding 

		NodeListIter nit4 = Net.GetNodeListIter();
		for(Node node1=nit4.GetFirstNode();node1!=null;node1=nit4.GetNextNode())
		{
			System.out.print(Candidates.GetInt(node1)+" ");
			System.out.println();
		}

		// Find the locations of all the facilities. 
		

		System.out.println("\n");
		
		NodeArray CoveredNodes = new NodeArray(NodeCount,"CoveredNodes");
		CoveredNodes.Fill(0);
		// CoveredNodes = 1 if it's covered =0 otherwise. 

		Node NodeSelected;    NodeSelected = null;
		int MaxDemands;		
		System.out.println("FacilityCount" +FacilityCount);

		while(FacilityCount<P && TotalDemandsCovered<TotalDemands)
		{
			
			MaxDemands = 0;  
			
			NodeListIter nit5 = Net.GetNodeListIter();
			for(Node node1=nit5.GetFirstNode();node1!=null;node1=nit5.GetNextNode())
			{
				// NodeCoverage is the node array with integer value equal to the demands covered by each node. 
				if( NodeCoverage.GetInt(node1) > MaxDemands && Candidates.GetInt(node1)==(-1) )
				{	
					NodeSelected = node1; 
					MaxDemands = NodeCoverage.GetInt(node1);
				}
				
			}
			
			System.out.println("NodeSelected"+NodeCoverage.GetInt(NodeSelected));
			
			CoveredNodes.Put(NodeSelected, 1) ; 

			// Animation
			UI.SingleStep(NodeSelected,Selected);
			UI.Trace(Selected);
			UI.ShowPseudoCodeText("NextFacility");	
			UI.Update();
			
			EdgeListIter eit2 = NodeSelected.GetAdjListIter();
			for( Edge e=eit2.GetFirstEdge(); e!=null; e=eit2.GetNextEdge() )
			{
				if(Coverage.GetInt(NodeSelected,e.GetOpposite(NodeSelected))==1)
				{	
					//update edge.
					UI.PutEdgeVisibilty(e,true);
					UI.SingleStep(e,Selected);
				    UI.Trace(Selected);
					//update the node covered by NodeSelected
					Node NodeCovered = e.GetOpposite(NodeSelected) ;
					if( Candidates.GetInt(NodeCovered)!=1 ) 
					{	
						CoveredNodes.Put(NodeCovered, 1) ; 
						UI.SingleStep(NodeCovered,Covered);
						UI.Trace(Covered); 
					}
				}
			}
			UI.ShowPseudoCodeText("DemandsCovered");	
			UI.Update();

			NodeListIter nit31 = Net.GetNodeListIter();
			for(Node uncovered = nit31.GetFirstNode();uncovered!=null;uncovered=nit31.GetNextNode())
			{
				if(CoveredNodes.GetInt(uncovered)==0)
				{	UI.SingleStep(uncovered,Uncovered);
					UI.Trace(Uncovered); 

					EdgeListIter eit32 = uncovered.GetAdjListIter();
					for( Edge e=eit32.GetFirstEdge(); e!=null; e=eit32.GetNextEdge() )
					{
						UI.PutEdgeVisibilty(e,false);
					}
				}
			}

			UI.Update();

			// update

			TotalDemandsCovered = TotalDemandsCovered + MaxDemands; 
			FacilityCount = FacilityCount + 1;
			System.out.println("FacilityCount" +FacilityCount);

			UI.StatusLine("Total Demands Covered = " + TotalDemandsCovered);	

			Candidates.Put(NodeSelected, 1);      //locate a facility at that node. 

			//NodeCoverage.Put(NodeSelected, 0);
			NodeListIter nit11 = Net.GetNodeListIter();
			for(Node CommonNode=nit11.GetFirstNode();CommonNode!=null;CommonNode=nit11.GetNextNode())
			{
				// DemandArray[n1,n2] is 0 if D[n1,n2]>CDist; = Demand[n2] otherwise. 
				if( DemandCover.GetInt(NodeSelected,CommonNode)>0 )
				{
					NodeListIter nit12 = Net.GetNodeListIter();
					for(Node n=nit12.GetFirstNode(); n!=null; n=nit12.GetNextNode())
						if( DemandCover.GetInt(n,CommonNode)>0 )
							NodeCoverage.Put(n, NodeCoverage.GetInt(n)-DemandCover.GetInt(n,CommonNode));
				}
			}
			

							

			//Facility.AddItem(NodeSelected); 
			System.out.println("NodeCoverage");
			NodeListIter nit7 = Net.GetNodeListIter();
			for(Node node2=nit7.GetFirstNode();node2!=null;node2=nit7.GetNextNode())
			{
				System.out.print(NodeCoverage.GetInt(node2)+" ");
				System.out.println("\n");
			}
			
			System.out.println("Candidates");
			NodeListIter nit8 = Net.GetNodeListIter();
			for(Node node2=nit8.GetFirstNode();node2!=null;node2=nit8.GetNextNode())
			{
				System.out.print(Candidates.GetInt(node2)+" ");
				System.out.println("\n");
			}





		
		}

		
	}








}




	
	
		
