package giden.userSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;

/*
 * WE MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. WE SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY USER AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * SolverMenu is an interface class that provides implementors
 * with a method for creating menu selections (CreateMenu) and
 * responding to those selections (CreateSolver).  The menus are
 * created in a sequence of two steps: first create the problem
 * type (AddProblem) and  then create one or more solvers
 * (AddSolver) for that type of problem.  These problem types
 * and solver names then become menu items under the GIDEN Solver
 * menu selection. After a user selects a particular solver,
 * GIDEN calls CreateSolver method and passes it the name of the
 * solver (as it appears in the menu).  The CreateSolver method
 * then creates the solver and returns the base class of
 * that solver for execution by GIDEN.
 * <p>
 * The SolverMenu class includes two methods to include
 * the GIDEN solvers on the menu and to create them.
 * These methods are called by CreateMenu and CreateSolver.
 * <p>
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @see Network
 * @see SolverMenuBase
 *
 * @version 	2.0b 25 Sep 99
 * @author 	    D. S. Dilworth
 */


public class SolverMenu extends SolverMenuBase
{
    /*----------------------------------------------------------
    | Create strings for implementor solvers
    ----------------------------------------------------------*/
    
    final String TEST_SOLVER="Test Solver";  
    final String FEATURE_EXAMPLE="FeatureExample";
    final String CALL_KRUSKAL="CallKruskal";
    
    final String TSP_SOLVER_EXHAUSTIVE="TspSolverExhaustive";
    final String TSP_SOLVER_NEAREST_NEIGHBOR="TspSolverNearesNeighbor";
    final String TSP_SOLVER_CHRISTO="TspChristofides-Francis&Sahyouni" ;
    final String BIPARTITE_SOLVER ="BipartiteMaxCardMatching-Ilhan&Yuen";
    final String NODE_COVER="NodeCover-Lien&Yang";
    final String MAX_DEMAND="MaxDemand-Lien&Yang";
    final String GLOBAL_MIN_CUT="GlobalMinCut-Curtis&Arda";

    /*----------------------------------------------------------
    | Construct the SolverMenu object without frame
    ----------------------------------------------------------*/
    public SolverMenu()
    {
        CreateMenu();
    }

    //----------------------------------------------------------
    /** Creates the Problem and Solver menu picks.
     * This method is maintained by implementors and is called
     * once during the GIDEN initialization.
     * This method provides implementors with a mechanism
     * for creating the menu picks that appear under the
     * GIDEN Solver menu selection.
     *
     * @see SolverMenuBase#AddProblem
     * @see SolverMenuBase#AddSolver */
    //----------------------------------------------------------
    public void CreateMenu()
    {
        AddProblem("TSP");
        AddSolver(TSP_SOLVER_EXHAUSTIVE);
        AddSolver(TSP_SOLVER_NEAREST_NEIGHBOR);
        AddProblem("Example");
        AddSolver(TEST_SOLVER);
        AddSolver(FEATURE_EXAMPLE);
        AddSolver(CALL_KRUSKAL);    
        AddProblem("IE452W04");    
        AddSolver(TSP_SOLVER_CHRISTO) ;
        AddSolver(BIPARTITE_SOLVER);
        AddSolver(NODE_COVER);
        AddSolver(MAX_DEMAND);
        AddSolver(GLOBAL_MIN_CUT);
        
   }

    //----------------------------------------------------------
    /** Creates a solver, given its name and other GIDEN information.
     * This method is maintained by implementors and is called by
     * GIDEN after a user selects a Solver from the menu.  The
     * implementor must create a UserIO object and pass it to the
     * Solver's constructor.  The Solver constructor returns an
     * ExecBase object that this method returns.
     * @param name   The name of the Solver selected by the user
     * @param draw   The GidenDraw object for user interaction
     * @param net    The network for this solver
     * @return The ExecBase object for the created Solver, or
     * null if the solver couldn't be created*/
    //----------------------------------------------------------
    public ExecBase CreateSolver(String name,SolverServices solvserv,GUISolver gui)
    {
        UserBase user = new UserIO(solvserv,gui);
        ExecBase exec =null;
        
                
        //------------------------------------------------
        // create the solver that was selected by the user
        //------------------------------------------------
        
        if (name.equals(TEST_SOLVER))
            exec = new TestSolver(user);
        if (name.equals(FEATURE_EXAMPLE))
            exec = new FeatureExample(user);
        if (name.equals(CALL_KRUSKAL))
            exec = new CallKruskal(user);         
        if (name.equals(TSP_SOLVER_EXHAUSTIVE))
            exec = new TspSolverExhaustive(user);   
        if (name.equals(TSP_SOLVER_NEAREST_NEIGHBOR))
            exec = new TspSolverNearestNeighbor(user);   
        if (name.equals(TSP_SOLVER_CHRISTO))
            exec = new TspSolverChristo(user) ;
        if (name.equals(BIPARTITE_SOLVER))
            exec = new BipartiteSolver(user);
        if (name.equals(NODE_COVER))
            exec = new NodeCover(user);
        if (name.equals(MAX_DEMAND))
            exec = new MaxDemand(user);
        if (name.equals(GLOBAL_MIN_CUT))
            exec = new GlobalMinCutSolver(user);

        return exec;
    }
}
