package giden.contrib.samples;
import giden.gidenSOLVERS.*;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;

/*
 * Copyright (c) 1993-1999 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * WE MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. WE SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY USER AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <p>
 * Test Solver is an example program that shows some simple animation
 * and network features.
 * <p>
 *
 * @see Network
 * @see SolverMenu
 * @see UserIO
 *
 * @version 	2.2 18 Jan 03
 * @author 	    D. S. Dilworth
 */


public class CallKruskal extends ExecBase
{
    UserBase UI;                  // user interactions
    Network Net;                  // access to network
    
    AnimationSet StartSet;        // StartSet data member
    AnimationSet EndSet;          // EndSet data member  
    String EdgeArrayName;         // name of edge array for length
   
    /** Initializes invariant objects */
    public CallKruskal (UserBase userinfo)
    {
        super(userinfo);        // mandatory call to the base class
        UI =  userinfo;         // local access for user interactions
        StartSet = UI.CreateAnimationSet("",Color.gray);
        EndSet   = UI.CreateAnimationSet("End",Color.red);       
        UI.CreateAnimationButtons();
    }
    
    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
        Net=net;                        // local access to the network
        EdgeArrayName = "Euclidean Length";
        UI.StatusLine("Executing with "+ EdgeArrayName + " edge array");         // status line     
        UI.MoveAll(StartSet);           // nodes/edges -> StartSet
        return true;
    }
    
    /** Executes the solver */
    public boolean Execute()
    {      
        UI.PutDirected(false);      // draw undirected                  
        
        UserBase nullbase = new UserBase(UI.Services);      // UserBase for Kruskal
        Kruskal kruskal = new Kruskal(nullbase);            // create Kruskal
        EdgeArray length = Net.GetEdgeArray(EdgeArrayName); // get length array
        kruskal.PutLength(length);                          // give array to Kruskal
        kruskal.Setup(Net);                                 // setup Kruskal
        kruskal.Execute();                                  // run Kruskal
        int forest_length = kruskal.GetForestLength();      // get forest length
        // get the forest edge list, and clone it, Kruskal clears the list at shutdown
        LinkList forest_edges = (LinkList)(kruskal.GetForestEdges().clone());
        kruskal.Shutdown();                                 // shutdown Kruskal
        
        // show the results
        Edge edge;
        Node node;      
        EdgeListIter eit = new EdgeListIter(forest_edges);
        for(edge = eit.GetFirstEdge(); edge!=null; edge=eit.GetNextEdge())
        {
            UI.SingleStep(edge,EndSet);     // draw the edge
            node = edge.GetSource();        // get the source node
            UI.SingleStep(node,EndSet);     // draw the source node
            UI.PutLabel(node,"");           // clear source node label
            node = edge.GetTarget();        // get the target node
            UI.SingleStep(node,EndSet);     // draw the target node
            UI.PutLabel(node,"");           // clear the target node lab
        }
        UI.Trace(EndSet);
        
        UI.StatusLine("Forest Length from Kruskal = "+forest_length);   // show length
        
        return true;
    }   
}
