package giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;

/**********************************************************************/
// Cycle Canceling class
/**********************************************************************/
public class CycleCanceling extends MinCostFlow {

    private AnimationSet Cycle;
    private ColorSet     Bottleneck;

    private boolean feasible;

    /****************************************************************/
    // CycleCanceling constructor
    /****************************************************************/
    public CycleCanceling(UserBase userinfo) {
        super(userinfo);

        Cycle      = UI.CreateAnimationSet("Cycle",Color.blue);
        Bottleneck = UI.CreateColorSet(Color.red);
    }

    public String getSolverName() {return "Cycle Canceling";}

    /****************************************************************/
    // Setup
    //   - calls MinCostFlow base class Setup()
    //   - initializes solver specific information
    /****************************************************************/
    public boolean Setup(Network net) {
        if(super.Setup(net)==false)
            return false;

        InitializeNetworkLabels(false);
        feasible = true;

        return true;
    }

    /****************************************************************/
    // Shutdown
    /****************************************************************/
    public boolean Shutdown() {
        UI.StatusLine("");
        return super.Shutdown();
    }

    /****************************************************************/
    // CycleCanceling Execute
    /****************************************************************/
    public boolean Execute() {
        //        EdgeArray Cost = GetCostArray();
        //        EdgeArray Capacity = GetCapacityArray();
        //        NodeArray Supply = GetSupplyArray();
        EdgeArray Flow = GetFlowArray();
        NodeArray Potential = GetPotentialArray();

        Network Net = GetNetwork();
        
        CycleCancelingSolver(Net,Cost,Flow,Capacity,Supply);
        // FindOptimalNodePotentials();
        UI.ShowPseudoCodeText("end");
        if (feasible)
            DisplayFinalSolution(Net,Cost,Flow,Capacity,Potential,Supply);
        else
            ReportMaxFlowInfeasibility(Net,Flow,Capacity,Supply);

        return true;
    }

    /****************************************************************/
    // CycleCanceling CycleCancelingSolver
    /****************************************************************/
    private void CycleCancelingSolver(Network   Net,
                                      EdgeArray Cost,
                                      EdgeArray Flow,
                                      EdgeArray Capacity,
                                      NodeArray Supply) {
        UI.StatusLine("Cycle Canceling Solver begins...");
        UI.ShowPseudoCodeText("start");
        UI.StatusLine("");
        
        int cycle_count = 0;
        NodeArray Pred = new NodeArray(Net.GetNodeIndex(),"Pred");

	GetInitialFlow(Net,Cost,Flow,Capacity,Supply);
        if (!feasible) return;

        Node i;
        UI.ShowPseudoCodeText("main loop");
        Node CycleRoot = FindNegativeCostCycle(Net,Cost,Flow,Capacity,Pred);
        while (CycleRoot != null) {
            cycle_count++;

            // augment flow
            AugmentFlow(Net,CycleRoot,Cost,Flow,Capacity,Pred);

            // check for next cycle
            CycleRoot = FindNegativeCostCycle(Net,Cost,Flow,Capacity,Pred);

            UI.ShowPseudoCodeText("main loop");
        }
    }

    /****************************************************************/
    private void AugmentFlow(Network   Net,
                             Node      CycleRoot,
                             EdgeArray Cost,
                             EdgeArray Flow,
                             EdgeArray Capacity,
                             NodeArray Pred) {
        int delta;
        int cost;
        Edge e;
        Node i;


        // calculate the cost and the residual capacity of cycle
        e = Pred.GetEdge(CycleRoot);
        if (CycleRoot == e.GetTarget()) {
            delta = Capacity.GetInt(e) - Flow.GetInt(e);
            cost = Cost.GetInt(e);
        }
        else {
            delta = Flow.GetInt(e);
            cost = -Cost.GetInt(e);
        }
        UI.SingleStep(CycleRoot,Cycle);
        UI.SingleStep(e,Cycle);

        i = e.GetOpposite(CycleRoot);
        e = Pred.GetEdge(i);
        while (i != CycleRoot) {
            if (i == e.GetTarget()) {
                delta = Math.min(delta, Capacity.GetInt(e)-Flow.GetInt(e));
                cost += Cost.GetInt(e);
            }
            else {
                delta = Math.min(delta, Flow.GetInt(e));
                cost -= Cost.GetInt(e);
            }
            UI.SingleStep(i,Cycle);
            UI.SingleStep(e,Cycle);

            i = e.GetOpposite(i);
            e = Pred.GetEdge(i);
        }
        UI.ShowPseudoCodeText("id cycle");
        UI.StatusLine("Cycle residual capacity = "+delta+".  "+
                      "Cycle cost = "+cost);
        UI.Trace(Cycle);
        UI.ShowPseudoCodeText("calc residual");
        UI.Update();

        // update flow along the cycle
        i = CycleRoot;
        e = Pred.GetEdge(CycleRoot);
        do {
            if (i == e.GetTarget()) {
                Flow.Put(e, Flow.GetInt(e)+delta);
                PutEdgeLabel(e,Flow,Capacity,Cost);
                if (Flow.GetInt(e) == Capacity.GetInt(e))
                    UI.SingleStep(e,Bottleneck);
            }
            else {
                Flow.Put(e, Flow.GetInt(e)-delta);
                PutEdgeLabel(e,Flow,Capacity,Cost);
                if (Flow.GetInt(e) == 0)
                    UI.SingleStep(e,Bottleneck);
            }
            i = e.GetOpposite(i);
            e = Pred.GetEdge(i);
        } while (i != CycleRoot);
        FlowCost += delta*cost;
        UI.StatusLine("Flow Updated.  New flow cost = $"+FlowCost+".");
        UI.SingleStep(Cycle);
        UI.Trace(Cycle);
        UI.ShowPseudoCodeText("augment flow");
        UI.Update();
        UI.StatusLine("");


        // return cycle to initial set
        i = CycleRoot;
        e = Pred.GetEdge(CycleRoot);
        do {
            UI.SingleStep(i,InitialSet);
            UI.SingleStep(e,InitialSet);

            i = e.GetOpposite(i);
            e = Pred.GetEdge(i);
        } while (i != CycleRoot);
        
    }

    /****************************************************************/
    private Node GetCycleRoot(Network   Net,
                              Node      StartNode,
                              NodeArray Pred) {
        NodeArray Seen = new NodeArray(Net.GetNodeIndex(),"Seen");
        Seen.Fill(Boolean.FALSE);

        Node i=StartNode;
        Seen.Put(StartNode,Boolean.TRUE);

        // identify the cycle
        i = (Pred.GetEdge(StartNode)).GetOpposite(StartNode);
        while (Seen.GetObj(i).equals(Boolean.FALSE)) {
            Seen.Put(i,Boolean.TRUE);
            i = (Pred.GetEdge(i)).GetOpposite(i);
        }

        // we know that node i is in the cycle
        return i;
    }

    /****************************************************************/
    private Node FindNegativeCostCycle(Network   Net,
                                       EdgeArray Cost,
                                       EdgeArray Flow,
                                       EdgeArray Capacity,
                                       NodeArray Pred) {
        Edge e;
        Node i,j;
        EdgeListIter eit;
        NodeListIter nit;

        // FIFO Label Correcting (see AMO)
        BoundedQueue BQ = new BoundedQueue(Net.GetNodeIndex());
        NodeArray InQueue = new NodeArray(Net.GetNodeIndex(),"InQueue");
        NodeArray NumExaminations = new NodeArray(Net.GetNodeIndex(),"NumExams");
        NodeArray Distance = new NodeArray(Net.GetNodeIndex(),"Distance");

        int MAX_WT = 1;
        eit = Net.GetEdgeListIter();
        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
            MAX_WT += Cost.GetInt(e);
        Distance.Fill(MAX_WT);

        InQueue.Fill(Boolean.FALSE);
        NumExaminations.Fill(0);

        // setup
        Pred.Fill(null);
        nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode()) {
            BQ.Append(i);
            InQueue.Put(i,Boolean.TRUE);
        }

        Distance.Put((Node)BQ.GetTop(),0);
        while (!BQ.IsEmpty()) {
            i = (Node) BQ.Pop();
            InQueue.Put(i,Boolean.FALSE);

            //----------------------------------------------------------
            // examine forward edges in original network with residual
            // capacity that can be used to improve the current node
            // labels
            //----------------------------------------------------------
            eit = i.GetOutListIter();
            for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
                j = e.GetTarget();
                if (Capacity.GetInt(e) - Flow.GetInt(e) > 0 && 
                    Distance.GetInt(j) > Distance.GetInt(i)+Cost.GetInt(e)) {
                    Pred.Put(j,e);

                    NumExaminations.Put(j,NumExaminations.GetInt(j)+1);
                    if (NumExaminations.GetInt(j) >= Net.GetNodeCount()) {
                        // negative-cost cycle detected
                        return GetCycleRoot(Net,i,Pred); 
                    }

                    Distance.Put(j,Distance.GetInt(i)+Cost.GetInt(e));
                    if (InQueue.GetObj(j).equals(Boolean.FALSE)) {
                        InQueue.Put(j,Boolean.TRUE);
                        BQ.Append(j);
                    }
                }
            }

            //----------------------------------------------------------
            // examine reverse edges in original network with residual
            // capacity that can be used to improve the current node
            // labels
            //----------------------------------------------------------
            eit = i.GetInListIter();
            for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
                j = e.GetSource();
                if (Flow.GetInt(e) > 0 &&
                    Distance.GetInt(j) > Distance.GetInt(i)-Cost.GetInt(e)) {
                    Pred.Put(j,e);
                    NumExaminations.Put(j,NumExaminations.GetInt(j)+1);
                    if (NumExaminations.GetInt(j) >= Net.GetNodeCount()) {
                        // negative-cost cycle detected
                        return GetCycleRoot(Net,i,Pred); 
                    }

                    Distance.Put(j,Distance.GetInt(i)-Cost.GetInt(e));
                    if (InQueue.GetObj(j).equals(Boolean.FALSE)) {
                        InQueue.Put(j,Boolean.TRUE);
                        BQ.Append(j);
                    }
                }
            }
        }
        
        BQ.DeleteContents();


        return (Node) null;
    }

    /****************************************************************/
    private void GetInitialFlow(Network   Net,
                                EdgeArray Cost,
                                EdgeArray Flow,
                                EdgeArray Capacity,
                                NodeArray Supply) {

        GraphState SaveGraph = new GraphState(Net);

        Node asource = Net.CreateNode();
        Node asink = Net.CreateNode();

        int TotalSupply = 0;

        Node n;
        Edge e;
        NodeListIter nit = Net.GetNodeListIter();
        for (n=nit.GetFirstNode() ; n!=null ; n=nit.GetNextNode()) {
            if (Supply.GetInt(n) > 0) {
                e = Net.CreateEdge(asource,n);
                Capacity.Put(e,Supply.GetInt(n));
                TotalSupply += Supply.GetInt(n);
            }
            else if (Supply.GetInt(n) < 0) {
                e = Net.CreateEdge(n,asink);
                Capacity.Put(e,-Supply.GetInt(n));
            }
        }
        
        // call a max-flow solver to get an initial feasible flow
        UserBase nullbase = new UserBase(UI.Services);
        MaxFlow solver = new PreFlowPush(nullbase);
        
        solver.Setup(Net);
        
        solver.Source = asource;
        solver.Sink = asink;
        solver.Capacity = Capacity;
        
        solver.Execute();
        GIDEN.out.println("Flow Value= "+solver.FlowValue);
        
        // restore network and save flow values
        SaveGraph.Restore();
        EdgeListIter eit = Net.GetEdgeListIter();
        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
            Flow.Put(e, solver.Flow.GetInt(e));
            PutEdgeLabel(e,Flow,Capacity,Cost);
            FlowCost += Flow.GetInt(e)*Cost.GetInt(e);
        }
        
        // check feasibility; is flow value equal to sum of supplies (demands)?
        feasible = (solver.FlowValue == TotalSupply);
            
        solver.Shutdown();

        if (!feasible) {
            UI.StatusLine("Could not find an initial feasible flow!");
            UI.ShowPseudoCodeText("setup");
            UI.StatusLine("");
            return;
        }

        // show initial flow, if the cycle animation set is selected
        UI.StatusLine("Initial flow generated using a max-flow solver.  "+
                      "Flow cost = $"+FlowCost+".");
        UI.ShowPseudoCodeText("setup");
        UI.SingleStep(Cycle);
        UI.Trace(Cycle);
        UI.Update();
        UI.StatusLine("");
    }


    /****************************************************************/
}
