package giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;

/**********************************************************************/
// Dijkstra class
/**********************************************************************/
public class Dijkstra extends ShortestPath
{
    AnimationSet Trial;
    AnimationSet Acquired;
    AnimationSet Discarded;

    boolean NegLengths;

    public boolean ValidateInput() {

        // extract length array
        Object data = input.getItemData("Length");
        if (!(data instanceof EdgeArray))
            return false;

        // check length data (no negative-length edges allowed by Dijkstra)
        if (NegLengthValuesExist(GetNetwork(), (EdgeArray) data)) {
            UI.NotifyOfSolverError("Negative values exist in selected \"Length\" array.");
            return false;
        }

        return true;
    }


    /******************************************************************/
    // Dijkstra constructor
    /******************************************************************/
    public Dijkstra(UserBase userinfo) {
        super(userinfo);

        Trial     = UI.CreateAnimationSet("Trial",Color.red);
        Acquired  = UI.CreateAnimationSet("Acquired",new Color(48,175,48));
        Discarded = UI.CreateAnimationSet("Discarded",Color.yellow);
    }

    public String getSolverName() {return "Dijkstra";}

    /******************************************************************/
    // Setup
    //   - calls ShortestPath base class Setup()
    //   - initializes solver specific information
    //   - makes sure that input data is valid for solver
    /******************************************************************/
    public boolean Setup(Network net) {
        if(super.Setup(net)==false)
            return false;

        if (input.isValidated()) 
            return true;

        // if input hasn't already been checked, then do it here
        return (!NegLengthValuesExist(net,GetLengthArray()));
    }

    /******************************************************************/
    // Dijkstra Execute
    /******************************************************************/
    public boolean Execute() {
        if (NegLengths) {
            UI.StatusLine("Please EXIT Dijkstra --- network contains negative edge lengths.");
            return false;
        }

        Node Source = GetSourceNode();
        //        EdgeArray Length = GetLengthArray();
        //        NodeArray Distance = GetDistanceArray();
        //        NodeArray Pred = GetPredArray();

        Network Net = GetNetwork();

        DijkstraSolver(Net,Source,Length,Distance,Pred);
        DisplaySPTree(Net,Distance,Pred,Length);

        return true;
    }

    /******************************************************************/
    // Dijkstra DijkstraSolver
    /******************************************************************/
    public void DijkstraSolver(Network   Net,
                               Node      Source,
                               EdgeArray Length,
                               NodeArray Distance,
                               NodeArray Pred) {

        UI.StatusLine("Dijkstra Solver begins...");
        UI.ShowPseudoCodeText("start");
        UI.StatusLine("");

        PQList PQ;
        Node i,j;
        Edge e;
        EdgeListIter eit;
        NodeListIter nit;


        // init the arrays
        Distance.Fill(INFINITY);
        Pred.Fill(null);

        UI.ShowPseudoCodeText("init dist");
        UI.ShowPseudoCodeText("init sets");

        // distance to source node is 0
        Distance.Put(Source,0);
        UI.PutLabel(Source,"0");
        UI.ShowPseudoCodeText("dist(s)");
        UI.ShowPseudoCodeText("move s");

        // fill the PQ
        PQ = new PQList();
        nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i != null ; i=nit.GetNextNode())
            PQ.Insert(i,Distance.GetInt(i));

        // main loop
        while (PQ.NotEmpty()) {
            i = PQ.DeleteMinNode();

            if (Distance.GetInt(i) == INFINITY) {
                UI.StatusLine("Network is not connected.");
                break;
            }
            UI.ShowPseudoCodeText("main loop");

            UI.SingleStep(i,Acquired);
            if (Pred.GetEdge(i) != null) {
                e = Pred.GetEdge(i);
                UI.SingleStep(e,Acquired);
                TreeLength += Length.GetInt(e);
            }

            UI.ShowPseudoCodeText("select node");
            UI.ShowPseudoCodeText("move node");
            UI.Trace(Acquired);
            UI.Update();

            // if PQ is empty, then we are done
            if (PQ.IsEmpty())
                break;

            eit = i.GetOutListIter();
            for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {

                UI.SingleStep(e,Trial);
                UI.ShowPseudoCodeText("adj nodes");

                j = e.GetOpposite(i);
                UI.ShowPseudoCodeText("check dist");
                if (Distance.GetInt(j) > Distance.GetInt(i) + Length.GetInt(e)) {
                    Distance.Put(j,Distance.GetInt(i) + Length.GetInt(e));
                    PutNodeLabel(j);
                    UI.ShowPseudoCodeText("update dist");
                    PQ.DecreaseValue(j,Distance.GetInt(j));
                    if (Pred.GetEdge(j) != null)
                        UI.SingleStep(Pred.GetEdge(j),Discarded);
                    Pred.Put(j,e);
                    UI.SingleStep(j,Trial);
                    UI.ShowPseudoCodeText("update pred");
                    UI.ShowPseudoCodeText("update set");
                }
                else
                    UI.SingleStep(e,Discarded);

                UI.Trace(Trial);
                UI.Trace(Discarded);
                UI.Update();
            }
        }
        UI.ShowPseudoCodeText("main loop");

        UI.ShowPseudoCodeText("end");

        UI.StatusLine("Dijkstra Solver finished.  "
                      +"Length of shortest path tree = "
                      +TreeLength +" units.");
    }
}
