package giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;

/**********************************************************************/
// LabelCorrecting (FIFO) class
/**********************************************************************/
public class  LabelCorrect extends ShortestPath {

    protected Node CycleRoot;

    private AnimationSet Accepted;
    private AnimationSet Current;
    private AnimationSet Discarded;

    final int IN_BQ = 1;
    final int NOT_IN_BQ = 2;

    /****************************************************************/
    // LabelCorrect constructor
    /****************************************************************/
    public LabelCorrect(UserBase userinfo) {
        super(userinfo);

        Accepted  = UI.CreateAnimationSet("Accepted",new Color(48,175,48));
        Current   = UI.CreateAnimationSet("Current",Color.red);
        Discarded = UI.CreateAnimationSet("Discarded",Color.yellow);
    }

    public String getSolverName() {return "Label Correcting";}

    /****************************************************************/
    // Setup
    //   - calls ShortestPath base class Setup()
    //   - initializes solver specific information
    /****************************************************************/
    public boolean Setup(Network net) {
        if(super.Setup(net)==false)
            return false;

	CycleRoot = null;

        return true;
    }

    /********************************************************************/
    // Shutdown
    /********************************************************************/
    public boolean Shutdown() {
        CycleRoot = null;

        return super.Shutdown();
    }

    /****************************************************************/
    // LabelCorrect Execute
    /****************************************************************/
    public boolean Execute() {
        Node Source = GetSourceNode();
        //        EdgeArray Length = GetLengthArray();
        //        NodeArray Distance = GetDistanceArray();
        //        NodeArray Pred = GetPredArray();

        Network Net = GetNetwork();

        CycleRoot = LabelCorrectSolver(Net,Source,Length,Distance,Pred);
        if (CycleRoot != null)
            ExamineNegCycle(Net,CycleRoot,Pred,Length);
        else
            DisplaySPTree(Net,Distance,Pred,Length);

        return true;
    }

    /****************************************************************/
    // LabelCorrect LabelCorrectSolver
    /****************************************************************/
    public Node LabelCorrectSolver(Network   Net,
                                   Node      Source,
                                   EdgeArray Length,
                                   NodeArray Distance,
                                   NodeArray Pred) {

        UI.StatusLine("FIFO Label-Correcting begins...");
        UI.ShowPseudoCodeText("start");
        UI.StatusLine("");

        Edge e;
        Node i,j;
        EdgeListIter eit;
        BoundedQueue BQ = new BoundedQueue(Net.GetNodeIndex());
        NodeArray NodeStatus = new NodeArray(Net.GetNodeIndex(),"NodeStatus");
        NodeArray NumExaminations = new NodeArray(Net.GetNodeIndex(),"NumExams");

        // init the arrays
        Pred.Fill(null);
        Distance.Fill(INFINITY);
        NodeStatus.Fill(NOT_IN_BQ);
        NumExaminations.Fill(0);

        // Setup
        Distance.Put(Source,0);
        UI.ShowPseudoCodeText("dist(s)");
        UI.ShowPseudoCodeText("init dist");

        BQ.Append(Source);
        NodeStatus.Put(Source,IN_BQ);
        NumExaminations.Put(Source,1);
        PutNodeLabel(Source);
        UI.SingleStep(Source,Accepted);
        UI.ShowPseudoCodeText("init queue");
        UI.Trace(Accepted);
        UI.Update();        

        while (!BQ.IsEmpty()) {
            UI.ShowPseudoCodeText("main loop");
            
            i = (Node) BQ.Pop();
            NodeStatus.Put(i,NOT_IN_BQ);
            UI.SingleStep(i,Current);
            UI.ShowPseudoCodeText("pop queue");
            UI.Trace(Current);
            UI.Update();
            
            eit = i.GetOutListIter();
            for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
                UI.SingleStep(e,Current);
                UI.ShowPseudoCodeText("adj nodes");
                UI.Trace(Current);
                UI.Update();
                
                j = e.GetTarget();
                UI.ShowPseudoCodeText("check dist");
                if (Distance.GetInt(j) > Distance.GetInt(i) + Length.GetInt(e)) {
                    
                    if (Pred.GetEdge(j) != null) {
                        TreeLength -= Length.GetInt(Pred.GetEdge(j));
                        UI.SingleStep(Pred.GetEdge(j),Discarded);
                    }
                    TreeLength += Length.GetInt(e);
                    UI.SingleStep(e,Accepted);
                    
                    Distance.Put(j,Distance.GetInt(i) + Length.GetInt(e));
                    PutNodeLabel(j);
                    
                    
                    UI.ShowPseudoCodeText("update dist");
                    Pred.Put(j,e);
                    UI.ShowPseudoCodeText("update pred");
                    if (NodeStatus.GetInt(j) == NOT_IN_BQ) {
                        BQ.Append(j);
                        NodeStatus.Put(j,IN_BQ);
                        
                        UI.SingleStep(j,Accepted);
                        NumExaminations.Put(j,NumExaminations.GetInt(j)+1);
                        if (NumExaminations.GetInt(j) >= Net.GetNodeCount()) {
                            return i;
                        }
                    }
                    UI.ShowPseudoCodeText("update queue");
                    UI.Trace(Discarded); // trace here if switch preds so seems simultaneous
                    UI.Trace(Accepted);
                    UI.Update();
                }
                else {
                    UI.SingleStep(e,Discarded);
                    UI.Trace(Discarded);
                    UI.Update();
                }
            }
            UI.SingleStep(i,Accepted);
            UI.Trace(Accepted);
            UI.Update();
        }
        UI.ShowPseudoCodeText("main loop");
        
        UI.StatusLine("FIFO Label-Correcting Solver finished.");
        UI.ShowPseudoCodeText("end");
        
        return ((Node) null);
    }
}
