package  giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;


/**********************************************************************/
// Minimum Spanning Tree Problem base class
/**********************************************************************/
abstract public class MinSpan extends ExecBase {

    protected GidenSolverInput input;

    protected UserBase UI;
  
    // problem data
    protected EdgeArray Length = null;
    protected int       ForestLength;
    private   LinkList  Forest;


    private Network Net;
    private ColorSet InitialSet;
    private FinalSet MinSpanForest;
    
    protected boolean UseResults; // default=false, true if old results in network
                        // dsd 010401 and below in Setup
                        
    public void PutLength(EdgeArray length)
    {
        Length=length;
    }
    
    public int GetForestLength()
    {
        return ForestLength;
    }
    
    public LinkList GetForestEdges()
    {
        return Forest;
    }
    
    
    /********************************************************************/
    // MinSpan constructor
    /********************************************************************/
    public MinSpan(UserBase userinfo) {
        super(userinfo);

        UI=userinfo;
        UseResults=false;
        InitialSet    = UI.CreateColorSet(Color.lightGray);
        MinSpanForest = UI.CreateFinalSet("Minimum Spanning Forest",
                                          Color.orange);

        input = new GidenSolverInput();
        input.addItem(SolverInputItem.INTEGER_EDGE_ARRAY,"Length","Weight");
    }

    abstract public String getSolverName();

    /********************************************************************/
    // Setup
    /********************************************************************/
    public boolean Setup(Network net) {
        GIDEN.out.println("** MinSpan Setup()");
        if(net==null)
            return false;
        
        super.Setup(net);

        Net = net;
        UI.PutDirected(false);
        
        if(UseResults)      // dsd 010401 and else brackets
        {
            Length = Net.GetEdgeArray("PrimEdgeLengths");
        }
        else
        {

            if (!UI.NullBase) {
                UI.CreateSolverInputDialog(getSolverName(), this, net, input);
                if (!input.isValidated())
                    return false;

                // extract input info
                Object data = input.getItemData("Length");
                if (data instanceof EdgeArray)
                    Length = (EdgeArray) data;
            }

            if (Length == null)
                Length = Net.GetEdgeArray("Weight");
            if (Length == null)
                Length = Net.GetEdgeArray("Euclidean Length");
        }

        Forest = new LinkList();
        ForestLength = 0;

        InitializeNetworkLabels();

        UI.MoveAll(InitialSet);
    
        return true;
    }
    
    /********************************************************************/
    // Shutdown
    /********************************************************************/
    public boolean Shutdown() {
        GIDEN.out.println("** MinSpan Shutdown()");
        ForestLength = 0;
        Forest.DeleteContents();

        return super.Shutdown();
    }

    /********************************************************************/
    /********************************************************************/
    protected LinkList GetForestList() {return Forest;}
    protected Network GetNetwork() {return Net;}

    /********************************************************************/
    protected void PutEdgeLabel(Edge e) {
        String label = "" + Length.GetInt(e);
        UI.PutLabel(e,label);
    }

    /********************************************************************/
    protected void PutNodeLabel(Node i) {
        UI.PutLabel(i,"");
    }

    /********************************************************************/
    private void InitializeNetworkLabels() {
        // initialize edge labels
        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge())
            PutEdgeLabel(e);

        // initialize node labels
        NodeListIter nit = Net.GetNodeListIter();
        for (Node i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode())
            PutNodeLabel(i);
    }

    /********************************************************************/
    protected void DisplayMinSpanForest(Network   Net,
                                        LinkList  Forest,
                                        EdgeArray Length) {
        Edge e;
        int tmpForestLength = 0;
        NodeListIter nit = Net.GetNodeListIter();

        UI.MoveAll(InitialSet);
        
        int edgecount = 0;

        NodeArray Reached = new NodeArray(Net.GetNodeIndex(),"Reached");
        Reached.Fill(Boolean.FALSE);

        ListEntry entry=Forest.GetFirstEntry();
        while (entry != null) {
            e = (Edge) entry.GetItem();
            tmpForestLength += Length.GetInt(e);

            UI.SingleStep(e.GetSource(),MinSpanForest);
            UI.SingleStep(e.GetTarget(),MinSpanForest);
            Reached.Put(e.GetSource(),Boolean.TRUE);
            Reached.Put(e.GetTarget(),Boolean.TRUE);

            UI.SingleStep(e,MinSpanForest);
            entry = entry.GetNext();

            edgecount++;
        }

        boolean allNodesReached = true;
        for (Node i = nit.GetFirstNode(); i != null ; i = nit.GetNextNode())
            if (Reached.GetObj(i).equals(Boolean.FALSE))
                allNodesReached = false;

        if (tmpForestLength != ForestLength)
            UI.StatusLine("Error detected in forest length calculation!");

        else if (edgecount < Net.GetNodeCount()-1 && !allNodesReached) {
            UI.StatusLine("Network is not connected.  "
                          +"R = {i in N : d(i) < \"inf\"}, U = N\\R.  "
                          +"Length of min span tree on R = "
                          +ForestLength+" units.");
            MinSpanForest.PutLabel("Minimum Spanning Tree on R");
        }
        else if (edgecount < Net.GetNodeCount()-1 && allNodesReached) {
            UI.StatusLine("Forest length = "+ForestLength+" units.");
            MinSpanForest.PutLabel("Forest of Component-wise Minimum Spanning Trees");
        }
        else { // edgecount == Net.GetNodeCount()-1 && allNodesReached
            UI.StatusLine("Minimum spanning tree length = "
                          +ForestLength+" units.");
            MinSpanForest.PutLabel("Minimum Spanning Tree");
        }

        //        UI.ShowPseudoCodeText("end");
        UI.Trace(MinSpanForest);
        UI.Update();
    }
}
