package  giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.util.Enumeration;
import java.util.Vector;
import java.awt.Color;

/**********************************************************************/
// Network Simplex class
/**********************************************************************/
public class NetworkSimplex extends MinCostFlow {

    private AnimationSet Tree;
    private AnimationSet Entering;
    private AnimationSet Leaving;

    private boolean feasible;

    private final int INFINITY = Integer.MAX_VALUE;

    private int iteration_count;

    /****************************************************************/
    // NetworkSimplex constructor
    /****************************************************************/
    public NetworkSimplex(UserBase userinfo) {
        super(userinfo);

        Tree     = UI.CreateAnimationSet("Tree",new Color(48,175,48));
        Entering = UI.CreateAnimationSet("Entering Edge",Color.red);
        Leaving  = UI.CreateAnimationSet("Leaving Edge",Color.yellow);
    }

    public String getSolverName() {return "Network Simplex";}

    /****************************************************************/
    // Setup
    //   - calls MinCostFlow base class Setup()
    //   - initializes solver specific information
    /****************************************************************/
    public boolean Setup(Network net) {
        if(super.Setup(net)==false)
            return false;

        InitializeNetworkLabels(true);
        feasible = true;
        iteration_count = 0;

        return true;
    }

    /****************************************************************/
    // Shutdown
    /****************************************************************/
    public boolean Shutdown() {
        UI.StatusLine("");
        iteration_count = 0;

        GIDEN.out.println("** Shutdown()");

        return super.Shutdown();
    }

    /****************************************************************/
    // NetworkSimplex Execute
    /****************************************************************/
    public boolean Execute() {
        //        EdgeArray Cost = GetCostArray();
        //        EdgeArray Capacity = GetCapacityArray();
        //        NodeArray Supply = GetSupplyArray();
        EdgeArray Flow = GetFlowArray();
        NodeArray Potential = GetPotentialArray();
        
        Network Net = GetNetwork();

        NetworkSimplexSolver(Net,Cost,Flow,Capacity,Potential,Supply);
        if (!feasible)
            ReportMaxFlowInfeasibility(Net,Flow,Capacity,Supply);

        return true;
    }

    /****************************************************************/
    // NetworkSimplex NetworkSimplexSolver
    /****************************************************************/
    private void NetworkSimplexSolver(Network   Net,
                                      EdgeArray Cost,
                                      EdgeArray Flow,
                                      EdgeArray Capacity,
                                      NodeArray Pi,
                                      NodeArray Supply) {
        UI.StatusLine("Network Simplex Solver begins...");
        UI.ShowPseudoCodeText("start");
        UI.StatusLine("");

        //        FindCycleFreeFlow(Net,Cost,Flow,Capacity,Supply);
        // strongly feasible tree storage information
        NodeArray Depth  = new NodeArray(Net.GetNodeIndex(),"Depth");
        NodeArray Pred   = new NodeArray(Net.GetNodeIndex(),"Pred");
        NodeArray Thread = new NodeArray(Net.GetNodeIndex(),"Thread");

        // edge lists
        Vector T = new Vector(Net.GetNodeCount());
        Vector L = new Vector(Net.GetEdgeCount());
        Vector U = new Vector(Net.GetEdgeCount());


        //------------------------------------------------------------
        // Network Simplex Phase-1:
        //     - find an initial feasible flow
        //     - find a strongly feasible spanning tree structure
        //------------------------------------------------------------
        EdgeArray Phase1Cost = new EdgeArray(Net.GetEdgeIndex(),"Phase1Cost");
        GraphState SaveGraph = new GraphState(Net);
        Node root = InitializeTree(Net,Flow,Pi,Capacity,Phase1Cost,Cost,
                                   T,L,U,Depth,Pred,Thread);

        // for phase-1, run the network simplex solver with "dummy" userbase
        UserBase nullbase = new UserBase(UI.Services);
        UserBase realbase = UI;
        UI = nullbase;
        NetworkSimplexAlgorithm(root,Flow,Pi,Supply,Capacity,Phase1Cost,
                                T,L,U,Depth,Pred,Thread);

        // check feasibility of problem
        int infeasible_flow = 0;
        Edge e;
        EdgeListIter eit = root.GetInListIter();
        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
            infeasible_flow += Flow.GetInt(e);
        if (infeasible_flow > 0) {
            UI = realbase;
            feasible = false;
            UI.StatusLine("Phase-I detected infeasibility!");
            UI.ShowPseudoCodeText("feasible tree");
            UI.StatusLine("");
            SaveGraph.Restore();
            UI.ShowPseudoCodeText("end");
            return;
        }


        //------------------------------------------------------------
        // Network Simplex Phase-2
        //------------------------------------------------------------
        UI = realbase; // use the "real" userbase for phase-2

        // using the "real" user info, place the tree edges in the right set
        Enumeration myenum = T.elements();
        while (myenum.hasMoreElements())
            UI.SingleStep((Edge) myenum.nextElement(),Tree);
        UI.ShowPseudoCodeText("feasible tree");


        // reset the node potentials for the actual problem, now that
        // an initial basis has been determined
        Node i=root;
        Pi.Put(i,0);
        //PutNodeLabel(i,Supply,Pi); // not needed for artificial node, but
                                   // included for sake of completeness
                                   // dsd 000723 - not created via UiNetwork .. no label
        i = Thread.GetNode(i);
        while (i != root) {
            e = Pred.GetEdge(i);

            if (i == e.GetTarget())
                Pi.Put(i, Pi.GetInt(e.GetSource())-Cost.GetInt(e));
            else // i == e.GetSource()
                Pi.Put(i, Pi.GetInt(e.GetTarget())+Cost.GetInt(e));

            PutNodeLabel(i,Supply,Pi);
            i = Thread.GetNode(i);
        }
        
        // calculate flow cost for initial flow
        FlowCost = 0;
        eit = Net.GetEdgeListIter();
        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
            // KLUDGE --- putedgelabel fails for non-gui nodes/edges
            if (e.GetSource() != root && e.GetTarget() != root)
                PutEdgeLabel(e,Flow,Capacity,Cost);
            // PutEdgeLabel(e,Flow,Capacity,Cost); // 000723 dsd restored kludge

            FlowCost += Cost.GetInt(e)*Flow.GetInt(e);
        }
        UI.StatusLine("Initial flow cost = $"+FlowCost+".  ("
                      +iteration_count+" phase-1 iterations)");
        UI.ShowPseudoCodeText("initial flow");
        UI.Trace(Tree);
        UI.Update();
        UI.StatusLine("");
        final int phase1_ic = iteration_count;

        NetworkSimplexAlgorithm(root,Flow,Pi,Supply,Capacity,Cost,
                                T,L,U,Depth,Pred,Thread);
        SaveGraph.Restore();
        UI.ShowPseudoCodeText("end");

        DisplayFinalSolution(Net,Cost,Flow,Capacity,Pi,Supply);

        UI.StatusLine("Min-Cost Flow cost = $"+FlowCost+".  "+
                      "Total number of pivots:  "+
                      iteration_count+"   ("+phase1_ic+" Phase-I pivots)");
        //        UI.SingleStep();
        //        UI.Trace(Tree);UI.Trace(Entering);UI.Trace(Leaving);
        //        UI.Update();
        //        UI.StatusLine("");
    }

    /****************************************************************/
    // Actual network simplex solver 
    /****************************************************************/
    private void NetworkSimplexAlgorithm(Node      root,
                                         EdgeArray Flow,
                                         NodeArray Pi,
                                         NodeArray Supply,
                                         EdgeArray Capacity,
                                         EdgeArray Cost,
                                         Vector    T,
                                         Vector    L,
                                         Vector    U,
                                         NodeArray Depth,
                                         NodeArray Pred,
                                         NodeArray Thread) {
        Edge entering = (Edge) null;
        Edge leaving = (Edge) null;

        int entering_rc;
        entering = FindEnteringEdge(root,Cost,Pi,L,U);
        UI.ShowPseudoCodeText("phase-2 loop");
        while (entering != null) {
            entering_rc = Cost.GetInt(entering)-Pi.GetInt(entering.GetSource())
                +Pi.GetInt(entering.GetTarget());

            UI.StatusLine("Entering edge's reduced cost:  $"+entering_rc);
            UI.SingleStep(entering,Entering);
            UI.ShowPseudoCodeText("select entering");
            UI.Trace(Entering);
            UI.Update();
            UI.StatusLine("");
            leaving = FindLeavingEdgeAndUpdateSolution(entering,Flow,Capacity,
                                                       T,L,U,Depth,Pred);
            iteration_count++;

            UI.ShowPseudoCodeText("tree change");
            if (entering != leaving) {
                UpdateTreeAndPotentials(entering,leaving,
                                        Cost,Supply,Pi,Depth,Pred,Thread);
                UI.StatusLine("Updated node potentials.");
                UI.ShowPseudoCodeText("update pi");
                //                UI.SingleStep(Entering);
                //                UI.Trace(Entering);
                //                UI.Update();
                UI.StatusLine("");
            }

            entering = FindEnteringEdge(root,Cost,Pi,L,U);
            UI.ShowPseudoCodeText("phase-2 loop");
        }
    }


    /****************************************************************/
    /****************************************************************/
    private Edge FindEnteringEdge(Node      root,
                                  EdgeArray Cost,
                                  NodeArray Pi,
                                  Vector    L,
                                  Vector    U) {
        
        Node i,j;
        Edge e;
        Enumeration myenum;

        // if the solution is sub-optimal, find an entering edge
        myenum = L.elements();
        while (myenum.hasMoreElements()) {
            e = (Edge) myenum.nextElement();
            i = e.GetSource();
            j = e.GetTarget();

            // accept a non-artificial edge in L with negative reduced cost
            if (i!=root && j !=root && Cost.GetInt(e)-Pi.GetInt(i)+Pi.GetInt(j) < 0)
                return e;
        }

        myenum = U.elements();
        while (myenum.hasMoreElements()) {
            e = (Edge) myenum.nextElement();
            i = e.GetSource();
            j = e.GetTarget();

            // accept a non-artificial edge in U with positive reduced cost
            if (i!=root && j !=root && Cost.GetInt(e)-Pi.GetInt(i)+Pi.GetInt(j) > 0)
                return e;
        }

        return (Edge) null;
    }

    /****************************************************************/
    /****************************************************************/
    private Node InitializeTree(Network   Net,
                                EdgeArray Flow,
                                NodeArray Pi,
                                EdgeArray Capacity,
                                EdgeArray Phase1Cost,     // phase-I costs
                                EdgeArray Cost,     // phase-I costs
                                Vector    T,
                                Vector    L,
                                Vector    U,
                                NodeArray Depth,
                                NodeArray Pred,
                                NodeArray Thread) {

        Edge e;
        EdgeListIter eit;
        Node i,j;
        NodeListIter nit;


        Node root = Net.CreateNode();

        // phase-I cost
        Phase1Cost.Fill(0);

        nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode()) {
            if (i != root) {
                if (Supply.GetInt(i) <= 0) { // demand or transshipment node
                    e = Net.CreateEdge(root,i);
                    Capacity.Put(e,-Supply.GetInt(i));
                    Flow.Put(e,-Supply.GetInt(i));
                }
                else { // supply node
                    e = Net.CreateEdge(i,root);
                    Capacity.Put(e,Supply.GetInt(i));
                    Flow.Put(e,Supply.GetInt(i));
                }
                Pred.Put(i,e);
                Phase1Cost.Put(e,1);
                Cost.Put(e,0);
            }
        }

        // initialize Depth array
        Depth.Fill(1);
        Depth.Put(root,0);


        // initialize Thread array
        i=root;
        nit = Net.GetNodeListIter();
        for (j=nit.GetFirstNode() ; j!=null ; j=nit.GetNextNode()) {
            if (j!=root) {
                Thread.Put(j,i);
                i=j;
            }
        }
        Thread.Put(root,i);


        // initialize Lists
        T.removeAllElements();  // empty
        L.removeAllElements();  // empty
        U.removeAllElements();  // empty
        eit = Net.GetEdgeListIter();
        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
            if (e.GetSource() == root || e.GetTarget() == root) {
                T.addElement(e);
                UI.SingleStep(e,Tree);
            }
            else
                L.addElement(e);
        }
        UI.Trace(Tree);
        UI.Update();


        // initialize Potential
        Pi.Put(root,0);
        eit = root.GetAdjListIter();
        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
            if (e.GetSource() == root)
                Pi.Put(e.GetTarget(),-1);
            else // e.GetTarget() == root
                Pi.Put(e.GetSource(),1);
        }

        return root; // return the artificial root node
    }

    /****************************************************************/
    /****************************************************************/
    private void UpdatePotentials(Edge      entering,
                                  Node      e1,
                                  Node      f2,
                                  EdgeArray Cost,
                                  NodeArray Supply,
                                  NodeArray Pi,
                                  NodeArray Depth,
                                  NodeArray Thread) {
        Node y,z;
        int change;

        y = f2;
        change = Cost.GetInt(entering) - Pi.GetInt(entering.GetSource())
            + Pi.GetInt(entering.GetTarget());
        if (e1 == entering.GetSource())
            change = -change;

        Pi.Put(y, Pi.GetInt(y)+change);
        PutNodeLabel(y,Supply,Pi);
        z = Thread.GetNode(y);
        while (Depth.GetInt(z) > Depth.GetInt(y)) {
            Pi.Put(z, Pi.GetInt(z)+change);
            PutNodeLabel(z,Supply,Pi);
            z = Thread.GetNode(z);
        }
    }

    /****************************************************************/
    /****************************************************************/
    private void UpdateTreeAndPotentials(Edge      entering,
                                         Edge      leaving,
                                         EdgeArray Cost,
                                         NodeArray Supply,
                                         NodeArray Pi,
                                         NodeArray Depth,
                                         NodeArray Pred,
                                         NodeArray Thread) {
        Node i,j;

        // label end points of entering and leaving edges; consistent
        // notation with (V. Chvtal --- 1983)
        Node e1,e2,f1,f2;

        f1 = leaving.GetSource();
        f2 = leaving.GetTarget();
        if (Depth.GetInt(f2) != Depth.GetInt(f1) + 1) {
            i=f1 ; f1=f2 ; f2=i;  // swap f1 and f2
        }

        i = entering.GetSource();
        j = entering.GetTarget();
        while (j != f2 && i != f2) {
            if (Pred.GetEdge(i) != null) i = Pred.GetEdge(i).GetOpposite(i);
            if (Pred.GetEdge(j) != null) j = Pred.GetEdge(j).GetOpposite(j);
        }
        if (i == f2) {
            e2 = entering.GetSource();
            e1 = entering.GetTarget();
        }
        else {
            e1 = entering.GetSource();
            e2 = entering.GetTarget();
        }


        // update node potentials before updating tree structure
        UpdatePotentials(entering,e1,f2,Cost,Supply,Pi,Depth,Thread);


        // update the tree info "Depth", "Pred", and "Thread"
        int depth_incr;
        Node a,b,k,r;
        Edge temp,old_pred;

        a = f1;
        while (Thread.GetNode(a) != f2)
            a = Thread.GetNode(a);

        b = Thread.GetNode(e1);
        i = e2;

        old_pred = Pred.GetEdge(i);
        Pred.Put(i,entering);
        depth_incr = Depth.GetInt(e1)-Depth.GetInt(e2)+1;

        k = i;
        while (Depth.GetInt(Thread.GetNode(k)) > Depth.GetInt(i)) {
            k = Thread.GetNode(k);
            Depth.Put(k, Depth.GetInt(k)+depth_incr);
        }
        Depth.Put(i, Depth.GetInt(i)+depth_incr);

        r = Thread.GetNode(k);

        while (i != f2) {
            j = i;
            i = old_pred.GetOpposite(i);
            Thread.Put(k,i);
            temp = Pred.GetEdge(i);
            Pred.Put(i,old_pred);
            old_pred = temp;
            depth_incr += 2;

            k = i;
            while (Thread.GetNode(k) != j) {
                k = Thread.GetNode(k);
                Depth.Put(k,Depth.GetInt(k)+depth_incr);
            }
                
            if (Depth.GetInt(r) > Depth.GetInt(i)) {
                Thread.Put(k,r);
                while (Depth.GetInt(Thread.GetNode(k)) > Depth.GetInt(i)) {
                    k = Thread.GetNode(k);
                    Depth.Put(k, Depth.GetInt(k)+depth_incr);
                }

                r = Thread.GetNode(k);
            }
            Depth.Put(i, Depth.GetInt(i)+depth_incr);
        }

        // Invariant:  i == f2
        if (e1 == a) {
            Thread.Put(e1,e2);
            Thread.Put(k,r);
        }
        else {
            Thread.Put(a,r);
            Thread.Put(e1,e2);
            Thread.Put(k,b);
        }

        Depth.Put(e2, Depth.GetInt(e1)+1);
        Pred.Put(e2, entering);
    }

    /****************************************************************/
    /****************************************************************/
    private Edge FindLeavingEdgeAndUpdateSolution(Edge      entering,
                                                  EdgeArray Flow,
                                                  EdgeArray Capacity,
                                                  Vector    T,
                                                  Vector    L,
                                                  Vector    U,
                                                  NodeArray Depth,
                                                  NodeArray Pred) {

        Edge e;
        Node i,j;

        if (L.contains(entering)) {
            i = entering.GetSource();
            j = entering.GetTarget();
        }
        else {  // U.contains(entering) == true
            i = entering.GetTarget();
            j = entering.GetSource();
        }


        // find the leaving edge that preserves "strong feasible" spanning tree
        int di,dj;
        int delta = Capacity.GetInt(entering);
        Edge leaving = entering;        
        while (i!=j) {
            di = Depth.GetInt(i);
            dj = Depth.GetInt(j);

            if (di >= dj) {
                e = Pred.GetEdge(i);
                if (i==e.GetTarget() && 
                    Capacity.GetInt(e)-Flow.GetInt(e) < delta) {
                    leaving = e;
                    delta = Capacity.GetInt(e)-Flow.GetInt(e);
                }
                else if (i==e.GetSource() && Flow.GetInt(e) < delta) {
                    leaving = e;
                    delta = Flow.GetInt(e);
                }
                i = e.GetOpposite(i);
            }

            if (dj >= di) {
                e = Pred.GetEdge(j);
                if (j==e.GetSource() && 
                    Capacity.GetInt(e)-Flow.GetInt(e) <= delta) {
                    leaving = e;
                    delta = Capacity.GetInt(e) - Flow.GetInt(e);
                }
                else if (j == e.GetTarget() && Flow.GetInt(e) <= delta) {
                    leaving = e;
                    delta = Flow.GetInt(e);
                }
                j = e.GetOpposite(j);
            }
        }
        if (entering == leaving)
            UI.StatusLine("Entering edge is selected as the leaving edge; "+
                          "edge's residual capacity = "
                          +delta+((delta==1)?" unit.":" units."));
        else
            UI.StatusLine("Leaving edge's residual capacity = "
                          +delta+((delta==1)?" unit.":" units."));
        UI.SingleStep(leaving,Leaving);
        UI.ShowPseudoCodeText("select leaving");
        UI.Trace(Leaving);
        UI.Update();
        UI.StatusLine("");

        // update the flow
        if (L.contains(entering)) {
            i = entering.GetSource();
            j = entering.GetTarget();
            Flow.Put(entering,delta);
            FlowCost += delta*Cost.GetInt(entering);
            L.removeElement(entering);
        }
        else {  // U.contains(entering) == true
            i = entering.GetTarget();
            j = entering.GetSource();
            Flow.Put(entering,Capacity.GetInt(entering)-delta);
            FlowCost -= delta*Cost.GetInt(entering);
            U.removeElement(entering);
        }
        T.addElement(entering);
        UI.SingleStep(entering,Tree);
        PutEdgeLabel(entering,Flow,Capacity,Cost);
        while (i != j) {
            di = Depth.GetInt(i);
            dj = Depth.GetInt(j);

            if (di >= dj) {
                e = Pred.GetEdge(i);
                if (i == e.GetTarget()) {
                    Flow.Put(e,Flow.GetInt(e)+delta);
                    FlowCost += delta*Cost.GetInt(e);
                }
                else {
                    Flow.Put(e,Flow.GetInt(e)-delta);
                    FlowCost -=delta*Cost.GetInt(e);
                }
                PutEdgeLabel(e,Flow,Capacity,Cost);
                i = e.GetOpposite(i);
            }

            if (dj >= di) {
                e = Pred.GetEdge(j);
                if (j == e.GetSource()) {
                    Flow.Put(e,Flow.GetInt(e)+delta);
                    FlowCost += delta*Cost.GetInt(e);
                }
                else {
                    Flow.Put(e,Flow.GetInt(e)-delta);
                    FlowCost -=delta*Cost.GetInt(e);
                }
                PutEdgeLabel(e,Flow,Capacity,Cost);
                j = e.GetOpposite(j);
            }
        }
        T.removeElement(leaving);
        if (Flow.GetInt(leaving) == 0)
            L.addElement(leaving);
        else
            U.addElement(leaving);
        UI.StatusLine("Flow augmented by "+delta+
                      ((delta==1)?" unit":" units")+".  New flow cost = $"+
                      FlowCost+".");
        UI.SingleStep(leaving,InitialSet);
        UI.ShowPseudoCodeText("update flow");
        UI.Trace(Tree);
        UI.Update();
        UI.StatusLine("");

        return leaving;
    }















    /****************************************************************/
    /****************************************************************/
    /*
    private void FindCycleFreeFlow(Network   Net,
                                   EdgeArray Cost,
                                   EdgeArray Flow,
                                   EdgeArray Capacity,
                                   NodeArray Supply) {

        NodeArray Pred = new NodeArray(Net.GetNodeIndex(),"Pred");

        GetInitialFlow(Net,Cost,Flow,Capacity,Supply);

        Node CycleRoot = FindFreeArcCycle(Net,Pred,Flow,Capacity);
        while (CycleRoot != null) {
            AugmentCycleFlow(Net,CycleRoot,Pred,Flow,Capacity,Cost);
            CycleRoot = FindFreeArcCycle(Net,Pred,Flow,Capacity);
        }
    }
    */
    /****************************************************************/
    /****************************************************************/
    /*
    private void AugmentCycleFlow(Network   Net,
                                  Node      CycleRoot,
                                  NodeArray Pred,
                                  EdgeArray Flow,
                                  EdgeArray Capacity,
                                  EdgeArray Cost) {
        Edge e;
        Node i;
        int delta;
        int cost;

        // calculate the cost and the residual capacity of cycle
        e = Pred.GetEdge(CycleRoot);
        if (CycleRoot == e.GetTarget()) {
            delta = Capacity.GetInt(e) - Flow.GetInt(e);
            cost = Cost.GetInt(e);
        }
        else {
            delta = Flow.GetInt(e);
            cost = -Cost.GetInt(e);
        }
        UI.SingleStep(CycleRoot,Cycle);
        UI.SingleStep(e,Cycle);

        i = e.GetOpposite(CycleRoot);
        e = Pred.GetEdge(i);
        while (i != CycleRoot) {
            if (i == e.GetTarget()) {
                delta = Math.min(delta, Capacity.GetInt(e)-Flow.GetInt(e));
                cost += Cost.GetInt(e);
            }
            else {
                delta = Math.min(delta, Flow.GetInt(e));
                cost -= Cost.GetInt(e);
            }
            UI.SingleStep(i,Cycle);
            UI.SingleStep(e,Cycle);

            i = e.GetOpposite(i);
            e = Pred.GetEdge(i);
        }
        UI.StatusLine("Cycle residual capacity = "+delta+".  "+
                      "Cycle cost = "+cost);
        UI.Trace(Cycle);
        UI.Update();

        // update the cycle flow
        i = CycleRoot;
        e = Pred.GetEdge(CycleRoot);
        do {
            if (i == e.GetTarget()) {
                Flow.Put(e, Flow.GetInt(e)+delta);
                PutEdgeLabel(e,Flow,Capacity,Cost);
            }
            else {
                Flow.Put(e, Flow.GetInt(e)-delta);
                PutEdgeLabel(e,Flow,Capacity,Cost);
            }
            i = e.GetOpposite(i);
            e = Pred.GetEdge(i);
        } while (i != CycleRoot);
        FlowCost += delta*cost;

        UI.StatusLine("Flow Updated.  New flow cost = $"+FlowCost+".");
        UI.SingleStep(Cycle);
        UI.Trace(Cycle);
        UI.Update();
    }
    */
    /****************************************************************/
    /****************************************************************/
    /*
    private Node FindFreeArcCycle(Network   Net,
                                  NodeArray Pred,
                                  EdgeArray Flow,
                                  EdgeArray Capacity) {
        Pred.Fill(null);

        NodeArray Dist = new NodeArray(Net.GetNodeIndex(),"Dist");
        Dist.Fill(INFINITY);

        Node i,j;
        PQList PQ = new PQList();
        NodeListIter nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i != null ; i=nit.GetNextNode())
            PQ.Insert(i,Dist.GetInt(i));

        Edge e;
        EdgeListIter eit;
        while (PQ.NotEmpty()) {
            i = PQ.DeleteMinNode();

            // this is the "source node" for all nodes in its
            // reachable component
            if (Dist.GetInt(i) == INFINITY) 
                Dist.Put(i,0);

            eit = i.GetAdjListIter();
            for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
                // make sure this isn't node i's predecessor edge,
                // and make sure that it is a "free edge"
                if (e != Pred.GetEdge(i) &&
                    (Flow.GetInt(e)>0 && Flow.GetInt(e)<Capacity.GetInt(e))) {
                    j = e.GetOpposite(i);

                    // check for presence of a cycle
                    if (Dist.GetInt(j) < INFINITY) {
                        // cycle detected; need to identify

                        // we will trace pred(j) and pred(i) until
                        // there is a common root; notice that nodes i
                        // and j are necessarily in the same component
                        // of the network, and that the distances of i
                        // and j gives the number of edges between
                        // each node and the "source" node of the
                        // component.

                        // make sure that node "j" is the closer node
                        if (Dist.GetInt(j) > Dist.GetInt(i)) {
                            // conjecture that this will never happen
                            GIDEN.out.println("******* Warning (owenj-NS1)");

                            // swap "i" and "j"
                            Node temp = j;
                            j=i;
                            i=temp;
                        }

                        while (Dist.GetInt(i) > Dist.GetInt(j))
                            i = (Pred.GetEdge(i)).GetOpposite(i);

                        // update pred edges for j's path to form cycle
                        Edge old_pred_j = Pred.GetEdge(j);
                        Edge new_pred_j = e;

                        // same "distance"/"depth", so keep moving up
                        // until a common ancestor
                        while (i != j) {
                            i = (Pred.GetEdge(i)).GetOpposite(i);

                            // update preds for j and move "up"
                            // j = (Pred.GetEdge(j)).GetOpposite(j);
                            Pred.Put(j,new_pred_j);
                            j = old_pred_j.GetOpposite(j);
                            new_pred_j = old_pred_j;
                            old_pred_j = Pred.GetEdge(j);
                        }
                        // the cycle root is node i, which equals node j
                        Pred.Put(j,new_pred_j);
                        return i;
                    }

                    Pred.Put(j,e);
                    Dist.Put(j,Dist.GetInt(i)+1);
                    PQ.DecreaseValue(j,Dist.GetInt(j));
                }
            }
        }

        return (Node) null;

    }
    */
    /****************************************************************/
    /****************************************************************/
    /*
    private Node OLDFindFreeArcCycle(Network   Net,
                                  NodeArray Pred,
                                  EdgeArray Flow,
                                  EdgeArray Capacity) {

        // This is a variation on the FIFO label correcting algorithm.
        // Without the connectedness assumption, we add all nodes to
        // the list initially, which has the same effect as an
        // artificial node connected to all other nodes.  Also, we
        // consider the cost of all edges in the residual network to
        // be "-1" so that we can identify the cycle by adentifying a
        // negative cost cycle.  We work on the original network, but
        // only consider "free arcs" as admissible edges in the
        // residual network.

        NodeArray Dist = new NodeArray(Net.GetNodeIndex(),"Dist");
        Dist.Fill(0);

        Pred.Fill(null);

        NodeArray NumExam = new NodeArray(Net.GetNodeIndex(),"NumExam");
        NumExam.Fill(0);

        Node i,j;
        BoundedQueue Q = new BoundedQueue(Net.GetNodeIndex());
        NodeArray InQueue = new NodeArray(Net.GetNodeIndex(),"InQueue");
        NodeListIter nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode()) {
            Q.Append(i);
            InQueue.Put(i,Boolean.TRUE);
        }

        Edge e;
        EdgeListIter eit;
        while (!Q.IsEmpty()) {
            i = (Node) Q.Pop();
            InQueue.Put(i,Boolean.FALSE);

            if (NumExam.GetInt(i) >= Net.GetNodeCount()-1)
                return GetCycleRoot(Net,i,Pred);
            else
                NumExam.Put(i,NumExam.GetInt(i)+1);

            eit = i.GetAdjListIter();
            for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
                j = e.GetOpposite(i);

                // check distance label as qualifying, and make sure
                // that we don't add the same edge as a predecessor
                // for each endpoint
                if (Dist.GetInt(j) > Dist.GetInt(i)-1 &&
                    Pred.GetEdge(i) != e) {

                    // make sure edge is a "free arc" with 0 < x_ij < u_ij
                    //if ((i==e.GetSource() && Flow.GetInt(e) > 0) ||
                    // (i==e.GetTarget() && Capacity.GetInt(e)-Flow.GetInt(e) > 0)) {
                    if (Flow.GetInt(e)>0 && Flow.GetInt(e)<Capacity.GetInt(e)) {
                        Dist.Put(j,Dist.GetInt(i)-1);
                        Pred.Put(j,e);
                        if (InQueue.GetObj(j).equals(Boolean.FALSE)) {
                            Q.Append(j);
                            InQueue.Put(j,Boolean.TRUE);
                        }
                    }
                }
            }
        }

        // only reach here if there isn't a cycle
        return (Node) null;
    }
    */
    /****************************************************************/
    /*
    private Node GetCycleRoot(Network   Net,
                              Node      StartNode,
                              NodeArray Pred) {
        NodeArray Seen = new NodeArray(Net.GetNodeIndex(),"Seen");
        Seen.Fill(Boolean.FALSE);

        Node i=StartNode;
        Seen.Put(StartNode,Boolean.TRUE);

        // identify the cycle
        i = (Pred.GetEdge(StartNode)).GetOpposite(StartNode);
        while (Seen.GetObj(i).equals(Boolean.FALSE)) {
            Seen.Put(i,Boolean.TRUE);
            i = (Pred.GetEdge(i)).GetOpposite(i);
        }

        // we know that node i is in the cycle
        return i;
    }
    */
    /****************************************************************/
    /****************************************************************/
    /*
    private void GetInitialFlow(Network   Net,
                                EdgeArray Cost,
                                EdgeArray Flow,
                                EdgeArray Capacity,
                                NodeArray Supply) {

        GraphState SaveGraph = new GraphState(Net);

        Node asource = Net.CreateNode();
        Node asink = Net.CreateNode();

        NodeListIter nit = Net.GetNodeListIter();
        Node n;
        Edge e;
        for (n=nit.GetFirstNode() ; n!=null ; n=nit.GetNextNode()) {
            if (Supply.GetInt(n) > 0) {
                e = Net.CreateEdge(asource,n);
                Capacity.Put(e,Supply.GetInt(n));
            }
            else if (Supply.GetInt(n) < 0) {
                e = Net.CreateEdge(n,asink);
                Capacity.Put(e,-Supply.GetInt(n));
            }
        }

        // call a max-flow solver to get an initial feasible flow
        UserBase nullbase = new UserBase(UI.SolverIo);
        MaxFlow solver = new PreFlowPush(nullbase);

        solver.Setup(Net);

        solver.Source = asource;
        solver.Sink = asink;
        solver.Capacity = Capacity;

        solver.Execute();
        GIDEN.out.println("Flow Value= "+solver.FlowValue);

        // restore network and save flow values
        SaveGraph.Restore();
        EdgeListIter eit = Net.GetEdgeListIter();
        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
            Flow.Put(e, solver.Flow.GetInt(e));
            PutEdgeLabel(e,Flow,Capacity,Cost);
            FlowCost += Flow.GetInt(e)*Cost.GetInt(e);
        }

        solver.Shutdown();

        // OWENJ: KLUDGE FOR MinCostFlow-2 network to check on cycle
        // free flow routine
        //eit = Net.GetEdgeListIter();
        //for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
        //    Capacity.Put(e,Capacity.GetInt(e)+1);
        //    PutEdgeLabel(e,Flow,Capacity,Cost);
        //}



        // show initial flow, if the cycle animation set is selected
        UI.StatusLine("Initial flow generated using a max-flow solver.  "+
                      "Flow cost = $"+FlowCost+".");
        UI.SingleStep(Cycle);
        UI.Trace(Cycle);
        UI.Update();
        UI.StatusLine("");
    }
    */
    /****************************************************************/
}
