package  giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;


/**********************************************************************/
// Prim solver class
/**********************************************************************/
public class Prim extends MinSpan {

    // prooblem data
    Node Source;

    private AnimationSet Trial;
    private AnimationSet Acquired;
    private AnimationSet Discarded;
    
    private final int PERMANENTLY_LABELED = 1;
    private final int LABELED = 0;
    private final int UNLABELED = -1;
    private final int INFINITY = Integer.MAX_VALUE;

    
    
    /****************************************************************/
    // Prim constructor
    /****************************************************************/
    public Prim(UserBase userinfo) {
        super(userinfo);

        Trial = UI.CreateAnimationSet("Trial",Color.red);
        Acquired = UI.CreateAnimationSet("Acquired",new Color(48,175,48));
        Discarded = UI.CreateAnimationSet("Discarded",Color.yellow);        
    }
    
    public String getSolverName() {return "Prim";}

    /****************************************************************/
    // Setup
    //   - calls MinSpan base class Setup()
    //   - initializes solver specific information
    /****************************************************************/
    public boolean Setup(Network net) {
        if(super.Setup(net)==false)
            return false;

        Source = null;

        return true;        
    }
    
    /****************************************************************/
    // Prim Execute
    /****************************************************************/
    public boolean Execute() {
        LinkList Forest = GetForestList();
        Network Net = GetNetwork();
        // owenj 09-Apr-1999       Length = GetLengthArray(); 

        while (Source == null)
            Source = UI.UserMouseNodeInput("Click on Starting Node");

        PrimSolver(Source,Net,Length,Forest);
        DisplayMinSpanForest(Net,Forest,Length);

        return true;
    }
    
    /********************************************************************/
    // Prim Shutdown
    /********************************************************************/
    public boolean Shutdown() {
        Source = null;

        return super.Shutdown();
    }
    
    
    /****************************************************************/
    // Prim PrimSolver
    /****************************************************************/
    private void PrimSolver(Node      Source,
                            Network   Net,
                            EdgeArray Length,
                            LinkList  Forest) {

        UI.StatusLine("Prim Solver begins...");
        UI.ShowPseudoCodeText("start");
        UI.StatusLine("");
        
        Edge e;
        Node i,j;
        NodeListIter nit;
        EdgeListIter eit;
        
        
        PQList L = new PQList();
        NodeArray Distance = new NodeArray(Net.GetNodeIndex(),"Distance");
        NodeArray Pred = new NodeArray(Net.GetNodeIndex(),"Pred");
        NodeArray Status = new NodeArray(Net.GetNodeIndex(),"Status");
        
        // initialize inheap, preds, distances and fill the heap
        nit = Net.GetNodeListIter();
        Status.Fill(UNLABELED);
        Pred.Fill(null);
        Distance.Fill(INFINITY);
        
        for (i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode())
            UI.PutLabel(i,"-");
        UI.ShowPseudoCodeText("setup");
        UI.ShowPseudoCodeText("init sets");
        
        Distance.Put(Source,0);
        UI.PutLabel(Source,""+Distance.GetInt(Source));
        UI.ShowPseudoCodeText("source dist");
        UI.SingleStep(Source,Trial);
        UI.ShowPseudoCodeText("source to L");
        UI.Trace(Trial);
        UI.Update();
        L.Insert(Source,Distance.GetInt(Source));
        Status.Put(Source,LABELED);
        
        while (L.NotEmpty()) {
            i = L.DeleteMinNode();
            e = Pred.GetEdge(i);
            Status.Put(i,PERMANENTLY_LABELED);
            
            UI.ShowPseudoCodeText("main loop");
            UI.ShowPseudoCodeText("select node");
            
            UI.SingleStep(i,Acquired);
            if (e != null) {
                UI.SingleStep(e,Acquired);
                Forest.AddItem(e);
                ForestLength += Length.GetInt(e);
            }
            UI.ShowPseudoCodeText("node to P");
            UI.Trace(Acquired);
            UI.Update();
            
            eit = i.GetAdjListIter();
            for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
                
                j = e.GetOpposite(i);
                if (Status.GetInt(j) != PERMANENTLY_LABELED) {
                    
                    UI.SingleStep(e,Trial);
                    
                    UI.ShowPseudoCodeText("examine adj");
                    UI.ShowPseudoCodeText("check dist");
                    
                    if (Distance.GetInt(j) > Length.GetInt(e)) {
                        
                        Distance.Put(j,Length.GetInt(e));
                        UI.PutLabel(j,Length.GetInt(e));
                        UI.ShowPseudoCodeText("update dist");
                        
                        if (Pred.GetEdge(j) != null)
                            UI.SingleStep(Pred.GetEdge(j),Discarded);
                        Pred.Put(j,e);
                        UI.ShowPseudoCodeText("update pred");
                        
                        if (Status.GetInt(j) == UNLABELED) {
                            L.Insert(j,Distance.GetInt(j));
                            Status.Put(j,LABELED);
                        }
                        
                        L.DecreaseValue(j,Distance.GetInt(j));
                        UI.SingleStep(j,Trial);
                        UI.ShowPseudoCodeText("update set");
                        
                        UI.Trace(Discarded);
                        UI.Trace(Trial);
                        UI.Update();
                    }
                    else {
                        UI.Trace(Trial);
                        UI.SingleStep(e,Discarded);
                        UI.Trace(Discarded);
                        UI.Update();
                    }
                }
            }
        }
        UI.ShowPseudoCodeText("main loop");

        // owenj (05-aug-1999); moved to "DisplayMinSpanForest" in MinSpan
        UI.ShowPseudoCodeText("end");
        
        int finalstatus = PERMANENTLY_LABELED;
        for (i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode())
            if (Status.GetInt(i) == UNLABELED)
                finalstatus = UNLABELED;

        if (finalstatus == UNLABELED)
            UI.StatusLine("Network is not connected.  "
                          +"R = {i in N : d(i) < \"inf\"}, U = N\\R.  "
                          +"Length of min span tree on R = "
                          +ForestLength+" units.");
        else
            UI.StatusLine("Prim Solver finished.  "
                          +"Length of minimum spanning tree = "
                          +ForestLength+" units.");
    }
}
