package  giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;

/**********************************************************************/
// Shortest Augmenting Path class
/**********************************************************************/
public class ShortAugPath extends MaxFlow {

    private AnimationSet Trial;
    private AnimationSet Acquired;
    private AnimationSet Discarded;

    private int PathCount;
    private int RelabelCount;

    private final int UNDEFINED_LABEL_UPDATES = 0;
    private final int SMART_LABEL_UPDATES = 1;
    private final int BASIC_LABEL_UPDATES = 2;

    private final int INFINITY = Integer.MAX_VALUE;

    private int SelectedLabelUpdateStyle;

    /****************************************************************/
    // ShortAugPath constructor
    /****************************************************************/
    public ShortAugPath(UserBase userinfo) {
        super(userinfo);

        Trial = UI.CreateAnimationSet("Trial",Color.red);
        Acquired = UI.CreateAnimationSet("Acquired",new Color(48,175,48));
        Discarded = UI.CreateAnimationSet("Discarded",Color.yellow);
    }

    public String getSolverName() {return "Shortest Augmenting Path";}

    /****************************************************************/
    // Setup
    //   - calls MaxFlow base class Setup()
    //   - initializes solver specific information
    /****************************************************************/
    public boolean Setup(Network net) {
        if(super.Setup(net)==false)
            return false;

        SelectedLabelUpdateStyle = UNDEFINED_LABEL_UPDATES;

        return true;
    }

    /****************************************************************/
    // ShortAugPath Execute
    /****************************************************************/
    public boolean Execute() {
        PathCount = 0;
        RelabelCount = 0;

        Node Source = GetSourceNode();
        Node Sink = GetSinkNode();
        //        EdgeArray Capacity = GetCapacityArray();
        EdgeArray Flow = GetFlowArray();
        NodeArray Reachable = GetReachableArray();

        Network Net = GetNetwork();

        int[] CntDist = new int[Net.GetNodeIndex()+1];
        NodeArray Pred = new NodeArray(Net.GetNodeIndex(),"Pred");
        NodeArray Dist = new NodeArray(Net.GetNodeIndex(),"Dist");
        EdgeArray MaxResidual = new EdgeArray(Net.GetEdgeIndex(),"MaxResidual");
        NodeArray CurrentEIT = new NodeArray(Net.GetNodeIndex(),"CurrentEIT");

        NodeListIter nit = Net.GetNodeListIter();
        for (Node i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode())
            InitializeAdjEdgeIter(i,CurrentEIT);

        int LabelUpdateStyle = SelectedLabelUpdateStyle;

        ShortAugPathSolver(Net,Source,Sink,Capacity,Flow,Reachable,CntDist,
                           Pred,Dist,MaxResidual,CurrentEIT,LabelUpdateStyle);
        // NOTE:  WE could find the reachable nodes during algorithm progression
        //        at the same time as early termination.....
        FindSourceReachableNodes(Net,Source,Reachable,Flow,Capacity);
        DisplayMinCut(Net,Reachable,Capacity);

        return true;
    }

    /****************************************************************/
    // ShortAugPath ShortAugPathSolver
    /****************************************************************/
    private void ShortAugPathSolver(Network   Net,
                                 Node      Source,
                                 Node      Sink,
                                 EdgeArray Capacity,
                                 EdgeArray Flow,
                                 NodeArray Reached,
                                 int[]     CntDist,
                                 NodeArray Pred,
                                 NodeArray Dist,
                                 EdgeArray MaxResidual,
                                 NodeArray CurrentEIT,
                                 int       LabelUpdateStyle) {
        UI.StatusLine("Shortest Augmenting Path Solver begins...");
        UI.ShowPseudoCodeText("start");
        UI.StatusLine("");

        UI.ShowPseudoCodeText("init flow");

        Node i;
        Node j = null;
        Edge e;

        int residual;

        InitDistanceLabels(Net,Sink,Reached,Dist,CntDist);
        // owenj 12-apr-1999 (no such method)        UI.UpdateLabels();
        UI.ShowPseudoCodeText("init dist");

        i = Source;
        UI.SingleStep(i,Trial);
        UI.Trace(Trial);
        UI.Update();
        UI.ShowPseudoCodeText("start node");

        UI.ShowPseudoCodeText("main loop");
        while (Dist.GetInt(Source) < Net.GetNodeIndex()) {

            UI.ShowPseudoCodeText("is admissible?");
            e = AdmissibleAdjArc(i,Capacity,Flow,Dist,CurrentEIT,Pred);
            if (e != null) {
                UI.ShowPseudoCodeText("call advance");
                i = Advance(i,e,Pred);

                UI.ShowPseudoCodeText("is path?");
                if (i == Sink) {
                    UI.ShowPseudoCodeText("call augment");
                    AugmentFlow(Sink,Pred,Flow,Capacity);

                    UI.MoveAll(InitialSet);
                    // owenj 12-apr-1999 (no such method)     UI.PaintGraph();
                    
                    UI.ShowPseudoCodeText("reset");
                    i = Source;

                    UI.SingleStep(i,Trial);
                    UI.Trace(Trial);
                    UI.Update();
                }
            }
            else {
                CntDist[Dist.GetInt(i)]--;

                // Check for possible early termination
                if (CntDist[Dist.GetInt(i)] == 0
                    && LabelUpdateStyle == UNDEFINED_LABEL_UPDATES
                    ) { // owenj 12-Apr-1999 (no method)   && !UI.FinalMode) {

                    LabelUpdateStyle = GetLabelUpdateStyle();
                    if (LabelUpdateStyle == SMART_LABEL_UPDATES) {
                        Dist.Put(Source,Net.GetNodeIndex());
                        continue;
                    }
                }

                UI.ShowPseudoCodeText("call retreat");

                // Retreat
                UI.ShowPseudoCodeText("proc retreat");
                Relabel(Net,i,Capacity,Flow,Dist);

                UI.ShowPseudoCodeText("retreat dist");
                CntDist[Dist.GetInt(i)]++;
                InitializeAdjEdgeIter(i,CurrentEIT);

                PutNodeLabel(i,Dist);
                // owenj 12-apr-1999 (no such method)       UI.UpdateLabels();
                UI.ShowPseudoCodeText("retreat update");

                if (i != Source) {
                    e = Pred.GetEdge(i);
                    UI.SingleStep(i,Discarded);
                    UI.SingleStep(e,Discarded);
                    UI.Trace(Discarded);
                    UI.Update();

                    i = e.GetOpposite(i);
                }
            }
            UI.ShowPseudoCodeText("main loop");
        }

        UI.MoveAll(InitialSet);
        // owenj 12-apr-1999 (no such method)        UI.PaintGraph();

        UI.StatusLine("Shortest Augmenting Path Solver finished.");
        UI.ShowPseudoCodeText("end");
    }


    /****************************************************************/
    //   Put Edge label for edge e
    /****************************************************************/
    private void PutEdgeLabel(Edge      e,
                              EdgeArray Flow,
                              EdgeArray Capacity) {
        int residual = Capacity.GetInt(e) - Flow.GetInt(e);
        String label = "(" + Flow.GetInt(e) + "," + residual + ")";

        UI.PutLabel(e,label);
    }


    /****************************************************************/
    // Put Node label for i
    /****************************************************************/
    private void PutNodeLabel(Node      i,
                              NodeArray Dist) {
        UI.PutLabel(i,Dist.GetInt(i));
    }


    /****************************************************************/
    //   Select approach for distance label updates
    /****************************************************************/
    private int GetLabelUpdateStyle() {
        String temp;

        do {
            temp = UI.UserTextInput("Use \"smart\" label updates? ","Yes");
            if (temp == null) temp = new String("YES");

            temp = temp.toUpperCase();
        } while (temp.compareTo("YES") != 0 && temp.compareTo("NO") != 0);

        if (temp.compareTo("YES") == 0)
            return SMART_LABEL_UPDATES;
        else
            return BASIC_LABEL_UPDATES;
    }

    /****************************************************************/
    //   Advance the current node saving the predecessor edge
    /****************************************************************/
    private Node Advance(Node i, Edge e, NodeArray Pred) {
        UI.ShowPseudoCodeText("proc advance");

        Node j = e.GetOpposite(i);
        Pred.Put(j,e);

        UI.SingleStep(e,Trial);
        UI.ShowPseudoCodeText("advance edge");
        UI.SingleStep(j,Trial);
        UI.Trace(Trial);
        UI.ShowPseudoCodeText("advance update");
        UI.Update();

        return j;
    }

    /****************************************************************/
    //   Set edge iterator for the current node's first adjacent edge
    /****************************************************************/
    private void InitializeAdjEdgeIter(Node i, NodeArray CurrentEIT) {
        EdgeListIter eit = i.GetAdjListIter();
        eit.GetFirstEdge();
        CurrentEIT.Put(i,eit);
    }


    /****************************************************************/
    //   Return the edge pointed to by the current edge iterator
    /****************************************************************/
    private Edge CurrentEdge(Node i, NodeArray CurrentEIT) {
        EdgeListIter eit = (EdgeListIter) CurrentEIT.GetObj(i);
        return ((Edge) eit.GetItem());
    }

    /****************************************************************/
    //   Increment the edge iterator and return next edge in the list
    /****************************************************************/
    private Edge NextEdge(Node i, NodeArray CurrentEIT) {
        EdgeListIter eit = (EdgeListIter) CurrentEIT.GetObj(i);
        Edge nextedge = eit.GetNextEdge();
        CurrentEIT.Put(i,eit);
        return nextedge;
    }

    /****************************************************************/
    // Return an admissible adjacent arc if one exists (or null)
    /****************************************************************/
    private Edge AdmissibleAdjArc(Node      i,
                                  EdgeArray Capacity,
                                  EdgeArray Flow,
                                  NodeArray Dist,
                                  NodeArray CurrentEIT,
                                  NodeArray Pred) {
        Node j;
        Edge e = CurrentEdge(i,CurrentEIT);
        int residual;

        while (e != null) {
            j = e.GetOpposite(i);

            if (e.GetSource() == i)
                residual = Capacity.GetInt(e) - Flow.GetInt(e);
            else
                residual = Flow.GetInt(e);

            if (Dist.GetInt(i) == Dist.GetInt(j) + 1 && residual > 0)
                return e;

            if (e != Pred.GetEdge(i)) {
                UI.SingleStep(e,Discarded);
                UI.Trace(Discarded);
                UI.Update();
            }

            e = NextEdge(i,CurrentEIT);
        }
        return ((Edge) null);
    }


    /****************************************************************/
    //   Send flow along the augmenting path between Source and Sink
    //    - determine the minimum residual capacity
    //    - send the flow along the augmenting path
    /****************************************************************/
    private void AugmentFlow(Node Sink,
                             NodeArray Pred,
                             EdgeArray Flow,
                             EdgeArray Capacity) {
        Node v;
        Edge e;
        int delta;
        int residual;

        PathCount++;

        UI.ShowPseudoCodeText("proc augment");

        v = Sink;
        e = Pred.GetEdge(Sink);
        UI.SingleStep(Sink,Acquired);
        UI.SingleStep(e,Acquired);

        if (e.GetTarget() == Sink) {
            v = e.GetSource();
            delta = Capacity.GetInt(e) - Flow.GetInt(e);
        }
        else {
            v = e.GetTarget();
            delta = Flow.GetInt(e);
        }
        UI.SingleStep(v,Acquired);
        e = Pred.GetEdge(v);

        while (e != null) {
            UI.SingleStep(e,Acquired);

            if (e.GetTarget() == v) {
                v = e.GetSource();
                residual = Capacity.GetInt(e) - Flow.GetInt(e);
            }
            else {
                v = e.GetTarget();
                residual = Flow.GetInt(e);
            }

            if (residual < delta)
                delta = residual;

            UI.SingleStep(v,Acquired);
            e = Pred.GetEdge(v);
        }
        UI.ShowPseudoCodeText("augment path");
        UI.StatusLine("Residual capacity delta = "+delta+" units.");
        UI.ShowPseudoCodeText("augment delta");
        UI.Trace(Acquired);
        UI.Update();
        UI.StatusLine("");

        v = Sink;
        e = Pred.GetEdge(Sink);

        while (e != null) {
            if (v == e.GetTarget()) {
                Flow.Put(e,Flow.GetInt(e)+delta);
                v = e.GetSource();
            }
            else {
                Flow.Put(e,Flow.GetInt(e)-delta);
                v = e.GetTarget();
            }
            PutEdgeLabel(e,Flow,Capacity);
            e = Pred.GetEdge(v);
        }
        // owenj 12-apr-1999 (no such method)        UI.UpdateLabels();
        FlowValue += delta;
        UI.StatusLine("Flow augmented. New flow value = "+FlowValue+" units.");
        UI.ShowPseudoCodeText("augment update");
        UI.StatusLine("");
    }


    /****************************************************************/
    //   Update the distance label of the current active node so that
    //   excess can later be pushed along an arc with residual capacity
    /****************************************************************/
    private void Relabel(Network   Net,
                         Node      i,
                         EdgeArray Capacity,
                         EdgeArray Flow,
                         NodeArray Dist) {
        Node j;
        Edge e;
        int residual;
        int current_min_dist;
        EdgeListIter eit;

        RelabelCount++;
        
        current_min_dist = Net.GetNodeIndex() - 1;

        eit = i.GetAdjListIter();
        for (e = eit.GetFirstEdge() ; e != null ; e =eit.GetNextEdge()) {
            if (e.GetSource() == i) {
                j = e.GetTarget();
                residual = Capacity.GetInt(e) - Flow.GetInt(e);
            }
            else { // e.GetTarget() == i
                j = e.GetSource();
                residual = Flow.GetInt(e);
            }

            if (residual > 0 && Dist.GetInt(j) < current_min_dist)
                current_min_dist = Dist.GetInt(j);
        }

        Dist.Put(i,current_min_dist + 1);
    }

    /****************************************************************/
    //   Initialize the node distance labels via a BFS from the Sink node
    /****************************************************************/
    private void InitDistanceLabels(Network   Net,
                                    Node      Sink,
                                    NodeArray Reached,
                                    NodeArray Dist,
                                    int[]     CntDist) {
        Reached.Fill(NOT_REACHED);
        BoundedQueue BQ = new BoundedQueue(Net.GetNodeIndex());

        Node v;
        Node w;
        Edge e;

        EdgeListIter eit;
        
        Dist.Fill(INFINITY);

        Dist.Put(Sink,0);
        CntDist[0] = 1;
        Reached.Put(Sink,REACHED);
        BQ.Append(Sink);

        while (!BQ.IsEmpty()) {
            v = (Node) BQ.Pop();

            eit = v.GetInListIter();
            for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
                w =e.GetSource();

                if (Reached.GetInt(w) == NOT_REACHED) {
                    Reached.Put(w,REACHED);
                    Dist.Put(w,Dist.GetInt(v) + 1);
                    PutNodeLabel(w,Dist);
                    CntDist[Dist.GetInt(w)]++;
                    BQ.Append(w);
                }
            }
        }

        // DECISION:  WE SHOLD HANDLE "if (!reached[Source)"
    }

}
