import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

class GUIConsole extends Frame implements MouseListener {

    private TextArea ta;
    private final String prompt = "giden%  ";
    private int comm_start_pos;


    GUIConsole() {
        super("GIDEN Console");

        ta = new TextArea("",20,81,TextArea.SCROLLBARS_VERTICAL_ONLY);
        ta.setFont(new Font("Monospaced",Font.PLAIN,12));
        ta.setEditable(true);
        ta.addKeyListener(new KeyEventHandler());
        ta.addMouseListener(this);
        add(ta, BorderLayout.CENTER);

        pack();
        show();

        prepareForCommandInput();
    }

    synchronized private void prepareForCommandInput() {
        ta.append(prompt);
        comm_start_pos = (ta.getText()).length();
        ta.setCaretPosition(comm_start_pos);
    }





    // store key events in a vector, just to make life easy for now;
    // we can refine later as desired
    Vector buffered_key_events = new Vector();


    synchronized public boolean isKeyEventAvailable() {
        return !buffered_key_events.isEmpty();
    }

    synchronized public KeyEvent getKeyEvent() {
        if (buffered_key_events.isEmpty())
            return (KeyEvent) null;
        else {
            // NOTE: deleting the front element causes items to shift,
            // and is probably inefficient (depending on the
            // implementation of Vector); we could likely do better by
            // manually managing an array

            KeyEvent ke = (KeyEvent) buffered_key_events.elementAt(0);
            buffered_key_events.removeElementAt(0);
            return ke;
        }
    }

    synchronized public void putTerminalChar(char c) {
        ta.append((new Character(c)).toString());
    }





    class KeyEventHandler extends KeyAdapter {
        public void keyPressed(KeyEvent evt) {
            char c = evt.getKeyChar();
            int key_code = evt.getKeyCode();

            // consume these naigational commands
            if (key_code == KeyEvent.VK_UP || key_code == KeyEvent.VK_DOWN) {
                evt.consume();
                return;
            }


            // handle "enter" as a special case (since we want to print prompt)
            if (key_code == KeyEvent.VK_ENTER) {
                ta.append("\n");
                prepareForCommandInput();
                buffered_key_events.addElement(evt);
                evt.consume();
                return;
            }


            // check to see if we should consume certain "backup" commands
            if (key_code == KeyEvent.VK_BACK_SPACE || 
                key_code == KeyEvent.VK_DELETE ||
                key_code == KeyEvent.VK_LEFT) {

                // prevent moving back too far (would be nice to beep
                // or something, too...)
                if (ta.getCaretPosition() <= comm_start_pos) {
                    evt.consume();
                    return;
                }
            }

            // save events that have made it this far ("enter" events
            // saved above as a special case)
            buffered_key_events.addElement(evt);
        }
    }

    // mouse event handlers
    public void mouseClicked(MouseEvent evt) {evt.consume();}
    public void mouseEntered(MouseEvent evt) {}
    public void mouseExited(MouseEvent evt) {}
    public void mousePressed(MouseEvent evt) {evt.consume();}
    public void mouseReleased(MouseEvent evt) {evt.consume();}
}
