import java.awt.*;
import java.awt.event.*;
import java.io.*;

class GUIConsole2 extends Frame implements MouseListener {

    private TextArea ta;
    private final String prompt = "giden%  ";
    private int comm_start_pos;
    private int state;

    public static final int ACCEPTING_NEW_COMMAND               = 0;
    public static final int WAITING_FOR_COMMAND_COMPLETION      = 1;


    private PrintWriter stdin; // command process stdin stream
    class SystemCommand extends Thread {
        private String comm;

        public SystemCommand(String comm) {this.comm = new String(comm);}

        class output_listener extends Thread {
            private InputStream s;
            public boolean active = false;

            public output_listener(InputStream stream) {
                active = true;
                this.s = stream;
            }
            public void run() {
                int num, avail;
                byte buf[] = new byte[1024];
                try {
                    avail = (int) Math.max(1,Math.min(s.available(),1024));

                    while ((num=s.read(buf,0,avail)) != -1) {
                        append(new String(buf,0,num));
                        avail = (int) Math.max(1,Math.min(s.available(),1024));
                    }

                    active = false;
                    s.close();
                }
                catch (IOException e) {
                    GIDEN.err.println(e);
                }
            }
        }


        public void run() {
            output_listener err, out;
            try {
                Process proc = Runtime.getRuntime().exec(comm);
                stdin = new PrintWriter(proc.getOutputStream());

                // kludge to get prompts set in ampl
                if (comm.equalsIgnoreCase("ampl")) {
                    stdin.println("option prompt2 'ampl? ';\n");
                    stdin.println("option prompt1 'ampl: ';\n");
                    stdin.flush();
                }
                
                out = new output_listener(proc.getInputStream());
                out.start();

                err = new output_listener(proc.getErrorStream());
                err.start();

                while (out.active || err.active) {
                    try {
                        Thread.sleep(50);
                    } catch (InterruptedException e) {GIDEN.err.println(e);}
                }
                try {
                    proc.waitFor();
                    stdin.close();
                } catch(InterruptedException e) {e.printStackTrace();}
                if (proc.exitValue() != 0)
                    GIDEN.out.println("** Exit code:  "+proc.exitValue());
            } catch(IOException e) {GIDEN.err.println(e);}

            prepareForCommandInput();
        }
    }





    GUIConsole2() {
        super("GIDEN Console (Type-2)");

        ta = new TextArea("",20,81,TextArea.SCROLLBARS_VERTICAL_ONLY);
        ta.setFont(new Font("Monospaced",Font.PLAIN,12));
        ta.setEditable(true);
        ta.addKeyListener(new KeyEventHandler());
        ta.addMouseListener(this);
        add(ta, BorderLayout.CENTER);

        pack();
        show();

        prepareForCommandInput();
    }

    synchronized private void append(String s) {
        ta.append(s);
        ta.setCaretPosition(ta.getText().length());
    }

    synchronized private void executeSystemCommand(String comm) {
        if ("".equals(comm)) {
            prepareForCommandInput();
            return;
        }
        if (comm.equalsIgnoreCase("exit")) {
            this.dispose();
            return;
        }
        SystemCommand sc = new SystemCommand(comm);
        sc.start();
    }

    synchronized private void prepareForCommandInput() {
        ta.append(prompt);
        comm_start_pos = (ta.getText()).length();
        ta.setCaretPosition(comm_start_pos);
        state = ACCEPTING_NEW_COMMAND;
    }

    StringBuffer kbrd_input = new StringBuffer(128);
    int offset = 0;
    class KeyEventHandler extends KeyAdapter {
        public void keyPressed(KeyEvent evt) {
            char c = evt.getKeyChar();
            int key_code = evt.getKeyCode();

            if (key_code == KeyEvent.VK_ENTER) {
                // append character and consume event
                append(String.valueOf(c));
                evt.consume();

                if (state != ACCEPTING_NEW_COMMAND) {
                    // append to input buffer or running process and flush
                    kbrd_input.append(c);
                    stdin.println(kbrd_input.toString());
                    stdin.flush();
                }
                else {
                    // set state and execute command
                    state = WAITING_FOR_COMMAND_COMPLETION;
                    executeSystemCommand(kbrd_input.toString());
                }

                // reset the keyboard input buffer
                kbrd_input = new StringBuffer(128);
                offset = 0;

                return;
            }

            if (key_code == KeyEvent.VK_BACK_SPACE || 
                key_code == KeyEvent.VK_DELETE) {

                // remove last typed key (if any) from input buffer
                if (kbrd_input.length() > 0)
                    kbrd_input.setLength(kbrd_input.length()-1);

                // prevent moving back too far in the text area component 
                // (would be nice to beep or something, too...)
                if (ta.getCaretPosition() <= comm_start_pos)
                    evt.consume();

                return;
            }

            if (key_code == KeyEvent.VK_UP ||
                key_code == KeyEvent.VK_DOWN) {
                evt.consume();
                return;
            }
            if (key_code == KeyEvent.VK_RIGHT) {
                if (offset < kbrd_input.length())
                    offset++;
                return;
            }
            if (key_code == KeyEvent.VK_LEFT) {
                if (offset > 0)
                    offset--;
                else
                    evt.consume();
                return;
            }

            // insert all other typed characters to the input buffer
            kbrd_input.insert(offset++,c);
        }
    }

    // mouse event handlers to prevent using the mouse to set the
    // caret position in the text area component
    public void mouseClicked(MouseEvent evt) {evt.consume();}
    public void mouseEntered(MouseEvent evt) {}
    public void mouseExited(MouseEvent evt) {}
    public void mousePressed(MouseEvent evt) {evt.consume();}
    public void mouseReleased(MouseEvent evt) {evt.consume();}
}
