import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.util.Date;
import java.util.Random;
import java.util.Vector;
import java.util.Enumeration;

class SymmetricTSP extends ExecBase {
    private UserBase UI;
    private Network  Net;

    private final int INFINITY = Integer.MAX_VALUE;

    private boolean initialized = false;

    private boolean continuous_animation = false;

    private ColorSet     InitialSet;
    private ColorSet     Action;
    private ColorSet     Action2;
    private AnimationSet Tour;

    SymmetricTSP(UserBase userinfo) {
        super(userinfo);
        this.UI = userinfo;

        InitialSet = UI.CreateColorSet(Color.lightGray);
        Action     = UI.CreateColorSet(Color.red);
        Action2    = UI.CreateColorSet(Color.blue);
        Tour       = UI.CreateAnimationSet("Tour",Color.orange);
    }

    private int dist[][];
    private NodeArray nidx;
    private void initDistanceMatrix(Network Net) {
        nidx = new NodeArray(Net.GetNodeIndex(),"nidx");
        nidx.Fill(0);

        NodeListIter nit = Net.GetNodeListIter();
        int count=0;
        for (Node i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode())
            nidx.Put(i,count++);
            
        dist = new int[count][count];

        NodeListIter nit1, nit2;
        nit1 = Net.GetNodeListIter();
        Node i,j;
        int i_idx, j_idx;
        for (i=nit1.GetFirstNode() ; i!=null ; i=nit1.GetNextNode()) {
            nit2 = Net.GetNodeListIter();
            for (j=nit2.GetFirstNode() ; j!=null ; j=nit2.GetNextNode())
                if (!i.equals(j)) {
                    i_idx = nidx.GetInt(i);
                    j_idx = nidx.GetInt(j);

                    dist[i_idx][j_idx] = GUIUserUtil.getEuclideanDistance(Node2UiNode(i),Node2UiNode(j));
                    dist[j_idx][i_idx] = dist[i_idx][j_idx];
                }
        }
    }
    private int dist(Node i, Node j) {
        return dist[nidx.GetInt(i)][nidx.GetInt(j)];
        /*
        return GUIUserUtil.getEuclideanDistance(Node2UiNode(i),
                                                Node2UiNode(j));
        */
    }


    private void removeAllEdges(Network net) {
        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
            Net.DeleteEdge(e);
    }

    private void removeAllNodes(Network net) {
        NodeListIter nit = Net.GetNodeListIter();
        for (Node i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode())
            Net.DeleteNode(i);
    }

    public boolean Setup(Network net) {
        if (net == null)
            return false;

        super.Setup(net);

        this.Net = net;
        UI.PutDirected(false);

        initDistanceMatrix(Net);

        if (!initialized) {
            removeAllEdges(Net);
            initialized = true;
        }

        UI.MoveAll(InitialSet);


        return true;
    }

    public boolean Shutdown() {
        removeAllEdges(Net);
        return super.Shutdown();
    }

    public boolean Execute() {
        String action = UI.UserTextInput("Action:  ","r");
        while (!action.equalsIgnoreCase("q")) {
            GIDEN.out.println(" ** action = \""+action+"\"");

            if (action.equalsIgnoreCase("s"))
                // NearestNeighbor(UI,Net,selectRandomNode(Net));
                NearestNeighbor(UI,Net,null);
            else if (action.equalsIgnoreCase("n"))
                // NearestInsertion(UI,Net,selectRandomNode(Net));
                NearestInsertion(UI,Net,null);
            else if (action.equalsIgnoreCase("f"))
                // FarthestInsertion(UI,Net,selectRandomNode(Net));
                FarthestInsertion(UI,Net,null);
            else if (action.equalsIgnoreCase("m"))
                ManualTourBuilder(UI,Net);
            else if (action.equalsIgnoreCase("r"))
                RandomTourBuilder(UI,Net);
            else if (action.equalsIgnoreCase("g")) {
                RandomGridNetworkBuilder(UI,Net);
                initDistanceMatrix(Net);
            }
            else if (action.equalsIgnoreCase("b")) {
                RandomNetworkBuilder(UI,Net);
                initDistanceMatrix(Net);
            }
            else if (action.equalsIgnoreCase("ca")) {
                continuous_animation = true;
                GIDEN.out.println(" ** continuous animation style");
            }
            else if (action.equalsIgnoreCase("da")) {
                continuous_animation = false;
                GIDEN.out.println(" ** discrete animation style");
            }
            else if (action.equalsIgnoreCase("2")) {
                (new TwoOpt(Net)).improveTour();
            }
            else if (action.equalsIgnoreCase("3")) {
                (new ThreeOpt(Net)).improveTour();
            }

            GIDEN.out.println(" --- action completed");

            UI.MoveAll(InitialSet);
            action = UI.UserTextInput("Action:  ","r");
        }
        return true;
    }

    private void RandomNetworkBuilder(UserBase UI,
                                      Network  Net) {
        if (UI.NullBase)
            return;

        removeAllNodes(Net);
        // removeAllEdges(Net); // removing nodes should also remove all edges

        // KLUDGE:  manually get from GUIDrawCanvas; need a better way
        Dimension d = GetUiNet().GUI.draw_canvas.getSize();



        int radius=10;   // desired node radius
        // int p_space=6;   // preferred space between node placements
        int r_space = 2; // required space between node placements


        Random rand = new Random();
        Vector nodes = new Vector();


        int num = -1;
        /*
        int max_size = ((d.width/((2*radius)+r_space+p_space))
                        *(d.height/((2*radius)+r_space+p_space)))/1;
        */
        int max_size = ((d.width/((2*radius)+r_space))
                        *(d.height/((2*radius)+r_space)))/4;
        while (num < 0 || num > max_size)
            num = Integer.parseInt(UI.UserTextInput("Number of nodes in random network (0--"+max_size+"):  ", String.valueOf(max_size/5)));

        Node i,j;
        GUINodeInfo info;
        boolean acceptable;
        Enumeration enum;
        int x,y;
        int dist;
        while (nodes.size() < num) {
            x = (int) Math.floor(rand.nextDouble()*(d.width-2*radius));
            y = (int) Math.floor(rand.nextDouble()*(d.height-2*radius));
            x += radius;
            y += radius;
            //            i = UI.CreateNode(x,y);
            i = UI.CreateNode(x,y,GUINode.SMALL_NODE_STYLE);

            acceptable = true;
            enum = nodes.elements();
            while (enum.hasMoreElements()) {
                j = (Node) enum.nextElement();
                dist = GUIUserUtil.getEuclideanDistance(Node2UiNode(i),
                                                        Node2UiNode(j));
                if (dist < 2*radius+r_space) {
                    acceptable = false;
                    break;
                }
            }
            if (acceptable) {
                /*
                info = ((GUINode) Node2UiNode(i)).GetInfo();
                info.radius = radius;
                info.shape = GUINodeInfo.CIRCLE_SHAPE;
                ((GUINode) Node2UiNode(i)).PutInfo(info);
                */
                UI.PutLabel(i,"");
                nodes.addElement(i);
            }
            else
                Net.DeleteNode(i);
        }
    }

    private void RandomGridNetworkBuilder(UserBase UI,
                                          Network  Net) {
        if (UI.NullBase)
            return;

        removeAllNodes(Net);
        // removeAllEdges(Net); // removing nodes should also remove all edges

        // KLUDGE:  manually get from GUIDrawCanvas; need a better way
        Dimension d = GetUiNet().GUI.draw_canvas.getSize();



        Random rand = new Random();
        Vector points = new Vector();


        /*
        // DRAWS IN A GRID WITH MINOR RANDOMNESS IN THE PADDING
        int radius=10;   // desired node radius
        int p_space=6;   // preferred space between node placements
        int r_space = 2; // required space between node placements

        int x,y,xc,yc;
        int increment = (2*radius)+p_space+r_space;
        for (x=0 ; x <= d.width-increment ; x+=increment)
            for (y=0 ; y <= d.height-increment ; y+=increment) {
                xc = x+radius+((int) Math.floor(rand.nextDouble()*p_space));
                yc = y+radius+((int) Math.floor(rand.nextDouble()*p_space));
                points.addElement(new Point(xc,yc));
            }
        */

        // DRAWS IN A STRICT GRID
        int radius=10;   // desired node radius
        int space=6;     // space between node placements

        int x,y;
        int increment = (2*radius)+space;
        for (x=increment ; x < d.width-increment ; x+=increment)
            for (y=increment ; y < d.height-increment ; y+=increment)
                points.addElement(new Point(x,y));



        int num = -1;
        while (num < 0 || num > points.size())
            num = Integer.parseInt(UI.UserTextInput("Number of nodes in random network (0--"+
                                   points.size()+"):  ",
                                   String.valueOf(points.size()/5)));

        int count=0;
        int random_idx;
        Point coord;
        //        Random rand = new Random();
        Node i;
        GUINodeInfo info;
        while (count < num) {
            random_idx = (int) Math.floor(rand.nextDouble()*points.size());
            coord = (Point) points.elementAt(random_idx);
            points.removeElement(coord);
            /*
            i = UI.CreateNode(coord.x,coord.y);
            info = ((GUINode) Node2UiNode(i)).GetInfo();
            info.radius = radius;
            info.shape = GUINodeInfo.CIRCLE_SHAPE;
            ((GUINode) Node2UiNode(i)).PutInfo(info);
            */
            i = UI.CreateNode(coord.x,coord.y,GUINode.SMALL_NODE_STYLE);
            UI.PutLabel(i,"");
            count++;
        }
    }

    private void RandomTourBuilder(UserBase UI,
                                   Network  Net) {
        removeAllEdges(Net);

        // initialize "NotVisited" list of nodes
        Vector NotVisited = new Vector(Net.GetNodeCount());
        NodeListIter nit = Net.GetNodeListIter();
        for (Node i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode()) {
            NotVisited.addElement(i);
            UI.PutLabel(i,"");
        }

        // set the start node
        Node start = selectRandomNode(Net);
        NotVisited.removeElement(start);
        UI.PutLabel(start,"s");
        UI.SingleStep(start,Tour);



        Edge e;
        Node last,current;
        int TourDistance = 0;
        int random_idx;
        Random rand = new Random();

        last = start;
        while (NotVisited.isEmpty() == false) {
            random_idx = (int) Math.floor(rand.nextDouble()*NotVisited.size());
            current = (Node) NotVisited.elementAt(random_idx);

            NotVisited.removeElement(current);
            TourDistance += dist(last,current);
            UI.StatusLine("Current distance:  "+TourDistance);
            e = UI.CreateEdge(last,current,GUIEdge.THIN_EDGE_STYLE);
            UI.PutLabel(e,"");  // UI.PutLabel(e,""+dist(last,current));
            UI.SingleStep(e, Tour);
            UI.SingleStep(current, Tour);

            last = current;
        }
        TourDistance += dist(last,start);
        UI.StatusLine("Total tour distance:  "+TourDistance);
        e = UI.CreateEdge(last,start,GUIEdge.THIN_EDGE_STYLE);
        UI.PutLabel(e,"");  // UI.PutLabel(e,""+dist(last,current));
        UI.SingleStep(e, Tour);
        UI.Trace(Tour);
        UI.Update();
        UI.StatusLine("");
        UI.PutLabel(start,"");
    }


    private void ManualTourBuilder(UserBase UI,
                                   Network  Net) {
        if (UI.NullBase)
            return;

        removeAllEdges(Net);

        // initialize "NotVisited" list of nodes
        Vector NotVisited = new Vector(Net.GetNodeCount());
        NodeListIter nit = Net.GetNodeListIter();
        for (Node i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode()) {
            NotVisited.addElement(i);
            UI.PutLabel(i,"");
        }

        // set the start node
        Node start = null;
        while (start == null)
            start = UI.UserMouseNodeInput("Select a starting node.");
        NotVisited.removeElement(start);
        UI.PutLabel(start,"s");
        UI.SingleStep(start,Tour);
        

        Edge e;
        Node last,current;
        int TourDistance = 0;

        last = start;
        while (NotVisited.isEmpty() == false) {
            current = (Node) null;
            while (current == null || NotVisited.contains(current) == false)
                current = UI.UserMouseNodeInput("Select the next tour node.");

            NotVisited.removeElement(current);
            TourDistance += dist(last,current);
            UI.StatusLine("Current distance:  "+TourDistance);
            e = UI.CreateEdge(last,current,GUIEdge.THIN_EDGE_STYLE);
            //            UI.PutLabel(e,""+nearest_dist);
            UI.PutLabel(e,"");
            UI.SingleStep(e, Tour);
            UI.SingleStep(current, Tour);

            last = current;
        }
        TourDistance += dist(last,start);
        UI.StatusLine("Total tour distance:  "+TourDistance);
        e = UI.CreateEdge(last,start,GUIEdge.THIN_EDGE_STYLE);
        //        UI.PutLabel(e,""+curr_dist);
        UI.PutLabel(e,"");
        UI.SingleStep(e, Tour);
        UI.Trace(Tour);
        UI.Update();
        UI.StatusLine("");
        UI.PutLabel(start,"");
    }

    private void NearestInsertion(UserBase UI,
                                  Network  Net,
                                  Node     start) {
        Node i,j,k;
        NodeListIter nit;

        removeAllEdges(Net);

        // initialize "Visited" and "NotVisited" lists of nodes
        Vector Visited = new Vector(Net.GetNodeCount());
        Vector NotVisited = new Vector(Net.GetNodeCount());
        nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode()) {
            NotVisited.addElement(i);
            UI.PutLabel(i,"");
        }

        // make sure that we have a starting node
        if (start == null) {
            if (UI.NullBase)
                start = selectRandomNode(Net);
            else
                while (start == null)
                    start = UI.UserMouseNodeInput("Click on starting node.");
        }
        Visited.addElement(start);
        NotVisited.removeElement(start);
        UI.StatusLine("Using start node \"s\" for nearest insertion heuristic.");
        UI.PutLabel(start,"s");
        UI.SingleStep(start,Tour);
        UI.StatusLine("");

        int TourDistance = 0;
        int curr_dist, best_dist;
        Edge e, deletion_edge;
        EdgeListIter eit;
        Node nearest_node = null;
        Enumeration enum_t, enum_u;

        while (NotVisited.isEmpty() == false) {

            /*
            // take arbitrary node as the incumbant choice
            k = (Node) NotVisited.firstElement();
            j = (Node) Visited.firstElement();
            nearest_node = k;
            best_dist = dist(j,k);
            */
            best_dist = INFINITY;

            // consider all unvisited nodes and select the nearest
            enum_u = NotVisited.elements();
            while (enum_u.hasMoreElements()) {
                k = (Node) enum_u.nextElement();

                enum_t = Visited.elements();
                while (enum_t.hasMoreElements()) {
                    j = (Node) enum_t.nextElement();

                    curr_dist = dist(k,j);
                    if (curr_dist < best_dist) {
                        nearest_node = k;
                        best_dist = curr_dist;
                    }
                }
            }

            // now find the best place to insert "nearest_node"
            eit = Net.GetEdgeListIter(); // Net only contains "tour" edges
            deletion_edge = null;
            best_dist = INFINITY;
            for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
                i = e.GetSource();
                j = e.GetTarget();
                curr_dist = dist(i,nearest_node) + dist(nearest_node,j) 
                    - dist(i,j);
                if (curr_dist < best_dist) {
                    deletion_edge = e;
                    best_dist = curr_dist;
                }
            }
            // prepare to insert "nearest_node" into current tour
            if (deletion_edge == null) {
                // current tour is a single node; set i == j in tour
                i = (Node) Visited.firstElement();
                j = i;
                best_dist = dist(i,nearest_node) + dist(nearest_node,j);
            }
            else {
                // set i and j, and delete edge (i,j) from current tour
                i = deletion_edge.GetSource();
                j = deletion_edge.GetTarget();
                Net.DeleteEdge(deletion_edge);
            }

            // update tour cost
            TourDistance += best_dist;
            UI.StatusLine("Current distance:  "+TourDistance);

            // insert edges (i,nearest_node) and (nearest_node,j) into tour
            e = UI.CreateEdge(i,nearest_node,GUIEdge.THIN_EDGE_STYLE);
            UI.PutLabel(e,"");  // UI.PutLabel(e,""+curr_dist);
            UI.SingleStep(e, Tour);
            e = UI.CreateEdge(nearest_node,j,GUIEdge.THIN_EDGE_STYLE);
            UI.PutLabel(e,"");  // UI.PutLabel(e,""+curr_dist);
            UI.SingleStep(e, Tour);

            // finally add "nearest_node" to tour
            UI.SingleStep(nearest_node, Tour);
            NotVisited.removeElement(nearest_node);
            Visited.addElement(nearest_node);
animate();
        }
        UI.StatusLine("Total tour distance:  "+TourDistance);
        UI.Trace(Tour);
        UI.Update();
        UI.StatusLine("");
        UI.PutLabel(start,"");
    }

    private void FarthestInsertion(UserBase UI,
                                   Network  Net,
                                   Node     start) {
        Node i,j,k;
        NodeListIter nit;

        removeAllEdges(Net);

        // initialize "Visited" and "NotVisited" lists of nodes
        Vector Visited = new Vector(Net.GetNodeCount());
        Vector NotVisited = new Vector(Net.GetNodeCount());
        nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode()) {
            NotVisited.addElement(i);
            UI.PutLabel(i,"");
        }

        // make sure that we have a starting node
        if (start == null) {
            if (UI.NullBase)
                start = selectRandomNode(Net);
            else
                while (start == null)
                    start = UI.UserMouseNodeInput("Click on starting node.");
        }
        Visited.addElement(start);
        NotVisited.removeElement(start);
        UI.StatusLine("Using start node \"s\" for nearest insertion heuristic.");
        UI.PutLabel(start,"s");
        UI.SingleStep(start,Tour);
        UI.StatusLine("");

        int TourDistance = 0;
        int curr_dist, best_dist, local_best_dist;
        Edge e, deletion_edge;
        EdgeListIter eit;
        Node nearest_node = null;
        Enumeration enum_t, enum_u;

        while (NotVisited.isEmpty() == false) {

            /*
            // take arbitrary node as the incumbant choice
            k = (Node) NotVisited.firstElement();
            j = (Node) Visited.firstElement();
            nearest_node = k;
            best_dist = dist(k,j);
            */
            best_dist = -INFINITY;

            // consider all unvisited nodes and select the nearest
            enum_u = NotVisited.elements();
            while (enum_u.hasMoreElements()) {
                k = (Node) enum_u.nextElement();

                local_best_dist = INFINITY;

                enum_t = Visited.elements();
                while (enum_t.hasMoreElements()) {
                    j = (Node) enum_t.nextElement();

                    curr_dist = dist(k,j);
                    if (curr_dist < local_best_dist)
                        local_best_dist = curr_dist;
                }

                if (local_best_dist > best_dist) {
                    nearest_node = k;
                    best_dist = local_best_dist;
                }
            }

            // now find the best place to insert "nearest_node"
            eit = Net.GetEdgeListIter(); // Net only contains "tour" edges
            deletion_edge = null;
            best_dist = INFINITY;
            for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {
                i = e.GetSource();
                j = e.GetTarget();
                curr_dist = dist(i,nearest_node) + dist(nearest_node,j) 
                    - dist(i,j);
                if (curr_dist < best_dist) {
                    deletion_edge = e;
                    best_dist = curr_dist;
                }
            }
            // prepare to insert "nearest_node" into current tour
            if (deletion_edge == null) {
                // current tour is a single node; set i == j in tour
                i = (Node) Visited.firstElement();
                j = i;
                best_dist = dist(i,nearest_node) + dist(nearest_node,j);
            }
            else {
                // set i and j, and delete edge (i,j) from current tour
                i = deletion_edge.GetSource();
                j = deletion_edge.GetTarget();
                Net.DeleteEdge(deletion_edge);
            }

            // update tour cost
            TourDistance += best_dist;
            UI.StatusLine("Current distance:  "+TourDistance);

            // insert edges (i,nearest_node) and (nearest_node,j) into tour
            e = UI.CreateEdge(i,nearest_node,GUIEdge.THIN_EDGE_STYLE);
            UI.PutLabel(e,"");  // UI.PutLabel(e,""+curr_dist);
            UI.SingleStep(e, Tour);
            e = UI.CreateEdge(nearest_node,j,GUIEdge.THIN_EDGE_STYLE);
            UI.PutLabel(e,"");  // UI.PutLabel(e,""+curr_dist);
            UI.SingleStep(e, Tour);

            // finally add "nearest_node" to tour
            UI.SingleStep(nearest_node, Tour);
            NotVisited.removeElement(nearest_node);
            Visited.addElement(nearest_node);
animate();
        }
        UI.StatusLine("Total tour distance:  "+TourDistance);
        UI.Trace(Tour);
        UI.Update();
        UI.StatusLine("");
        UI.PutLabel(start,"");
    }


    private void NearestNeighbor(UserBase UI,
                                 Network  Net,
                                 Node     start) {
        Node i,j;
        NodeListIter nit;

        removeAllEdges(Net);

        // initialize "NotVisited" list of nodes
        Vector NotVisited = new Vector(Net.GetNodeCount());
        nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode()) {
            NotVisited.addElement(i);
            UI.PutLabel(i,"");
        }

        // make sure that we have a starting node
        if (start == null) {
            if (UI.NullBase)
                start = selectRandomNode(Net);
            else
                while (start == null)
                    start = UI.UserMouseNodeInput("Click on starting node.");
        }
        NotVisited.removeElement(start);
        UI.StatusLine("Using start node \"s\" for nearest neighbor heuristic.");
        UI.PutLabel(start,"s");
        UI.SingleStep(start,Tour);
        UI.StatusLine("");

        int TourDistance = 0;
        int curr_dist, nearest_dist;
        Edge e;
        Node last, nearest_node;
        Enumeration enum;

        last = start;
        while (NotVisited.isEmpty() == false) {

            // take arbitrary node as the incumbant choice
            j = (Node) NotVisited.firstElement();
            nearest_node = j;
            nearest_dist = dist(last,j);

            // consider all unvisited nodes and select the nearest
            enum = NotVisited.elements();
            while (enum.hasMoreElements()) {
                j = (Node) enum.nextElement();

                curr_dist = dist(last,j);
                if (curr_dist < nearest_dist) {
                    nearest_node = j;
                    nearest_dist = curr_dist;
                }
            }

            NotVisited.removeElement(nearest_node);
            TourDistance += nearest_dist;
            UI.StatusLine("Current distance:  "+TourDistance);
            e = UI.CreateEdge(last,nearest_node,GUIEdge.THIN_EDGE_STYLE);
            //            UI.PutLabel(e,""+nearest_dist);
            UI.PutLabel(e,"");
            UI.SingleStep(e, Tour);
            UI.SingleStep(nearest_node, Tour);
animate();
            UI.StatusLine("");

            last = nearest_node;
        }
        
        curr_dist = dist(last,start);
        TourDistance += curr_dist;
        UI.StatusLine("Total tour distance:  "+TourDistance);
        e = UI.CreateEdge(last,start,GUIEdge.THIN_EDGE_STYLE);
        //        UI.PutLabel(e,""+curr_dist);
        UI.PutLabel(e,"");
        UI.SingleStep(e, Tour);
        UI.Trace(Tour);
        UI.Update();
        UI.StatusLine("");
        UI.PutLabel(start,"");
    }


    class ThreeOpt {
        private Network Net;
        private int TourDistance;
        private Node T[]; // store nodes in the "tour" order
        private Node tt[]; // temp storage for tour modifications
        private int n;

        private final int NO_IMPROVEMENT = -1;
        private final int TYPE_1         =  1;
        private final int TYPE_2         =  2;
        private final int TYPE_3         =  3;
        private final int TYPE_4         =  4;

        ThreeOpt(Network net) {
            this.Net = net;
            n = Net.GetNodeCount();

            T = new Node[n+1];
            tt = new Node[n+1];
            // Node source = (Net.GetNodeListIter()).GetFirstNode();
            Node source = selectRandomNode(Net);
            // (just a rough check)     Node source = selectRandomNode(Net);
UI.PutLabel(source,"s");
            int position = 0;
            T[position++] = source;

            EdgeListIter eit;
            Edge curr = (source.GetAdjListIter()).GetFirstEdge();
            Node i = curr.GetOpposite(source);
            Edge prev;
            while (!i.equals(source)) {
                T[position++] = i;
                eit = i.GetAdjListIter();
                prev = curr;
                curr = eit.GetFirstEdge();
                if (curr.equals(prev))
                    curr = eit.GetNextEdge();
                i = curr.GetOpposite(i);
            }
            T[position] = i;


            // brute force to calculate tour distance; we should
            // maintain this as global information
            TourDistance = 0;
            for (int k=1 ; k <= n ; k++)
                TourDistance += dist(T[k-1],T[k]);
            GIDEN.out.println(" ** 3-opt: calculated initial tour distance = "
                               +TourDistance);
        }

        public void improveTour() {
            Edge old_e1, old_e2, old_e3, new_e1, new_e2, new_e3;
            int i1,i2,i3,j1,j2,j3;
            int idx_T,idx_tt;

            int type;
            int current, candidate, best; // improvement values

            boolean interchange_optimal = false;
            while (!interchange_optimal) {
                interchange_optimal = true;

                for (i1=0 ; i1<=n-5 ; i1++)
                    for (i2=i1+2 ; i2<=n-3 ; i2++)
                        for (i3=i2+2 ; i3<=n-1 ; i3++) {
                            if (!(i1==0 && i3==n-1)) {
                                j1 = i1+1;
                                j2 = i2+1;
                                j3 = i3+1;
UI.SingleStep(findEdge(T[i1],T[j1]),Action2);
UI.SingleStep(findEdge(T[i2],T[j2]),Action2);
UI.SingleStep(findEdge(T[i3],T[j3]),Action2);

                                // calculate current edge costs
                                current = dist(T[i1],T[j1]) +
                                    dist(T[i2],T[j2]) + dist(T[i3],T[j3]);
                                type = NO_IMPROVEMENT;
                                best = current;

                                candidate = dist(T[i1],T[i2]) +
                                    dist(T[j1],T[i3]) + dist(T[j2],T[j3]);
                                if (candidate < best) {
                                    best = candidate;
                                    type = TYPE_1;
                                }

                                candidate = dist(T[i1],T[j2]) +
                                    dist(T[i3],T[j1]) + dist(T[i2],T[j3]);
                                if (candidate < best) {
                                    best = candidate;
                                    type = TYPE_2;
                                }

                                candidate = dist(T[i1],T[j2]) +
                                    dist(T[i3],T[i2]) + dist(T[j1],T[j3]);
                                if (candidate < best) {
                                    best = candidate;
                                    type = TYPE_3;
                                }

                                candidate = dist(T[i1],T[i3]) +
                                    dist(T[j2],T[j1]) + dist(T[i2],T[j3]);
                                if (candidate < best) {
                                    best = candidate;
                                    type = TYPE_4;
                                }

                                if (type != NO_IMPROVEMENT) {
                                    old_e1 = findEdge(T[i1],T[j1]);
                                    old_e2 = findEdge(T[i2],T[j2]);
                                    old_e3 = findEdge(T[i3],T[j3]);
                                    UI.SingleStep(old_e1,Action);
                                    UI.SingleStep(old_e2,Action);
                                    UI.SingleStep(old_e3,Action);

                                    TourDistance -= (current-best);

                                    idx_tt = j1;

                                    if (type == TYPE_1) {
                                        idx_T = i2;
                                        while (idx_T >= j1)
                                            tt[idx_tt++] = T[idx_T--];
                                        idx_T = i3;
                                        while (idx_T >= j2)
                                            tt[idx_tt++] = T[idx_T--];

                                        new_e1 = UI.CreateEdge(T[i1],T[i2],GUIEdge.THIN_EDGE_STYLE);
                                        new_e2 = UI.CreateEdge(T[j1],T[i3],GUIEdge.THIN_EDGE_STYLE);
                                        new_e3 = UI.CreateEdge(T[j2],T[j3],GUIEdge.THIN_EDGE_STYLE);
                                    }
                                    else if (type == TYPE_2) {
                                        idx_T = j2;
                                        while (idx_T <= i3)
                                            tt[idx_tt++] = T[idx_T++];
                                        idx_T = j1;
                                        while (idx_T <= i2)
                                            tt[idx_tt++] = T[idx_T++];

                                        new_e1 = UI.CreateEdge(T[i1],T[j2],GUIEdge.THIN_EDGE_STYLE);
                                        new_e2 = UI.CreateEdge(T[i3],T[j1],GUIEdge.THIN_EDGE_STYLE);
                                        new_e3 = UI.CreateEdge(T[i2],T[j3],GUIEdge.THIN_EDGE_STYLE);
                                    }
                                    else if (type == TYPE_3) {
                                        idx_T = j2;
                                        while (idx_T <= i3)
                                            tt[idx_tt++] = T[idx_T++];
                                        idx_T = i2;
                                        while (idx_T >= j1)
                                            tt[idx_tt++] = T[idx_T--];

                                        new_e1 = UI.CreateEdge(T[i1],T[j2],GUIEdge.THIN_EDGE_STYLE);
                                        new_e2 = UI.CreateEdge(T[i3],T[i2],GUIEdge.THIN_EDGE_STYLE);
                                        new_e3 = UI.CreateEdge(T[j1],T[j3],GUIEdge.THIN_EDGE_STYLE);
                                    }
                                    else { // (type == TYPE_4)
                                        idx_T = i3;
                                        while (idx_T >= j2)
                                            tt[idx_tt++] = T[idx_T--];
                                        idx_T = j1;
                                        while (idx_T <= i2)
                                            tt[idx_tt++] = T[idx_T++];

                                        new_e1 = UI.CreateEdge(T[i1],T[i3],GUIEdge.THIN_EDGE_STYLE);
                                        new_e2 = UI.CreateEdge(T[j2],T[j1],GUIEdge.THIN_EDGE_STYLE);
                                        new_e3 = UI.CreateEdge(T[i2],T[j3],GUIEdge.THIN_EDGE_STYLE);
                                    }

                                    // copy the relevant tour info
                                    for (idx_T=j1 ; idx_T <= i3 ; idx_T++)
                                        T[idx_T] = tt[idx_T];

                                    UI.PutLabel(new_e1,"");
                                    UI.PutLabel(new_e2,"");
                                    UI.PutLabel(new_e3,"");
                                    UI.SingleStep(new_e1, Tour);
                                    UI.SingleStep(new_e2, Tour);
                                    UI.SingleStep(new_e3, Tour);
animate();
                                    // clean up from interchange
                                    UI.SingleStep(new_e1, InitialSet);
                                    UI.SingleStep(new_e2, InitialSet);
                                    UI.SingleStep(new_e3, InitialSet);
                                    Net.DeleteEdge(old_e1);
                                    Net.DeleteEdge(old_e2);
                                    Net.DeleteEdge(old_e3);

                                    // mark for another pass
                                    interchange_optimal = false;
                                }
else {
    UI.SingleStep(findEdge(T[i1],T[j1]),InitialSet);
    UI.SingleStep(findEdge(T[i2],T[j2]),InitialSet);
    UI.SingleStep(findEdge(T[i3],T[j3]),InitialSet);
}
                            }
                        }
            }
            UI.StatusLine("Current tour distance:  "+TourDistance);
            UI.Trace(Tour);
            UI.Update();
            UI.StatusLine("");
UI.PutLabel(T[0],"");
        }

        private Edge findEdge(Node i, Node j) {
            EdgeListIter eit = i.GetAdjListIter();
            Edge e = eit.GetFirstEdge();

            // we should check e!=null for robustness, but if we have
            // a tour then this is safe
            while (!j.equals(e.GetOpposite(i)))
                e = eit.GetNextEdge();

            return e;
        }
    }

    class TwoOpt {
        private Network Net;
        private int TourDistance;
        private Node T[]; // store nodes in the "tour" order
        private int n;

        TwoOpt(Network net) {
            // relies on Net only containing the edges in a tour
            // (i.e., each node is incident to exactly two edges)

            // probably should maintain a global variable indicating
            // if a tour is available; we could then check immediately
            // and return if not.

            this.Net = net;
            n = Net.GetNodeCount();

            T = new Node[n+1];
            Node source = (Net.GetNodeListIter()).GetFirstNode();
            // (just a rough check)     Node source = selectRandomNode(Net);
            int position = 0;
            T[position++] = source;

            EdgeListIter eit;
            Edge curr = (source.GetAdjListIter()).GetFirstEdge();
            Node i = curr.GetOpposite(source);
            Edge prev;
            while (!i.equals(source)) {
                T[position++] = i;
                eit = i.GetAdjListIter();
                prev = curr;
                curr = eit.GetFirstEdge();
                if (curr.equals(prev))
                    curr = eit.GetNextEdge();
                i = curr.GetOpposite(i);
            }
            T[position] = i;


            // brute force to calculate tour distance; we should
            // maintain this as global information
            TourDistance = 0;
            for (int k=1 ; k <= n ; k++)
                TourDistance += dist(T[k-1],T[k]);
            GIDEN.out.println(" ** 2-opt: calculated initial tour distance = "
                               +TourDistance);
        }

        public void improveTour() {
            Edge old_e1, old_e2, new_e1, new_e2;
            Node temp;
            int i1,i2,j1,j2;
            int idx1,idx2;

            boolean interchange_optimal = false;
            while (!interchange_optimal) {
                interchange_optimal = true;

                for (i1=0 ; i1<=n-3 ; i1++)
                    for (i2=i1+2 ; i2<=n-1 ; i2++) {
                        if (!(i1==0 && i2==n-1)) {
                            j1 = i1+1;
                            j2 = i2+1;
UI.SingleStep(findEdge(T[i1],T[j1]),Action2);
UI.SingleStep(findEdge(T[i2],T[j2]),Action2);
                            // check for improvement
                            if (dist(T[i1],T[j1]) + dist(T[i2],T[j2]) >
                                dist(T[i1],T[i2]) + dist(T[j1],T[j2])) {

                                /*
                                // label nodes of interest
                                UI.PutLabel(T[i1],"i1");
                                UI.PutLabel(T[j1],"j1");
                                UI.PutLabel(T[i2],"i2");
                                UI.PutLabel(T[j2],"j2");
                                */

                                // mark old edges (to be deleted)
                                old_e1 = findEdge(T[i1],T[j1]);
                                old_e2 = findEdge(T[i2],T[j2]);
                                UI.SingleStep(old_e1,Action);
                                UI.SingleStep(old_e2,Action);

                                // create new edges
                                new_e1 = UI.CreateEdge(T[i1],T[i2],GUIEdge.THIN_EDGE_STYLE);
                                new_e2 = UI.CreateEdge(T[j1],T[j2],GUIEdge.THIN_EDGE_STYLE);
                                UI.PutLabel(new_e1,"");
                                UI.PutLabel(new_e2,"");
                                UI.SingleStep(new_e1, Tour);
                                UI.SingleStep(new_e2, Tour);

                                // update TourDistance
                                TourDistance -= (dist(T[i1],T[j1]) 
                                                 + dist(T[i2],T[j2]));
                                TourDistance += (dist(T[i1],T[i2]) 
                                                 + dist(T[j1],T[j2]));

                                // update order of nodes in tour
                                idx1 = j1;
                                idx2 = i2;
                                while (idx1 < idx2) {
                                    temp = T[idx1];
                                    T[idx1++] = T[idx2];
                                    T[idx2--] = temp;
                                }

animate();
                                // clean up from interchange
                                UI.SingleStep(new_e1, InitialSet);
                                UI.SingleStep(new_e2, InitialSet);
                                /*
                                UI.PutLabel(T[i1],"");
                                UI.PutLabel(T[j1],"");
                                UI.PutLabel(T[i2],"");
                                UI.PutLabel(T[j2],"");
                                */
                                Net.DeleteEdge(old_e1);
                                Net.DeleteEdge(old_e2);
 
                                // mark for another pass
                                interchange_optimal = false;
                            }
else {
    UI.SingleStep(findEdge(T[i1],T[j1]),InitialSet);
    UI.SingleStep(findEdge(T[i2],T[j2]),InitialSet); 
}
                        }
                    }
            }
            UI.StatusLine("Current tour distance:  "+TourDistance);
            UI.Trace(Tour);
            UI.Update();
            UI.StatusLine("");
        }

        private Edge findEdge(Node i, Node j) {
            EdgeListIter eit = i.GetAdjListIter();
            Edge e = eit.GetFirstEdge();

            // we should check e!=null for robustness, but if we have
            // a tour then this is safe
            while (!j.equals(e.GetOpposite(i)))
                e = eit.GetNextEdge();

            return e;
        }
    }



    private Node selectRandomNode(Network Net) {
        int random_idx = (int) Math.floor(((new Random()).nextDouble())
                                          *Net.GetNodeCount());

        int count;
        Node i;
        NodeListIter nit = Net.GetNodeListIter();
        for (count=0, i=nit.GetFirstNode() ; i!= null ; i=nit.GetNextNode(),count++)
            if (count == random_idx) return i;

        GIDEN.out.println("** Error!  Could not select random node");
        return (Net.GetNodeListIter()).GetFirstNode();
    }

    
    private Date lastTime = new Date();
    private void animate() {animate(100);}
    private void animate(long ms) {
        if (!continuous_animation) return;

        UI.DrawNetwork();
        //        GetUiNet().GUI.draw_canvas.repaint();
        Date currTime = new Date();
        try {
            Thread.sleep(ms);
            /*
            Thread.sleep(Math.max((long)0, 
                                  ms-(currTime.getTime()-lastTime.getTime())));
            */
        } catch(InterruptedException e){}
        lastTime = currTime;
    }
}
