package giden.contrib.NU_IEMS_452_W04;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;
import java.awt.Point;

/*
* talk about algorithm here
*/

public class NodeCover extends ExecBase
{
	UserBase UI;
	Network Net;
	GraphState State;

	Color InitialColor;
	Color ElimColor;
	Color SelecColor;
		
	AnimationSet InitialSet;
	AnimationSet Eliminated;	//eliminated facility location
	AnimationSet Selected;	//selected facility location
	FinalSet Cover;
		
	NodeArray2dInt Coverage;
	NodeArray Facility;
	NodeArray Demand;
	EdgeArray Lengths;

	int NodeCount;
	int CoverCount=0;
	int CoveredCount=0;
	
	int CDist;

	boolean ValidNetwork;

	//******************************************
	public NodeCover(UserBase userinfo)
	{
		super(userinfo);
		UI = userinfo;
	
		InitialColor = Color.cyan;
		ElimColor = Color.lightGray;
		SelecColor = Color.green;
		
		InitialSet = UI.CreateAnimationSet("",InitialColor);
		Eliminated = UI.CreateAnimationSet("Eliminated",ElimColor);
		Selected = UI.CreateAnimationSet("Cover",SelecColor);
		Cover = UI.CreateFinalSet("Cover",Color.green);
		
		UI.CreateAnimationButtons();
		UI.CreateFinalButtons();
	}

	//********************************************
	public boolean Setup(Network net)
	{
		Net = net;
		State = new GraphState(Net);

		NodeCount = Net.GetNodeCount();
		if(NodeCount<2)
		{
			UI.UserMouseNodeInput("Solver needs two or more nodes (click anywhere)");
			ValidNetwork = false;
			return false;
		}
		if(Net.GetEdgeCount() != 0)
		{
			UI.UserMouseNodeInput("Network should have no edges (click anywhere)");
			ValidNetwork = false;
			return false;
		}

		CoverCount = 0;
		CoveredCount=0;	
	
		UI.ShowPseudoCodeText("CDist");
		String temp = UI.UserTextInput("Enter Coverage Distance:","0");
		CDist = Integer.parseInt(temp);

		Lengths= Net.GetEdgeArray("Euclidean Length");
		FillEdgeArray(Lengths);

		Facility = new NodeArray(NodeCount,"Facility");
		Facility.Fill(-NodeCount);

		Demand = new NodeArray(NodeCount,"Demand");
		Demand.Fill(0);

		UI.PutDirected(false);
		UI.MoveAll(InitialSet);
		
		ValidNetwork=true;
		return true;
	}

	//**********************************************
	public boolean Shutdown()
	{
		super.Shutdown();
		State.Restore();
		return true;
	}

	//**************************************************
	public boolean Execute()
	{
		if(ValidNetwork==false)
			return false;
		UI.StatusLine("Executing");
		NodeCoverSolver();
		UI.StatusLine("Final network - Number of Cover Nodes = "+CoverCount);
		UI.ShowPseudoCodeText("end");
		DrawCover();
		return true;
	}

	//***************************************************
	void FillEdgeArray(EdgeArray lengths)
	{
		UI.StatusLine("Network with edges less than " + CDist + " in length.");	
		UI.ShowPseudoCodeText("CovMatrix");

		Coverage = new NodeArray2dInt(Net);
	
		NodeListIter nit1 = Net.GetNodeListIter();
		for(Node node1=nit1.GetFirstNode();node1!=null;node1=nit1.GetNextNode())
		{
			NodeListIter nit2 = Net.GetNodeListIter();
			for(Node node2=nit2.GetFirstNode();node2!=null;node2=nit2.GetNextNode())
			{
				if(node1.GetIndex()<node2.GetIndex())
				{
					Edge edge=Net.CreateEdge(node1,node2);
					UI.AddEdge(edge);
					UI.PutEdgeVisibilty(edge,false);
					UI.PutLabel(edge,lengths.GetInt(edge));
					int dist=lengths.GetInt(edge);
					if (dist<=CDist)
					{
						Coverage.PutInt(node1,node2,1);
						Coverage.PutInt(node2,node1,1);
						UI.PutEdgeVisibilty(edge,true);
					}
					else
					{
						Coverage.PutInt(node1,node2,0);
						Coverage.PutInt(node2,node1,0);
					}
				}
				else if(node1.GetIndex()==node2.GetIndex())
				{
					Coverage.PutInt(node1,node2,1);
				}
			}
		}
		System.out.println();
		System.out.println("Coverage Matrix");
		NodeListIter nit3 = Net.GetNodeListIter();
		for(Node node1=nit3.GetFirstNode();node1!=null;node1=nit3.GetNextNode())
		{
			NodeListIter nit4 = Net.GetNodeListIter();
			for(Node node2=nit4.GetFirstNode();node2!=null;node2=nit4.GetNextNode())
			{
				 System.out.print(Coverage.GetInt(node1,node2)+" ");
			}
			System.out.println();
		}
	}

	//*****************************************************
	//int Evaluate()
	//void DrawCover()

	//******************************************************
	public void NodeCoverSolver()
	{
		boolean change=true;
		boolean elim=true;
		int sum=0;	
		int index=0;
		Node check;
	
		while(change)
		{
			UI.StatusLine("Eliminating Inferior Nodes");
			UI.ShowPseudoCodeText("Inferior");
			change=false;
			
			NodeListIter nit1 = Net.GetNodeListIter();
			for(Node fac1=nit1.GetFirstNode();fac1!=null;fac1=nit1.GetNextNode())
			{
				if(Facility.GetInt(fac1)==(-NodeCount))
				{
					NodeListIter nit2 = Net.GetNodeListIter();
					for(Node fac2=nit2.GetFirstNode();fac2!=null;fac2=nit2.GetNextNode())
					{
						if(Facility.GetInt(fac2)==(-NodeCount) && fac1.GetIndex()!=fac2.GetIndex())
						{
							elim=true;
		
							NodeListIter nit3 = Net.GetNodeListIter();
							for(Node dem1=nit3.GetFirstNode();dem1!=null;dem1=nit3.GetNextNode())
							{
								if(Demand.GetInt(dem1)==0 && Coverage.GetInt(dem1,fac2)>Coverage.GetInt(dem1,fac1))
								{
									elim=false;
								}
							}
							
		
							if(elim)
							{
								Facility.Put(fac2,0);
								
								UI.SingleStep(fac2,Eliminated);
								UI.Trace(Eliminated);
								EdgeListIter eit3 = fac2.GetAdjListIter();
								for(Edge e=eit3.GetFirstEdge(); e!=null; e=eit3.GetNextEdge())
								{
									UI.SingleStep(e,Eliminated);
									UI.Trace(Eliminated);
								}
								System.out.println();
								System.out.println("Node "+fac1.GetIndex() + " dominates node " + fac2.GetIndex());	
								UI.Update();
								change=true;
								
							}
						}
					}
				}
			}
			
		
			UI.StatusLine("Eliminating Constraints");
			UI.ShowPseudoCodeText("Covered");
			NodeListIter nit21 = Net.GetNodeListIter();
			for(Node fac1=nit21.GetFirstNode();fac1!=null;fac1=nit21.GetNextNode())
			{
				if(Demand.GetInt(fac1)==0)
				{
					NodeListIter nit22 = Net.GetNodeListIter();
					for(Node fac2=nit22.GetFirstNode();fac2!=null;fac2=nit22.GetNextNode())
					{
						if(Demand.GetInt(fac2)==0 && fac1.GetIndex()!=fac2.GetIndex())
						{
							elim=true;
		
							NodeListIter nit23 = Net.GetNodeListIter();
							for(Node dem1=nit23.GetFirstNode();dem1!=null;dem1=nit23.GetNextNode())
							{
								if(Facility.GetInt(dem1)==(-NodeCount) && Coverage.GetInt(fac2,dem1)<Coverage.GetInt(fac1,dem1))
								{
									elim=false;
								}
							}
							
		
							if(elim)
							{
								Demand.Put(fac2,1);
								CoveredCount=CoveredCount+1;
								change=true;
								System.out.println();
								System.out.println("Satisfying node "+ fac1.GetIndex() + " will satisfy node " + fac2.GetIndex());
								UI.PutLabel(fac2,fac2.GetIndex() +" C");			
								UI.SingleStep();
								UI.Trace(InitialSet);
								UI.Update();
							}
						}
					}
				}

			}
		
		
			UI.StatusLine("Checking for necessary cover nodes.");
			UI.ShowPseudoCodeText("Must");
			NodeListIter nit4 = Net.GetNodeListIter();
			for(Node dem1=nit4.GetFirstNode();dem1!=null;dem1=nit4.GetNextNode())
			{
				if(Demand.GetInt(dem1)==0)
				{
					sum=0;
					
					NodeListIter nit5 = Net.GetNodeListIter();
					for(Node fac1=nit5.GetFirstNode();fac1!=null;fac1=nit5.GetNextNode())
					{
						if(Facility.GetInt(fac1)==(-NodeCount))
						{
							sum=sum+Coverage.GetInt(dem1,fac1);
						}
					}

					if(sum==1)
					{
					
						NodeListIter nit6 = Net.GetNodeListIter();
						check=nit6.GetFirstNode();		
						index=0;

						while(index==0)	
						{
							if(Coverage.GetInt(dem1,check)==1 && Facility.GetInt(check)==(-NodeCount))
							{
								Facility.Put(check,1);
								CoverCount=CoverCount+1;
								change=true;
								System.out.println();
								System.out.println("Must select node "+check.GetIndex()+" for cover node.");

								UI.SingleStep(check,Selected);
								UI.Trace(Selected);
								UI.Update();
																

								NodeListIter nit7 = Net.GetNodeListIter();
								for(Node dem2=nit7.GetFirstNode();dem2!=null;dem2=nit7.GetNextNode())
								{	
									if(Coverage.GetInt(dem2,check)==1 && Demand.GetInt(dem2)==0)
									{
										Demand.Put(dem2,1);
										CoveredCount=CoveredCount+1;
										System.out.println();
										System.out.println("Cover node "+check.GetIndex()+" covers "+dem2.GetIndex());
										UI.ShowPseudoCodeText("MustC");
										UI.PutLabel(dem2,dem2.GetIndex() +" C");			
										UI.SingleStep();
										UI.Trace(InitialSet);
										UI.Update();
									}
								}
								
								index=1;
							}
							check=nit6.GetNextNode();
						}
					}
				}
			}

			
		}
		
		
		NodeListIter nit11 = Net.GetNodeListIter();
		for(Node f=nit11.GetFirstNode();f!=null;f=nit11.GetNextNode())
		{
			if(Facility.GetInt(f)==1)
			{
				EdgeListIter eit1 = f.GetAdjListIter();
				for(Edge e=eit1.GetFirstEdge(); e!=null; e=eit1.GetNextEdge())
				{
					UI.SingleStep(e,Selected);
					UI.Trace(Selected);
				}
			}
		}
		UI.StatusLine("Current Cover Nodes and Coverage.");
		UI.Update();
		
		int max=0;
		Node ind=null;	

		UI.StatusLine("Running greedy algorithm");

		System.out.println();
		System.out.println("******Begin Greedy Algorithm******");
		UI.ShowPseudoCodeText("Greedy");
		UI.SingleStep();
		UI.Trace(InitialSet);
		UI.Update();
		
		while(CoveredCount!=NodeCount)
		{
			max=0;
				
			NodeListIter nit8 = Net.GetNodeListIter();
			for(Node fac1=nit8.GetFirstNode();fac1!=null;fac1=nit8.GetNextNode())
			{								
				sum=0;
			
				if(Facility.GetInt(fac1)==(-NodeCount))
				{
					NodeListIter nit9 = Net.GetNodeListIter();
					for(Node dem1=nit9.GetFirstNode();dem1!=null;dem1=nit9.GetNextNode())
					{
						if(Demand.GetInt(dem1)==0)
						{
							sum=sum+Coverage.GetInt(dem1,fac1);
						}
					}
					
					if(sum>max)
					{
						max=sum;
						ind=fac1;
					}
				}
			}
			
			Facility.Put(ind,1);
			CoverCount=CoverCount+1;

			UI.SingleStep(ind,Selected);
			UI.Trace(Selected);
			EdgeListIter eit2 = ind.GetAdjListIter();
			for(Edge e=eit2.GetFirstEdge(); e!=null; e=eit2.GetNextEdge())
			{
				UI.SingleStep(e,Selected);
				UI.Trace(Selected);
			}
			
			System.out.println();
			System.out.println("Select node " +ind.GetIndex()+ " for cover node.");
			UI.ShowPseudoCodeText("GreedyS");	
			UI.Update();

			NodeListIter nit10 = Net.GetNodeListIter();
			for(Node dem2=nit10.GetFirstNode();dem2!=null;dem2=nit10.GetNextNode())
			{	
				if(Coverage.GetInt(dem2,ind)==1 && Demand.GetInt(dem2)==0)
				{
					Demand.Put(dem2,1);
					CoveredCount=CoveredCount+1;
					System.out.println();
					System.out.println("Cover node "+ind.GetIndex()+" covers "+dem2.GetIndex());
					UI.ShowPseudoCodeText("GreedyC");
					UI.PutLabel(dem2,dem2.GetIndex() +" C");			
					UI.SingleStep();
					UI.Trace(InitialSet);
					UI.Update();									
				}
						
			}
		}
		System.out.println();
		System.out.println("All nodes are covered.");
	}

	public void DrawCover()
	{
		UI.MoveAll(InitialSet);

		EdgeListIter eit=Net.GetEdgeListIter();
		for(Edge e=eit.GetFirstEdge(); e!=null; e=eit.GetNextEdge())
		{
			UI.PutEdgeVisibilty(e,false);
		}
		
		NodeListIter nit1=Net.GetNodeListIter();
		for(Node fac1=nit1.GetFirstNode();fac1!=null;fac1=nit1.GetNextNode())
		{
			if(Facility.GetInt(fac1)==1)
			{
				UI.SingleStep(fac1,Cover);
				
				EdgeListIter eit1=fac1.GetAdjListIter();
				for(Edge e=eit1.GetFirstEdge(); e!=null; e=eit1.GetNextEdge())
				{
					if(Lengths.GetInt(e)<=CDist)
					{
						UI.PutEdgeVisibilty(e,true);
						UI.SingleStep(e,Cover);
					}
				}
			}
		}
		UI.Trace(Cover);
		UI.Update(); 

	}


}
				
		
