// pete@northwestern.edu

package giden.contrib.NU_IEMS_452_W04;
import giden.contrib.TSP.*;
import giden.gidenSOLVERS.*;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;

/**
 * TspSolverChristo
 * Solves the TSP using the Christofides heuristic
 * (c) Kristin Sahyouni, Peter Francis - March 2004
 * v 0.6 2004 Mar 06
 */

public class TspSolverChristo extends TspSolverBase
{ 
	Node StartNode ;
	int mwm_choice ;
	float mwmCompBestCost = 1000000000 ;
    NodeArray Busy;             // state of nodes (in or out of tour)
    AnimationSet AcquiredSet;   // nodes in the tour
    AnimationSet OddSet ;
    AnimationSet AugmentedSet ;
    AnimationSet EulerSet ;
    AnimationSet TourSet ;
    LinkList kruskalEdges ;
    LinkList oddNodes ;
    LinkList mwmEdges ;
    LinkList augmentedEdges ;
    LinkList eulerTour ;
    
    /** Initializes invariant objects */
    public TspSolverChristo (UserBase userinfo)
    {
	    // call the base-class TspSolverBase constructor. This creates:
	    // 	(1) a UserInfo object called UI
	    //	(2)	a gray AnimationSet object called InitialSet
	    // 	(3) a green FinalSet object called Tour
        super(userinfo);        
        // make an animation set to hold acquired nodes
        System.out.println("\n\n\n*******Constructor***********\n\n\n") ;
        AcquiredSet = UI.CreateAnimationSet("Acquired",Color.pink);
        OddSet = UI.CreateAnimationSet("OddDegree",Color.yellow) ;
		AugmentedSet = UI.CreateAnimationSet("AugmentedSet",Color.cyan) ;
		EulerSet = UI.CreateAnimationSet("Euler Tour", Color.black) ;
		TourSet = UI.CreateAnimationSet("TSP Tour", Color.red) ;
        UI.CreateAnimationButtons();
    }
    
    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
	    String choice = new String() ;
	    // call the base-class TspSolverBase setup function. This does:
	    // * makes an object of the network called Net
	    // * checks for validity of Net (>2 nodes, no edges)
	    // * makes an array of all edges in EdgeArray distance_array
	    // * moves all nodes into the InitialSet object
	    mwmCompBestCost = 1000000000 ;
	    mwm_choice = 0 ;
	    System.out.println("\n\n\n*******Restarting***********\n\n\n") ;
        if(super.Setup(net)==false) // was Setup() successful?
            return false;
        UI.StatusLine("TspSolverChristo Setup");
		UI.ShowPseudoCodeText("start");
        // StartNode = UI.UserMouseNodeInput("Click on Start Node");
        /*
         if(StartNode!=null)      // if the user clicked on a start node
        {
            Busy = new NodeArray(NodeCount,""); // create busy array
            Busy.Fill(0);                       // fill with 0s
            UI.PutLabel(StartNode,"s");     // Draw StartNode with "s"
            UI.SingleStep(StartNode,AcquiredSet);
            UI.Trace(AcquiredSet);          // trace the set        
        }
        else
        {
            UI.StatusLine("User Canceled"); // no StartNode
        }
        */
        choice = "" ; mwm_choice = 0 ;
        while ( (mwm_choice!=1)&&(mwm_choice!=2)&&(mwm_choice!=3) )
        {
        	choice = UI.UserTextInput("Not a valid choice *"+mwm_choice+"*. Choose a heuristic to use for matching: [1] Greedy or [2]Enumeration?", "1") ;
        	try {
        	mwm_choice = Integer.valueOf(choice).intValue() ;
    		}
    		catch (Exception e) { mwm_choice = 0 ; }
    	}
		// UI.StatusLine("Will use heuristic "+mwm_choice+" for MWM matching segment.") ;
    	if ( mwm_choice == 1 )
    		UI.StatusLine("Will use greedy heuristic for MWM matching section.") ;
    	else if ( mwm_choice == 2 )
    		UI.StatusLine("Will use enumeration method for MWM matching section.") ;
    	else
    		UI.StatusLine("Will use **secret** method. Bwaa ha ha ha ha!") ;    
		return true;
    }
       
    /** Executes the solver */
    public boolean Shutdown()
    {
        super.Shutdown();
        return true;
    }

    /** Solve TSP */
    public boolean ExecuteTspSolver()
    {
	    
	    // System.out.println("Peter\n") ;
// 	    System.out.println("In christo.executeTSPSolver()\n") ;
		System.out.println("\n\n\n*******Starting TSP Solver***********\n\n\n") ;
	    Christofides() ;
        // return Christofides();        
        return false ;
    }
    
	/** the main Christofides procedure */
   	boolean Christofides()
    {
// 	    private GidenSolverInput input = new GidenSolverInput() ;
// 	    int choice ;
	    boolean result=true;    // set solver result to default
	    // input.AddItem(1
// 	    System.out.println("In christofides()\n") ;
	    // call kruskal
            UI.ShowPseudoCodeText("Kruskal");
	    MakeMST() ;
	   
	    UI.StatusLine("Done forming MST. Identifying odd degree nodes...");  
	    // identify odd degree nodes
	    UI.ShowPseudoCodeText("OddNodes");
	    MarkOddDegree() ;
	    UI.Trace(OddSet) ; UI.Update() ;

	    UI.ShowPseudoCodeText("Matching");
	    UI.Trace(OddSet) ; UI.Update() ;
	    UI.Trace(OddSet) ; UI.Update() ;
	    UI.StatusLine("Odd degree nodes identified. Making matching set...");  
	    // connect odd-degree nodes with a matching
	    
		if ( mwm_choice == 1 )
	    	MakeGreedyMatching() ;
	    else if ( mwm_choice == 2 )
	    	MakeCompleteMatching() ;
	    else
	    	MakeSimpleMatching() ;
	    
	    UI.StatusLine("Matching Set made. Augmenting MST set with this matching...");  
	    UI.ShowPseudoCodeText("Augment");
	    MakeAugmentedSet() ;
	    // make euler tour
	    UI.StatusLine("Making Euler tour on augmented set...");  
	    MakeEuler() ;
	 	// shortcut the euler tour   
	 	UI.StatusLine("Creating TSP tour from Euler tour...");  
 	 	MakeTSP() ;
		UI.ShowPseudoCodeText("end");
 	 	System.out.println("\n\n***********MWM... COST = "+mwmCompBestCost+"*************") ;
	 	return result ;
 	}
 	
 	/** Make a TSP tour from the given Euler tour */
 	boolean MakeTSP()
 	{
	 	UI.ShowPseudoCodeText("TSP");	
		int tourPosition = 0 ;
	 	float tourLen = 0 ;
	 	Node currNode, startNode, tailNode, headNode, connPoint ;
	 	Edge currEdge, nextEdge, insEdge ;
	 	NodeListIter nit = Net.GetNodeListIter() ;
	 	NodeArray visitedNodes = new NodeArray(nit.GetSize(),"") ;
 	 	for (currNode=nit.GetFirstNode(); currNode!=null; currNode=nit.GetNextNode())
	 	{
		 	visitedNodes.Put(currNode, 0) ;
	 	}
		EdgeListIter eulerWalk = new EdgeListIter(eulerTour) ;
		for (currEdge=eulerWalk.GetFirstEdge(); currEdge!=null; currEdge=eulerWalk.GetNextEdge()) 
		{
// 			System.out.println("Edge: "+currEdge.GetIndex()) ;
			UI.PutLabel(currEdge, "") ;
		}
	 	currEdge = eulerWalk.GetFirstEdge() ;
	 	// figure out where the true start node is 
	 	startNode = currEdge.GetSource() ;
	 	currNode = currEdge.GetTarget() ;
	 	tailNode = startNode; headNode = currNode ;
	 	nextEdge = eulerWalk.GetNextEdge() ;
// 	 	System.out.println("Current start = "+startNode.GetIndex()) ;
// 	 	System.out.println("Alternative start = "+currNode.GetIndex()) ;
// 	 	System.out.println("Next Source = "+((Node)nextEdge.GetSource()).GetIndex()) ;
// 	 	System.out.println("Next Target = "+((Node)nextEdge.GetTarget()).GetIndex()) ;
	 	if ( (nextEdge.GetSource()==startNode)||(nextEdge.GetTarget()==startNode) )
	 	{
		 	startNode = currEdge.GetTarget() ; 
		 	currNode = currEdge.GetSource() ;
		 	tailNode = startNode ; 
		 	headNode = currNode ; 
// 		 	System.out.println("Swapped! Start node is now "+startNode.GetIndex()) ;
	 	}
	 	visitedNodes.Put(startNode,1) ;
	 	UI.SingleStep(startNode, TourSet) ; 
	 	UI.PutLabel(startNode, tourPosition) ;
	 	tourPosition++ ;
	 	UI.SingleStep(currEdge, TourSet) ;
	 	tourLen = tourLen + super.NodesToDistance.GetInt(startNode, currNode) ;
	 	UI.SingleStep(currNode, TourSet) ;
	 	visitedNodes.Put(currNode,1) ;
	 	UI.PutLabel(currNode, tourPosition) ;
	 	tourPosition++ ;
	 	UI.Trace(TourSet) ; UI.Update() ;
	 	tailNode = currNode ; connPoint = tailNode ;
	 	
// 	 	System.out.println("Node : "+tailNode.GetIndex()) ;
 	 	headNode = currEdge.GetTarget() ;
 	 	int firstPass = 1 ;
 	 	do
 	 	{
	 	 	do 
	 	 	{
		 	 	if ( firstPass == 1 )
		 	 	{
		 	 		nextEdge = eulerWalk.GetFirstEdge() ;
		 	 		firstPass = 0 ; 
		 	 	//	nextEdge = eulerWalk.GetNextEdge() ;
	 	 		}
		 	//  else
		 	 	 	nextEdge = eulerWalk.GetNextEdge() ;
		 	 	 	if (nextEdge ==null) { firstPass = 2 ; break ; }
// 		 	 	System.out.println("Considering edge "+((Node)nextEdge.GetSource()).GetIndex()+"->"+((Node)nextEdge.GetTarget()).GetIndex()) ;
// 		 	 	System.out.println("connPoint = "+connPoint.GetIndex()) ;
		 	 	if ( nextEdge.GetSource() == connPoint ) 
		 	 	{
			 	 	headNode = nextEdge.GetTarget() ;
		 	 	}
		 	 	else if ( nextEdge.GetTarget() == connPoint )
		 	 	{
			 	 	headNode = nextEdge.GetSource() ;
		 	 	}
		 	 	else 
		 	 	{
			 	 	System.out.println("Inconsistency in eulerTour set. No incident edges here!") ;
			 	 	return false ;
		 	 	}
// 		 	 	System.out.println("Considering node "+headNode.GetIndex() ) ;
		 	 	if ( visitedNodes.GetInt(headNode) == 0 )
		 	 	{
			 	 	break ;
		 	 	}
		 	 	else
		 	 	{
			 	 	connPoint = headNode ;	
		 	 	}
	 	 	} while ( true ) ;
	 	 	if ( firstPass == 2 ) 
	 	 	{
		 	 	// tailNode = connPoint; 
		 	 	headNode = startNode ; 
		 	 	// break ;
	 	 	}
	 		insEdge = NodesToEdge.GetEdge(tailNode, headNode) ;
// 	 		UI.PutLabel(insEdge,insEdge.GetIndex()) ;
// 	 		System.out.println("Edge being added:"+insEdge.GetIndex()) ;
	 		UI.PutEdgeVisibility(insEdge,true);
	 	 	UI.SingleStep(insEdge, TourSet); 
	 	 	tourLen = tourLen + super.NodesToDistance.GetInt(tailNode, headNode) ;
	 	 	tailNode = headNode ; connPoint = tailNode ;
// 	 	 	System.out.println("Added Node : "+tailNode.GetIndex()) ;
// 	 	 	System.out.println("Node flag for node "+tailNode.GetIndex()+" is now = "+visitedNodes.GetInt(tailNode)) ;
	 	 	UI.SingleStep(tailNode, TourSet) ;
	 	 	UI.PutLabel(tailNode, tourPosition) ;
	 	 	// UI.PutLabel(tailNode, tailNode.GetIndex()) ;
	 	 	tourPosition++ ;
	 	 	UI.Trace(TourSet); UI.Update() ;
	 	 	visitedNodes.Put(tailNode,1) ;
 	 	} while ( headNode != startNode ) ;
 	 	UI.StatusLine("TSP Tour found! Tour length = "+tourLen) ;
		return true ;
 	}
 	
 	/** Make an Euler tour from the given augmented set  */
	boolean MakeEuler()
 	{
		// key is to make a list of nodes and update that accordingly
		LinkList eulerNodes = new LinkList() ;
		LinkList freshEdges = (LinkList) augmentedEdges.clone() ;
		Node currNode, startNode, tailNode, headNode ;
		Edge currEdge ;
		ListEntry currItem, currEntry, nextEntry, candPoint, insPoint, delEdgeItem ;
		// pick a random nodes to start from
		currEntry = freshEdges.GetFirstEntry() ;
		currEdge = (Edge) currEntry.GetItem() ;
		startNode = currEdge.GetSource() ;
		// put that into the list
		insPoint = eulerNodes.AddItem(startNode) ;		
		
		// find lowest numbered vertex that is in the 
		// tour and has freshEdges incident on it
		while (freshEdges.GetSize()!=0)
		{
			insPoint = null ;
			for (candPoint=eulerNodes.GetFirstEntry(); candPoint!=null; candPoint=eulerNodes.GetNextEntry(candPoint))
			{
				currNode = (Node) candPoint.GetItem() ;
				// we have a node item to search edges for
				for (currItem=freshEdges.GetFirstEntry(); currItem!=null; currItem=freshEdges.GetNextEntry(currItem))
				{
					if ( (((Edge)currItem.GetItem()).GetSource()==currNode)||(((Edge)currItem.GetItem()).GetTarget()==currNode) )
					{
						// we have a match
						insPoint = candPoint ;
						break ;					// break out of the edge search
					}
				}
				if ( insPoint != null ) break ;	// break out of the node search
			}
			if ( insPoint == null ) { System.out.println("Error: all edges not used and yet algorithm could not find an isthmus point.") ; return false ; }
		
			// at this point we have the lowest numbered vertex 
			// eulerNodes.InsertBefore( insPoint, new ListEntry( insPoint.GetItem() ) ) ;
			tailNode = (Node) insPoint.GetItem() ;
			while ( true ) // keep iterating until we hit a stopping criterion
			{
// 				System.out.println("*******************************") ;
				headNode = null; delEdgeItem = null ;
				for ( currItem=freshEdges.GetFirstEntry(); currItem!=null;currItem=freshEdges.GetNextEntry(currItem))
				{
					currEdge = (Edge) currItem.GetItem() ;
					if (currEdge.GetSource()==tailNode)
					{
						headNode = currEdge.GetTarget() ;
						delEdgeItem = currItem ;
						break ;
					}
					else if (((Edge)currItem.GetItem()).GetTarget()==tailNode)
					{
						headNode = (Node) currEdge.GetSource() ;
						delEdgeItem = currItem ;
						break ;
					}
				}
				if ((headNode == null )||(delEdgeItem==null) )
				{	
// 					System.out.println("Could not find an incident edge while edge-following an Euler subtour!") ; 
					break ;
				}
				ListEntry entryToIns = new ListEntry(headNode) ;
// 				System.out.println("Insert node idx = "+((Node)insPoint.GetItem()).GetIndex()) ;
// 				System.out.println("New node idx= "+((Node)entryToIns.GetItem()).GetIndex()) ;
// 				System.out.println("Size of eulerNodes list = "+eulerNodes.GetSize() ) ;
				eulerNodes.InsertBefore(insPoint, entryToIns) ;
				insPoint = entryToIns ;
				tailNode = headNode ;
				freshEdges.DeleteEntry(delEdgeItem) ;
			}
			// if we have broken out then we have no more nodes in that subbranch
			// go back to searching the entire set of nodes for more isthmus nodes
		}
		// when we are here, there are no more freshEdges
		// create a list of edges and move them into the EulerSet animationset
// 		System.out.println("\n\n*******************Wrapping up**************") ;
		eulerTour = new LinkList() ;
		currEntry = eulerNodes.GetFirstEntry() ; 
		UI.SingleStep(((Node)currEntry.GetItem()), EulerSet) ;
		UI.ShowPseudoCodeText("Euler");
		nextEntry = currEntry.GetNext() ;
		while ( nextEntry != null )
		{
			currEdge = NodesToEdge.GetEdge(((Node)currEntry.GetItem()), ((Node)nextEntry.GetItem())) ;
			// if ( nextEntry == null) break ;
// 			UI.PutLabel(currEdge, currEdge.GetIndex()) ;
// 			System.out.println("Euler Edge: "+currEdge.GetIndex()) ;
			eulerTour.AddItem(currEdge) ;
			// System.out.println("Added that edge!") ;
			UI.SingleStep(currEdge, EulerSet) ; UI.SingleStep(((Node)nextEntry.GetItem()), EulerSet) ;
			UI.Trace(EulerSet) ; UI.Update() ;
			currEntry = nextEntry ;
			nextEntry = nextEntry.GetNext() ;
		}
		UI.Trace(EulerSet) ; UI.Update() ;
		UI.Trace(EulerSet) ; UI.Update() ;
		return true ;
	}

	/** Make an augmented animation set */
	boolean MakeAugmentedSet()
	{
		Node currNode ;
		Edge currEdge ;
		augmentedEdges = (LinkList) kruskalEdges.clone() ;
		augmentedEdges.JoinList(mwmEdges) ;
		NodeListIter nit = Net.GetNodeListIter() ;
		EdgeListIter eit = new EdgeListIter(augmentedEdges) ;
		for(currNode=nit.GetFirstNode();currNode!=null;currNode=nit.GetNextNode())
		{
			UI.SingleStep(currNode, AugmentedSet) ;
		}
		for (currEdge=eit.GetFirstEdge(); currEdge!=null; currEdge=eit.GetNextEdge()) 
		{
			UI.SingleStep(currEdge, AugmentedSet) ;

		}
		UI.Trace(AugmentedSet) ;
		UI.Update() ;
		return true ;
	}
	
	/** Make complete matching */
	boolean MakeCompleteMatching()
	{
		mwmEdges = new LinkList() ;
		LinkList oddNodesList = new LinkList() ;
		LinkList candList = new LinkList() ;
		float cumCost = 0 ;
		Edge currEdge ;
		oddNodesList = (LinkList) oddNodes.clone() ;
// 		System.out.println("\n\n********* Starting to make complete enumeration for matching! *********\n\n") ;
// 		CompEnumMaker((Node)oddNodesList.GetFirstEntry().GetItem(), null, oddNodesList, null, candList, cumCost, bestCost) ;
		if ( CompEnumMaker(null, oddNodesList, null, candList, cumCost) )
		{
// 			System.out.println("\n\nComplete enumeration results in "+mwmEdges.GetSize()+" edges\n\n") ;
			EdgeListIter eit = new EdgeListIter(mwmEdges) ;
			for (currEdge=eit.GetFirstEdge(); currEdge!=null; currEdge=eit.GetNextEdge()) 
			{
				UI.PutEdgeVisibility(currEdge, true) ;
				UI.SingleStep(currEdge, OddSet) ;
				UI.ShowPseudoCodeText("CE Matching");
				UI.Trace(OddSet) ;
				UI.Update() ;
			}			
		}
		else
		{
			System.out.println("******************** Damn Bro! Something went wrong!********************") ;
		}
		return true ;
	}
	
	boolean CompEnumMaker(Node removeNode, LinkList noList, Edge newEdge, LinkList candidateList, float cumulCost) 
	{
		// if removeNode!=null, remove it
		// make an edge with another node by iterating; 
			// for every possible node k
				// find corresp edge
				// update cumulCost and candList
				// if cumulCost > bestCost, go to the next node combination k 
				// mark k as the node to remove and call CompEnumMaker
		// if nodes list is empty, this is a viable combination
			// double check by making sure that cand list is exactly nodes/2 long
			// check its cost and replace bestEdges if necessary
		LinkList nList ;
		LinkList candList ;
		ListEntry currEntry, nextEntry, trialEntry ;
		Node otherNode, startNode ;
		Edge candEdge ;
		float candCost ;
// 		System.out.print(".") ;
		nList = (LinkList) noList.clone() ;
		candList = (LinkList) candidateList.clone() ;
		if ( newEdge!=null )
		{
			candList.AddItem(newEdge) ;
			UI.PutLabel(newEdge, ".") ;
// 			System.out.print("+") ;
		}
		if ( removeNode!=null )
		{
			currEntry = nList.GetFirstEntry() ;
			while (currEntry!=null)
			{
// 				System.out.print("-") ;
				nextEntry = currEntry.GetNext() ;
				if ( (Node)currEntry.GetItem() == removeNode )
					nList.DeleteEntry(currEntry) ;
				currEntry = nextEntry ;
			}
		}
// 		System.out.println("Size = "+nList.GetSize()) ;
		if ( nList.GetSize()>2 )
		{
			currEntry = nList.GetFirstEntry() ;
			startNode = (Node) currEntry.GetItem() ;
			nList.DeleteEntry(currEntry) ;
			NodeListIter nit = new NodeListIter(nList) ;
			for (otherNode=nit.GetFirstNode(); otherNode!=null; otherNode=nit.GetNextNode()) 
			{
// 				System.out.print("*") ;
				candEdge = NodesToEdge.GetEdge(startNode, otherNode) ;
				candCost = super.NodesToDistance.GetInt(startNode, otherNode) + cumulCost ;
				if ( candCost < mwmCompBestCost )
				{
					if ( !CompEnumMaker(otherNode, nList, candEdge, candList, candCost) ) 
						return false ;
				}
			}
		}
		else
		{
			// there are only two nodes left. 
			// match the start node with the other node
			// and perform end of iteration work
			if ( nList.GetSize() == 2 )
			{
				startNode = (Node) nList.GetFirstEntry().GetItem() ;
				otherNode = (Node) nList.GetLastEntry().GetItem() ;
				candEdge = NodesToEdge.GetEdge(startNode, otherNode) ;
				candCost = super.NodesToDistance.GetInt(startNode, otherNode) + cumulCost ;
// 				System.out.println("One MWM combo cost: "+candCost) ;
				if ( candCost < mwmCompBestCost )
				{
					candList.AddItem(candEdge) ;
					UI.PutLabel(candEdge, ".") ;
					System.out.print("+") ;
					if ( (candList.GetSize()*2) != oddNodes.GetSize() )
					{
						System.out.println("Something is seriously wrong here!!!!!!! Not enough nodes to candidates!") ;
						System.out.println("I have "+candList.GetSize()+" edges and "+oddNodes.GetSize()+" nodes!") ;
						UI.Trace(OddSet) ; UI.Update() ; return false ;
					}
					mwmEdges = (LinkList) candList.clone() ;
					mwmCompBestCost = candCost ;
// 					System.out.println("Best MWM cost is now :"+mwmCompBestCost) ;
				}
			}
			else
			{
				System.out.println("*********************** Something is seriously wrong here dawg***************") ;
				UI.ShowPseudoCodeText("Matching");
				UI.Trace(OddSet) ; 
				UI.Update() ;
				return false ;
			}
		}
// 		System.out.print("-#-") ;
		return true ;
	}

	/** Makes a greedy matching */
	boolean MakeGreedyMatching()
	{
		// make edge list
		// find lowest edge 
		// store its end points
		// delete current edge
		// walk through edge list
		// delete any edge with end points same as that edge
		// return to second step
		ListEntry bestEntry, currEntry, walkEntry ;
		Node aNode, bNode, currNode ;
		Edge aEdge, currEdge ;
		float bestLen = 10000000 ;
		float currLen ;
		LinkList candEdges = new LinkList() ;
		NodeListIter nit = new NodeListIter(oddNodes) ;
		mwmEdges = new LinkList() ;
		
		mwmCompBestCost = 0 ;
		
		walkEntry = oddNodes.GetFirstEntry() ;
		bestEntry = null ;
		while ( walkEntry != null ) 
		{
			aNode = (Node) walkEntry.GetItem() ;
			for ( currNode=nit.GetFirstNode() ; currNode!=null; currNode=nit.GetNextNode()) 
			{
				if ( currNode != aNode )
				{
					aEdge = NodesToEdge.GetEdge(aNode, currNode) ;
					currLen = super.NodesToDistance.GetInt(aNode, currNode) ;
					currEntry = candEdges.AddItem(aEdge) ;
// 					System.out.println("Edge "+aEdge.GetIndex()+":"+aNode.GetIndex()+"->"+currNode.GetIndex()+" with len "+currLen+" added to possibles set.") ;
					if (currLen < bestLen)	
					{ 
						bestEntry = currEntry; bestLen = currLen ; 
// 						System.out.println("Edge "+aEdge.GetIndex()+" is best so far. Best cost = "+currLen) ;
					}
				}
			}
			walkEntry = walkEntry.GetNext() ;
		}
		if ( bestEntry == null ) 
		{
			System.out.println("Something is wrong in finding the best entry for the greedy method!") ;
			return false ;
		}
		mwmCompBestCost = bestLen ;
			
		while ( candEdges.GetSize() != 0  )
		{
			aEdge = (Edge) bestEntry.GetItem() ;
			UI.PutEdgeVisibility(aEdge, true) ;
			mwmEdges.AddItem(aEdge) ;
			UI.SingleStep(aEdge, OddSet) ;
  			UI.ShowPseudoCodeText("Greedy Matching");
			UI.Trace(OddSet) ;
			UI.Update() ;
			aNode = (Node) aEdge.GetSource() ;
			bNode = (Node) aEdge.GetTarget() ;
// 			System.out.println("Searching for non "+aNode.GetIndex()+","+bNode.GetIndex()+" nodes among "+candEdges.GetSize()+" edges.") ;
			bestLen = 10000000 ;
			bestEntry = null ;
			currEntry=candEdges.GetFirstEntry() ;
			while ( currEntry != null ) 
			{
				currEdge = (Edge) currEntry.GetItem() ;
				walkEntry = candEdges.GetNextEntry(currEntry) ;
// 				System.out.println("Looking at edge "+currEdge.GetIndex()) ;
				if ( (currEdge.GetSource()==aNode)||(currEdge.GetTarget()==aNode)||(currEdge.GetSource()==bNode)||(currEdge.GetTarget()==bNode) )
				{
					candEdges.DeleteEntry(currEntry) ;
// 					System.out.println("Deleted edge"+currEdge.GetIndex()+":"+((Node)currEdge.GetSource()).GetIndex()+"->"+((Node)currEdge.GetTarget()).GetIndex()) ;
				}
				else
				{
					currLen = super.NodesToDistance.GetInt(currEdge.GetSource(), currEdge.GetTarget()) ;
					if ( currLen < bestLen) 
					{ 
						bestEntry = currEntry ; bestLen = currLen ; 
// 						System.out.println("Edge"+currEdge.GetIndex()+": has the best cost of "+currLen) ;
					}
				}
				currEntry = walkEntry ;
			}
			if ( bestEntry != null ) mwmCompBestCost = mwmCompBestCost + bestLen ;
			if ( bestEntry == null ) 
			{
// 				System.out.println("Something is wrong in finding the best entry for the greedy method!") ;
// 				return false ;
			}
			
		}
		return true ;
	}


	/** Makes a simple matching one with the other */
	boolean MakeSimpleMatching()
	{
		Node currNode ;
		Node matchingNode ;
		Edge edgeToAdd ;
		Edge currEdge ;
		NodeListIter nit = new NodeListIter(oddNodes) ;
		mwmEdges = new LinkList() ;
		mwmEdges.DeleteContents() ;
		for (currNode=nit.GetFirstNode(); currNode!=null;currNode=nit.GetNextNode()) 
		{
			matchingNode = nit.GetNextNode() ;
			edgeToAdd = NodesToEdge.GetEdge(currNode, matchingNode) ;
			UI.PutEdgeVisibility(edgeToAdd,true);
			mwmEdges.AddItem(edgeToAdd) ;
			UiEdgeInfo info = UI.GetEdgeInfo(edgeToAdd) ;
// 			if ( info!=null )
// 			{
// 				info.color = Color.blue ;
// 				UI.PutEdgeInfo(edgeToAdd, info) ;
// 			}
			EdgeListIter eit = new EdgeListIter(mwmEdges) ;
			for (currEdge=eit.GetFirstEdge(); currEdge!=null; currEdge=eit.GetNextEdge()) 
			{
				UI.SingleStep(currEdge, OddSet) ;
				UI.ShowPseudoCodeText("Matching");
				UI.Trace(OddSet) ;
				UI.Update() ;
			}			
		}	
		mwmCompBestCost = 0 ;	
		return true ;
	}
	

	/** Identify OddDegreeNodes */
	boolean MarkOddDegree()
	{
		Node node ; 
		Edge currEdge ;
		int edgeCount ;
// 		System.out.println("Mark Odd degree: Point 1") ;
		NodeListIter nit = Net.GetNodeListIter();
//	        LinkList adjedges = new SingleLinkList();
//	  	LinkList oddit = new SingleLinkList();
		oddNodes = new LinkList() ;
		oddNodes.DeleteContents() ;
// 		System.out.println("Mark Odd degree: Point 2") ;
	  	EdgeListIter eit = new EdgeListIter(kruskalEdges);
//         	System.out.println("Inside MarkOddDegree()... ") ;
	        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
        	{
			edgeCount = 0 ;
		        for (currEdge=eit.GetFirstEdge(); currEdge!=null; currEdge=eit.GetNextEdge()) 
		        {
		        	if ( currEdge.IsSource(node)||currEdge.IsTarget(node) )
					edgeCount++ ;
	        	}
// 		        System.out.print(".") ;
			// edgeCount = node.GetEdgeCount() % 2 ;
			edgeCount = edgeCount % 2 ;
			if ( edgeCount == 1 )
			{
				// put into OddSet
//				oddit.AddItem(node) ;
// 				System.out.print("+") ;
				UI.SingleStep(node, OddSet) ;
				oddNodes.AddItem(node) ;
			}
			UI.ShowPseudoCodeText("OddNodes");
			UI.Trace(OddSet) ;
			UI.Update() ;
	        }        
        	return true ;
	}	// MarkOddDegree()
	
	/** Display a Min Spanning Tree using a call to Kruskal's algorithm */
	
	boolean MakeMST()
	{
		boolean result = true ;
	 	UserBase nullbase = new UserBase(UI.Services);      // UserBase for Kruskal
        Kruskal kruskal = new Kruskal(nullbase);            // create Kruskal
        EdgeArray length = Net.GetEdgeArray("Euclidean Length"); // get length array
        UI.StatusLine("Edges = "+Net.GetEdgeCount()) ;
        kruskal.PutLength(length);                          // give array to Kruskal
        // kruskal.PutLength(distance_array);   
        kruskal.Setup(Net);     					// setup Kruskal
//         System.out.println("About to execute Kruskal()\n") ;                          
        kruskal.Execute();                                  // run Kruskal
//         System.out.println("Done executing kruskal()\n") ;                          
        int forest_length = kruskal.GetForestLength();      // get forest length
        // get the forest edge list, and clone it, Kruskal clears the list at shutdown
        kruskalEdges = (LinkList)(kruskal.GetForestEdges().clone());
        kruskal.Shutdown();                                 // shutdown Kruskal
// 		System.out.println("Kruskal() closed\n") ;                          
		
        // show the results
        Edge edge;
        Node node;      
        EdgeListIter eit = new EdgeListIter(kruskalEdges);
//         System.out.println("Got edge list\n") ;
        for(edge = eit.GetFirstEdge(); edge!=null; edge=eit.GetNextEdge())
        {
	        UI.PutEdgeVisibility(edge,true);
            UI.SingleStep(edge,AcquiredSet);     // draw the edge
         
	    node = edge.GetSource();        // get the source node
            UI.SingleStep(node,AcquiredSet);     // draw the source node
            UI.PutLabel(node,"");           // clear source node label
            node = edge.GetTarget();        // get the target node
            UI.SingleStep(node,AcquiredSet);     // draw the target node
            UI.PutLabel(node,"");           // clear the target node lab
//             System.out.print(".") ;
        }
//         System.out.println(" ... done ") ;
        
        
	UI.Trace(AcquiredSet);
	UI.Update() ;
        System.out.println("Forest length = "+forest_length) ;
        
        return true;
    }
}


 