package giden.userSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;

/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * SolverMenu is an interface class that provides implementors
 * with a method for creating menu selections (CreateMenu) and
 * responding to those selections (CreateSolver).  The menus are
 * created in a sequence of two steps: first create the problem
 * type (AddProblem) and  then create one or more solvers
 * (AddSolver) for that type of problem.  These problem types
 * and solver names then become menu items under the GIDEN Solver
 * menu selection. After a user selects a particular solver,
 * GIDEN calls CreateSolver method and passes it the name of the
 * solver (as it appears in the menu).  The CreateSolver method
 * then creates the solver and returns the base class of
 * that solver for execution by GIDEN.
 * <p>
 * The SolverMenu class includes two methods to include
 * the GIDEN solvers on the menu and to create them.
 * These methods are called by CreateMenu and CreateSolver.
 * <p>
 * Copyright (c) 1993-2003 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @see Network
 * @see SolverMenuBase
 *
 * @version 	4.0a 2004 08 29
 * @author 	    D. S. Dilworth
 */


public class SolverMenu extends SolverMenuBase
{
    /*----------------------------------------------------------
    | Create strings for implementor solvers
    ----------------------------------------------------------*/
    
    final String TEST_SOLVER="TestSolver";
    final String FEATURE_EXAMPLE="FeatureExample";
    final String TSP_CONTROLLER="TspController";
    final String CALL_ANIMATED_SOLVER="CallAnimatedSolver";
    

    //final String DIALOG_SOLVER="DialogSolver";
    

    /*----------------------------------------------------------
    | Construct the SolverMenu object without frame
    ----------------------------------------------------------*/
    public SolverMenu()
    {
        CreateMenu();
    }

    //----------------------------------------------------------
    /** Creates the Problem and Solver menu picks.
     * This method is maintained by implementors and is called
     * once during the GIDEN initialization.
     * This method provides implementors with a mechanism
     * for creating the menu picks that appear under the
     * GIDEN Solver menu selection.
     *
     * @see SolverMenuBase#AddProblem
     * @see SolverMenuBase#AddSolver */
    //----------------------------------------------------------
    public void CreateMenu()
    {
        AddProblem("Example");
        AddSolver(TEST_SOLVER);
        AddSolver(FEATURE_EXAMPLE);
        AddSolver(TSP_CONTROLLER);
        AddSolver(CALL_ANIMATED_SOLVER);
        
    //    AddSolver(DIALOG_SOLVER);
    }

    //----------------------------------------------------------
    /** Creates a solver, given its name and other GIDEN information.
     * This method is maintained by implementors and is called by
     * GIDEN after a user selects a Solver from the menu.  The
     * implementor must create a UserIO object and pass it to the
     * Solver's constructor.  The Solver constructor returns an
     * ExecBase object that this method returns.
     * @param name   The name of the Solver selected by the user
     * @param draw   The GidenDraw object for user interaction
     * @param net    The network for this solver
     * @return The ExecBase object for the created Solver, or
     * null if the solver couldn't be created*/
    //----------------------------------------------------------
    public ExecBase CreateSolver(String name,SolverServices solvserv,GUISolver gui)
    {
        UserBase user = new UserIO(solvserv,gui);
        ExecBase exec =null;
                       
        //------------------------------------------------
        // create the solver that was selected by the user
        //------------------------------------------------
        
        if (name.equals(TEST_SOLVER))
            exec = new TestSolver(user);
        if (name.equals(FEATURE_EXAMPLE))
            exec = new FeatureExample(user);
        if (name.equals(TSP_CONTROLLER))
            exec = new TspController(user);               
        if (name.equals(CALL_ANIMATED_SOLVER))
            exec = new CallAnimatedSolver(user);                     
          
    //    if (name.equals(DIALOG_SOLVER))
    //        exec = new DialogSolver(user);
        
        return exec;
    }
}
