package giden.userSOLVERS;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;

/*
 * Copyright (c) 1993-1999 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * WE MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. WE SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY USER AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <p>
 * Test Solver is an example program that shows some simple animation
 * and network features.
 * <p>
 *
 * @see Network
 * @see SolverMenu
 * @see UserIO
 *
 * @version 	4.0a 2004 08 29
 * @author 	    D. S. Dilworth
 */


public class TestSolver extends ExecBase
{
    UserBase UI;                  // user interactions
    Network Net;                  // access to network
    
    AnimationSet StartSet;        // StartSet data member
    AnimationSet EndSet;          // EndSet data member
    
    /** Initializes invariant objects */
    public TestSolver (UserBase userinfo)
    {
        super(userinfo);        // mandatory call to the base class
        UI =  userinfo;         // local access for user interactions
        
        // create the start and end Sets
        StartSet = UI.CreateAnimationSet("",Color.red);
        EndSet   = UI.CreateAnimationSet("End",Color.green);    
        UI.CreateAnimationButtons();
    }
    
    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
        Net=net;                        // local access to the network
        UI.StatusLine("Setup (User)");         // status line
        UI.MoveAll(StartSet);           // nodes/edges -> StartSet
        UI.ShowPseudoCodeText("start"); // pseudocode window
        return true;
    }
    
    /** Executes the solver */
    public boolean Execute()
    {
        UI.StatusLine("Executing");         // status line
        UI.ShowPseudoCodeText("execute");   // pseudocode window
        
        ExecuteTestSolver();                // execute the solver
        
        UI.StatusLine("Completed");         // status line
        UI.ShowPseudoCodeText("end");       // pseudocode window
        return true;
    }
    
    /** Individually moves nodes and edges into the EndSet */
    void ExecuteTestSolver()
    {
        NodeListIter nit = Net.GetNodeListIter();    // node list iterator
        for(Node node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
        {
            UI.SingleStep(node,EndSet); // move node into EndSet
        }                
        EdgeListIter eit = Net.GetEdgeListIter();    // edge list iterator
        for(Edge edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
        {
            UI.SingleStep(edge,EndSet); // move edge into EndSet
        }    
        UI.Trace(EndSet);               // trace the EndSet
        UI.Update();                    // display animation            
    }
}
