package giden.userSOLVERS;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;
import giden.contrib.TSP.*;

/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 * <p>
 * @see SolverController
 *
 * @version 	4.0.1a 2004 09 01
 * @author 	    D. S. Dilworth
 */



public class TspController extends SolverController
{
    UserBase UI;                  // user interactions
    Network Net;                  // access to network
    
    AnimationSet StartSet;        // StartSet data member
    AnimationSet EndSet;          // EndSet data member
    TspSolverBase LastSolver;
    RowDialogController Exhaust;
    RowDialogController Nearest;
    RowDialogController TwoSwap;   
    TspSolverExhaustive ExhaustSolver;
    TspSolverNearestNeighbor NearestSolver;
    TspSolverTwoSwap TwoSwapSolver;
    LinkList ExhaustTour;
    int ExhaustLength;
    LinkList NearestTour;
    int NearestLength;
    LinkList TwoSwapTour;
    int TwoSwapLength;
    
    /** Used to delete nodes or edges created by a solver */
    GraphState State;

    

    
    /** Initializes invariant objects */
    public TspController (UserBase userinfo)
    {
        super(userinfo);        // mandatory call to the base class
        UI =  userinfo;         // local access for user interactions
        
        // create the start and end Sets
        StartSet = UI.CreateAnimationSet("",Color.red);
        EndSet   = UI.CreateAnimationSet("End",Color.green);    
        UI.CreateAnimationButtons();
        LastSolver=null;
        ExhaustSolver = null;
        NearestSolver = null;
        TwoSwapSolver=null;
        ExhaustTour=null;
        ExhaustLength=0;
        NearestTour=null;
        NearestLength=0;
        TwoSwapTour=null;
        TwoSwapLength=0;
        
    }
    
    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
        Net=net;                        // local access to the network
        State = new GraphState(Net);    // get state so created edges can be removed
        UI.StatusLine("Setup");         // status line
        UI.MoveAll(StartSet);           // nodes/edges -> StartSet
        UI.ShowPseudoCodeText("start"); // pseudocode window
        UI.PutDirected(false);          // draw as undirected
        return true;
    }
    
    /** Executes the solver */
    public boolean Execute()
    {
        UI.StatusLine("Executing");         // status line
        UI.ShowPseudoCodeText("execute");   // pseudocode window
        
        ExecuteTestSolver();                // execute the solver
        
        UI.StatusLine("Completed");         // status line
        UI.ShowPseudoCodeText("end");       // pseudocode window
        return true;
    }
    /** Cleanup after solver execution is completed */
    public boolean Shutdown()
    {
        super.Shutdown();
        State.Restore();
        return true;
    }

    
    /** Individually moves nodes and edges into the EndSet */
    void ExecuteTestSolver()
    {
        
//    boolean InteractWithUser()
//    {
        RowDialogMain Dialog = UI.CreateRowDialog("TSP Controller");
        Exhaust = Dialog.AddRowController("Exhaustive","n/a",this);
        Nearest = Dialog.AddRowController("Nearest","n/a",this);      
        TwoSwap = Dialog.AddRowController("Two Swap","n/a",this);       
        
        Dialog.AddButton("Cancel");
        Dialog.AddButton("OK");
        
        boolean loop=true;
        while(loop)
        {
            String pushed_button = Dialog.Interact(); 
            System.out.println(pushed_button);

            if(pushed_button.equals("RunButton"))
            {
                RunButton(Dialog.ControllerButtonResult);
            }
            else if(pushed_button.equals("ResultsButton"))
            {
                ResultsButton(Dialog.ControllerButtonResult);
            }
            
            else
                loop=false;
        }        
        Dialog.dispose();
//        return pushed_button.equals("OK");
//    }    
      
    }
    public void RunButton(RowDialogController source)
    {
        Net.DeleteAllEdges();
        if(source==Exhaust)
        {
            if(ExhaustSolver==null)
            {
                UserBase nulle = new UserBase(UI.Services);      // UserBase for Kruskal
                ExhaustSolver = new TspSolverExhaustive(nulle);            // create Kruskal
                ExhaustSolver.ControllerUI=UI;
            }
            LastSolver=ExhaustSolver;
            ExhaustSolver.Setup(Net);                                 // setup Kruskal
            ExhaustSolver.Execute();                                  // run Kruskal
            ExhaustLength = ExhaustSolver.GetLength();
            ExhaustTour = ExhaustSolver.GetTour();
            ExhaustSolver.Shutdown();     
            
            System.out.println("E="+ExhaustLength);
            Exhaust.PutResults(ExhaustLength);
            DrawFinalTour(ExhaustSolver);
            
        }
        if(source==Nearest)
        {
            
            if(NearestSolver==null)
            {
                UserBase nullbase = new UserBase(UI.Services);      // UserBase for Kruskal
                NearestSolver = new TspSolverNearestNeighbor(nullbase);            // create Kruskal
                NearestSolver.ControllerUI=UI;
                Node start = Net.GetNode("1");
                NearestSolver.StartNode=start;
            }
            LastSolver=NearestSolver;
            NearestSolver.Setup(Net);                                 // setup Kruskal
            NearestSolver.Execute();                                  // run Kruskal
            NearestLength = NearestSolver.GetLength();
            NearestTour = NearestSolver.GetTour();
            NearestSolver.Shutdown(); 
            
            System.out.println("N="+NearestLength);
            Nearest.PutResults(NearestLength);
            DrawFinalTour(NearestSolver);           
        }
        if(source==TwoSwap)
        {
            if(TwoSwapSolver==null)
            {
                UserBase nullbase = new UserBase(UI.Services);      // UserBase for Kruskal
                TwoSwapSolver = new TspSolverTwoSwap(nullbase);            // create Kruskal
                TwoSwapSolver.ControllerUI=UI;
            }
            TwoSwapSolver.Setup(Net);                                 // setup Kruskal
            TwoSwapSolver.PutExistingTour(LastSolver);
            TwoSwapSolver.Execute();                                  // run Kruskal
            //TwoSwapSolver.PickEdges();
            TwoSwapLength = TwoSwapSolver.GetLength();
            TwoSwapTour = TwoSwapSolver.GetTour();
            TwoSwapSolver.Shutdown(); 
            
            System.out.println("N="+TwoSwapLength);
            TwoSwap.PutResults(TwoSwapLength);
            DrawFinalTour(TwoSwapSolver);           
        }
        
    }
    public void ResultsButton(RowDialogController source)
    {
        Net.DeleteAllEdges();
        
        if(source==Exhaust)
        {
            LastSolver=ExhaustSolver;
            DrawFinalTour(ExhaustSolver);           
        }
        if(source==Nearest)
        {
            LastSolver=NearestSolver;
            DrawFinalTour(NearestSolver);           
        }
    }
    
    /** Draws the tour.  There is likely a simpler way to do this */
    void DrawFinalTour(TspSolverBase tsp)
    {
        Edge edge;
        Node node1;
        Node node2 = null; 
        NodeListIter nit = new NodeListIter(tsp.GetTour());
        for(node1=nit.GetFirstNode();node1!=null;node1=nit.GetNextNode())
        {
            if(node2!=null)
            {
                edge = Net.CreateEdge(node1,node2);
                UI.AddEdge(edge);
                UI.PutLabel(edge,"");                       // no lable on edge               
                //UI.SingleStep(edge,EndSet); // move edge into EndSet
                UI.PutEdgeColor(edge,Color.green);

            }
            node2=node1;
        }
        node1 = nit.GetFirstNode();
        edge = Net.CreateEdge(node1,node2);
        UI.AddEdge(edge);
        UI.PutLabel(edge,"");                       // no lable on edge  
        UI.PutEdgeColor(edge,Color.green);
        
        //UI.SingleStep(edge,EndSet);     // move edge into EndSet        
        //UI.Trace(EndSet);               // trace the EndSets
        //UI.Update();
        UI.DrawNetwork();
    } 
    
}
